@@ -30,8 +30,8 @@ class HTMLListAccessible : public HyperTextAccessibleWrap {
                                        HyperTextAccessibleWrap)
 
   // Accessible
-  virtual a11y::role NativeRole() override;
-  virtual uint64_t NativeState() override;
+  virtual a11y::role NativeRole() const override;
+  virtual uint64_t NativeState() const override;
 
  protected:
   virtual ~HTMLListAccessible() {}
@@ -50,11 +50,12 @@ class HTMLLIAccessible : public HyperTextAccessibleWrap {
 
   // Accessible
   virtual void Shutdown() override;
-  virtual nsIntRect Bounds() const override;
-  virtual a11y::role NativeRole() override;
-  virtual uint64_t NativeState() override;
+  virtual nsRect BoundsInAppUnits() const override;
+  virtual a11y::role NativeRole() const override;
+  virtual uint64_t NativeState() const override;
 
   virtual bool InsertChildAt(uint32_t aIndex, Accessible* aChild) override;
+  virtual void RelocateChild(uint32_t aNewIndex, Accessible* aChild) override;
 
   // HTMLLIAccessible
   HTMLListBulletAccessible* Bullet() const { return mBullet; }
@@ -77,9 +78,9 @@ class HTMLListBulletAccessible : public LeafAccessible {
 
   // Accessible
   virtual nsIFrame* GetFrame() const override;
-  virtual ENameValueFlag Name(nsString& aName) override;
-  virtual a11y::role NativeRole() override;
-  virtual uint64_t NativeState() override;
+  virtual ENameValueFlag Name(nsString& aName) const override;
+  virtual a11y::role NativeRole() const override;
+  virtual uint64_t NativeState() const override;
   virtual void AppendTextTo(nsAString& aText, uint32_t aStartOffset = 0,
                             uint32_t aLength = UINT32_MAX) override;
 
