@@ -24,8 +24,9 @@ uint64_t NativeState() const;
 
 /*
  * Set aName to the name of the proxied accessible.
+ * Return the ENameValueFlag passed from Accessible::Name
  */
-void Name(nsString& aName) const;
+uint32_t Name(nsString& aName) const;
 
 /*
  * Set aValue to the value of the proxied accessible.
@@ -62,12 +63,14 @@ bool IsSearchbox() const;
 
 nsAtom* LandmarkRole() const;
 
-nsAtom* ARIARoleAtom() const;
+nsStaticAtom* ARIARoleAtom() const;
 
-int32_t GetLevelInternal();
+mozilla::a11y::GroupPos GroupPosition();
 void ScrollTo(uint32_t aScrollType);
 void ScrollToPoint(uint32_t aScrollType, int32_t aX, int32_t aY);
 
+void Announce(const nsString& aAnnouncement, uint16_t aPriority);
+
 int32_t CaretLineNumber();
 int32_t CaretOffset();
 void SetCaretOffset(int32_t aOffset);
@@ -247,6 +250,7 @@ ProxyAccessible* FocusedChild();
 ProxyAccessible* ChildAtPoint(int32_t aX, int32_t aY,
                               Accessible::EWhichChildAtPoint aWhichChild);
 nsIntRect Bounds();
+nsIntRect BoundsInCSSPixels();
 
 void Language(nsString& aLocale);
 void DocType(nsString& aType);
@@ -256,9 +260,6 @@ void MimeType(nsString aMime);
 void URLDocTypeMimeType(nsString& aURL, nsString& aDocType,
                         nsString& aMimeType);
 
-ProxyAccessible* AccessibleAtPoint(int32_t aX, int32_t aY,
-                                   bool aNeedsScreenCoords);
-
 void Extents(bool aNeedsScreenCoords, int32_t* aX, int32_t* aY, int32_t* aWidth,
              int32_t* aHeight);
 
