@@ -39,24 +39,27 @@ var FullZoom = {
 
   // nsISupports
 
-  QueryInterface: XPCOMUtils.generateQI([Ci.nsIDOMEventListener,
-                                         Ci.nsIObserver,
-                                         Ci.nsIContentPrefObserver,
-                                         Ci.nsISupportsWeakReference,
-                                         Ci.nsISupports]),
+  QueryInterface: ChromeUtils.generateQI([
+    Ci.nsIObserver,
+    Ci.nsIContentPrefObserver,
+    Ci.nsISupportsWeakReference,
+  ]),
 
   // Initialization & Destruction
 
   init: function FullZoom_init() {
-    gBrowser.addEventListener("ZoomChangeUsingMouseWheel", this);
+    gBrowser.addEventListener("DoZoomEnlargeBy10", this);
+    gBrowser.addEventListener("DoZoomReduceBy10", this);
 
     // Register ourselves with the service so we know when our pref changes.
-    this._cps2 = Cc["@mozilla.org/content-pref/service;1"].
-                 getService(Ci.nsIContentPrefService2);
+    this._cps2 = Cc["@mozilla.org/content-pref/service;1"].getService(
+      Ci.nsIContentPrefService2
+    );
     this._cps2.addObserverForName(this.name, this);
 
-    this.updateBackgroundTabs =
-      Services.prefs.getBoolPref("browser.zoom.updateBackgroundTabs");
+    this.updateBackgroundTabs = Services.prefs.getBoolPref(
+      "browser.zoom.updateBackgroundTabs"
+    );
 
     // Listen for changes to the browser.zoom branch so we can enable/disable
     // updating background tabs and per-site saving and restoring of zoom levels.
@@ -81,20 +84,21 @@ var FullZoom = {
   destroy: function FullZoom_destroy() {
     Services.prefs.removeObserver("browser.zoom.", this);
     this._cps2.removeObserverForName(this.name, this);
-    gBrowser.removeEventListener("ZoomChangeUsingMouseWheel", this);
+    gBrowser.removeEventListener("DoZoomEnlargeBy10", this);
+    gBrowser.removeEventListener("DoZoomReduceBy10", this);
   },
 
-
   // Event Handlers
 
-  // nsIDOMEventListener
+  // EventListener
 
   handleEvent: function FullZoom_handleEvent(event) {
     switch (event.type) {
-      case "ZoomChangeUsingMouseWheel":
-        let browser = this._getTargetedBrowser(event);
-        this._ignorePendingZoomAccesses(browser);
-        this._applyZoomToPref(browser);
+      case "DoZoomEnlargeBy10":
+        this.changeZoomBy(this._getTargetedBrowser(event), 0.1);
+        break;
+      case "DoZoomReduceBy10":
+        this.changeZoomBy(this._getTargetedBrowser(event), -0.1);
         break;
     }
   },
@@ -106,14 +110,15 @@ var FullZoom = {
       case "nsPref:changed":
         switch (aData) {
           case "privacy.resistFingerprinting":
-            // fall through
+          // fall through
           case "browser.zoom.siteSpecific":
             // Invalidate pref cache.
             this._siteSpecificPref = undefined;
             break;
           case "browser.zoom.updateBackgroundTabs":
-            this.updateBackgroundTabs =
-              Services.prefs.getBoolPref("browser.zoom.updateBackgroundTabs");
+            this.updateBackgroundTabs = Services.prefs.getBoolPref(
+              "browser.zoom.updateBackgroundTabs"
+            );
             break;
         }
         break;
@@ -122,11 +127,20 @@ var FullZoom = {
 
   // nsIContentPrefObserver
 
-  onContentPrefSet: function FullZoom_onContentPrefSet(aGroup, aName, aValue, aIsPrivate) {
+  onContentPrefSet: function FullZoom_onContentPrefSet(
+    aGroup,
+    aName,
+    aValue,
+    aIsPrivate
+  ) {
     this._onContentPrefChanged(aGroup, aValue, aIsPrivate);
   },
 
-  onContentPrefRemoved: function FullZoom_onContentPrefRemoved(aGroup, aName, aIsPrivate) {
+  onContentPrefRemoved: function FullZoom_onContentPrefRemoved(
+    aGroup,
+    aName,
+    aIsPrivate
+  ) {
     this._onContentPrefChanged(aGroup, undefined, aIsPrivate);
   },
 
@@ -138,7 +152,11 @@ var FullZoom = {
    * @param aValue  The new value of the changed preference.  Pass undefined to
    *                indicate the preference's removal.
    */
-  _onContentPrefChanged: function FullZoom__onContentPrefChanged(aGroup, aValue, aIsPrivate) {
+  _onContentPrefChanged: function FullZoom__onContentPrefChanged(
+    aGroup,
+    aValue,
+    aIsPrivate
+  ) {
     if (this._isNextContentPrefChangeInternal) {
       // Ignore changes that FullZoom itself makes.  This works because the
       // content pref service calls callbacks before notifying observers, and it
@@ -148,31 +166,37 @@ var FullZoom = {
     }
 
     let browser = gBrowser.selectedBrowser;
-    if (!browser.currentURI)
+    if (!browser.currentURI) {
       return;
+    }
+
+    if (this._isPDFViewer(browser)) {
+      return;
+    }
 
     let ctxt = this._loadContextFromBrowser(browser);
     let domain = this._cps2.extractDomain(browser.currentURI.spec);
     if (aGroup) {
-      if (aGroup == domain && ctxt.usePrivateBrowsing == aIsPrivate)
+      if (aGroup == domain && ctxt.usePrivateBrowsing == aIsPrivate) {
         this._applyPrefToZoom(aValue, browser);
+      }
       return;
     }
 
-    this._globalValue = aValue === undefined ? aValue :
-                          this._ensureValid(aValue);
-
     // If the current page doesn't have a site-specific preference, then its
     // zoom should be set to the new global preference now that the global
     // preference has changed.
     let hasPref = false;
     let token = this._getBrowserToken(browser);
     this._cps2.getByDomainAndName(browser.currentURI.spec, this.name, ctxt, {
-      handleResult() { hasPref = true; },
+      handleResult() {
+        hasPref = true;
+      },
       handleCompletion: () => {
-        if (!hasPref && token.isCurrent)
+        if (!hasPref && token.isCurrent) {
           this._applyPrefToZoom(undefined, browser);
-      }
+        }
+      },
     });
   },
 
@@ -189,7 +213,11 @@ var FullZoom = {
    * @param aBrowser
    *        (optional) browser object displaying the document
    */
-  onLocationChange: function FullZoom_onLocationChange(aURI, aIsTabSwitch, aBrowser) {
+  onLocationChange: function FullZoom_onLocationChange(
+    aURI,
+    aIsTabSwitch,
+    aBrowser
+  ) {
     let browser = aBrowser || gBrowser.selectedBrowser;
 
     // If we haven't been initialized yet but receive an onLocationChange
@@ -209,10 +237,30 @@ var FullZoom = {
       return;
     }
 
-    // Avoid the cps roundtrip and apply the default/global pref.
     if (aURI.spec == "about:blank") {
-      this._applyPrefToZoom(undefined, browser,
-                            this._notifyOnLocationChange.bind(this, browser));
+      if (
+        !browser.contentPrincipal ||
+        browser.contentPrincipal.isNullPrincipal
+      ) {
+        // For an about:blank with a null principal, zooming any amount does not
+        // make any sense - so simply do 100%.
+        this._applyPrefToZoom(
+          1,
+          browser,
+          this._notifyOnLocationChange.bind(this, browser)
+        );
+      } else {
+        // If it's not a null principal, there may be content loaded into it,
+        // so use the global pref. This will avoid a cps2 roundtrip if we've
+        // already loaded the global pref once. Really, this should probably
+        // use the contentPrincipal's origin if it's an http(s) principal.
+        // (See bug 1457597)
+        this._applyPrefToZoom(
+          undefined,
+          browser,
+          this._notifyOnLocationChange.bind(this, browser)
+        );
+      }
       return;
     }
 
@@ -224,12 +272,27 @@ var FullZoom = {
       return;
     }
 
+    // The PDF viewer zooming isn't handled by `ZoomManager`, ensure that the
+    // browser zoom level always gets reset to 100% on load (to prevent the
+    // UI elements of the PDF viewer from being zoomed in/out on load).
+    if (this._isPDFViewer(browser)) {
+      this._applyPrefToZoom(
+        1,
+        browser,
+        this._notifyOnLocationChange.bind(this, browser)
+      );
+      return;
+    }
+
     // See if the zoom pref is cached.
     let ctxt = this._loadContextFromBrowser(browser);
     let pref = this._cps2.getCachedByDomainAndName(aURI.spec, this.name, ctxt);
     if (pref) {
-      this._applyPrefToZoom(pref.value, browser,
-                            this._notifyOnLocationChange.bind(this, browser));
+      this._applyPrefToZoom(
+        pref.value,
+        browser,
+        this._notifyOnLocationChange.bind(this, browser)
+      );
       return;
     }
 
@@ -237,15 +300,20 @@ var FullZoom = {
     let value = undefined;
     let token = this._getBrowserToken(browser);
     this._cps2.getByDomainAndName(aURI.spec, this.name, ctxt, {
-      handleResult(resultPref) { value = resultPref.value; },
+      handleResult(resultPref) {
+        value = resultPref.value;
+      },
       handleCompletion: () => {
         if (!token.isCurrent) {
           this._notifyOnLocationChange(browser);
           return;
         }
-        this._applyPrefToZoom(value, browser,
-                              this._notifyOnLocationChange.bind(this, browser));
-      }
+        this._applyPrefToZoom(
+          value,
+          browser,
+          this._notifyOnLocationChange.bind(this, browser)
+        );
+      },
     });
   },
 
@@ -260,23 +328,66 @@ var FullZoom = {
   // Setting & Pref Manipulation
 
   /**
-   * Reduces the zoom level of the page in the current browser.
+   * If browser in reader mode sends message to reader in order to decrease font size,
+   * Otherwise reduces the zoom level of the page in the current browser.
    */
-  reduce: function FullZoom_reduce() {
-    ZoomManager.reduce();
+  async reduce() {
     let browser = gBrowser.selectedBrowser;
-    this._ignorePendingZoomAccesses(browser);
-    this._applyZoomToPref(browser);
+    if (browser.currentURI.spec.startsWith("about:reader")) {
+      browser.messageManager.sendAsyncMessage("Reader:ZoomOut");
+    } else if (this._isPDFViewer(browser)) {
+      browser.messageManager.sendAsyncMessage("PDFJS:ZoomOut");
+    } else {
+      ZoomManager.reduce();
+      this._ignorePendingZoomAccesses(browser);
+      await this._applyZoomToPref(browser);
+    }
   },
 
   /**
-   * Enlarges the zoom level of the page in the current browser.
+   * If browser in reader mode sends message to reader in order to increase font size,
+   * Otherwise enlarges the zoom level of the page in the current browser.
    */
-  enlarge: function FullZoom_enlarge() {
-    ZoomManager.enlarge();
+  async enlarge() {
     let browser = gBrowser.selectedBrowser;
-    this._ignorePendingZoomAccesses(browser);
-    this._applyZoomToPref(browser);
+    if (browser.currentURI.spec.startsWith("about:reader")) {
+      browser.messageManager.sendAsyncMessage("Reader:ZoomIn");
+    } else if (this._isPDFViewer(browser)) {
+      browser.messageManager.sendAsyncMessage("PDFJS:ZoomIn");
+    } else {
+      ZoomManager.enlarge();
+      this._ignorePendingZoomAccesses(browser);
+      await this._applyZoomToPref(browser);
+    }
+  },
+
+  /**
+   * If browser in reader mode sends message to reader in order to increase font size,
+   * Otherwise enlarges the zoom level of the page in the current browser.
+   * This function is not async like reduce/enlarge, because it is invoked by our
+   * event handler. This means that the call to _applyZoomToPref is not awaited and
+   * will happen asynchronously.
+   */
+  changeZoomBy(aBrowser, aValue) {
+    if (aBrowser.currentURI.spec.startsWith("about:reader")) {
+      const message = aValue > 0 ? "Reader::ZoomIn" : "Reader:ZoomOut";
+      aBrowser.messageManager.sendAsyncMessage(message);
+      return;
+    } else if (this._isPDFViewer(aBrowser)) {
+      const message = aValue > 0 ? "PDFJS::ZoomIn" : "PDFJS:ZoomOut";
+      aBrowser.messageManager.sendAsyncMessage(message);
+      return;
+    }
+    let zoom = ZoomManager.getZoomForBrowser(aBrowser);
+    zoom += aValue;
+    if (zoom < ZoomManager.MIN) {
+      zoom = ZoomManager.MIN;
+    } else if (zoom > ZoomManager.MAX) {
+      zoom = ZoomManager.MAX;
+    }
+    ZoomManager.setZoomForBrowser(aBrowser, zoom);
+    this._ignorePendingZoomAccesses(aBrowser);
+    this._applyZoomToPref(aBrowser);
   },
 
   /**
@@ -284,6 +395,9 @@ var FullZoom = {
    * point value, where 1 is the default zoom level.
    */
   setZoom(value, browser = gBrowser.selectedBrowser) {
+    if (this._isPDFViewer(browser)) {
+      return;
+    }
     ZoomManager.setZoomForBrowser(browser, value);
     this._ignorePendingZoomAccesses(browser);
     this._applyZoomToPref(browser);
@@ -296,10 +410,19 @@ var FullZoom = {
    * @return A promise which resolves when the zoom reset has been applied.
    */
   reset: function FullZoom_reset(browser = gBrowser.selectedBrowser) {
+    let forceValue;
+    if (browser.currentURI.spec.startsWith("about:reader")) {
+      browser.messageManager.sendAsyncMessage("Reader:ResetZoom");
+    } else if (this._isPDFViewer(browser)) {
+      browser.messageManager.sendAsyncMessage("PDFJS:ZoomReset");
+      // Ensure that the UI elements of the PDF viewer won't be zoomed in/out
+      // on reset, even if/when browser default zoom value is not set to 100%.
+      forceValue = 1;
+    }
     let token = this._getBrowserToken(browser);
-    let result = this._getGlobalValue(browser).then(value => {
+    let result = ZoomUI.getGlobalValue().then(value => {
       if (token.isCurrent) {
-        ZoomManager.setZoomForBrowser(browser, value === undefined ? 1 : value);
+        ZoomManager.setZoomForBrowser(browser, forceValue || value);
         this._ignorePendingZoomAccesses(browser);
       }
     });
@@ -330,8 +453,12 @@ var FullZoom = {
    * @param aBrowser   The zoom is set in this browser.  Required.
    * @param aCallback  If given, it's asynchronously called when complete.
    */
-  _applyPrefToZoom: function FullZoom__applyPrefToZoom(aValue, aBrowser, aCallback) {
-    if (!this.siteSpecific || gInPrintPreviewMode) {
+  _applyPrefToZoom: function FullZoom__applyPrefToZoom(
+    aValue,
+    aBrowser,
+    aCallback
+  ) {
+    if (gInPrintPreviewMode) {
       this._executeSoon(aCallback);
       return;
     }
@@ -339,22 +466,29 @@ var FullZoom = {
     // The browser is sometimes half-destroyed because this method is called
     // by content pref service callbacks, which themselves can be called at any
     // time, even after browsers are closed.
-    if (!aBrowser.parentNode || aBrowser.isSyntheticDocument) {
+    if (
+      !aBrowser.mInitialized ||
+      aBrowser.isSyntheticDocument ||
+      (!this.siteSpecific && aBrowser.tabHasCustomZoom)
+    ) {
       this._executeSoon(aCallback);
       return;
     }
 
-    if (aValue !== undefined) {
+    if (aValue !== undefined && this.siteSpecific) {
       ZoomManager.setZoomForBrowser(aBrowser, this._ensureValid(aValue));
       this._ignorePendingZoomAccesses(aBrowser);
       this._executeSoon(aCallback);
       return;
     }
 
+    // Above, we check if site-specific zoom is enabled before setting
+    // the tab browser zoom, however global zoom should work independent
+    // of the site-specific pref, so we do no checks here.
     let token = this._getBrowserToken(aBrowser);
-    this._getGlobalValue(aBrowser).then(value => {
+    ZoomUI.getGlobalValue().then(value => {
       if (token.isCurrent) {
-        ZoomManager.setZoomForBrowser(aBrowser, value === undefined ? 1 : value);
+        ZoomManager.setZoomForBrowser(aBrowser, value);
         this._ignorePendingZoomAccesses(aBrowser);
       }
       this._executeSoon(aCallback);
@@ -368,17 +502,31 @@ var FullZoom = {
    * @param browser  The zoom of this browser will be saved.  Required.
    */
   _applyZoomToPref: function FullZoom__applyZoomToPref(browser) {
-    if (!this.siteSpecific ||
-        gInPrintPreviewMode ||
-        browser.isSyntheticDocument)
-      return;
+    if (
+      !this.siteSpecific ||
+      gInPrintPreviewMode ||
+      browser.isSyntheticDocument
+    ) {
+      // If site-specific zoom is disabled, we have called this function
+      // to adjust our tab's zoom level. It is now considered "custom"
+      // and we mark that here.
+      browser.tabHasCustomZoom = !this.siteSpecific;
+      return null;
+    }
 
-    this._cps2.set(browser.currentURI.spec, this.name,
-                   ZoomManager.getZoomForBrowser(browser),
-                   this._loadContextFromBrowser(browser), {
-      handleCompletion: () => {
-        this._isNextContentPrefChangeInternal = true;
-      },
+    return new Promise(resolve => {
+      this._cps2.set(
+        browser.currentURI.spec,
+        this.name,
+        ZoomManager.getZoomForBrowser(browser),
+        this._loadContextFromBrowser(browser),
+        {
+          handleCompletion: () => {
+            this._isNextContentPrefChangeInternal = true;
+            resolve();
+          },
+        }
+      );
     });
   },
 
@@ -388,8 +536,9 @@ var FullZoom = {
    * @param browser  The zoom of this browser will be removed.  Required.
    */
   _removePref: function FullZoom__removePref(browser) {
-    if (browser.isSyntheticDocument)
+    if (browser.isSyntheticDocument) {
       return;
+    }
     let ctxt = this._loadContextFromBrowser(browser);
     this._cps2.removeByDomainAndName(browser.currentURI.spec, this.name, ctxt, {
       handleCompletion: () => {
@@ -414,8 +563,9 @@ var FullZoom = {
    */
   _getBrowserToken: function FullZoom__getBrowserToken(browser) {
     let map = this._browserTokenMap;
-    if (!map.has(browser))
+    if (!map.has(browser)) {
       map.set(browser, 0);
+    }
     return {
       token: map.get(browser),
       get isCurrent() {
@@ -424,14 +574,14 @@ var FullZoom = {
         // hasn't been received yet.  In that case, the browser is unusable, it
         // has no properties, so return false.  Check for this case by getting a
         // property, say, docShell.
-        return map.get(browser) === this.token && browser.parentNode;
+        return map.get(browser) === this.token && browser.mInitialized;
       },
     };
   },
 
   /**
    * Returns the browser that the supplied zoom event is associated with.
-   * @param event  The ZoomChangeUsingMouseWheel event.
+   * @param event  The zoom event.
    * @return  The associated browser element, if one exists, otherwise null.
    */
   _getTargetedBrowser: function FullZoom__getTargetedBrowser(event) {
@@ -439,18 +589,23 @@ var FullZoom = {
 
     // With remote content browsers, the event's target is the browser
     // we're looking for.
-    const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
-    if (target instanceof window.XULElement &&
-        target.localName == "browser" &&
-        target.namespaceURI == XUL_NS)
+    const XUL_NS =
+      "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
+    if (
+      target instanceof window.XULElement &&
+      target.localName == "browser" &&
+      target.namespaceURI == XUL_NS
+    ) {
       return target;
+    }
 
     // With in-process content browsers, the event's target is the content
     // document.
-    if (target.nodeType == Node.DOCUMENT_NODE)
-      return gBrowser.getBrowserForDocument(target);
+    if (target.nodeType == Node.DOCUMENT_NODE) {
+      return target.ownerGlobal.docShell.chromeEventHandler;
+    }
 
-    throw new Error("Unexpected ZoomChangeUsingMouseWheel event source");
+    throw new Error("Unexpected zoom event source");
   },
 
   /**
@@ -460,7 +615,9 @@ var FullZoom = {
    *
    * @param browser  Pending accesses in this browser will be ignored.
    */
-  _ignorePendingZoomAccesses: function FullZoom__ignorePendingZoomAccesses(browser) {
+  _ignorePendingZoomAccesses: function FullZoom__ignorePendingZoomAccesses(
+    browser
+  ) {
     let map = this._browserTokenMap;
     map.set(browser, (map.get(browser) || 0) + 1);
   },
@@ -468,45 +625,21 @@ var FullZoom = {
   _ensureValid: function FullZoom__ensureValid(aValue) {
     // Note that undefined is a valid value for aValue that indicates a known-
     // not-to-exist value.
-    if (isNaN(aValue))
+    if (isNaN(aValue)) {
       return 1;
+    }
 
-    if (aValue < ZoomManager.MIN)
+    if (aValue < ZoomManager.MIN) {
       return ZoomManager.MIN;
+    }
 
-    if (aValue > ZoomManager.MAX)
+    if (aValue > ZoomManager.MAX) {
       return ZoomManager.MAX;
+    }
 
     return aValue;
   },
 
-  /**
-   * Gets the global browser.content.full-zoom content preference.
-   *
-   * @param browser   The browser pertaining to the zoom.
-   * @returns Promise<prefValue>
-   *                  Resolves to the preference value when done.
-   */
-  _getGlobalValue: function FullZoom__getGlobalValue(browser) {
-    // * !("_globalValue" in this) => global value not yet cached.
-    // * this._globalValue === undefined => global value known not to exist.
-    // * Otherwise, this._globalValue is a number, the global value.
-    return new Promise(resolve => {
-      if ("_globalValue" in this) {
-        resolve(this._globalValue);
-        return;
-      }
-      let value = undefined;
-      this._cps2.getGlobal(this.name, this._loadContextFromBrowser(browser), {
-        handleResult(pref) { value = pref.value; },
-        handleCompletion: (reason) => {
-          this._globalValue = this._ensureValid(value);
-          resolve(this._globalValue);
-        }
-      });
-    });
-  },
-
   /**
    * Gets the load context from the given Browser.
    *
@@ -530,8 +663,17 @@ var FullZoom = {
   },
 
   _executeSoon: function FullZoom__executeSoon(callback) {
-    if (!callback)
+    if (!callback) {
       return;
+    }
     Services.tm.dispatchToMainThread(callback);
   },
+
+  _isPDFViewer(browser) {
+    return !!(
+      browser.contentPrincipal &&
+      browser.contentPrincipal.URI &&
+      browser.contentPrincipal.URI.spec == "resource://pdf.js/web/viewer.html"
+    );
+  },
 };
