@@ -1,14 +1,16 @@
 /* eslint-disable mozilla/no-arbitrary-setTimeout */
-ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
-ChromeUtils.import("resource://gre/modules/Services.jsm");
 
-const PREF_DISABLE_OPEN_NEW_WINDOW = "browser.link.open_newwindow.disabled_in_fullscreen";
-const PREF_BLOCK_TOPLEVEL_DATA = "security.data_uri.block_toplevel_data_uri_navigations";
-const isOSX = (Services.appinfo.OS === "Darwin");
+const PREF_DISABLE_OPEN_NEW_WINDOW =
+  "browser.link.open_newwindow.disabled_in_fullscreen";
+const PREF_BLOCK_TOPLEVEL_DATA =
+  "security.data_uri.block_toplevel_data_uri_navigations";
+const isOSX = Services.appinfo.OS === "Darwin";
 
 const TEST_FILE = "file_fullscreen-window-open.html";
-const gHttpTestRoot = getRootDirectory(gTestPath).replace("chrome://mochitests/content/",
-                                                          "http://127.0.0.1:8888/");
+const gHttpTestRoot = getRootDirectory(gTestPath).replace(
+  "chrome://mochitests/content/",
+  "http://127.0.0.1:8888/"
+);
 
 var newWin;
 var newBrowser;
@@ -60,7 +62,6 @@ function runNextTest() {
   }
 }
 
-
 // Test for window.open() with no feature.
 function test_open() {
   waitForTabOpen({
@@ -142,8 +143,11 @@ function test_open_with_dialog() {
 function test_open_when_open_new_window_by_pref() {
   const PREF_NAME = "browser.link.open_newwindow";
   Services.prefs.setIntPref(PREF_NAME, Ci.nsIBrowserDOMWindow.OPEN_NEWWINDOW);
-  is(Services.prefs.getIntPref(PREF_NAME), Ci.nsIBrowserDOMWindow.OPEN_NEWWINDOW,
-     PREF_NAME + " is nsIBrowserDOMWindow.OPEN_NEWWINDOW at this time");
+  is(
+    Services.prefs.getIntPref(PREF_NAME),
+    Ci.nsIBrowserDOMWindow.OPEN_NEWWINDOW,
+    PREF_NAME + " is nsIBrowserDOMWindow.OPEN_NEWWINDOW at this time"
+  );
 
   waitForTabOpen({
     message: {
@@ -171,15 +175,15 @@ function test_open_with_pref_to_disable_in_fullscreen() {
   });
 }
 
-
 // Test for window.open() called from chrome context.
 function test_open_from_chrome() {
   waitForWindowOpenFromChrome({
     message: {
       title: "test_open_from_chrome",
       param: "",
+      option: "noopener",
     },
-    finalizeFn() {}
+    finalizeFn() {},
   });
 }
 
@@ -200,8 +204,11 @@ function waitForTabOpen(aOptions) {
 
     let tab = aEvent.target;
     whenTabLoaded(tab, function() {
-      is(tab.linkedBrowser.contentTitle, message.title,
-         "Opened Tab is expected: " + message.title);
+      is(
+        tab.linkedBrowser.contentTitle,
+        message.title,
+        "Opened Tab is expected: " + message.title
+      );
 
       if (aOptions.successFn) {
         aOptions.successFn();
@@ -219,9 +226,10 @@ function waitForTabOpen(aOptions) {
     runNextTest();
   };
 
-  const URI = "data:text/html;charset=utf-8,<!DOCTYPE html><html><head><title>" +
-              message.title +
-              "<%2Ftitle><%2Fhead><body><%2Fbody><%2Fhtml>";
+  const URI =
+    "data:text/html;charset=utf-8,<!DOCTYPE html><html><head><title>" +
+    message.title +
+    "<%2Ftitle><%2Fhead><body><%2Fbody><%2Fhtml>";
 
   executeWindowOpenInContent({
     uri: URI,
@@ -230,7 +238,6 @@ function waitForTabOpen(aOptions) {
   });
 }
 
-
 function waitForWindowOpen(aOptions) {
   let message = aOptions.message;
   let url = aOptions.url || "about:blank";
@@ -251,10 +258,14 @@ function waitForWindowOpen(aOptions) {
     runNextTest();
   };
 
-  let listener = new WindowListener(message.title, getBrowserURL(), {
-    onSuccess: aOptions.successFn,
-    onFinalize,
-  });
+  let listener = new WindowListener(
+    message.title,
+    AppConstants.BROWSER_CHROME_URL,
+    {
+      onSuccess: aOptions.successFn,
+      onFinalize,
+    }
+  );
   Services.wm.addListener(listener);
 
   executeWindowOpenInContent({
@@ -265,11 +276,15 @@ function waitForWindowOpen(aOptions) {
 }
 
 function executeWindowOpenInContent(aParam) {
-  ContentTask.spawn(newBrowser.selectedBrowser, JSON.stringify(aParam), async function(dataTestParam) {
-    let testElm = content.document.getElementById("test");
-    testElm.setAttribute("data-test-param", dataTestParam);
-    testElm.click();
-  });
+  SpecialPowers.spawn(
+    newBrowser.selectedBrowser,
+    [JSON.stringify(aParam)],
+    async function(dataTestParam) {
+      let testElm = content.document.getElementById("test");
+      testElm.setAttribute("data-test-param", dataTestParam);
+      testElm.click();
+    }
+  );
 }
 
 function waitForWindowOpenFromChrome(aOptions) {
@@ -292,10 +307,14 @@ function waitForWindowOpenFromChrome(aOptions) {
     runNextTest();
   };
 
-  let listener = new WindowListener(message.title, getBrowserURL(), {
-    onSuccess: aOptions.successFn,
-    onFinalize,
-  });
+  let listener = new WindowListener(
+    message.title,
+    AppConstants.BROWSER_CHROME_URL,
+    {
+      onSuccess: aOptions.successFn,
+      onFinalize,
+    }
+  );
   Services.wm.addListener(listener);
 
   newWin.open(url, message.title, message.option);
@@ -308,7 +327,6 @@ function WindowListener(aTitle, aUrl, aCallBackObj) {
   this.callBack_onFinalize = aCallBackObj.onFinalize;
 }
 WindowListener.prototype = {
-
   test_title: null,
   test_url: null,
   callback_onSuccess: null,
@@ -317,11 +335,13 @@ WindowListener.prototype = {
   onOpenWindow(aXULWindow) {
     Services.wm.removeListener(this);
 
-    let domwindow = aXULWindow.QueryInterface(Ci.nsIInterfaceRequestor)
-                    .getInterface(Ci.nsIDOMWindow);
+    let domwindow = aXULWindow.docShell.domWindow;
     let onLoad = aEvent => {
-      is(domwindow.document.location.href, this.test_url,
-        "Opened Window is expected: " + this.test_title);
+      is(
+        domwindow.document.location.href,
+        this.test_url,
+        "Opened Window is expected: " + this.test_title
+      );
       if (this.callback_onSuccess) {
         this.callback_onSuccess();
       }
@@ -342,6 +362,5 @@ WindowListener.prototype = {
     domwindow.addEventListener("load", onLoad, true);
   },
   onCloseWindow(aXULWindow) {},
-  QueryInterface: XPCOMUtils.generateQI([Ci.nsIWindowMediatorListener,
-                                         Ci.nsISupports]),
+  QueryInterface: ChromeUtils.generateQI([Ci.nsIWindowMediatorListener]),
 };
