@@ -16,19 +16,21 @@ PromiseTestUtils.whitelistRejectionsGlobally(/NS_ERROR_ILLEGAL_VALUE/);
 
 const kEnginePref = "browser.translation.engine";
 const kApiKeyPref = "browser.translation.yandex.apiKeyOverride";
+const kDetectLanguagePref = "browser.translation.detectLanguage";
 const kShowUIPref = "browser.translation.ui.show";
 
-const {Translation} = ChromeUtils.import("resource:///modules/translation/Translation.jsm", {});
+const { Translation } = ChromeUtils.import(
+  "resource:///modules/translation/TranslationParent.jsm"
+);
 
 add_task(async function setup() {
-  Services.prefs.setCharPref(kEnginePref, "yandex");
-  Services.prefs.setCharPref(kApiKeyPref, "yandexValidKey");
-  Services.prefs.setBoolPref(kShowUIPref, true);
-
-  registerCleanupFunction(function() {
-    Services.prefs.clearUserPref(kEnginePref);
-    Services.prefs.clearUserPref(kApiKeyPref);
-    Services.prefs.clearUserPref(kShowUIPref);
+  await SpecialPowers.pushPrefEnv({
+    set: [
+      [kEnginePref, "Yandex"],
+      [kApiKeyPref, "yandexValidKey"],
+      [kDetectLanguagePref, true],
+      [kShowUIPref, true],
+    ],
   });
 });
 
@@ -37,7 +39,6 @@ add_task(async function setup() {
  * a sample page.
  */
 add_task(async function test_yandex_translation() {
-
   // Loading the fixture page.
   let url = constructFixtureURL("bug1022725-fr.html");
   let tab = await promiseTestPageLoad(url);
@@ -46,12 +47,19 @@ add_task(async function test_yandex_translation() {
   gBrowser.selectedTab = tab;
   let browser = tab.linkedBrowser;
 
-  await ContentTask.spawn(browser, null, async function() {
-    ChromeUtils.import("resource:///modules/translation/TranslationDocument.jsm");
-    ChromeUtils.import("resource:///modules/translation/YandexTranslator.jsm");
+  await SpecialPowers.spawn(browser, [], async function() {
+    const { TranslationDocument } = ChromeUtils.import(
+      "resource:///modules/translation/TranslationDocument.jsm"
+    );
+    const { YandexTranslator } = ChromeUtils.import(
+      "resource:///modules/translation/YandexTranslator.jsm"
+    );
 
     let client = new YandexTranslator(
-      new TranslationDocument(content.document), "fr", "en");
+      new TranslationDocument(content.document),
+      "fr",
+      "en"
+    );
     let result = await client.translate();
 
     Assert.ok(result, "There should be a result.");
@@ -76,19 +84,18 @@ add_task(async function test_yandex_attribution() {
   gBrowser.removeTab(tab);
 });
 
-
 add_task(async function test_preference_attribution() {
-    let prefUrl = "about:preferences#general";
-    let waitPrefLoaded = TestUtils.topicObserved("sync-pane-loaded", () => true);
-    let tab = await promiseTestPageLoad(prefUrl);
-    await waitPrefLoaded;
-    let browser = gBrowser.getBrowserForTab(tab);
-    let win = browser.contentWindow;
-    let bingAttribution = win.document.getElementById("bingAttribution");
-    ok(bingAttribution, "Bing attribution should exist.");
-    ok(bingAttribution.hidden, "Bing attribution should be hidden.");
+  let prefUrl = "about:preferences#general";
+  let waitPrefLoaded = TestUtils.topicObserved("sync-pane-loaded", () => true);
+  let tab = await promiseTestPageLoad(prefUrl);
+  await waitPrefLoaded;
+  let browser = gBrowser.getBrowserForTab(tab);
+  let win = browser.contentWindow;
+  let bingAttribution = win.document.getElementById("bingAttribution");
+  ok(bingAttribution, "Bing attribution should exist.");
+  ok(bingAttribution.hidden, "Bing attribution should be hidden.");
 
-    gBrowser.removeTab(tab);
+  gBrowser.removeTab(tab);
 });
 
 /**
@@ -99,11 +106,15 @@ add_task(async function test_preference_attribution() {
  */
 function constructFixtureURL(filename) {
   // Deduce the Mochitest server address in use from a pref that was pre-processed.
-  let server = Services.prefs.getCharPref("browser.translation.yandex.translateURLOverride")
-                             .replace("http://", "");
+  let server = Services.prefs
+    .getCharPref("browser.translation.yandex.translateURLOverride")
+    .replace("http://", "");
   server = server.substr(0, server.indexOf("/"));
-  let url = "http://" + server +
-    "/browser/browser/components/translation/test/fixtures/" + filename;
+  let url =
+    "http://" +
+    server +
+    "/browser/browser/components/translation/test/fixtures/" +
+    filename;
   return url;
 }
 
@@ -114,9 +125,13 @@ function constructFixtureURL(filename) {
  */
 function promiseTestPageLoad(url) {
   return new Promise(resolve => {
-    let tab = gBrowser.selectedTab = BrowserTestUtils.addTab(gBrowser, url);
+    let tab = (gBrowser.selectedTab = BrowserTestUtils.addTab(gBrowser, url));
     let browser = gBrowser.selectedBrowser;
-    BrowserTestUtils.browserLoaded(browser, false, (loadurl) => loadurl != "about:blank").then(() => {
+    BrowserTestUtils.browserLoaded(
+      browser,
+      false,
+      loadurl => loadurl != "about:blank"
+    ).then(() => {
       info("Page loaded: " + browser.currentURI.spec);
       resolve(tab);
     });
@@ -125,9 +140,14 @@ function promiseTestPageLoad(url) {
 
 function showTranslationUI(tab, aDetectedLanguage) {
   let browser = gBrowser.selectedBrowser;
-  Translation.documentStateReceived(browser, {state: Translation.STATE_OFFER,
-                                              originalShown: true,
-                                              detectedLanguage: aDetectedLanguage});
-  let ui = browser.translationUI;
-  return ui.notificationBox.getNotificationWithValue("translation");
+  let actor = browser.browsingContext.currentWindowGlobal.getActor(
+    "Translation"
+  );
+  actor.documentStateReceived({
+    state: Translation.STATE_OFFER,
+    originalShown: true,
+    detectedLanguage: aDetectedLanguage,
+  });
+
+  return actor.notificationBox.getNotificationWithValue("translation");
 }
