@@ -46,7 +46,7 @@ var TodayPane = {
     TodayPane.showTodayPaneStatusLabel();
 
     document.getElementById("today-splitter").addEventListener("command", () => {
-      document.dispatchEvent(new CustomEvent("viewresize", { bubbles: true }));
+      window.dispatchEvent(new CustomEvent("viewresize"));
     });
 
     Services.obs.addObserver(TodayPane, "defaultTimezoneChanged");
@@ -129,7 +129,7 @@ var TodayPane = {
       }
     }
 
-    document.dispatchEvent(new CustomEvent("viewresize", { bubbles: true }));
+    window.dispatchEvent(new CustomEvent("viewresize"));
   },
 
   /**
