@@ -46,7 +46,7 @@ add_task(async function testEventDialog() {
   createCalendar(controller, CALENDARNAME);
   // Since from other tests we may be elsewhere, make sure we start today.
   switchToView(controller, "day");
-  goToDate(controller, now.getFullYear(), now.getMonth() + 1, now.getDate());
+  goToDate(controller, now.getUTCFullYear(), now.getUTCMonth() + 1, now.getUTCDate());
   viewBack(controller, 1);
 
   // Open month view.
@@ -56,7 +56,7 @@ add_task(async function testEventDialog() {
 
   // Setup start- & endTime.
   // Next full hour except last hour of the day.
-  let hour = now.getHours();
+  let hour = now.getUTCHours();
   let startHour = hour == 23 ? hour : (hour + 1) % 24;
 
   let nextHour = cal.dtz.now();
@@ -110,7 +110,7 @@ add_task(async function testEventDialog() {
       description: EVENTDESCRIPTION,
       categories: [category],
       repeat: "daily",
-      repeatuntil: cal.dtz.dateTimeToJsDate(untildate),
+      repeatuntil: untildate,
       reminder: "5minutes",
       privacy: "private",
       attachment: { add: EVENTURL },
