@@ -7,7 +7,7 @@ var { Services } = ChromeUtils.import("resource:///modules/imServices.jsm");
 // _findCommands in order to avoid having to intercept command execution.
 var imCommands = {};
 Services.scriptloader.loadSubScript(
-  "resource:///components/imCommands.js",
+  "resource:///modules/imCommands.jsm",
   imCommands
 );
 
@@ -211,7 +211,7 @@ function run_test() {
 
     // Check which commands are available in which context.
     let cmdlist = cmdserv
-      .listCommandsForConversation(test.conv, {})
+      .listCommandsForConversation(test.conv)
       .map(aCmd => aCmd.name)
       .sort()
       .join(", ");
