@@ -3,6 +3,11 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+/* import-globals-from emailWizard.js */
+var { cleanUpHostName, isLegalHostNameOrIP } = ChromeUtils.import(
+  "resource:///modules/hostnameUtils.jsm"
+);
+
 /**
  * This is a generic input validation lib. Use it when you process
  * data from the network.
@@ -16,57 +21,57 @@
  * they throw exceptions.
  */
 
-ChromeUtils.import("resource:///modules/hostnameUtils.jsm");
-
-var sanitize =
-{
-  integer : function(unchecked)
-  {
-    if (typeof(unchecked) == "number" && !isNaN(unchecked))
+var sanitize = {
+  integer(unchecked) {
+    if (typeof unchecked == "number" && !isNaN(unchecked)) {
       return unchecked;
+    }
 
     var r = parseInt(unchecked);
-    if (isNaN(r))
+    if (isNaN(r)) {
       throw new MalformedException("no_number.error", unchecked);
+    }
 
     return r;
   },
 
-  integerRange : function(unchecked, min, max)
-  {
+  integerRange(unchecked, min, max) {
     var int = this.integer(unchecked);
-    if (int < min)
+    if (int < min) {
       throw new MalformedException("number_too_small.error", unchecked);
+    }
 
-    if (int > max)
+    if (int > max) {
       throw new MalformedException("number_too_large.error", unchecked);
+    }
 
     return int;
   },
 
-  boolean : function(unchecked)
-  {
-    if (typeof(unchecked) == "boolean")
+  boolean(unchecked) {
+    if (typeof unchecked == "boolean") {
       return unchecked;
+    }
 
-    if (unchecked == "true")
+    if (unchecked == "true") {
       return true;
+    }
 
-    if (unchecked == "false")
+    if (unchecked == "false") {
       return false;
+    }
 
     throw new MalformedException("boolean.error", unchecked);
   },
 
-  string : function(unchecked)
-  {
+  string(unchecked) {
     return String(unchecked);
   },
 
-  nonemptystring : function(unchecked)
-  {
-    if (!unchecked)
+  nonemptystring(unchecked) {
+    if (!unchecked) {
       throw new MalformedException("string_empty.error", unchecked);
+    }
 
     return this.string(unchecked);
   },
@@ -76,11 +81,11 @@ var sanitize =
    *
    * Empty strings not allowed (good idea?).
    */
-  alphanumdash : function(unchecked)
-  {
+  alphanumdash(unchecked) {
     var str = this.nonemptystring(unchecked);
-    if (!/^[a-zA-Z0-9\-\_]*$/.test(str))
+    if (!/^[a-zA-Z0-9\-\_]*$/.test(str)) {
       throw new MalformedException("alphanumdash.error", unchecked);
+    }
 
     return str;
   },
@@ -91,28 +96,41 @@ var sanitize =
    * Empty strings not allowed.
    * Currently does not support IDN (international domain names).
    */
-  hostname : function(unchecked)
-  {
+  hostname(unchecked) {
     let str = cleanUpHostName(this.nonemptystring(unchecked));
 
     // Allow placeholders. TODO move to a new hostnameOrPlaceholder()
     // The regex is "anything, followed by one or more (placeholders than
     // anything)".  This doesn't catch the non-placeholder case, but that's
     // handled down below.
-    if (/^[a-zA-Z0-9\-\.]*(%[A-Z0-9]+%[a-zA-Z0-9\-\.]*)+$/.test(str))
+    if (/^[a-zA-Z0-9\-\.]*(%[A-Z0-9]+%[a-zA-Z0-9\-\.]*)+$/.test(str)) {
       return str;
+    }
 
-    if (!isLegalHostNameOrIP(str))
+    if (!isLegalHostNameOrIP(str)) {
       throw new MalformedException("hostname_syntax.error", unchecked);
+    }
 
     return str.toLowerCase();
   },
+
+  /**
+   * A value which resembles an email address.
+   */
+  emailAddress(unchecked) {
+    let str = this.nonemptystring(unchecked);
+    if (!/^[a-z0-9\-%+_\.\*]+@[a-z0-9\-\.]+\.[a-z]+$/i.test(str)) {
+      throw new MalformedException("emailaddress_syntax.error", unchecked);
+    }
+
+    return str.toLowerCase();
+  },
+
   /**
    * A non-chrome URL that's safe to request.
    */
-  url : function (unchecked)
-  {
-    var str =  this.string(unchecked);
+  url(unchecked) {
+    var str = this.string(unchecked);
 
     // DANGER ZONE: data:text/javascript or data:text/html can contain
     // JavaScript code, run in the caller's security context, and might allow
@@ -120,8 +138,10 @@ var sanitize =
     // PNG and JPEG data: URLs are fine.  But SVG is again dangerous,
     // it can contain javascript, so it would create a critical security hole.
     // Talk to BenB or bz before relaxing *any* of the checks in this function.
-    if (str.startsWith("data:image/png;") ||
-        str.startsWith("data:image/jpeg;")) {
+    if (
+      str.startsWith("data:image/png;") ||
+      str.startsWith("data:image/jpeg;")
+    ) {
       return new URL(str).href;
     }
 
@@ -137,8 +157,9 @@ var sanitize =
       throw new MalformedException("url_parsing.error", unchecked);
     }
 
-    if (uri.scheme != "http" && uri.scheme != "https")
+    if (uri.scheme != "http" && uri.scheme != "https") {
       throw new MalformedException("url_scheme.error", unchecked);
+    }
 
     return uri.spec;
   },
@@ -146,8 +167,7 @@ var sanitize =
   /**
    * A value which should be shown to the user in the UI as label
    */
-  label : function(unchecked)
-  {
+  label(unchecked) {
     return this.string(unchecked);
   },
 
@@ -162,16 +182,16 @@ var sanitize =
    *       no |defaultValue| is passed.
    * @throws MalformedException
    */
-  enum : function(unchecked, allowedValues, defaultValue)
-  {
-    for (let allowedValue of allowedValues)
-    {
-      if (allowedValue == unchecked)
+  enum(unchecked, allowedValues, defaultValue) {
+    for (let allowedValue of allowedValues) {
+      if (allowedValue == unchecked) {
         return allowedValue;
+      }
     }
     // value is bad
-    if (typeof(defaultValue) == "undefined")
+    if (typeof defaultValue == "undefined") {
       throw new MalformedException("allowed_value.error", unchecked);
+    }
     return defaultValue;
   },
 
@@ -192,29 +212,29 @@ var sanitize =
    *       no |defaultValue| is passed.
    * @throws MalformedException
    */
-  translate : function(unchecked, mapping, defaultValue)
-  {
-    for (var inputValue in mapping)
-    {
-      if (inputValue == unchecked)
+  translate(unchecked, mapping, defaultValue) {
+    for (var inputValue in mapping) {
+      if (inputValue == unchecked) {
         return mapping[inputValue];
+      }
     }
     // value is bad
-    if (typeof(defaultValue) == "undefined")
+    if (typeof defaultValue == "undefined") {
       throw new MalformedException("allowed_value.error", unchecked);
+    }
     return defaultValue;
-  }
+  },
 };
 
-function MalformedException(msgID, uncheckedBadValue)
-{
+function MalformedException(msgID, uncheckedBadValue) {
   var stringBundle = getStringBundle(
-      "chrome://messenger/locale/accountCreationUtil.properties");
+    "chrome://messenger/locale/accountCreationUtil.properties"
+  );
   var msg = stringBundle.GetStringFromName(msgID);
-  if (kDebug)
-    msg += " (bad value: " + new String(uncheckedBadValue) + ")";
+  if (typeof kDebug != "undefined" && kDebug) {
+    msg += " (bad value: " + uncheckedBadValue + ")";
+  }
   Exception.call(this, msg);
 }
 MalformedException.prototype = Object.create(Exception.prototype);
 MalformedException.prototype.constructor = MalformedException;
-
