@@ -4,7 +4,6 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* import-globals-from ../../../../mailnews/addrbook/content/abResultsPane.js */
 /* import-globals-from abCommon.js */
 
 var { getSearchTokens, getModelQuery, generateQueryURI } = ChromeUtils.import(
@@ -16,6 +15,19 @@ function GetAbViewListener() {
   return null;
 }
 
+/**
+ * Handle the command event on abContextMenuButton (click, Enter, spacebar).
+ */
+function abContextMenuButtonOnCommand(event) {
+  showContextMenu("sidebarAbContextMenu", event, [
+    event.target,
+    "after_end",
+    0,
+    0,
+    true,
+  ]);
+}
+
 /**
  * Handle the context menu event of results tree (right-click, context menu key
  * press, etc.). Show the respective context menu for selected contact(s) or
@@ -98,10 +110,8 @@ function addSelectedAddresses(aRecipientType) {
   var cards = GetSelectedAbCards();
 
   // Turn each card into a properly formatted address.
-  var addressArray = cards
-    .map(GenerateAddressFromCard)
-    .filter(addr => addr != "");
-  parent.AddRecipientsArray(aRecipientType, addressArray);
+  let addresses = cards.map(makeMailboxObjectFromCard).filter(addr => addr);
+  parent.awAddRecipientsArray(aRecipientType, addresses);
 }
 
 function AddressBookMenuListChange(aValue) {
@@ -225,16 +235,17 @@ function onEnterInSearchBar() {
   }
 
   let searchURI = getSelectedDirectoryURI();
+  let searchQuery;
   let searchInput = document.getElementById("peopleSearchInput");
 
   // Use helper method to split up search query to multi-word search
   // query against multiple fields.
   if (searchInput) {
     let searchWords = getSearchTokens(searchInput.value);
-    searchURI += generateQueryURI(gQueryURIFormat, searchWords);
+    searchQuery = generateQueryURI(gQueryURIFormat, searchWords);
   }
 
-  SetAbView(searchURI);
+  SetAbView(searchURI, searchQuery);
 }
 
 /**
