@@ -8,23 +8,9 @@ this.tabs = class extends ExtensionAPI {
   getAPI(context) {
     return {
       tabs: {
-        connect(tabId, connectInfo) {
-          let name = "";
-          if (connectInfo && connectInfo.name !== null) {
-            name = connectInfo.name;
-          }
-          let recipient = {
-            extensionId: context.extension.id,
-            tabId,
-          };
-          if (connectInfo && connectInfo.frameId !== null) {
-            recipient.frameId = connectInfo.frameId;
-          }
-          return context.messenger.connect(
-            context.messageManager,
-            name,
-            recipient
-          );
+        connect(tabId, options) {
+          let { frameId = null, name = "" } = options || {};
+          return context.messenger.nm.connect({ name, tabId, frameId });
         },
 
         sendMessage(tabId, message, options, responseCallback) {
