@@ -557,7 +557,7 @@ this.addressBook = class extends ExtensionAPI {
             promises.push(
               new Promise(resolve => {
                 book.item.search(searchQuery, {
-                  onSearchFinished(result, errorMsg) {
+                  onSearchFinished(status, secInfo, location) {
                     resolve();
                   },
                   onSearchFoundCard(contact) {
