@@ -79,27 +79,113 @@ async function openComposeWindow(relatedMessageId, type, details, extension) {
     });
   }
 
-  // ForwardInline is totally broken, see bug 1513824.
-  if (type == Ci.nsIMsgCompType.ForwardInline) {
+  let format = Ci.nsIMsgCompFormat.Default;
+  let identity = null;
+
+  if (details) {
+    if (details.body !== null && details.plainTextBody !== null) {
+      throw new ExtensionError(
+        "Only one of body and plainTextBody can be specified."
+      );
+    }
+
+    if (details.isPlainText === false || details.body !== null) {
+      if (details.plainTextBody !== null) {
+        throw new ExtensionError(
+          "Cannot specify plainTextBody when isPlainText is false. Use body instead."
+        );
+      }
+      format = Ci.nsIMsgCompFormat.HTML;
+    }
+    if (details.isPlainText === true || details.plainTextBody !== null) {
+      if (details.body !== null) {
+        throw new ExtensionError(
+          "Cannot specify body when isPlainText is true. Use plainTextBody instead."
+        );
+      }
+      format = Ci.nsIMsgCompFormat.PlainText;
+    }
+    if (details.identityId !== null) {
+      if (!extension.hasPermission("accountsRead")) {
+        throw new ExtensionError(
+          'Using identities requires the "accountsRead" permission'
+        );
+      }
+
+      identity = MailServices.accounts.allIdentities.find(
+        i => i.key == details.identityId
+      );
+      if (!identity) {
+        throw new ExtensionError(`Identity not found: ${details.identityId}`);
+      }
+    }
+  }
+
+  // ForwardInline is totally broken, see bug 1513824. Fake it 'til we make it.
+  if (
+    [
+      Ci.nsIMsgCompType.ForwardInline,
+      Ci.nsIMsgCompType.EditAsNew,
+      Ci.nsIMsgCompType.Template,
+    ].includes(type)
+  ) {
     let msgHdr = null;
     let msgURI = null;
-    let hdrIdentity = null;
     if (relatedMessageId) {
       msgHdr = messageTracker.getMessage(relatedMessageId);
       msgURI = msgHdr.folder.getUriForMsg(msgHdr);
     }
     let newWindowPromise = waitForWindow();
+
+    // For the types in this code path, OpenComposeWindow only uses
+    // nsIMsgCompFormat.Default or OppositeOfDefault. Check which is needed.
+    // See https://hg.mozilla.org/comm-central/file/592fb5c396ebbb75d4acd1f1287a26f56f4164b3/mailnews/compose/src/nsMsgComposeService.cpp#l395
+    if (format != Ci.nsIMsgCompFormat.Default) {
+      // The mimeConverter used in this code path is not setting any format but
+      // defaults to plaintext if no identity and also no default account is set.
+      // The "mail.identity.default.compose_html" preference is NOT used.
+      let usedIdentity =
+        identity || MailServices.accounts.defaultAccount?.defaultIdentity;
+      let defaultFormat = usedIdentity?.composeHtml
+        ? Ci.nsIMsgCompFormat.HTML
+        : Ci.nsIMsgCompFormat.PlainText;
+      format =
+        format == defaultFormat
+          ? Ci.nsIMsgCompFormat.Default
+          : Ci.nsIMsgCompFormat.OppositeOfDefault;
+    }
+
     MailServices.compose.OpenComposeWindow(
       null,
       msgHdr,
       msgURI,
       type,
-      0,
-      hdrIdentity,
+      format,
+      identity,
       null,
       null
     );
-    return newWindowPromise;
+    let composeWindow = await newWindowPromise;
+    if (details) {
+      await setComposeDetails(composeWindow, details, extension);
+
+      if (details.attachments != null) {
+        let attachments = [];
+        for (let data of details.attachments) {
+          let attachment = Cc[
+            "@mozilla.org/messengercompose/attachment;1"
+          ].createInstance(Ci.nsIMsgAttachment);
+          attachment.name = data.name || data.file.name;
+          attachment.size = data.file.size;
+          attachment.url = await fileURLForFile(data.file);
+          attachments.push(attachment);
+        }
+        composeWindow.AddAttachments(attachments);
+      }
+
+      composeWindow.gContentChanged = false;
+    }
+    return composeWindow;
   }
 
   let params = Cc[
@@ -113,36 +199,14 @@ async function openComposeWindow(relatedMessageId, type, details, extension) {
     let msgHdr = messageTracker.getMessage(relatedMessageId);
     params.originalMsgURI = msgHdr.folder.getUriForMsg(msgHdr);
   }
+
   params.type = type;
+  params.format = format;
+  if (identity) {
+    params.identity = identity;
+  }
+
   if (details) {
-    if (details.body !== null) {
-      if (details.plainTextBody !== null) {
-        throw new ExtensionError(
-          "Only one of body and plainTextBody can be specified."
-        );
-      }
-      if (details.isPlainText) {
-        throw new ExtensionError(
-          "Cannot specify body when isPlainText is true. Use plainTextBody instead."
-        );
-      }
-    }
-
-    if (details.identityId !== null) {
-      if (!extension.hasPermission("accountsRead")) {
-        throw new ExtensionError(
-          'Using identities requires the "accountsRead" permission'
-        );
-      }
-
-      let identity = MailServices.accounts.allIdentities.find(
-        i => i.key == details.identityId
-      );
-      if (!identity) {
-        throw new ExtensionError(`Identity not found: ${details.identityId}`);
-      }
-      params.identity = identity;
-    }
     for (let field of ["to", "cc", "bcc", "replyTo", "followupTo"]) {
       composeFields[field] = await parseComposeRecipientList(details[field]);
     }
@@ -160,9 +224,6 @@ async function openComposeWindow(relatedMessageId, type, details, extension) {
       composeFields.body = details.body;
     }
     if (details.plainTextBody != null) {
-      if (details.isPlainText) {
-        params.format = Ci.nsIMsgCompFormat.PlainText;
-      }
       composeFields.body = details.plainTextBody;
     }
 
@@ -228,6 +289,19 @@ async function setComposeDetails(composeWindow, details, extension) {
       "Only one of body and plainTextBody can be specified."
     );
   }
+  // Check for body usage on a plain text composer and throw a helpful error.
+  // Otherwise, this will throw a NS_UNEXPECTED_ERROR later.
+  if (details.body && !composeWindow.IsHTMLEditor()) {
+    throw new ExtensionError(
+      "Cannot use body on a plain text compose window. Use plainTextBody instead."
+    );
+  }
+  // For consistency, using plainTextBody on a html compsoser is not allowed.
+  if (details.plainTextBody && composeWindow.IsHTMLEditor()) {
+    throw new ExtensionError(
+      "Cannot use plainTextBody on an HTML compose window. Use body instead."
+    );
+  }
 
   if (details.identityId) {
     if (!extension.hasPermission("accountsRead")) {
@@ -512,10 +586,18 @@ this.compose = class extends ExtensionAPI {
             };
           },
         }).api(),
-        async beginNew(details) {
+        async beginNew(messageId, details) {
+          let type = Ci.nsIMsgCompType.New;
+          if (messageId) {
+            let msgHdr = messageTracker.getMessage(messageId);
+            type =
+              msgHdr.flags & Ci.nsMsgMessageFlags.Template
+                ? Ci.nsIMsgCompType.Template
+                : Ci.nsIMsgCompType.EditAsNew;
+          }
           let composeWindow = await openComposeWindow(
-            null,
-            Ci.nsIMsgCompType.New,
+            messageId,
+            type,
             details,
             extension
           );
