@@ -1445,6 +1445,9 @@ class WindowManager extends WindowManagerBase {
  * @return {Object}
  */
 function convertMailIdentity(account, identity) {
+  if (!account || !identity) {
+    return null;
+  }
   identity = identity.QueryInterface(Ci.nsIMsgIdentity);
   return {
     accountId: account.key,
@@ -1454,6 +1457,7 @@ function convertMailIdentity(account, identity) {
     email: identity.email,
     replyTo: identity.replyTo,
     organization: identity.organization,
+    composeHtml: identity.composeHtml,
   };
 }
 
@@ -1466,7 +1470,19 @@ function convertMailIdentity(account, identity) {
  * Convert a folder URI to a human-friendly path.
  * @return {String}
  */
-function folderURIToPath(uri) {
+function folderURIToPath(accountId, uri) {
+  let server = MailServices.accounts.getAccount(accountId).incomingServer;
+  let rootURI = server.rootFolder.URI;
+  if (rootURI == uri) {
+    return "/";
+  }
+  // The .URI property of an IMAP folder doesn't have %-encoded characters, but
+  // may include literal % chars. Services.io.newURI(uri) applies encodeURI to
+  // the returned filePath, but will not encode any literal % chars, which will
+  // cause decodeURIComponent to fail (bug 1707408).
+  if (server.type == "imap") {
+    return uri.substring(rootURI.length);
+  }
   let path = Services.io.newURI(uri).filePath;
   return path
     .split("/")
@@ -1516,7 +1532,11 @@ const folderTypeMap = new Map([
 ]);
 
 /**
- * Converts an nsIMsgFolder to a simple object for use in messages.
+ * Converts an nsIMsgFolder to a simple object for use in API messages.
+ *
+ * @param {nsIMsgFolder} folder - The folder to convert.
+ * @param {string} [accountId] - An optimization to avoid looking up the
+ *     account. The value from nsIMsgHdr.accountKey must not be used here.
  * @return {Object}
  */
 function convertFolder(folder, accountId) {
@@ -1532,7 +1552,7 @@ function convertFolder(folder, accountId) {
   let folderObject = {
     accountId,
     name: folder.prettyName,
-    path: folderURIToPath(folder.URI),
+    path: folderURIToPath(accountId, folder.URI),
   };
 
   for (let [flag, typeName] of folderTypeMap.entries()) {
@@ -1545,14 +1565,23 @@ function convertFolder(folder, accountId) {
 }
 
 /**
- * Traverses a nsIMsgFolder to get all subfolders.
+ * Converts an nsIMsgFolder and all subfolders to a simple object for use in
+ * API messages.
+ *
+ * @param {nsIMsgFolder} folder - The folder to convert.
+ * @param {string} [accountId] - An optimization to avoid looking up the
+ *     account. The value from nsIMsgHdr.accountKey must not be used here.
  * @return {Array}
  */
-function traverseSubfolders(folder) {
-  let f = convertFolder(folder);
+function traverseSubfolders(folder, accountId) {
+  let f = convertFolder(folder, accountId);
   f.subFolders = [];
-  for (let subFolder of fixIterator(folder.subFolders, Ci.nsIMsgFolder)) {
-    f.subFolders.push(traverseSubfolders(subFolder));
+  if (folder.hasSubFolders) {
+    for (let subFolder of fixIterator(folder.subFolders, Ci.nsIMsgFolder)) {
+      f.subFolders.push(
+        traverseSubfolders(subFolder, accountId || f.accountId)
+      );
+    }
   }
   return f;
 }
@@ -1605,7 +1634,7 @@ function convertMessage(msgHdr, extension) {
     junkScore,
   };
   if (extension.hasPermission("accountsRead")) {
-    messageObject.folder = convertFolder(msgHdr.folder, msgHdr.accountKey);
+    messageObject.folder = convertFolder(msgHdr.folder);
   }
   let tags = msgHdr.getProperty("keywords");
   tags = tags ? tags.split(" ") : [];
