@@ -185,6 +185,10 @@ this.messages = class extends ExtensionAPI {
           let uri = folderPathToURI(accountId, path);
           let folder = MailServices.folderLookup.getFolderForURL(uri);
 
+          if (!folder) {
+            throw new ExtensionError(`Folder not found: ${path}`);
+          }
+
           return messageListTracker.startList(
             folder.messages,
             context.extension
@@ -203,12 +207,16 @@ this.messages = class extends ExtensionAPI {
           );
         },
         async getFull(messageId) {
-          return new Promise(resolve => {
+          let mimeMsg = await new Promise(resolve => {
             let msgHdr = messageTracker.getMessage(messageId);
             MsgHdrToMimeMessage(msgHdr, null, (_msgHdr, mimeMsg) => {
-              resolve(convertMessagePart(mimeMsg));
+              resolve(mimeMsg);
             });
           });
+          if (!mimeMsg) {
+            throw new ExtensionError(`Error reading message ${messageId}`);
+          }
+          return convertMessagePart(mimeMsg);
         },
         async getRaw(messageId) {
           let messenger = Cc["@mozilla.org/messenger;1"].createInstance(
@@ -281,9 +289,13 @@ this.messages = class extends ExtensionAPI {
                 'Querying by folder requires the "accountsRead" permission'
               );
             }
+            let { accountId, path } = queryInfo.folder;
             let folder = MailServices.folderLookup.getFolderForURL(
-              folderPathToURI(queryInfo.folder.accountId, queryInfo.folder.path)
+              folderPathToURI(accountId, path)
             );
+            if (!folder) {
+              throw new ExtensionError(`Folder not found: ${path}`);
+            }
             query.folder(folder);
           }
           if (queryInfo.fromDate || queryInfo.toDate) {
