@@ -2,8 +2,11 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-ChromeUtils.defineModuleGetter(this, "PromiseUtils",
-                               "resource://gre/modules/PromiseUtils.jsm");
+ChromeUtils.defineModuleGetter(
+  this,
+  "PromiseUtils",
+  "resource://gre/modules/PromiseUtils.jsm"
+);
 var { ExtensionError } = ExtensionUtils;
 
 /**
@@ -35,7 +38,7 @@ let tabListener = {
    * @param {Integer} flags                 The web progress flags for this change
    */
   onLocationChange(browser, webProgress, request, locationURI, flags) {
-    if (webProgress.isTopLevel) {
+    if (webProgress && webProgress.isTopLevel) {
       let tabmail = browser.ownerDocument.getElementById("tabmail");
       let nativeTabInfo = tabmail.getTabForBrowser(browser);
 
@@ -62,8 +65,13 @@ let tabListener = {
     if (!deferred) {
       deferred = PromiseUtils.defer();
       let browser = getTabBrowser(nativeTabInfo);
-      if (!this.initializingTabs.has(nativeTabInfo) &&
-          (browser.innerWindowID || browser.currentURI.spec === "about:blank")) {
+      if (
+        !this.initializingTabs.has(nativeTabInfo) &&
+        (browser.innerWindowID ||
+          ["about:blank", "about:blank?compose"].includes(
+            browser.currentURI.spec
+          ))
+      ) {
         deferred.resolve(nativeTabInfo);
       } else {
         this.initTabReady();
@@ -76,18 +84,14 @@ let tabListener = {
 
 // Attributes and properties used in the TabsUpdateFilterManager.
 const allAttrs = new Set(["favIconUrl", "title"]);
-const allProperties = new Set([
-  "favIconUrl",
-  "status",
-  "title",
-]);
+const allProperties = new Set(["favIconUrl", "status", "title"]);
 const restricted = new Set(["url", "favIconUrl", "title"]);
 
 /**
  * An EventManager for the tabs.onUpdated listener.
  */
 class TabsUpdateFilterEventManager extends EventManager {
-  constructor(context) {
+  constructor({ context }) {
     let { extension } = context;
     let { tabManager } = extension;
 
@@ -119,7 +123,7 @@ class TabsUpdateFilterEventManager extends EventManager {
       }
 
       function getWindowID(windowId) {
-        if (windowId === Window.WINDOW_ID_CURRENT) {
+        if (windowId === WindowBase.WINDOW_ID_CURRENT) {
           return windowTracker.getId(windowTracker.topWindow);
         }
         return windowId;
@@ -132,7 +136,10 @@ class TabsUpdateFilterEventManager extends EventManager {
         if (filter.tabId != null && tab.id != filter.tabId) {
           return false;
         }
-        if (filter.windowId != null && tab.windowId != getWindowID(filter.windowId)) {
+        if (
+          filter.windowId != null &&
+          tab.windowId != getWindowID(filter.windowId)
+        ) {
           return false;
         }
         if (filter.urls) {
@@ -157,7 +164,10 @@ class TabsUpdateFilterEventManager extends EventManager {
         let needed = [];
         if (event.type == "TabAttrModified") {
           let changed = event.detail.changed;
-          if (changed.includes("image") && filter.properties.has("favIconUrl")) {
+          if (
+            changed.includes("image") &&
+            filter.properties.has("favIconUrl")
+          ) {
             needed.push("favIconUrl");
           }
           if (changed.includes("label") && filter.properties.has("title")) {
@@ -206,14 +216,24 @@ class TabsUpdateFilterEventManager extends EventManager {
       };
     };
 
-    super(context, "tabs.onUpdated", register);
+    super({
+      context,
+      name: "tabs.onUpdated",
+      register,
+    });
   }
 
   addListener(callback, filter) {
     let { extension } = this.context;
-    if (filter && filter.urls &&
-        (!extension.hasPermission("tabs") && !extension.hasPermission("activeTab"))) {
-      Cu.reportError("Url filtering in tabs.onUpdated requires \"tabs\" or \"activeTab\" permission.");
+    if (
+      filter &&
+      filter.urls &&
+      !extension.hasPermission("tabs") &&
+      !extension.hasPermission("activeTab")
+    ) {
+      Cu.reportError(
+        'Url filtering in tabs.onUpdated requires "tabs" or "activeTab" permission.'
+      );
       return false;
     }
     return super.addListener(callback, filter);
@@ -259,86 +279,139 @@ this.tabs = class extends ExtensionAPI {
 
     return {
       tabs: {
-        onActivated: new EventManager(context, "tabs.onActivated", fire => {
-          let listener = (eventName, event) => {
-            fire.async(event);
-          };
+        onActivated: new EventManager({
+          context,
+          name: "tabs.onActivated",
+          register: fire => {
+            let listener = (eventName, event) => {
+              fire.async(event);
+            };
 
-          tabTracker.on("tab-activated", listener);
-          return () => {
-            tabTracker.off("tab-activated", listener);
-          };
+            tabTracker.on("tab-activated", listener);
+            return () => {
+              tabTracker.off("tab-activated", listener);
+            };
+          },
         }).api(),
 
-        onCreated: new EventManager(context, "tabs.onCreated", fire => {
-          let listener = (eventName, event) => {
-            fire.async(tabManager.convert(event.nativeTabInfo, event.currentTab));
-          };
+        onCreated: new EventManager({
+          context,
+          name: "tabs.onCreated",
+          register: fire => {
+            let listener = (eventName, event) => {
+              fire.async(
+                tabManager.convert(event.nativeTabInfo, event.currentTab)
+              );
+            };
 
-          tabTracker.on("tab-created", listener);
-          return () => {
-            tabTracker.off("tab-created", listener);
-          };
+            tabTracker.on("tab-created", listener);
+            return () => {
+              tabTracker.off("tab-created", listener);
+            };
+          },
         }).api(),
 
-        onAttached: new EventManager(context, "tabs.onAttached", fire => {
-          let listener = (eventName, event) => {
-            fire.async(event.tabId, { newWindowId: event.newWindowId, newPosition: event.newPosition });
-          };
+        onAttached: new EventManager({
+          context,
+          name: "tabs.onAttached",
+          register: fire => {
+            let listener = (eventName, event) => {
+              fire.async(event.tabId, {
+                newWindowId: event.newWindowId,
+                newPosition: event.newPosition,
+              });
+            };
 
-          tabTracker.on("tab-attached", listener);
-          return () => {
-            tabTracker.off("tab-attached", listener);
-          };
+            tabTracker.on("tab-attached", listener);
+            return () => {
+              tabTracker.off("tab-attached", listener);
+            };
+          },
         }).api(),
 
-        onDetached: new EventManager(context, "tabs.onDetached", fire => {
-          let listener = (eventName, event) => {
-            fire.async(event.tabId, { oldWindowId: event.oldWindowId, oldPosition: event.oldPosition });
-          };
+        onDetached: new EventManager({
+          context,
+          name: "tabs.onDetached",
+          register: fire => {
+            let listener = (eventName, event) => {
+              fire.async(event.tabId, {
+                oldWindowId: event.oldWindowId,
+                oldPosition: event.oldPosition,
+              });
+            };
 
-          tabTracker.on("tab-detached", listener);
-          return () => {
-            tabTracker.off("tab-detached", listener);
-          };
+            tabTracker.on("tab-detached", listener);
+            return () => {
+              tabTracker.off("tab-detached", listener);
+            };
+          },
         }).api(),
 
-        onRemoved: new EventManager(context, "tabs.onRemoved", fire => {
-          let listener = (eventName, event) => {
-            fire.async(event.tabId, { windowId: event.windowId, isWindowClosing: event.isWindowClosing });
-          };
+        onRemoved: new EventManager({
+          context,
+          name: "tabs.onRemoved",
+          register: fire => {
+            let listener = (eventName, event) => {
+              fire.async(event.tabId, {
+                windowId: event.windowId,
+                isWindowClosing: event.isWindowClosing,
+              });
+            };
 
-          tabTracker.on("tab-removed", listener);
-          return () => {
-            tabTracker.off("tab-removed", listener);
-          };
+            tabTracker.on("tab-removed", listener);
+            return () => {
+              tabTracker.off("tab-removed", listener);
+            };
+          },
         }).api(),
 
-        onMoved: new EventManager(context, "tabs.onMoved", fire => {
-          let moveListener = event => {
-            let nativeTab = event.originalTarget;
-            let nativeTabInfo = event.detail.tabInfo;
-            let tabmail = nativeTab.ownerDocument.getElementById("tabmail");
+        onMoved: new EventManager({
+          context,
+          name: "tabs.onMoved",
+          register: fire => {
+            let moveListener = event => {
+              let nativeTab = event.originalTarget;
+              let nativeTabInfo = event.detail.tabInfo;
+              let tabmail = nativeTab.ownerDocument.getElementById("tabmail");
 
-            fire.async(tabTracker.getId(nativeTabInfo), {
-              windowId: windowTracker.getId(nativeTab.ownerGlobal),
-              fromIndex: event.detail.idx,
-              toIndex: tabmail.tabInfo.indexOf(nativeTabInfo),
-            });
-          };
+              fire.async(tabTracker.getId(nativeTabInfo), {
+                windowId: windowTracker.getId(nativeTab.ownerGlobal),
+                fromIndex: event.detail.idx,
+                toIndex: tabmail.tabInfo.indexOf(nativeTabInfo),
+              });
+            };
 
-          windowTracker.addListener("TabMove", moveListener);
-          return () => {
-            windowTracker.removeListener("TabMove", moveListener);
-          };
+            windowTracker.addListener("TabMove", moveListener);
+            return () => {
+              windowTracker.removeListener("TabMove", moveListener);
+            };
+          },
         }).api(),
 
-        onUpdated: new TabsUpdateFilterEventManager(context).api(),
+        onUpdated: new TabsUpdateFilterEventManager({ context }).api(),
 
         async create(createProperties) {
-          let window = createProperties.windowId === null
-              ? windowTracker.topNormalWindow
-              : windowTracker.getWindow(createProperties.windowId, context);
+          let window = await new Promise((resolve, reject) => {
+            let window =
+              createProperties.windowId === null
+                ? windowTracker.topNormalWindow
+                : windowTracker.getWindow(createProperties.windowId, context);
+            if (!window.delayedStartupFinished) {
+              let obs = (finishedWindow, topic, data) => {
+                if (finishedWindow != window) {
+                  return;
+                }
+                Services.obs.removeObserver(
+                  obs,
+                  "mail-delayed-startup-finished"
+                );
+                resolve(window);
+              };
+              Services.obs.addObserver(obs, "mail-delayed-startup-finished");
+            } else {
+              resolve(window);
+            }
+          });
           let tabmail = window.document.getElementById("tabmail");
 
           let url;
@@ -385,18 +458,29 @@ this.tabs = class extends ExtensionAPI {
 
           for (let tabId of tabs) {
             let nativeTabInfo = tabTracker.getTab(tabId);
-            let browser = getTabBrowser(nativeTabInfo);
-            let tabmail = browser.ownerDocument.getElementById("tabmail");
+            if (nativeTabInfo instanceof Ci.nsIDOMWindow) {
+              nativeTabInfo.close();
+              continue;
+            }
+            let tabmail = getTabTabmail(nativeTabInfo);
             tabmail.closeTab(nativeTabInfo);
           }
         },
 
         async update(tabId, updateProperties) {
           let nativeTabInfo = getTabOrActive(tabId);
-          let browser = getTabBrowser(nativeTabInfo);
-          let tabmail = browser.ownerDocument.getElementById("tabmail");
+          if (nativeTabInfo instanceof Ci.nsIDOMWindow) {
+            throw new ExtensionError(
+              "tabs.update is not applicable to this tab."
+            );
+          }
+          let tabmail = getTabTabmail(nativeTabInfo);
 
           if (updateProperties.url) {
+            let browser = getTabBrowser(nativeTabInfo);
+            if (!browser) {
+              throw new ExtensionError("Cannot set a URL for this tab.");
+            }
             let url = context.uri.resolve(updateProperties.url);
 
             if (!context.checkLoadURL(url, { dontReportErrors: true })) {
@@ -405,8 +489,8 @@ this.tabs = class extends ExtensionAPI {
 
             let options = {
               flags: updateProperties.loadReplace
-                      ? Ci.nsIWebNavigation.LOAD_FLAGS_REPLACE_HISTORY
-                      : Ci.nsIWebNavigation.LOAD_FLAGS_NONE,
+                ? Ci.nsIWebNavigation.LOAD_FLAGS_REPLACE_HISTORY
+                : Ci.nsIWebNavigation.LOAD_FLAGS_NONE,
               triggeringPrincipal: context.principal,
             };
             browser.loadURI(url, options);
@@ -425,6 +509,11 @@ this.tabs = class extends ExtensionAPI {
 
         async reload(tabId, reloadProperties) {
           let nativeTabInfo = getTabOrActive(tabId);
+          if (nativeTabInfo instanceof Ci.nsIDOMWindow) {
+            throw new ExtensionError(
+              "tabs.reload is not applicable to this tab."
+            );
+          }
           let browser = getTabBrowser(nativeTabInfo);
 
           let flags = Ci.nsIWebNavigation.LOAD_FLAGS_NONE;
@@ -449,7 +538,10 @@ this.tabs = class extends ExtensionAPI {
         async query(queryInfo) {
           if (!extension.hasPermission("tabs")) {
             if (queryInfo.url || queryInfo.title !== null) {
-              return Promise.reject({ message: 'The "tabs" permission is required to use the query API with the "url" or "title" parameters' });
+              return Promise.reject({
+                message:
+                  'The "tabs" permission is required to use the query API with the "url" or "title" parameters',
+              });
             }
           }
 
@@ -465,27 +557,23 @@ this.tabs = class extends ExtensionAPI {
           // Make ext-tabs-base happy since it does a strict check.
           queryInfo.screen = null;
 
-          return Array.from(tabManager.query(queryInfo, context), tab => tab.convert());
+          return Array.from(tabManager.query(queryInfo, context), tab =>
+            tab.convert()
+          );
         },
 
         async executeScript(tabId, details) {
-          // TODO make this work
           let tab = await promiseTabWhenReady(tabId);
-
           return tab.executeScript(context, details);
         },
 
         async insertCSS(tabId, details) {
-          // TODO make this work
           let tab = await promiseTabWhenReady(tabId);
-
           return tab.insertCSS(context, details);
         },
 
         async removeCSS(tabId, details) {
-          // TODO make this work
           let tab = await promiseTabWhenReady(tabId);
-
           return tab.removeCSS(context, details);
         },
 
@@ -497,10 +585,14 @@ this.tabs = class extends ExtensionAPI {
 
           let destinationWindow = null;
           if (moveProperties.windowId !== null) {
-            destinationWindow = windowTracker.getWindow(moveProperties.windowId);
+            destinationWindow = windowTracker.getWindow(
+              moveProperties.windowId
+            );
             // Fail on an invalid window.
             if (!destinationWindow) {
-              return Promise.reject({ message: `Invalid window ID: ${moveProperties.windowId}` });
+              return Promise.reject({
+                message: `Invalid window ID: ${moveProperties.windowId}`,
+              });
             }
           }
 
@@ -516,6 +608,12 @@ this.tabs = class extends ExtensionAPI {
 
           let tabs = tabIds.map(tabId => tabTracker.getTab(tabId));
           for (let nativeTabInfo of tabs) {
+            if (nativeTabInfo instanceof Ci.nsIDOMWindow) {
+              throw new ExtensionError(
+                "tabs.move is not applicable to this tab."
+              );
+            }
+
             // If the window is not specified, use the window from the tab.
             let browser = getTabBrowser(nativeTabInfo);
 
@@ -530,7 +628,8 @@ this.tabs = class extends ExtensionAPI {
               continue;
             }
 
-            let insertionPoint = indexMap.get(tgtwindow) || moveProperties.index;
+            let insertionPoint =
+              indexMap.get(tgtwindow) || moveProperties.index;
             // If the index is -1 it should go to the end of the tabs.
             if (insertionPoint == -1) {
               insertionPoint = tgttabmail.tabInfo.length;
@@ -542,9 +641,11 @@ this.tabs = class extends ExtensionAPI {
             // the insertion point is the same as the last insertion and
             // the tab is further to the right than the current insertion point
             // then you need to bump up the insertion point. See bug 1323311.
-            if (lastInsertion.has(tgtwindow) &&
-                lastInsertion.get(tgtwindow) === insertionPoint &&
-                tabPosition > insertionPoint) {
+            if (
+              lastInsertion.has(tgtwindow) &&
+              lastInsertion.get(tgtwindow) === insertionPoint &&
+              tabPosition > insertionPoint
+            ) {
               insertionPoint++;
               indexMap.set(tgtwindow, insertionPoint);
             }
@@ -555,19 +656,31 @@ this.tabs = class extends ExtensionAPI {
             } else {
               // If the window we are moving the tab in is different, then move the tab
               // to the new window.
-              srctabmail.replaceTabWithWindow(nativeTabInfo, tgtwindow, insertionPoint);
-              nativeTabInfo = tgttabmail.tabInfo[insertionPoint] ||
+              srctabmail.replaceTabWithWindow(
+                nativeTabInfo,
+                tgtwindow,
+                insertionPoint
+              );
+              nativeTabInfo =
+                tgttabmail.tabInfo[insertionPoint] ||
                 tgttabmail.tabInfo[tgttabmail.tabInfo.length - 1];
             }
             lastInsertion.set(tgtwindow, tabPosition);
             tabsMoved.push(nativeTabInfo);
           }
 
-          return tabsMoved.map(nativeTabInfo => tabManager.convert(nativeTabInfo));
+          return tabsMoved.map(nativeTabInfo =>
+            tabManager.convert(nativeTabInfo)
+          );
         },
 
         duplicate(tabId) {
           let nativeTabInfo = tabTracker.getTab(tabId);
+          if (nativeTabInfo instanceof Ci.nsIDOMWindow) {
+            throw new ExtensionError(
+              "tabs.duplicate is not applicable to this tab."
+            );
+          }
           let browser = getTabBrowser(nativeTabInfo);
           let tabmail = browser.ownerDocument.getElementById("tabmail");
 
@@ -577,7 +690,9 @@ this.tabs = class extends ExtensionAPI {
           state.state.duplicate = true;
 
           if (mode.tabs.length && mode.tabs.length == mode.maxTabs) {
-            throw new ExtensionError(`Maximum number of ${state.mode} tabs reached`);
+            throw new ExtensionError(
+              `Maximum number of ${state.mode} tabs reached`
+            );
           } else {
             tabmail.restoreTab(state);
             return tabManager.convert(mode.tabs[mode.tabs.length - 1]);
