@@ -155,12 +155,13 @@ this.windows = class extends ExtensionAPI {
             createData.state = "normal";
           }
 
-          let createWindowArgs = urls => {
+          let createWindowArgs = (urls, allowScriptsToClose = false) => {
             let args = Cc["@mozilla.org/array;1"].createInstance(
               Ci.nsIMutableArray
             );
             let actionData = {
               action: "open",
+              allowScriptsToClose,
               tabs: urls.map(url => ({
                 tabType: "contentTab",
                 tabParams: { contentPage: url },
@@ -229,7 +230,7 @@ this.windows = class extends ExtensionAPI {
             let uris = Array.isArray(createData.url)
               ? createData.url
               : [createData.url];
-            let args = createWindowArgs(uris);
+            let args = createWindowArgs(uris, createData.allowScriptsToClose);
             window = Services.ww.openWindow(
               null,
               windowURL,
@@ -240,7 +241,10 @@ this.windows = class extends ExtensionAPI {
           } else {
             let args = null;
             if (!wantNormalWindow) {
-              args = createWindowArgs(["about:blank"]);
+              args = createWindowArgs(
+                ["about:blank"],
+                createData.allowScriptsToClose
+              );
             }
             window = Services.ww.openWindow(
               null,
@@ -276,6 +280,9 @@ this.windows = class extends ExtensionAPI {
             ) {
               win.state = createData.state;
             }
+            if (createData.titlePreface !== null) {
+              win.setTitlePreface(createData.titlePreface);
+            }
             return win.convert({ populate: true });
           });
         },
@@ -311,9 +318,11 @@ this.windows = class extends ExtensionAPI {
           win.updateGeometry(updateInfo);
 
           if (updateInfo.titlePreface !== null) {
+            win.setTitlePreface(updateInfo.titlePreface);
             if (win instanceof TabmailWindow) {
-              win.setTitlePreface(updateInfo.titlePreface);
               win.window.document.getElementById("tabmail").setDocumentTitle();
+            } else if (win.window.gBrowser?.updateTitlebar) {
+              win.window.gBrowser.updateTitlebar();
             }
           }
 
@@ -334,6 +343,22 @@ this.windows = class extends ExtensionAPI {
             windowTracker.addListener("domwindowclosed", listener);
           });
         },
+        openDefaultBrowser(url) {
+          let uri = null;
+          try {
+            uri = Services.io.newURI(url);
+          } catch (e) {
+            throw new ExtensionError(`Url "${url}" seems to be malformed.`);
+          }
+          if (!uri.schemeIs("http") && !uri.schemeIs("https")) {
+            throw new ExtensionError(
+              `Url scheme "${uri.scheme}" is not supported.`
+            );
+          }
+          Cc["@mozilla.org/uriloader/external-protocol-service;1"]
+            .getService(Ci.nsIExternalProtocolService)
+            .loadURI(uri);
+        },
       },
     };
   }
