@@ -1,23 +1,38 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, you can obtain one at http://mozilla.org/MPL/2.0/. */
+var { MailServices } = ChromeUtils.import(
+  "resource:///modules/MailServices.jsm"
+);
+
+let account = createAccount("pop3");
+createAccount("local");
+MailServices.accounts.defaultAccount = account;
 
-let account = createAccount();
 let defaultIdentity = addIdentity(account);
+defaultIdentity.composeHtml = true;
 let nonDefaultIdentity = addIdentity(account);
+nonDefaultIdentity.composeHtml = false;
+
 let rootFolder = account.incomingServer.rootFolder;
 rootFolder.createSubfolder("test", null);
 let folder = rootFolder.getChildNamed("test");
-createMessages(folder, 3);
+createMessages(folder, 4);
 
 add_task(async function testIdentity() {
   let files = {
     "background.js": async () => {
-      let [account] = await browser.accounts.list();
-      let [defaultIdentity, nonDefaultIdentity] = account.identities;
-      let folder = account.folders.find(f => f.name == "test");
+      let accounts = await browser.accounts.list();
+      browser.test.assertEq(2, accounts.length, "number of accounts");
+      browser.test.assertEq(
+        2,
+        accounts[0].identities.length,
+        "number of identities"
+      );
+      let [defaultIdentity, nonDefaultIdentity] = accounts[0].identities;
+      let folder = accounts[0].folders.find(f => f.name == "test");
       let { messages } = await browser.messages.list(folder);
-      browser.test.assertEq(3, messages.length);
+      browser.test.assertEq(4, messages.length, "number of messages");
 
       browser.test.log(defaultIdentity.id);
       browser.test.log(nonDefaultIdentity.id);
@@ -26,6 +41,9 @@ add_task(async function testIdentity() {
         { name: "beginNew", args: [] },
         { name: "beginReply", args: [messages[0].id] },
         { name: "beginForward", args: [messages[1].id, "forwardAsAttachment"] },
+        // Uses a different code path.
+        { name: "beginForward", args: [messages[2].id, "forwardInline"] },
+        { name: "beginNew", args: [messages[3].id] },
       ];
       let tests = [
         { args: [], isDefault: true },
@@ -47,8 +65,7 @@ add_task(async function testIdentity() {
           );
           browser.test.assertEq("object", typeof tab);
           browser.test.assertEq("number", typeof tab.id);
-          browser.test.sendMessage("checkIdentity", test.isDefault);
-          await window.waitForMessage();
+          await window.sendMessage("checkIdentity", test.isDefault);
         }
       }
 
@@ -96,10 +113,10 @@ add_task(async function testHeaders() {
       }
 
       let accounts = await browser.accounts.list();
-      browser.test.assertEq(1, accounts.length);
+      browser.test.assertEq(2, accounts.length, "number of accounts");
       let folder = accounts[0].folders.find(f => f.name == "test");
       let { messages } = await browser.messages.list(folder);
-      browser.test.assertEq(3, messages.length);
+      browser.test.assertEq(4, messages.length, "number of messages");
 
       let addressBook = await browser.addressBooks.create({
         name: "Baker Street",
@@ -207,6 +224,17 @@ add_task(async function testHeaders() {
         subject: `Fwd: ${messages[1].subject}`,
       });
 
+      // Forward a message inline. This uses a different code path.
+
+      createdWindowPromise = window.waitForEvent("windows.onCreated");
+      await browser.compose.beginForward(messages[2].id, "forwardInline", {
+        to: ["Mycroft Holmes <mycroft@bakerstreet.invalid>"],
+      });
+      await checkHeaders({
+        to: ["Mycroft Holmes <mycroft@bakerstreet.invalid>"],
+        subject: `Fwd: ${messages[2].subject}`,
+      });
+
       await browser.addressBooks.delete(addressBook);
       browser.test.notifyPass("finished");
     },
@@ -233,12 +261,29 @@ add_task(async function testHeaders() {
 add_task(async function testBody() {
   let files = {
     "background.js": async () => {
+      let accounts = await browser.accounts.list();
+      browser.test.assertEq(2, accounts.length, "number of accounts");
+      browser.test.assertEq(
+        2,
+        accounts[0].identities.length,
+        "number of identities"
+      );
+      let [htmlIdentity, plainTextIdentity] = accounts[0].identities;
+      let folder = accounts[0].folders.find(f => f.name == "test");
+      let { messages } = await browser.messages.list(folder);
+      browser.test.assertEq(4, messages.length, "number of messages");
+
+      let message0 = await browser.messages.getFull(messages[0].id);
+      let message0body = message0.parts[0].body;
+
       let emptyHTML = "<body>\n<p><br>\n</p>\n";
       let plainTextBodyTag =
         '<body style="font-family: -moz-fixed; white-space: pre-wrap; width: 72ch;">';
       let tests = [
         {
           // No arguments.
+          funcName: "beginNew",
+          arguments: [],
           expected: {
             isHTML: true,
             htmlIncludes: emptyHTML,
@@ -247,7 +292,8 @@ add_task(async function testBody() {
         },
         {
           // Empty arguments.
-          arguments: {},
+          funcName: "beginNew",
+          arguments: [{}],
           expected: {
             isHTML: true,
             htmlIncludes: emptyHTML,
@@ -256,7 +302,8 @@ add_task(async function testBody() {
         },
         {
           // Empty HTML.
-          arguments: { body: "" },
+          funcName: "beginNew",
+          arguments: [{ body: "" }],
           expected: {
             isHTML: true,
             htmlIncludes: emptyHTML,
@@ -265,7 +312,47 @@ add_task(async function testBody() {
         },
         {
           // Empty plain text.
-          arguments: { plainTextBody: "" },
+          funcName: "beginNew",
+          arguments: [{ plainTextBody: "" }],
+          expected: {
+            isHTML: false,
+            plainTextIs: "",
+          },
+        },
+        {
+          // Empty enforced plain text with default identity.
+          funcName: "beginNew",
+          arguments: [{ plainTextBody: "", isPlainText: true }],
+          expected: {
+            isHTML: false,
+            plainTextIs: "",
+          },
+        },
+        {
+          // Empty HTML for plaintext identity.
+          funcName: "beginNew",
+          arguments: [{ body: "", identityId: plainTextIdentity.id }],
+          expected: {
+            isHTML: true,
+            htmlIncludes: emptyHTML,
+            plainTextIs: "\n",
+          },
+        },
+        {
+          // Empty plain text for plaintext identity.
+          funcName: "beginNew",
+          arguments: [{ plainTextBody: "", identityId: plainTextIdentity.id }],
+          expected: {
+            isHTML: false,
+            plainTextIs: "",
+          },
+        },
+        {
+          // Empty HTML for plaintext identity enforcing HTML.
+          funcName: "beginNew",
+          arguments: [
+            { body: "", identityId: plainTextIdentity.id, isPlainText: false },
+          ],
           expected: {
             isHTML: true,
             htmlIncludes: emptyHTML,
@@ -274,12 +361,14 @@ add_task(async function testBody() {
         },
         {
           // Empty plain text and isPlainText.
-          arguments: { plainTextBody: "", isPlainText: true },
+          funcName: "beginNew",
+          arguments: [{ plainTextBody: "", isPlainText: true }],
           expected: { isHTML: false, plainTextIs: "" },
         },
         {
           // Non-empty HTML.
-          arguments: { body: "<p>I'm an HTML message!</p>" },
+          funcName: "beginNew",
+          arguments: [{ body: "<p>I'm an HTML message!</p>" }],
           expected: {
             isHTML: true,
             htmlIncludes: "<body>\n<p>I'm an HTML message!</p>\n</body>",
@@ -288,19 +377,23 @@ add_task(async function testBody() {
         },
         {
           // Non-empty plain text.
-          arguments: { plainTextBody: "I'm a plain text message!" },
+          funcName: "beginNew",
+          arguments: [{ plainTextBody: "I'm a plain text message!" }],
           expected: {
-            isHTML: true,
-            htmlIncludes: "<body>I'm a plain text message!</body>",
+            isHTML: false,
+            htmlIncludes: plainTextBodyTag + "I'm a plain text message!</body>",
             plainTextIs: "I'm a plain text message!",
           },
         },
         {
           // Non-empty plain text and isPlainText.
-          arguments: {
-            plainTextBody: "I'm a plain text message!",
-            isPlainText: true,
-          },
+          funcName: "beginNew",
+          arguments: [
+            {
+              plainTextBody: "I'm a plain text message!",
+              isPlainText: true,
+            },
+          ],
           expected: {
             isHTML: false,
             htmlIncludes: plainTextBodyTag + "I'm a plain text message!</body>",
@@ -309,21 +402,126 @@ add_task(async function testBody() {
         },
         {
           // HTML and plain text. Invalid.
-          arguments: { body: "", plainTextBody: "" },
+          funcName: "beginNew",
+          arguments: [{ body: "", plainTextBody: "" }],
           throws: true,
         },
         {
           // HTML and isPlainText. Invalid.
-          arguments: { body: "", isPlainText: true },
+          funcName: "beginNew",
+          arguments: [{ body: "", isPlainText: true }],
           throws: true,
         },
+        {
+          // HTML and isPlainText. Invalid.
+          funcName: "beginNew",
+          arguments: [{ plainTextBody: "", isPlainText: false }],
+          throws: true,
+        },
+        {
+          // Edit as new.
+          funcName: "beginNew",
+          arguments: [messages[0].id],
+          expected: {
+            isHTML: true,
+            htmlIncludes: message0body.trim(),
+          },
+        },
+        {
+          // Edit as new with plaintext identity
+          funcName: "beginNew",
+          arguments: [messages[0].id, { identityId: plainTextIdentity.id }],
+          expected: {
+            isHTML: false,
+            plainTextIs: message0body,
+          },
+        },
+        {
+          // Edit as new with default identity enforcing HTML
+          funcName: "beginNew",
+          arguments: [messages[0].id, { isPlainText: false }],
+          expected: {
+            isHTML: true,
+            htmlIncludes: message0body.trim(),
+          },
+        },
+        {
+          // Edit as new with plaintext identity enforcing HTML by setting a body.
+          funcName: "beginNew",
+          arguments: [
+            messages[0].id,
+            {
+              body: "<p>This is some HTML text</p>",
+              identityId: plainTextIdentity.id,
+            },
+          ],
+          expected: {
+            isHTML: true,
+            htmlIncludes: "<p>This is some HTML text</p>",
+          },
+        },
+        {
+          // Edit as new with html identity enforcing plain text by setting a plainTextBody.
+          funcName: "beginNew",
+          arguments: [
+            messages[0].id,
+            {
+              plainTextBody: "This is some plain text",
+              identityId: htmlIdentity.id,
+            },
+          ],
+          expected: {
+            isHTML: false,
+            plainText: "This is some plain text",
+          },
+        },
+        {
+          // ForwardInline with plaintext identity enforcing HTML
+          funcName: "beginForward",
+          arguments: [
+            messages[0].id,
+            { identityId: plainTextIdentity.id, isPlainText: false },
+          ],
+          expected: {
+            isHTML: true,
+            htmlIncludes: message0body.trim(),
+          },
+        },
+        {
+          // Reply.
+          funcName: "beginReply",
+          arguments: [messages[0].id],
+          expected: {
+            isHTML: true,
+            htmlIncludes: message0body.trim(),
+          },
+        },
+        {
+          // Forward inline.
+          funcName: "beginForward",
+          arguments: [messages[0].id],
+          expected: {
+            isHTML: true,
+            htmlIncludes: message0body.trim(),
+          },
+        },
+        {
+          // Forward as attachment.
+          funcName: "beginForward",
+          arguments: [messages[0].id, "forwardAsAttachment"],
+          expected: {
+            isHTML: true,
+            htmlIncludes: emptyHTML,
+            plainText: "",
+          },
+        },
       ];
 
       for (let test of tests) {
         browser.test.log(JSON.stringify(test));
         let createdWindowPromise = window.waitForEvent("windows.onCreated");
         try {
-          await browser.compose.beginNew(test.arguments);
+          await browser.compose[test.funcName](...test.arguments);
           if (test.throws) {
             browser.test.fail(
               "calling beginNew with these arguments should throw"
@@ -355,6 +553,7 @@ add_task(async function testBody() {
     files,
     manifest: {
       background: { scripts: ["utils.js", "background.js"] },
+      permissions: ["accountsRead", "messagesRead"],
     },
   });
   extension.onMessage("checkBody", async expected => {
@@ -372,10 +571,17 @@ add_task(async function testBody() {
       .replace(/\r/g, "");
     if ("htmlIncludes" in expected) {
       info(actualHTML);
-      ok(actualHTML.includes(expected.htmlIncludes), "HTML content is correct");
+      ok(
+        actualHTML.includes(expected.htmlIncludes.replace(/\r/g, "")),
+        "HTML content is correct"
+      );
     }
     if ("plainTextIs" in expected) {
-      is(actualPlainText, expected.plainTextIs, "plainText content is correct");
+      is(
+        actualPlainText,
+        expected.plainTextIs.replace(/\r/g, ""),
+        "plainText content is correct"
+      );
     }
 
     extension.sendMessage();
@@ -396,14 +602,14 @@ add_task(async function testAttachments() {
       let newTab = await browser.compose.beginNew({
         attachments: [
           { file: new File(["one"], "attachment1.txt") },
-          { file: new File(["two"], "attachment2.txt") },
+          { file: new File(["two"], "attachment-två.txt") },
         ],
       });
 
       let attachments = await browser.compose.listAttachments(newTab.id);
       browser.test.assertEq(2, attachments.length);
       browser.test.assertEq("attachment1.txt", attachments[0].name);
-      browser.test.assertEq("attachment2.txt", attachments[1].name);
+      browser.test.assertEq("attachment-två.txt", attachments[1].name);
 
       let replyTab = await browser.compose.beginReply(messages[0].id, {
         attachments: [
@@ -436,9 +642,41 @@ add_task(async function testAttachments() {
       // but it isn't.
       browser.test.assertEq(`${messages[1].subject}.eml`, attachments[2].name);
 
+      // Forward inline adds attachments differently, so check it works too.
+
+      let forwardTab2 = await browser.compose.beginForward(
+        messages[2].id,
+        "forwardInline",
+        {
+          attachments: [
+            { file: new File(["seven"], "attachment7.txt") },
+            { file: new File(["eight"], "attachment-åtta.txt") },
+          ],
+        }
+      );
+
+      attachments = await browser.compose.listAttachments(forwardTab2.id);
+      browser.test.assertEq(2, attachments.length);
+      browser.test.assertEq("attachment7.txt", attachments[0].name);
+      browser.test.assertEq("attachment-åtta.txt", attachments[1].name);
+
+      let newTab2 = await browser.compose.beginNew(messages[3].id, {
+        attachments: [
+          { file: new File(["nine"], "attachment9.txt") },
+          { file: new File(["ten"], "attachment10.txt") },
+        ],
+      });
+
+      attachments = await browser.compose.listAttachments(newTab2.id);
+      browser.test.assertEq(2, attachments.length);
+      browser.test.assertEq("attachment9.txt", attachments[0].name);
+      browser.test.assertEq("attachment10.txt", attachments[1].name);
+
       await browser.tabs.remove(newTab.id);
       await browser.tabs.remove(replyTab.id);
       await browser.tabs.remove(forwardTab.id);
+      await browser.tabs.remove(forwardTab2.id);
+      await browser.tabs.remove(newTab2.id);
 
       browser.test.notifyPass();
     },
