@@ -8,8 +8,68 @@ const { mailTestUtils } = ChromeUtils.import(
   "resource://testing-common/mailnews/MailTestUtils.jsm"
 );
 
+const URL_BASE =
+  "http://mochi.test:8888/browser/comm/mail/components/extensions/test/browser/data";
+
 const treeClick = mailTestUtils.treeClick.bind(null, EventUtils, window);
 
+/**
+ * Right-click on something and wait for the context menu to appear.
+ * For elements in the parent process only.
+ *
+ * @param {Element} menu     The <menu> that should appear.
+ * @param {Element} element  The element to be clicked on.
+ * @returns {Promise}        A promise that resolves when the menu appears.
+ */
+function rightClick(menu, element) {
+  let shownPromise = BrowserTestUtils.waitForEvent(menu, "popupshown");
+  EventUtils.synthesizeMouseAtCenter(
+    element,
+    { type: "contextmenu" },
+    element.ownerGlobal
+  );
+  return shownPromise;
+}
+
+/**
+ * Right-click on something in a content document and wait for the context
+ * menu to appear.
+ *
+ * @param {Element} menu     The <menu> that should appear.
+ * @param {string} selector  CSS selector of the element to be clicked on.
+ * @param {Element} browser  <browser> containing the element.
+ * @returns {Promise}        A promise that resolves when the menu appears.
+ */
+function rightClickOnContent(menu, selector, browser) {
+  let shownPromise = BrowserTestUtils.waitForEvent(menu, "popupshown");
+  BrowserTestUtils.synthesizeMouseAtCenter(
+    selector,
+    { type: "contextmenu" },
+    browser
+  );
+  return shownPromise;
+}
+
+/**
+ * Check the parameters of a browser.onShown event was fired.
+ *
+ * @see https://thunderbird-webextensions.readthedocs.io/en/latest/menus.html#menus-onshown
+ *
+ * @param extension
+ * @param {Object} expectedInfo
+ * @param {Array?} expectedInfo.menuIds
+ * @param {Array?} expectedInfo.contexts
+ * @param {Array?} expectedInfo.attachments
+ * @param {Object?} expectedInfo.displayedFolder
+ * @param {Object?} expectedInfo.selectedFolder
+ * @param {Array?} expectedInfo.selectedMessages
+ * @param {RegExp?} expectedInfo.pageUrl
+ * @param {string?} expectedInfo.selectionText
+ * @param {Object} expectedTab
+ * @param {boolean} expectedTab.active
+ * @param {integer} expectedTab.index
+ * @param {boolean} expectedTab.mailTab
+ */
 async function checkShownEvent(extension, expectedInfo, expectedTab) {
   let [info, tab] = await extension.awaitMessage("onShown");
   Assert.deepEqual(info.menuIds, expectedInfo.menuIds);
@@ -65,19 +125,57 @@ async function checkShownEvent(extension, expectedInfo, expectedTab) {
     Assert.ok(info.pageUrl.match(expectedInfo.pageUrl));
   }
 
+  Assert.equal(
+    !!info.selectionText,
+    !!expectedInfo.selectionText,
+    "selectionText in info"
+  );
+  if (expectedInfo.selectionText) {
+    Assert.equal(info.selectionText, expectedInfo.selectionText);
+  }
+
   Assert.equal(tab.active, expectedTab.active, "tab is active");
   Assert.equal(tab.index, expectedTab.index, "tab index");
   Assert.equal(tab.mailTab, expectedTab.mailTab, "tab is mailTab");
-
-  return [info, tab];
 }
 
-async function checkClickedEvent(extension, properties) {
+/**
+ * Check the parameters of a browser.onClicked event was fired.
+ *
+ * @see https://thunderbird-webextensions.readthedocs.io/en/latest/menus.html#menus-onclicked
+ *
+ * @param extension
+ * @param {Object} expectedInfo
+ * @param {string?} expectedInfo.selectionText
+ * @param {string?} expectedInfo.linkText
+ * @param {RegExp?} expectedInfo.pageUrl
+ * @param {RegExp?} expectedInfo.linkUrl
+ * @param {RegExp?} expectedInfo.srcUrl
+ * @param {Object} expectedTab
+ * @param {boolean} expectedTab.active
+ * @param {integer} expectedTab.index
+ * @param {boolean} expectedTab.mailTab
+ */
+async function checkClickedEvent(extension, expectedInfo, expectedTab) {
   let [info, tab] = await extension.awaitMessage("onClicked");
-  for (let [name, value] of Object.entries(properties)) {
-    Assert.equal(info[name], value);
+
+  Assert.equal(info.selectionText, expectedInfo.selectionText, "selectionText");
+  Assert.equal(info.linkText, expectedInfo.linkText, "linkText");
+
+  for (let infoKey of ["pageUrl", "linkUrl", "srcUrl"]) {
+    Assert.equal(
+      !!info[infoKey],
+      !!expectedInfo[infoKey],
+      `${infoKey} in info`
+    );
+    if (expectedInfo[infoKey]) {
+      Assert.ok(info[infoKey].match(expectedInfo[infoKey]));
+    }
   }
-  return [info, tab];
+
+  Assert.equal(tab.active, expectedTab.active, "tab is active");
+  Assert.equal(tab.index, expectedTab.index, "tab index");
+  Assert.equal(tab.mailTab, expectedTab.mailTab, "tab is mailTab");
 }
 
 function createExtension(...permissions) {
@@ -151,7 +249,13 @@ add_task(async function set_up() {
   gAccount = createAccount();
   addIdentity(gAccount);
   gFolders = [...gAccount.incomingServer.rootFolder.subFolders];
-  createMessages(gFolders[0], 10);
+  createMessages(gFolders[0], {
+    count: 1,
+    body: {
+      contentType: "text/html",
+      body: await fetch(`${URL_BASE}/content.html`).then(r => r.text()),
+    },
+  });
   gMessages = [...gFolders[0].messages];
 
   window.gFolderTreeView.selectFolder(gAccount.incomingServer.rootFolder);
@@ -199,16 +303,21 @@ add_task(async function test_folder_pane_no_permissions() {
 });
 
 async function subtest_message_panes(...permissions) {
+  window.gFolderTreeView.selectFolder(gFolders[0]);
+  if (window.IsMessagePaneCollapsed()) {
+    window.MsgToggleMessagePane();
+  }
+
   let extension = createExtension(...permissions);
   await extension.startup();
 
   // Test the thread pane in the 3-pane tab.
 
   let threadTree = document.getElementById("threadTree");
-  treeClick(threadTree, 1, 1, {});
+  treeClick(threadTree, 0, 1, {});
   let menu = document.getElementById("mailContext");
   let shownPromise = BrowserTestUtils.waitForEvent(menu, "popupshown");
-  treeClick(threadTree, 1, 1, { type: "contextmenu" });
+  treeClick(threadTree, 0, 1, { type: "contextmenu" });
 
   await shownPromise;
   Assert.ok(menu.querySelector("#test1_mochi_test-menuitem-_message_list"));
@@ -223,7 +332,7 @@ async function subtest_message_panes(...permissions) {
         ? { accountId: gAccount.key, path: "/Trash" }
         : undefined,
       selectedMessages: permissions.includes("messagesRead")
-        ? { id: null, messages: [{ subject: gMessages[1].subject }] }
+        ? { id: null, messages: [{ subject: gMessages[0].subject }] }
         : undefined,
     },
     { active: true, index: 0, mailTab: true }
@@ -234,44 +343,25 @@ async function subtest_message_panes(...permissions) {
   let messagePane = document.getElementById("messagepane");
   await awaitBrowserLoaded(messagePane);
 
-  EventUtils.synthesizeMouseAtCenter(messagePane, {});
-  shownPromise = BrowserTestUtils.waitForEvent(menu, "popupshown");
-  EventUtils.synthesizeMouseAtCenter(messagePane, { type: "contextmenu" });
-
-  await shownPromise;
-  Assert.ok(menu.querySelector("#test1_mochi_test-menuitem-_page"));
-  menu.hidePopup();
-
-  await checkShownEvent(
+  await subtest_content(
     extension,
-    {
-      menuIds: ["page"],
-      contexts: ["page", "all"],
-      pageUrl: permissions.includes("messagesRead") ? /^mailbox\:/ : undefined,
-    },
+    permissions.includes("messagesRead"),
+    messagePane,
+    /^mailbox\:/,
     { active: true, index: 0, mailTab: true }
   );
 
   // Test the message pane in a tab.
 
   window.MsgOpenSelectedMessages();
+  await BrowserTestUtils.waitForEvent(window, "MsgLoaded");
   await awaitBrowserLoaded(messagePane);
 
-  EventUtils.synthesizeMouseAtCenter(messagePane, {});
-  shownPromise = BrowserTestUtils.waitForEvent(menu, "popupshown");
-  EventUtils.synthesizeMouseAtCenter(messagePane, { type: "contextmenu" });
-
-  await shownPromise;
-  Assert.ok(menu.querySelector("#test1_mochi_test-menuitem-_page"));
-  menu.hidePopup();
-
-  await checkShownEvent(
+  await subtest_content(
     extension,
-    {
-      menuIds: ["page"],
-      contexts: ["page", "all"],
-      pageUrl: permissions.includes("messagesRead") ? /^mailbox\:/ : undefined,
-    },
+    permissions.includes("messagesRead"),
+    messagePane,
+    /^mailbox\:/,
     { active: true, index: 1, mailTab: false }
   );
 
@@ -283,39 +373,27 @@ async function subtest_message_panes(...permissions) {
   let displayWindowPromise = BrowserTestUtils.domWindowOpened();
   window.MsgOpenNewWindowForMessage();
   let displayWindow = await displayWindowPromise;
-  await BrowserTestUtils.waitForEvent(displayWindow, "load");
-  let displayDocument = displayWindow.document;
+  await BrowserTestUtils.waitForEvent(displayWindow, "MsgLoaded");
   await focusWindow(displayWindow);
 
+  let displayDocument = displayWindow.document;
   menu = displayDocument.getElementById("mailContext");
   messagePane = displayDocument.getElementById("messagepane");
   await awaitBrowserLoaded(messagePane);
 
-  EventUtils.synthesizeMouseAtCenter(messagePane, {}, displayWindow);
-  shownPromise = BrowserTestUtils.waitForEvent(menu, "popupshown");
-  EventUtils.synthesizeMouseAtCenter(
-    messagePane,
-    { type: "contextmenu" },
-    displayWindow
-  );
-
-  await shownPromise;
-  Assert.ok(menu.querySelector("#test1_mochi_test-menuitem-_page"));
-  menu.hidePopup();
-
-  await checkShownEvent(
+  await subtest_content(
     extension,
-    {
-      menuIds: ["page"],
-      contexts: ["page", "all"],
-      pageUrl: permissions.includes("messagesRead") ? /^mailbox\:/ : undefined,
-    },
+    permissions.includes("messagesRead"),
+    messagePane,
+    /^mailbox\:/,
     { active: true, index: 0, mailTab: false }
   );
 
   await extension.unload();
 
   await BrowserTestUtils.closeWindow(displayWindow);
+  window.gFolderTreeView.selectFolder(gAccount.incomingServer.rootFolder);
+  window.gFolderTreeView.selectFolder(gFolders[0]);
 }
 add_task(async function test_message_panes() {
   return subtest_message_panes("accountsRead", "messagesRead");
@@ -332,14 +410,7 @@ add_task(async function test_message_panes_no_permissions() {
 
 add_task(async function test_tab() {
   async function checkTabEvent(index, active, mailTab) {
-    let shownPromise = BrowserTestUtils.waitForEvent(menu, "popupshown");
-    EventUtils.synthesizeMouseAtCenter(
-      tabs[index],
-      { type: "contextmenu" },
-      window
-    );
-
-    await shownPromise;
+    await rightClick(menu, tabs[index]);
     Assert.ok(menu.querySelector("#test1_mochi_test-menuitem-_tab"));
     menu.hidePopup();
 
@@ -371,10 +442,129 @@ add_task(async function test_tab() {
   tabmail.closeOtherTabs(tabmail.tabModes.folder.tabs[0]);
 });
 
-add_task(async function test_selection() {
-  let extension = createExtension();
-  await extension.startup();
+async function subtest_content(
+  extension,
+  extensionHasPermission,
+  browser,
+  pageUrl,
+  tab
+) {
+  let { ownerDocument, ownerGlobal } = browser;
+  let menu = ownerDocument.getElementById(browser.getAttribute("context"));
 
+  BrowserTestUtils.synthesizeMouseAtCenter("body", {}, browser);
+
+  // Test a part of the page with no content.
+
+  await rightClickOnContent(menu, "body", browser);
+  Assert.ok(menu.querySelector("#test1_mochi_test-menuitem-_page"));
+  menu.hidePopup();
+
+  await checkShownEvent(
+    extension,
+    {
+      menuIds: ["page"],
+      contexts: ["page", "all"],
+      pageUrl: extensionHasPermission ? pageUrl : undefined,
+    },
+    tab
+  );
+
+  // Test selection.
+
+  ContentTask.spawn(browser, null, () => {
+    let text = content.document.querySelector("p");
+    content.getSelection().selectAllChildren(text);
+  });
+  await rightClickOnContent(menu, "p", browser);
+  Assert.ok(menu.querySelector("#test1_mochi_test-menuitem-_selection"));
+  await checkShownEvent(
+    extension,
+    {
+      pageUrl: extensionHasPermission ? pageUrl : undefined,
+      selectionText: extensionHasPermission ? "This is text." : undefined,
+      menuIds: ["selection"],
+      contexts: ["selection", "all"],
+    },
+    tab
+  );
+
+  EventUtils.synthesizeMouseAtCenter(
+    menu.querySelector("#test1_mochi_test-menuitem-_selection"),
+    {},
+    ownerGlobal
+  );
+  await checkClickedEvent(
+    extension,
+    {
+      pageUrl,
+      selectionText: "This is text.",
+    },
+    tab
+  );
+
+  BrowserTestUtils.synthesizeMouseAtCenter("body", {}, browser); // Select nothing.
+
+  // Test link.
+
+  await rightClickOnContent(menu, "a", browser);
+  Assert.ok(menu.querySelector("#test1_mochi_test-menuitem-_link"));
+  await checkShownEvent(
+    extension,
+    {
+      pageUrl: extensionHasPermission ? pageUrl : undefined,
+      menuIds: ["link"],
+      contexts: ["link", "all"],
+    },
+    tab
+  );
+
+  EventUtils.synthesizeMouseAtCenter(
+    menu.querySelector("#test1_mochi_test-menuitem-_link"),
+    {},
+    ownerGlobal
+  );
+  await checkClickedEvent(
+    extension,
+    {
+      pageUrl,
+      linkUrl: "http://mochi.test:8888/",
+      linkText: "This is a link with text.",
+    },
+    tab
+  );
+
+  // Test image.
+
+  await rightClickOnContent(menu, "img", browser);
+  Assert.ok(menu.querySelector("#test1_mochi_test-menuitem-_image"));
+  await checkShownEvent(
+    extension,
+    {
+      pageUrl: extensionHasPermission ? pageUrl : undefined,
+      menuIds: ["image"],
+      contexts: ["image", "all"],
+    },
+    tab
+  );
+
+  EventUtils.synthesizeMouseAtCenter(
+    menu.querySelector("#test1_mochi_test-menuitem-_image"),
+    {},
+    ownerGlobal
+  );
+  await checkClickedEvent(
+    extension,
+    {
+      pageUrl,
+      srcUrl: `${URL_BASE}/tb-logo.png`,
+    },
+    tab
+  );
+}
+
+add_task(async function test_content() {
+  window.gFolderTreeView.selectFolder(gFolders[0]);
   if (window.IsMessagePaneCollapsed()) {
     window.MsgToggleMessagePane();
   }
@@ -382,82 +572,81 @@ add_task(async function test_selection() {
   let oldPref = Services.prefs.getStringPref("mailnews.start_page.url");
   Services.prefs.setStringPref(
     "mailnews.start_page.url",
-    "http://mochi.test:8888/browser/comm/mail/components/extensions/test/browser/data/content.html"
+    `${URL_BASE}/content.html`
   );
 
-  let tabmail = document.getElementById("tabmail");
-  let menu = document.getElementById("mailContext");
+  let messagePane = document.getElementById("messagepane");
 
+  let loadPromise = BrowserTestUtils.browserLoaded(messagePane);
   window.loadStartPage();
-  await BrowserTestUtils.waitForEvent(
-    tabmail.selectedBrowser,
-    "DOMContentLoaded"
-  );
+  await loadPromise;
 
-  let { contentDocument, contentWindow } = tabmail.selectedBrowser;
+  let extension = createExtension("<all_urls>");
+  await extension.startup();
 
-  let text = contentDocument.querySelector("p");
-  EventUtils.synthesizeMouseAtCenter(text, {}, contentWindow);
-  contentWindow.getSelection().selectAllChildren(text);
-  let shownPromise = BrowserTestUtils.waitForEvent(menu, "popupshown");
-  EventUtils.synthesizeMouseAtCenter(
-    text,
-    { type: "contextmenu" },
-    contentWindow
-  );
-
-  await shownPromise;
-  Assert.ok(menu.querySelector("#test1_mochi_test-menuitem-_selection"));
-  await checkShownEvent(
+  await subtest_content(
     extension,
-    { menuIds: ["selection"], contexts: ["selection", "all"] },
+    true,
+    messagePane,
+    `${URL_BASE}/content.html`,
     { active: true, index: 0, mailTab: true }
   );
 
-  EventUtils.synthesizeMouseAtCenter(
-    menu.querySelector("#test1_mochi_test-menuitem-_selection"),
-    {}
-  );
-  await checkClickedEvent(extension, {
-    selectionText: "This is text.",
-  });
-
-  let link = contentDocument.querySelector("a");
-  shownPromise = BrowserTestUtils.waitForEvent(menu, "popupshown");
-  EventUtils.synthesizeMouseAtCenter(
-    link,
-    { type: "contextmenu" },
-    contentWindow
-  );
-
-  await shownPromise;
-  Assert.ok(menu.querySelector("#test1_mochi_test-menuitem-_selection"));
-  await checkShownEvent(
-    extension,
-    { menuIds: ["link", "selection"], contexts: ["link", "selection", "all"] },
-    { active: true, index: 0, mailTab: true }
-  );
-
-  shownPromise = BrowserTestUtils.waitForEvent(menu, "popupshown");
-  EventUtils.synthesizeMouseAtCenter(
-    menu.querySelector(`menu[id^="test1_mochi_test-menuitem"]`),
-    {}
-  );
-  await shownPromise;
-  EventUtils.synthesizeMouseAtCenter(
-    menu.querySelector("#test1_mochi_test-menuitem-_link"),
-    {}
-  );
-  await checkClickedEvent(extension, {
-    linkUrl: "http://mochi.test:8888/",
-    linkText: "This is a link with text.",
-    selectionText: "This is text.",
-  });
-
   await extension.unload();
+
   Services.prefs.setStringPref("mailnews.start_page.url", oldPref);
 });
 
+add_task(async function test_content_tab() {
+  let tab = window.openContentTab(`${URL_BASE}/content.html`);
+  await BrowserTestUtils.browserLoaded(tab.browser);
+
+  let extension = createExtension("<all_urls>");
+  await extension.startup();
+
+  await subtest_content(
+    extension,
+    true,
+    tab.browser,
+    `${URL_BASE}/content.html`,
+    { active: true, index: 1, mailTab: false }
+  );
+
+  await extension.unload();
+
+  let tabmail = document.getElementById("tabmail");
+  tabmail.closeOtherTabs(tabmail.tabModes.folder.tabs[0]);
+});
+
+add_task(async function test_content_window() {
+  let extensionWindowPromise = BrowserTestUtils.domWindowOpened();
+  window.openDialog(
+    "chrome://messenger/content/extensionPopup.xhtml",
+    "_blank",
+    "width=800,height=500",
+    `${URL_BASE}/content.html`
+  );
+  let extensionWindow = await extensionWindowPromise;
+  await BrowserTestUtils.waitForEvent(extensionWindow, "load");
+  await BrowserTestUtils.browserLoaded(extensionWindow.browser);
+  await focusWindow(extensionWindow);
+
+  let extension = createExtension("<all_urls>");
+  await extension.startup();
+
+  await subtest_content(
+    extension,
+    true,
+    extensionWindow.browser,
+    `${URL_BASE}/content.html`,
+    { active: true, index: 0, mailTab: false }
+  );
+
+  await extension.unload();
+
+  await BrowserTestUtils.closeWindow(extensionWindow);
+});
+
 async function subtest_compose(...permissions) {
   let extension = createExtension(...permissions);
   await extension.startup();
@@ -469,6 +658,10 @@ async function subtest_compose(...permissions) {
     "@mozilla.org/messengercompose/composefields;1"
   ].createInstance(Ci.nsIMsgCompFields);
 
+  params.composeFields.body = await fetch(
+    `${URL_BASE}/content_body.html`
+  ).then(r => r.text());
+
   for (let ordinal of ["first", "second", "third", "fourth"]) {
     let attachment = Cc[
       "@mozilla.org/messengercompose/attachment;1"
@@ -482,10 +675,24 @@ async function subtest_compose(...permissions) {
   let composeWindowPromise = BrowserTestUtils.domWindowOpened();
   MailServices.compose.OpenComposeWindowWithParams(null, params);
   let composeWindow = await composeWindowPromise;
-  await BrowserTestUtils.waitForEvent(composeWindow, "load");
+  await BrowserTestUtils.waitForEvent(composeWindow, "compose-editor-ready");
   let composeDocument = composeWindow.document;
   await focusWindow(composeWindow);
 
+  // Test the message being composed.
+
+  let messagePane = composeWindow.GetCurrentEditorElement();
+
+  await subtest_content(
+    extension,
+    permissions.includes("compose"),
+    messagePane,
+    /^about:blank\?compose$/,
+    { active: true, index: 0, mailTab: false }
+  );
+
+  // Test the attachments context menu.
+
   composeWindow.toggleAttachmentPane("show");
   let menu = composeDocument.getElementById("msgComposeAttachmentItemContext");
   let attachmentBucket = composeDocument.getElementById("attachmentBucket");
@@ -495,13 +702,7 @@ async function subtest_compose(...permissions) {
     {},
     composeWindow
   );
-  let shownPromise = BrowserTestUtils.waitForEvent(menu, "popupshowing");
-  EventUtils.synthesizeMouseAtCenter(
-    attachmentBucket.itemChildren[0],
-    { type: "contextmenu" },
-    composeWindow
-  );
-  await shownPromise;
+  await rightClick(menu, attachmentBucket.itemChildren[0], composeWindow);
   Assert.ok(
     menu.querySelector("#test1_mochi_test-menuitem-_compose_attachments")
   );
@@ -520,13 +721,7 @@ async function subtest_compose(...permissions) {
   );
 
   attachmentBucket.addItemToSelection(attachmentBucket.itemChildren[3]);
-  shownPromise = BrowserTestUtils.waitForEvent(menu, "popupshowing");
-  EventUtils.synthesizeMouseAtCenter(
-    attachmentBucket.itemChildren[0],
-    { type: "contextmenu" },
-    composeWindow
-  );
-  await shownPromise;
+  await rightClick(menu, attachmentBucket.itemChildren[0], composeWindow);
   Assert.ok(
     menu.querySelector("#test1_mochi_test-menuitem-_compose_attachments")
   );
