@@ -52,13 +52,9 @@ var preferencesTabType = {
     if (!gPrefTab) {
       return -1;
     }
-    let prefWindow = gPrefTab.browser.contentDocument.getElementById(
-      "MailPreferences"
-    );
-    gPrefTab.browser.contentWindow.selectPaneAndTab(
-      prefWindow,
+    gPrefTab.browser.contentWindow.selectPrefPane(
       aArgs.paneID,
-      aArgs.tabID,
+      aArgs.scrollPaneTo,
       aArgs.otherArgs
     );
     return document.getElementById("tabmail").tabInfo.indexOf(gPrefTab);
@@ -76,7 +72,7 @@ var preferencesTabType = {
     // First clone the page and set up the basics.
     let clone = document
       .getElementById("preferencesTab")
-      .firstChild.cloneNode(true);
+      .firstElementChild.cloneNode(true);
 
     clone.setAttribute("id", "preferencesTab" + this.lastBrowserId);
     clone.setAttribute("collapsed", false);
@@ -103,7 +99,7 @@ var preferencesTabType = {
 
     aTab.url = aArgs.contentPage;
     aTab.paneID = aArgs.paneID;
-    aTab.tabID = aArgs.tabID;
+    aTab.scrollPaneTo = aArgs.scrollPaneTo;
     aTab.otherArgs = aArgs.otherArgs;
 
     // Now set up the listeners.
@@ -121,8 +117,11 @@ var preferencesTabType = {
         if ("onLoad" in aArgs) {
           // Let selection of the initial pane complete before selecting another.
           // Otherwise we can end up with two panes selected at once.
-          setTimeout(() => {
-            aArgs.onLoad(event, aTab.browser);
+          aTab.browser.contentWindow.setTimeout(() => {
+            // By now, the tab could already be closed. Check that it isn't.
+            if (aTab.panel) {
+              aArgs.onLoad(event, aTab.browser);
+            }
           }, 0);
         }
       },
@@ -155,7 +154,7 @@ var preferencesTabType = {
     return {
       tabURI: aTab.url,
       paneID: aTab.paneID,
-      tabID: aTab.tabID,
+      scrollPaneTo: aTab.scrollPaneTo,
       otherArgs: aTab.otherArgs,
     };
   },
@@ -164,7 +163,7 @@ var preferencesTabType = {
     aTabmail.openTab("preferencesTab", {
       contentPage: aPersistedState.tabURI,
       paneID: aPersistedState.paneID,
-      tabID: aPersistedState.tabID,
+      scrollPaneTo: aPersistedState.scrollPaneTo,
       otherArgs: aPersistedState.otherArgs,
     });
   },
