@@ -13,15 +13,18 @@
 |   longer term, this code will be restructured to make it more reusable.      |
 ------------------------------------------------------------------------------*/
 
+var {BrowserUtils} =
+  ChromeUtils.import("resource://gre/modules/BrowserUtils.jsm");
 var {XPCOMUtils} = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
-var {Services} = ChromeUtils.import("resource://gre/modules/Services.jsm");
+var {Services} = ChromeUtils.import("resource://gre/modules/Services.jsm"
+var {LoginManagerContextMenu} =
+  ChromeUtils.import("resource://gre/modules/LoginManagerContextMenu.jsm");
 
-var gContextMenuContentData = null;
-
-XPCOMUtils.defineLazyGetter(this, "InlineSpellCheckerUI", function() {
-  let tmp = {};
-  ChromeUtils.import("resource://gre/modules/InlineSpellChecker.jsm", tmp);
-  return new tmp.InlineSpellChecker();
+XPCOMUtils.defineLazyGetter(this, "InlineSpellCheckerUI", () => {
+  let { InlineSpellChecker } = ChromeUtils.import(
+    "resource://gre/modules/InlineSpellChecker.jsm"
+  );
+  return new InlineSpellChecker();
 });
 
 XPCOMUtils.defineLazyGetter(this, "PageMenuParent", function() {
@@ -31,12 +34,18 @@ XPCOMUtils.defineLazyGetter(this, "PageMenuParent", function() {
 });
 
 XPCOMUtils.defineLazyModuleGetters(this, {
-  DevToolsShim: "chrome://devtools-shim/content/DevToolsShim.jsm",
+  SpellCheckHelper: "resource://gre/modules/InlineSpellChecker.jsm",
   findCssSelector: "resource://gre/modules/css-selector.js",
+  LoginHelper: "resource://gre/modules/LoginHelper.jsm",
+  LoginManagerContent: "resource://gre/modules/LoginManagerContent.jsm",
+  DevToolsShim: "chrome://devtools-shim/content/DevToolsShim.jsm",
   NetUtil: "resource://gre/modules/NetUtil.jsm",
   ShellService: "resource:///modules/ShellService.jsm",
+
 });
 
+var gContextMenuContentData = null;
+
 function nsContextMenu(aXulMenu, aIsShift, aEvent) {
   this.shouldDisplay = true;
   this.initMenu(aXulMenu, aIsShift, aEvent);
@@ -62,11 +71,11 @@ nsContextMenu.prototype = {
 
     this.initPopupPrincipal();
 
-    // Initialize (disable/remove) menu items.
-    this.initItems();
     // Initialize gContextMenuContentData.
     if (aEvent)
       this.initContentData(aEvent);
+    // Initialize (disable/remove) menu items.
+    this.initItems();
   },
 
   initContentData: function(aEvent) {
@@ -108,6 +117,7 @@ nsContextMenu.prototype = {
       event: aEvent,
       popupNode: popupNode,
       browser: this.browser,
+      principal: doc.nodePrincipal,
       addonInfo: addonInfo,
       documentURIObject: doc.documentURIObject,
       docLocation: doc.location.href,
@@ -119,6 +129,7 @@ nsContextMenu.prototype = {
       frameOuterWindowID: doc.defaultView.QueryInterface(Ci.nsIInterfaceRequestor)
                                          .getInterface(Ci.nsIDOMWindowUtils)
                                          .outerWindowID,
+      loginFillInfo: LoginManagerContent.getFieldContext(popupNode),
     };
   },
 
@@ -127,6 +138,7 @@ nsContextMenu.prototype = {
     InlineSpellCheckerUI.clearSuggestionsFromMenu();
     InlineSpellCheckerUI.clearDictionaryListFromMenu();
     InlineSpellCheckerUI.uninit();
+    LoginManagerContextMenu.clearLoginsFromMenu(document);
   },
 
   initItems: function() {
@@ -140,6 +152,7 @@ nsContextMenu.prototype = {
     this.initClipboardItems();
     this.initMetadataItems();
     this.initMediaPlayerItems();
+    this.initPasswordManagerItems();
   },
 
   initPageMenuSeparator: function() {
@@ -191,7 +204,8 @@ nsContextMenu.prototype = {
     this.showItem("context-sendlink", this.onSaveableLink);
 
     // Save image depends on having loaded its content, video and audio don't.
-    showSave = this.onLoadedImage || this.onStandaloneImage || this.onCanvas;
+    showSave = (this.onLoadedImage && this.onCompletedImage) ||
+               this.onStandaloneImage || this.onCanvas;
     if (showSave)
       goSetMenuValue("context-saveimage",
                      this.autoDownload ? "valueSave" : "valueSaveAs");
@@ -261,7 +275,8 @@ nsContextMenu.prototype = {
                        content.document.imageIsResized ? "true" : null);
     }
 
-    this.showItem("context-reloadimage", this.onImage);
+    // Reload image depends on an image that's not fully loaded
+    this.showItem("context-reloadimage", (this.onImage && !this.onCompletedImage));
 
     // View image depends on having an image that's not standalone
     // (or is in a frame), or a canvas.
@@ -280,6 +295,8 @@ nsContextMenu.prototype = {
     this.showItem("context-sep-viewbgimage", showView && !this.inSyntheticDoc);
     this.setItemAttr("context-viewbgimage", "disabled", this.hasBGImage ? null : "true");
 
+    this.showItem("context-viewimageinfo", this.onImage);
+
     // Hide Block and Unblock menuitems.
     this.showItem("context-blockimage", false);
     this.showItem("context-unblockimage", false);
@@ -340,21 +357,22 @@ nsContextMenu.prototype = {
     this.showItem("context-sep-popup", this.popupPrincipal);
 
     // BiDi UI
-    this.showItem("context-sep-bidi", gShowBiDi);
+    this.showItem("context-sep-bidi", !this.onNumeric && gShowBiDi);
     this.showItem("context-bidi-text-direction-toggle",
-                  this.onTextInput && gShowBiDi);
+                  this.onTextInput && !this.onNumeric && gShowBiDi);
     this.showItem("context-bidi-page-direction-toggle",
                   !this.onTextInput && gShowBiDi);
   },
 
   initSpellingItems: function() {
     var canSpell = InlineSpellCheckerUI.canSpellCheck &&
-                   !InlineSpellCheckerUI.initialSpellCheckPending;
+                   !InlineSpellCheckerUI.initialSpellCheckPending &&
+                   this.canSpellCheck;
+    let showDictionaries = canSpell && InlineSpellCheckerUI.enabled;
     var onMisspelling = InlineSpellCheckerUI.overMisspelling;
-    var showUndo = InlineSpellCheckerUI.enabled &&
-                   InlineSpellCheckerUI.canUndo();
+    var showUndo = canSpell && InlineSpellCheckerUI.canUndo();
     this.showItem("spell-check-enabled", canSpell);
-    this.showItem("spell-separator", canSpell || this.possibleSpellChecking);
+    this.showItem("spell-separator", canSpell);
     if (canSpell)
       this.setItemAttr("spell-check-enabled", "checked", InlineSpellCheckerUI.enabled);
     this.showItem("spell-add-to-dictionary", onMisspelling);
@@ -373,18 +391,23 @@ nsContextMenu.prototype = {
     }
 
     // dictionary list
-    this.showItem("spell-dictionaries", InlineSpellCheckerUI.enabled);
+    this.showItem("spell-dictionaries", showDictionaries);
     var dictMenu = document.getElementById("spell-dictionaries-menu");
     if (canSpell && dictMenu) {
       var dictSep = document.getElementById("spell-language-separator");
-      InlineSpellCheckerUI.addDictionaryListToMenu(dictMenu, dictSep);
+      let count = InlineSpellCheckerUI.addDictionaryListToMenu(dictMenu, dictSep);
+      this.showItem(dictSep, count > 0);
+      this.showItem("spell-add-dictionaries-main", false);
     }
-
-    // when there is no spellchecker but we might be able to spellcheck
-    // add the add to dictionaries item. This will ensure that people
-    // with no dictionaries will be able to download them
-    this.showItem("spell-add-dictionaries-main",
-                  !canSpell && this.possibleSpellChecking);
+    else if (this.onEditableArea) {
+      // when there is no spellchecker but we might be able to spellcheck
+      // add the add to dictionaries item. This will ensure that people
+      // with no dictionaries will be able to download them
+      this.showItem("spell-language-separator", showDictionaries);
+      this.showItem("spell-add-dictionaries-main", showDictionaries);
+    }
+    else
+      this.showItem("spell-add-dictionaries-main", false);
   },
 
   initClipboardItems: function() {
@@ -492,6 +515,60 @@ nsContextMenu.prototype = {
     this.showItem("context-media-sep-commands", onMedia);
   },
 
+  initPasswordManagerItems: function() {
+    let fillMenu = document.getElementById("fill-login");
+    // If no fill Menu, probably mailContext so nothing to set up.
+    if (!fillMenu)
+      return;
+
+    let loginFillInfo = gContextMenuContentData && gContextMenuContentData.loginFillInfo;
+
+    // If we could not find a password field we
+    // don't want to show the form fill option.
+    let showFill = loginFillInfo && loginFillInfo.passwordField.found;
+
+    // Disable the fill option if the user has set a master password
+    // or if the password field or target field are disabled.
+    let disableFill = !loginFillInfo ||
+                      !Services.logins ||
+                      !Services.logins.isLoggedIn ||
+                      loginFillInfo.passwordField.disabled ||
+                      (!this.onPassword && loginFillInfo.usernameField.disabled);
+
+    this.showItem("fill-login-separator", showFill);
+    this.showItem("fill-login", showFill);
+    this.setItemAttr("fill-login", "disabled", disableFill);
+
+    // Set the correct label for the fill menu
+    if (this.onPassword) {
+      fillMenu.setAttribute("label", fillMenu.getAttribute("label-password"));
+      fillMenu.setAttribute("accesskey", fillMenu.getAttribute("accesskey-password"));
+    } else {
+      fillMenu.setAttribute("label", fillMenu.getAttribute("label-login"));
+      fillMenu.setAttribute("accesskey", fillMenu.getAttribute("accesskey-login"));
+    }
+
+    if (!showFill || disableFill) {
+      return;
+    }
+    let documentURI = gContextMenuContentData.documentURIObject;
+    let fragment = LoginManagerContextMenu.addLoginsToMenu(this.target, this.browser, documentURI);
+
+    this.showItem("fill-login-no-logins", !fragment);
+
+    if (!fragment) {
+      return;
+    }
+    let popup = document.getElementById("fill-login-popup");
+    let insertBeforeElement = document.getElementById("fill-login-no-logins");
+    popup.insertBefore(fragment, insertBeforeElement);
+  },
+
+  openPasswordManager: function() {
+    // LoginHelper.openPasswordManager(window, gContextMenuContentData.documentURIObject.host);
+    toDataManager(gContextMenuContentData.documentURIObject.host + '|passwords');
+  },
+
   /**
    * Retrieve the array of CSS selectors corresponding to the provided node. The first item
    * of the array is the selector of the node in its owner document. Additional items are
@@ -531,12 +608,14 @@ nsContextMenu.prototype = {
     // Initialize contextual info.
     this.onImage               = false;
     this.onLoadedImage         = false;
+    this.onCompletedImage      = false;
     this.onStandaloneImage     = false;
     this.onCanvas              = false;
     this.onVideo               = false;
     this.onAudio               = false;
     this.onMetaDataItem        = false;
     this.onTextInput           = false;
+    this.onNumeric             = false;
     this.onKeywordField        = false;
     this.mediaURL              = "";
     this.onLink                = false;
@@ -547,6 +626,7 @@ nsContextMenu.prototype = {
     this.linkURL               = "";
     this.linkURI               = null;
     this.linkProtocol          = "";
+    this.linkHasNoReferrer     = false;
     this.onMathML              = false;
     this.inFrame               = false;
     this.inSyntheticDoc        = false;
@@ -556,39 +636,29 @@ nsContextMenu.prototype = {
     this.autoDownload          = false;
     this.isTextSelected        = false;
     this.isContentSelected     = false;
-    this.possibleSpellChecking = false;
+    this.onEditableArea        = false;
+    this.canSpellCheck         = false;
+    this.onPassword            = false;
 
     // Remember the node that was clicked.
     this.target = aNode;
 
-    if (aNode.namespaceURI == xulNS) {
+    if (aNode.nodeType == Node.DOCUMENT_NODE ||
+        // Not display on XUL element but relax for <label class="text-link">
+        (aNode.namespaceURI == xulNS && !isXULTextLinkLabel(aNode))) {
       this.shouldDisplay = false;
       return;
     }
 
-    this.autoDownload = Services.prefs.getBoolPref("browser.download.useDownloadDir");
+    let editFlags = SpellCheckHelper.isEditable(this.target, window);
+    this.browser = this.target.ownerDocument.defaultView
+                              .QueryInterface(Ci.nsIInterfaceRequestor)
+                              .getInterface(Ci.nsIWebNavigation)
+                              .QueryInterface(Ci.nsIDocShell)
+                              .chromeEventHandler;
+    this.principal = this.target.ownerDocument.nodePrincipal;
 
-    // if the document is editable, show context menu like in text inputs
-    var win = this.target.ownerDocument.defaultView;
-    if (win) {
-      var webNav = win.QueryInterface(Ci.nsIInterfaceRequestor)
-                      .getInterface(Ci.nsIWebNavigation);
-      this.browser = webNav.QueryInterface(Ci.nsIDocShell)
-                           .chromeEventHandler;
-      var editingSession = webNav.QueryInterface(Ci.nsIInterfaceRequestor)
-                                 .getInterface(Ci.nsIEditingSession);
-      if (editingSession.windowIsEditable(win) &&
-          this.isTargetEditable() && this.target.spellcheck) {
-        this.onTextInput           = true;
-        this.possibleSpellChecking = true;
-        InlineSpellCheckerUI.init(editingSession.getEditorForWindow(win));
-        InlineSpellCheckerUI.initFromEvent(aRangeParent, aRangeOffset);
-        var canSpell = InlineSpellCheckerUI.canSpellCheck;
-        this.showItem("spell-check-enabled", canSpell);
-        this.showItem("spell-separator", canSpell);
-        return;
-      }
-    }
+    this.autoDownload = Services.prefs.getBoolPref("browser.download.useDownloadDir");
 
     // Check if we are in a synthetic document (stand alone image, video, etc.).
     this.inSyntheticDoc = this.target.ownerDocument.mozSyntheticDocument;
@@ -603,6 +673,11 @@ nsContextMenu.prototype = {
           this.target.getRequest(Ci.nsIImageLoadingContent.CURRENT_REQUEST);
         if (request && (request.imageStatus & request.STATUS_SIZE_AVAILABLE))
           this.onLoadedImage = true;
+        if (request &&
+            (request.imageStatus & request.STATUS_LOAD_COMPLETE) &&
+            !(request.imageStatus & request.STATUS_ERROR)) {
+          this.onCompletedImage = true;
+        }
 
         this.mediaURL = this.target.currentURI.spec;
 
@@ -628,24 +703,16 @@ nsContextMenu.prototype = {
         this.onAudio = true;
         this.mediaURL = this.target.currentSrc || this.target.src;
       }
-      else if (this.target instanceof HTMLInputElement) {
-        this.onTextInput = this.isTargetATextBox(this.target);
-        // allow spellchecking UI on all writable text boxes except passwords
-        if (this.onTextInput && !this.target.readOnly &&
-            this.target.mozIsTextField(true) && this.target.spellcheck) {
-          this.possibleSpellChecking = true;
-          InlineSpellCheckerUI.init(this.target.editor);
-          InlineSpellCheckerUI.initFromEvent(aRangeParent, aRangeOffset);
-        }
-        this.onKeywordField = this.isTargetAKeywordField(this.target);
-      }
-      else if (this.target instanceof HTMLTextAreaElement) {
-        this.onTextInput = this.isTextBoxEnabled(this.target);
-        if (this.onTextInput && !this.target.readOnly && this.target.spellcheck) {
-          this.possibleSpellChecking = true;
+      else if (editFlags & (SpellCheckHelper.INPUT | SpellCheckHelper.TEXTAREA)) {
+        this.onTextInput = (editFlags & SpellCheckHelper.TEXTINPUT) !== 0;
+        this.onNumeric = (editFlags & SpellCheckHelper.NUMERIC) !== 0;
+        this.onEditableArea = (editFlags & SpellCheckHelper.EDITABLE) !== 0;
+        this.onPassword = (editFlags & SpellCheckHelper.PASSWORD) !== 0;
+        if (this.onEditableArea) {
           InlineSpellCheckerUI.init(this.target.editor);
           InlineSpellCheckerUI.initFromEvent(aRangeParent, aRangeOffset);
         }
+        this.onKeywordField = (editFlags & SpellCheckHelper.KEYWORD);
       }
       else if ( this.target instanceof HTMLHtmlElement ) {
         // pages with multiple <body>s are lame. we'll teach them a lesson.
@@ -696,32 +763,29 @@ nsContextMenu.prototype = {
           }
         }
       }
+
+      this.canSpellCheck = this._isSpellCheckEnabled(this.target);
+    }
+    else if (this.target.nodeType == Node.TEXT_NODE) {
+      // For text nodes, look at the parent node to determine the spellcheck attribute.
+      this.canSpellCheck = this.target.parentNode &&
+                           this._isSpellCheckEnabled(this.target);
     }
 
     // We have meta data on images.
     this.onMetaDataItem = this.onImage;
 
-    // See if the user clicked on MathML
-    const NS_MathML = "http://www.w3.org/1998/Math/MathML";
-    if ((this.target.nodeType == Node.TEXT_NODE &&
-         this.target.parentNode.namespaceURI == NS_MathML) ||
-        (this.target.namespaceURI == NS_MathML))
-      this.onMathML = true;
-
-    // See if the user clicked in a frame.
-    var docDefaultView = this.target.ownerDocument.defaultView;
-    if (docDefaultView != docDefaultView.top)
-      this.inFrame = true;
-
     // Bubble out, looking for items of interest
+    const NS_MathML = "http://www.w3.org/1998/Math/MathML";
     const XMLNS = "http://www.w3.org/XML/1998/namespace";
     var elem = this.target;
     while (elem) {
       if (elem.nodeType == Node.ELEMENT_NODE) {
         // Link?
         if (!this.onLink &&
-            ((elem instanceof HTMLAnchorElement && elem.href) ||
-             elem instanceof HTMLAreaElement ||
+            (isXULTextLinkLabel(elem) ||
+             (elem instanceof HTMLAnchorElement && elem.href) ||
+             (elem instanceof HTMLAreaElement && elem.href) ||
              elem instanceof HTMLLinkElement ||
              (elem.namespaceURI == NS_MathML && elem.hasAttribute("href")) ||
              elem.getAttributeNS("http://www.w3.org/1999/xlink", "type") == "simple")) {
@@ -736,6 +800,7 @@ nsContextMenu.prototype = {
           this.onMailtoLink = (this.linkProtocol == "mailto");
           // Remember if it is saveable.
           this.onSaveableLink = this.isLinkSaveable();
+          this.linkHasNoReferrer = BrowserUtils.linkHasNoReferrer(elem);
         }
 
         // Text input?
@@ -771,6 +836,51 @@ nsContextMenu.prototype = {
       }
       elem = elem.parentNode;
     }
+
+    // See if the user clicked on MathML
+    if ((this.target.nodeType == Node.TEXT_NODE &&
+         this.target.parentNode.namespaceURI == NS_MathML) ||
+        (this.target.namespaceURI == NS_MathML))
+      this.onMathML = true;
+
+    // See if the user clicked in a frame.
+    var docDefaultView = this.target.ownerDocument.defaultView;
+    if (docDefaultView != docDefaultView.top)
+      this.inFrame = true;
+
+    // if the document is editable, show context menu like in text inputs
+    if (!this.onEditableArea) {
+      if (editFlags & SpellCheckHelper.CONTENTEDITABLE) {
+        // If this.onEditableArea is false but editFlags is CONTENTEDITABLE,
+        // then the document itself must be editable.
+        this.onTextInput       = true;
+        this.onKeywordField    = false;
+        this.onImage           = false;
+        this.onLoadedImage     = false;
+        this.onCompletedImage  = false;
+        this.onMathML          = false;
+        this.inFrame           = false;
+        this.hasBGImage        = false;
+        this.onEditableArea    = true;
+        var win = this.target.ownerDocument.defaultView;
+        var editingSession = win.QueryInterface(Ci.nsIInterfaceRequestor)
+                                .getInterface(Ci.nsIWebNavigation)
+                                .QueryInterface(Ci.nsIInterfaceRequestor)
+                                .getInterface(Ci.nsIEditingSession);
+        InlineSpellCheckerUI.init(editingSession.getEditorForWindow(win));
+        InlineSpellCheckerUI.initFromEvent(aRangeParent, aRangeOffset);
+        var canSpell = InlineSpellCheckerUI.canSpellCheck && this.canSpellCheck;
+        this.showItem("spell-check-enabled", canSpell);
+        this.showItem("spell-separator", canSpell);
+      }
+    }
+
+    function isXULTextLinkLabel(node) {
+      return node.namespaceURI == xulNS &&
+             node.tagName == "label" &&
+             node.classList.contains('text-link') &&
+             node.href;
+    }
   },
 
   initPopupPrincipal: function() {
@@ -800,6 +910,23 @@ nsContextMenu.prototype = {
     }
   },
 
+  _isSpellCheckEnabled: function(aNode) {
+    // We can always force-enable spellchecking on textboxes
+    if (this.isTargetATextBox(aNode)) {
+      return true;
+    }
+    // We can never spell check something which is not content editable
+    var editable = aNode.isContentEditable;
+    if (!editable && aNode.ownerDocument) {
+      editable = aNode.ownerDocument.designMode == "on";
+    }
+    if (!editable) {
+      return false;
+    }
+    // Otherwise make sure that nothing in the parent chain disables spellchecking
+    return aNode.spellcheck;
+  },
+
   // Returns the computed style attribute for the given element.
   getComputedStyle: function(aElem, aProp) {
     return aElem.ownerDocument
@@ -851,26 +978,77 @@ nsContextMenu.prototype = {
     Services.perms.remove(uri, "image");
   },
 
+  _openLinkInParameters : function (extra) {
+    let params = { charset: gContextMenuContentData.charSet,
+                   originPrincipal: this.principal,
+                   triggeringPrincipal: this.principal,
+                   referrerURI: gContextMenuContentData.documentURIObject,
+                   referrerPolicy: gContextMenuContentData.referrerPolicy,
+                   noReferrer: this.linkHasNoReferrer || this.onPlainTextLink };
+    for (let p in extra) {
+      params[p] = extra[p];
+    }
+
+    // If we want to change userContextId, we must be sure that we don't
+    // propagate the referrer.
+    if ("userContextId" in params &&
+        params.userContextId != this.principal.originAttributes.userContextId) {
+      params.noReferrer = true;
+    }
+
+    return params;
+  },
+
   // Open linked-to URL in a new tab.
   openLinkInTab: function(aEvent) {
-    // Determine linked-to URL.
-    return openNewTabWith(this.linkURL, this.link, null, aEvent);
+    urlSecurityCheck(this.linkURL, this.principal);
+    let referrerURI = gContextMenuContentData.documentURIObject;
+
+    // if the mixedContentChannel is present and the referring URI passes
+    // a same origin check with the target URI, we can preserve the users
+    // decision of disabling MCB on a page for it's child tabs.
+    let persistAllowMixedContentInChildTab = false;
+
+    if (this.browser.docShell.mixedContentChannel) {
+      const sm = Services.scriptSecurityManager;
+      try {
+        let targetURI = this.linkURI;
+        sm.checkSameOriginURI(referrerURI, targetURI, false);
+        persistAllowMixedContentInChildTab = true;
+      }
+      catch (e) { }
+    }
+
+    let params = {
+      allowMixedContent: persistAllowMixedContentInChildTab,
+      userContextId: parseInt(aEvent.target.getAttribute('usercontextid')),
+    };
+
+    openLinkIn(this.linkURL,
+               aEvent && aEvent.shiftKey ? "tabshifted" : "tab",
+               this._openLinkInParameters(params));
   },
 
   // Open linked-to URL in a new window.
   openLinkInWindow: function() {
-    return openNewWindowWith(this.linkURL, this.link);
+    urlSecurityCheck(this.linkURL, this.principal);
+    openLinkIn(this.linkURL, "window", this._openLinkInParameters());
   },
 
   // Open linked-to URL in a private window.
   openLinkInPrivateWindow: function() {
-    return openNewPrivateWith(this.linkURL, this.link);
+    urlSecurityCheck(this.linkURL, this.principal);
+    openLinkIn(this.linkURL, "window",
+               this._openLinkInParameters({ private: true }));
   },
 
   // Open frame in a new tab.
   openFrameInTab: function(aEvent) {
-    return openNewTabWith(this.target.ownerDocument.location.href,
-                          this.target.ownerDocument, null, aEvent);
+    let referrer = gContextMenuContentData.referrer;
+    openLinkIn(gContextMenuContentData.docLocation,
+               aEvent && aEvent.shiftKey ? "tabshifted" : "tab",
+               { charset: gContextMenuContentData.charSet,
+                 referrerURI: referrer ? makeURI(referrer) : null });
   },
 
   // Reload clicked-in frame.
@@ -880,8 +1058,10 @@ nsContextMenu.prototype = {
 
   // Open clicked-in frame in its own window.
   openFrame: function() {
-    return openNewWindowWith(this.target.ownerDocument.location.href,
-                             this.target.ownerDocument);
+    let referrer = gContextMenuContentData.referrer;
+    openLinkIn(gContextMenuContentData.docLocation, "window",
+               { charset: gContextMenuContentData.charSet,
+                 referrerURI: referrer ? makeURI(referrer) : null });
   },
 
   printFrame: function() {
@@ -891,7 +1071,13 @@ nsContextMenu.prototype = {
 
   // Open clicked-in frame in the same window
   showOnlyThisFrame: function() {
-    openTopWin( this.target.ownerDocument.location.href, this.target.ownerDocument.defaultView );
+    urlSecurityCheck(gContextMenuContentData.docLocation,
+                     this.principal,
+                     Ci.nsIScriptSecurityManager.DISALLOW_SCRIPT);
+    let referrer = gContextMenuContentData.referrer;
+    openUILinkIn(gContextMenuContentData.docLocation, "current",
+                 { disallowInheritPrincipal: true,
+                   referrerURI: referrer ? makeURI(referrer) : null });
   },
 
   // View Partial Source
@@ -911,11 +1097,18 @@ nsContextMenu.prototype = {
   },
 
   viewInfo: function() {
-    BrowserPageInfo();
+    BrowserPageInfo(gContextMenuContentData.docLocation, null,
+                    null, null, this.browser);
+  },
+
+  viewImageInfo: function() {
+    BrowserPageInfo(gContextMenuContentData.docLocation, "mediaTab",
+                    this.target, null, this.browser);
   },
 
   viewFrameInfo: function() {
-    BrowserPageInfo(this.target.ownerDocument);
+    BrowserPageInfo(gContextMenuContentData.docLocation, null, null,
+                    gContextMenuContentData.frameOuterWindowID, this.browser);
   },
 
   toggleImageSize: function() {
@@ -924,28 +1117,34 @@ nsContextMenu.prototype = {
 
   // Reload image
   reloadImage: function() {
-    urlSecurityCheck(this.mediaURL, this.target.nodePrincipal,
-                     Ci.nsIScriptSecurityManager.ALLOW_CHROME);
+    urlSecurityCheck(this.mediaURL,
+                     this.target.nodePrincipal,
+                     Ci.nsIScriptSecurityManager.DISALLOW_SCRIPT);
     if (this.target instanceof Ci.nsIImageLoadingContent)
       this.target.forceReload();
   },
 
   // Change current window to the URL of the image, video, or audio.
-  viewMedia: function(aEvent) {
-    var viewURL;
-    if (this.onCanvas)
-      viewURL = this.target.toDataURL();
-    else {
-      viewURL = this.mediaURL;
-      urlSecurityCheck(viewURL, this.target.nodePrincipal,
-                       Ci.nsIScriptSecurityManager.ALLOW_CHROME);
+  viewMedia(e) {
+    let doc = this.target.ownerDocument;
+    let where = whereToOpenLink(e);
+    if (this.onCanvas) {
+      let blobUrl = URL.createObjectURL(this.target);
+      let systemPrincipal = Services.scriptSecurityManager
+                                    .getSystemPrincipal();
+      openUILinkIn(blobUrl, where,
+                   { referrerURI: doc.documentURIObject,
+                     triggeringPrincipal: systemPrincipal,
+                   });
+    } else {
+      urlSecurityCheck(this.mediaURL,
+                       this.target.nodePrincipal,
+                       Ci.nsIScriptSecurityManager.DISALLOW_SCRIPT);
+      openUILinkIn(this.mediaURL, where,
+                   { referrerURI: doc.documentURIObject,
+                     triggeringPrincipal: this.target.nodePrincipal,
+                   });
     }
-    var doc = this.target.ownerDocument;
-    var where = whereToOpenLink(aEvent);
-    if (where == "current")
-      openTopWin(viewURL, doc.defaultView);
-    else
-      openUILinkIn(viewURL, where, null, null, doc.documentURIObject);
   },
 
   saveVideoFrameAsImage: function () {
@@ -968,7 +1167,7 @@ nsContextMenu.prototype = {
                                   true, true,
                                   this.target.ownerDocument.documentURIObject,
                                   null, null, null, (gPrivate ? true : false),
-                                  this.target.ownerDocument.nodePrincipal);
+                                  this.principal);
   },
 
   // Full screen video playback
@@ -981,15 +1180,17 @@ nsContextMenu.prototype = {
   },
 
   // Change current window to the URL of the background image.
-  viewBGImage: function(aEvent) {
-    urlSecurityCheck(this.bgImageURL, this.target.nodePrincipal,
-                     Ci.nsIScriptSecurityManager.ALLOW_CHROME);
-    var doc = this.target.ownerDocument;
-    var where = whereToOpenLink(aEvent);
-    if (where == "current")
-      openTopWin(this.bgImageURL, doc.defaultView);
-    else
-      openUILinkIn(this.bgImageURL, where, null, null, doc.documentURIObject);
+  viewBGImage(e) {
+    urlSecurityCheck(this.bgImageURL,
+                     this.target.nodePrincipal,
+                     Ci.nsIScriptSecurityManager.DISALLOW_SCRIPT);
+
+    let doc = this.target.ownerDocument;
+    let where = whereToOpenLink(e);
+    openUILinkIn(this.bgImageURL, where,
+                 { referrerURI: doc.documentURIObject,
+                   triggeringPrincipal: this.target.nodePrincipal,
+                 });
   },
 
   setDesktopBackground: function() {
@@ -1008,7 +1209,7 @@ nsContextMenu.prototype = {
   // Save URL of clicked-on link.
   saveLink: function() {
     var doc = this.target.ownerDocument;
-    urlSecurityCheck(this.linkURL, this.target.nodePrincipal);
+    urlSecurityCheck(this.linkURL, this.principal);
     this.saveHelper(this.linkURL, this.linkText(), null, true, doc);
   },
 
@@ -1100,12 +1301,12 @@ nsContextMenu.prototype = {
     }
 
     // set up a channel to do the saving
-    var ios = Services.io;
-    var channel = ios.newChannel2(linkURL, null, null, null,
-                                  Services.scriptSecurityManager.getSystemPrincipal(),
-                                  null,
-                                  Ci.nsILoadInfo.SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
-                                  Ci.nsIContentPolicy.TYPE_OTHER);
+    var channel = NetUtil.newChannel({
+                    uri: makeURI(linkURL),
+                    loadUsingSystemPrincipal: true,
+                    securityFlags: Ci.nsILoadInfo.SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL
+                  });
+
     channel.notificationCallbacks = new Callbacks();
 
     var flags = Ci.nsIChannel.LOAD_CALL_CONTENT_SNIFFERS;
@@ -1145,15 +1346,17 @@ nsContextMenu.prototype = {
       saveImageURL(this.target.toDataURL(), "canvas.png", "SaveImageTitle",
                    true, false, referrerURI, null, null, null,
                    (gPrivate ? true : false),
-                   doc.nodePrincipal /* system, because blob: */);
+                   document.nodePrincipal /* system, because blob: */);
     else if (this.onImage) {
+      urlSecurityCheck(this.mediaURL, this.principal);
       saveImageURL(this.mediaURL, null, "SaveImageTitle", false,
                    false, referrerURI, null, gContextMenuContentData.contentType,
                    gContextMenuContentData.contentDisposition,
                    (gPrivate ? true : false),
-                   doc.nodePrincipal);
+                   this.principal);
     }
     else if (this.onVideo || this.onAudio) {
+      urlSecurityCheck(this.mediaURL, this.principal);
       var dialogTitle = this.onVideo ? "SaveVideoTitle" : "SaveAudioTitle";
       this.saveHelper(this.mediaURL, null, dialogTitle, false, doc);
     }
@@ -1463,18 +1666,6 @@ nsContextMenu.prototype = {
     return aNode instanceof HTMLTextAreaElement && this.isTextBoxEnabled(aNode);
   },
 
-  isTargetAKeywordField: function(aNode) {
-    if (!(aNode instanceof HTMLInputElement))
-      return false;
-
-    var form = aNode.form;
-    if (!form || !aNode.mozIsTextField(true))
-      return false;
-
-    return form.method == "get" || (form.method == "post" &&
-           form.enctype == "application/x-www-form-urlencoded");
-  },
-
   // Determines whether or not the separator with the specified ID should be
   // shown or not by determining if there are any non-hidden items between it
   // and the previous separator.
