@@ -98,12 +98,15 @@ function registerPlayPreview(mimeType, targetUrl) {
       aRequest.cancel(Cr.NS_BINDING_ABORTED);
 
       // Create a new channel that is viewer loaded as a resource.
-      var ioService = Services.io;
-      var channel = ios.newChannel2(targetUrl, null, null, null,
-                                    Services.scriptSecurityManager.getSystemPrincipal(),
-                                    null,
-                                    Ci.nsILoadInfo.SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
-                                    Ci.nsIContentPolicy.TYPE_OTHER);
+      var channel =
+        Services.io.newChannelFromURI(Services.io.newURI(targetUrl),
+                                      null,
+                                      Services.scriptSecurityManager
+                                              .getSystemPrincipal(),
+                                      null,
+                                      Ci.nsILoadInfo
+                                        .SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
+                                      Ci.nsIContentPolicy.TYPE_OTHER);
       channel.asyncOpen2(this.listener);
     },
 
@@ -190,7 +193,6 @@ function prepareTest(nextTest, url, skip) {
 // Tests a page with normal play preview registration (1/2)
 function test1a() {
   var notificationBox = gBrowser.getNotificationBox(gTestBrowser);
-  ok(!notificationBox.getNotificationWithValue("missing-plugins"), "Test 1a, Should not have displayed the missing plugin notification");
   ok(!notificationBox.getNotificationWithValue("blocked-plugins"), "Test 1a, Should not have displayed the blocked plugin notification");
 
   var pluginInfo = getTestPlugin();
