@@ -2,10 +2,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-ChromeUtils.import("resource:///modules/ABQueryUtils.jsm");
-ChromeUtils.import("resource:///modules/mailServices.js");
-ChromeUtils.import("resource://gre/modules/PluralForm.jsm");
-ChromeUtils.import("resource://gre/modules/Services.jsm");
+const {encodeABTermValue, getModelQuery} = ChromeUtils.import("resource:///modules/ABQueryUtils.jsm");
+const {MailServices} = ChromeUtils.import("resource:///modules/MailServices.jsm");
+const {PluralForm} = ChromeUtils.import("resource://gre/modules/PluralForm.jsm");
+var {Services} = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 const nsIAbListener = Ci.nsIAbListener;
 const kPrefMailAddrBookLastNameFirst = "mail.addr_book.lastnamefirst";
@@ -56,7 +56,7 @@ function OnLoadAddressBook()
 {
   gSearchInput = document.getElementById("searchInput");
 
-  verifyAccounts(null, false); 	// this will do migration, if we need to.
+  verifyAccounts(null, false); // this will do migration, if we need to.
 
   InitCommonJS();
 
@@ -100,25 +100,25 @@ function OnLoadAddressBook()
 
 function GetCurrentPrefs()
 {
-	// check "Show Name As" menu item based on pref
-	var menuitemID;
-	switch (Services.prefs.getIntPref(kPrefMailAddrBookLastNameFirst))
-	{
-		case kFirstNameFirst:
-			menuitemID = 'firstLastCmd';
-			break;
-		case kLastNameFirst:
-			menuitemID = 'lastFirstCmd';
-			break;
-		case kDisplayName:
-		default:
-			menuitemID = 'displayNameCmd';
-			break;
-	}
+  // check "Show Name As" menu item based on pref
+  var menuitemID;
+  switch (Services.prefs.getIntPref(kPrefMailAddrBookLastNameFirst))
+  {
+    case kFirstNameFirst:
+      menuitemID = 'firstLastCmd';
+      break;
+    case kLastNameFirst:
+      menuitemID = 'lastFirstCmd';
+      break;
+    case kDisplayName:
+    default:
+      menuitemID = 'displayNameCmd';
+      break;
+  }
 
-	var menuitem = top.document.getElementById(menuitemID);
-	if ( menuitem )
-		menuitem.setAttribute('checked', 'true');
+  var menuitem = top.document.getElementById(menuitemID);
+  if ( menuitem )
+    menuitem.setAttribute('checked', 'true');
 
   // show phonetic fields if indicated by the pref
   if (GetLocalizedStringPref("mail.addr_book.show_phonetic_fields") == "true")
@@ -126,7 +126,6 @@ function GetCurrentPrefs()
             .setAttribute("hidden", "false");
 }
 
-
 function SetNameColumn(cmd)
 {
   var prefValue;
@@ -153,6 +152,7 @@ function CommandUpdate_AddressBook()
   goUpdateCommand('button_delete');
   goUpdateCommand('cmd_printcardpreview');
   goUpdateCommand('cmd_printcard');
+  goUpdateCommand('cmd_print');
   goUpdateCommand('cmd_properties');
   goUpdateCommand('cmd_newlist');
 }
@@ -210,10 +210,10 @@ function AbPrintCardInternal(doPrintPreview, msgType)
   }
 
   printEngineWindow = window.openDialog("chrome://messenger/content/msgPrintEngine.xul",
-                                         "",
-                                         "chrome,dialog=no,all",
-                                         selectionArray.length, selectionArray,
-                                         statusFeedback, doPrintPreview, msgType);
+                                        "",
+                                        "chrome,dialog=no,all",
+                                        selectionArray.length, selectionArray,
+                                        statusFeedback, doPrintPreview, msgType);
 
   return;
 }
@@ -240,8 +240,8 @@ function AbPrintAddressBookInternal(doPrintPreview, msgType)
     return;
 
   var statusFeedback;
-	statusFeedback = Cc["@mozilla.org/messenger/statusfeedback;1"].createInstance();
-	statusFeedback = statusFeedback.QueryInterface(Ci.nsIMsgStatusFeedback);
+  statusFeedback = Cc["@mozilla.org/messenger/statusfeedback;1"].createInstance();
+  statusFeedback = statusFeedback.QueryInterface(Ci.nsIMsgStatusFeedback);
 
   /*
     turn "moz-abmdbdirectory://abook.mab" into
@@ -251,12 +251,13 @@ function AbPrintAddressBookInternal(doPrintPreview, msgType)
   var abURIArr = uri.split("://");
   var printUrl = "addbook://" + abURIArr[0] + "/" + abURIArr[1] + "?action=print"
 
-	printEngineWindow = window.openDialog("chrome://messenger/content/msgPrintEngine.xul",
-										"",
-										"chrome,dialog=no,all",
-										1, [printUrl], statusFeedback, doPrintPreview, msgType);
+  printEngineWindow = window.openDialog("chrome://messenger/content/msgPrintEngine.xul",
+                                        "",
+                                        "chrome,dialog=no,all",
+                                        1, [printUrl],
+                                        statusFeedback, doPrintPreview, msgType);
 
-	return;
+  return;
 }
 
 function AbPrintAddressBook()
@@ -352,7 +353,10 @@ function SetStatusText(total)
       }
     }
     else
-      statusText = gAddressBookBundle.getFormattedString("totalContactStatus", [gAbView.directory.dirName, total]);
+      statusText =
+        gAddressBookBundle.getFormattedString(
+          "totalContactStatus",
+          [getSelectedDirectory().dirName, total]);
 
     gStatusText.setAttribute("label", statusText);
   }
@@ -384,11 +388,12 @@ function onAdvancedAbSearch()
 function onEnterInSearchBar()
 {
   ClearCardViewPane();
-
   if (!gQueryURIFormat) {
-    // Get model query from pref, without preceding "?", so we need to add it again
-    gQueryURIFormat = "?" + getModelQuery("mail.addr_book.quicksearchquery.format");
+    // Get model query from pref. We don't want the query starting with "?"
+    // as we have to prefix "?and" to this format.
+    gQueryURIFormat = getModelQuery("mail.addr_book.quicksearchquery.format");
   }
+
   let searchURI = getSelectedDirectoryURI();
   if (!searchURI) return;
 
@@ -397,11 +402,18 @@ function onEnterInSearchBar()
    already has a query, like
    moz-abldapdirectory://nsdirectory.netscape.com:389/ou=People,dc=netscape,dc=com?(or(Department,=,Applications))
   */
-  if (gSearchInput.value != "") {
-    // replace all instances of @V with the escaped version
-    // of what the user typed in the quick search text input
-    searchURI += gQueryURIFormat.replace(/@V/g, encodeABTermValue(gSearchInput.value));
-  }
+  // Use helper method to split up search query to multi-word search
+  // query against multiple fields.
+  let searchWords = getSearchTokens(gSearchInput.value);
+  searchURI += generateQueryURI(gQueryURIFormat, searchWords);
+
+  if (searchURI == kAllDirectoryRoot)
+    searchURI += "?";
+
+  document.getElementById("localResultsOnlyMessage")
+          .setAttribute("hidden",
+                        !gDirectoryTreeView.hasRemoteAB ||
+                        searchURI != kAllDirectoryRoot + "?");
 
   SetAbView(searchURI);
 
