@@ -116,9 +116,6 @@ var gFilterTreeView = {
   isSelectable: function isSelectable(row, col) { return false; },
   setCellValue: function setCellValue(row, col, value) {},
   setCellText: function setCellText(row, col, value) { /* XXX Write me */ },
-  performAction: function performAction(action) {},
-  performActionOnRow: function performActionOnRow(action, row) {},
-  performActionOnCell: function performActionOnCell(action, row, col) {}
 }
 
 const nsMsgFilterMotion = Ci.nsMsgFilterMotion;
@@ -526,13 +523,8 @@ function getServerThatCanHaveFilters()
 
     // if it cannot, check all accounts to find a server
     // that can have filters
-    var allServers = MailServices.accounts.allServers;
-    var numServers = allServers.length;
-    for (var index = 0; index < numServers; index++)
+    for (let currentServer of MailServices.accounts.allServers)
     {
-        var currentServer =
-          allServers.queryElementAt(index, Ci.nsIMsgIncomingServer);
-
         if (currentServer.canHaveFilters)
             return currentServer;
     }
