@@ -3,9 +3,9 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
-ChromeUtils.import("resource://gre/modules/Services.jsm");
-ChromeUtils.import("resource:///modules/gloda/utils.js");
+var {XPCOMUtils} = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+var {Services} = ChromeUtils.import("resource://gre/modules/Services.jsm");
+const {GlodaUtils} = ChromeUtils.import("resource:///modules/gloda/utils.js");
 
 
 /* This is where functions related to displaying the headers for a selected message in the
@@ -22,10 +22,6 @@ ChromeUtils.import("resource:///modules/gloda/utils.js");
 ////////////////////////////////////////////////////////////////////////////////////
 
 var gViewAllHeaders = false;
-var gShowOrganization = false;
-var gShowUserAgent = false;
-var gShowReferences = false;
-var gShowMessageId = false;
 var gCollectIncoming = false;
 var gCollectOutgoing = false;
 var gCollectNewsgroup = false;
@@ -41,7 +37,6 @@ var gMessengerBundle;
 var gFileHandler;
 var gProfileDirURL;
 
-var gExtraExpandedHeaders;
 // Show the friendly display names for people I know, instead of the name + email address.
 var gShowCondensedEmailAddresses;
 
@@ -192,7 +187,7 @@ function initializeHeaderViewTables()
     gExpandedHeaderView[headerName] = new createHeaderEntry('expanded', gExpandedHeaderList[index]);
   }
 
-  var extraHeaders = gExtraExpandedHeaders.match(/[^ ]+/g);
+  var extraHeaders = Services.prefs.getCharPref("mailnews.headers.extraExpandedHeaders").match(/[^ ]+/g);
   if (extraHeaders) {
     for (let index = 0; index < extraHeaders.length; index++)
     {
@@ -201,19 +196,19 @@ function initializeHeaderViewTables()
     }
   }
 
-  if (gShowOrganization)
+  if (Services.prefs.getBoolPref("mailnews.headers.showOrganization"))
   {
     let organizationEntry = {name:"organization", outputFunction:updateHeaderValue};
     gExpandedHeaderView[organizationEntry.name] = new createHeaderEntry('expanded', organizationEntry);
   }
 
-  if (gShowUserAgent)
+  if (Services.prefs.getBoolPref("mailnews.headers.showUserAgent"))
   {
     let userAgentEntry = {name:"user-agent", outputFunction:updateHeaderValue};
     gExpandedHeaderView[userAgentEntry.name] = new createHeaderEntry('expanded', userAgentEntry);
   }
 
-  if (gShowMessageId)
+  if (Services.prefs.getBoolPref("mailnews.headers.showMessageId"))
   {
     let messageIdEntry = {name:"message-id", outputFunction:OutputMessageIds};
     gExpandedHeaderView[messageIdEntry.name] = new createHeaderEntry('expanded', messageIdEntry);
@@ -228,11 +223,6 @@ function OnLoadMsgHeaderPane()
   gCollectNewsgroup = Services.prefs.getBoolPref("mail.collect_email_address_newsgroup");
   gCollectOutgoing = Services.prefs.getBoolPref("mail.collect_email_address_outgoing");
   gShowCondensedEmailAddresses = Services.prefs.getBoolPref("mail.showCondensedAddresses");
-  gShowUserAgent = Services.prefs.getBoolPref("mailnews.headers.showUserAgent");
-  gShowOrganization = Services.prefs.getBoolPref("mailnews.headers.showOrganization");
-  gShowReferences = Services.prefs.getBoolPref("mailnews.headers.showReferences");
-  gShowMessageId = Services.prefs.getBoolPref("mailnews.headers.showMessageId");
-  gExtraExpandedHeaders = Services.prefs.getCharPref("mailnews.headers.extraExpandedHeaders");
 
   Services.prefs.addObserver("mail.showCondensedAddresses", MsgHdrViewObserver);
 
@@ -385,6 +375,12 @@ var messageHeaderSink = {
 
       EnsureSubjectValue(); // make sure there is a subject even if it's empty so we'll show the subject and the twisty
 
+      // Load feed web page if so configured. This entry point works for
+      // messagepane loads in 3pane folder tab, 3pane message tab, and the
+      // standalone message window.
+      if (!FeedMessageHandler.shouldShowSummary(gMessageDisplay.displayedMessage, false))
+        FeedMessageHandler.setContent(gMessageDisplay.displayedMessage, false);
+
       ShowMessageHeaderPane();
       UpdateMessageHeaders();
       ShowEditMessageBox();
@@ -692,7 +688,7 @@ function SetTagHeader()
   // get the list of known tags
   var tagService = Cc["@mozilla.org/messenger/tagservice;1"]
                    .getService(Ci.nsIMsgTagService);
-  var tagArray = tagService.getAllTags({});
+  var tagArray = tagService.getAllTags();
   var tagKeys = {};
   for (var tagInfo of tagArray)
     if (tagInfo.tag)
@@ -971,8 +967,9 @@ function UpdateMessageHeaders()
 
     if (headerEntry)
     {
+      let show = Services.prefs.getBoolPref("mailnews.headers.showReferences");
       if (headerName == "references" &&
-          !(gViewAllHeaders || gShowReferences ||
+          !(gViewAllHeaders || show ||
             (gDBView.msgFolder && gDBView.msgFolder.server.type == "nntp")))
       {
         // hide references header if view all headers mode isn't selected, the pref show references is
@@ -1800,23 +1797,20 @@ function HandleMultipleAttachments(commandPrefix, selectedAttachments)
    switch (commandPrefix)
    {
      case "saveAttachment":
-       messenger.saveAllAttachments(attachmentContentTypeArray.length,
-                                    attachmentContentTypeArray,
+       messenger.saveAllAttachments(attachmentContentTypeArray,
                                     attachmentUrlArray,
                                     attachmentDisplayNameArray,
                                     attachmentMessageUriArray);
        break;
      case "detachAttachment":
-       messenger.detachAllAttachments(attachmentContentTypeArray.length,
-                                      attachmentContentTypeArray,
+       messenger.detachAllAttachments(attachmentContentTypeArray,
                                       attachmentUrlArray,
                                       attachmentDisplayNameArray,
                                       attachmentMessageUriArray,
                                       true /* save */);
        break;
      case "deleteAttachment":
-       messenger.detachAllAttachments(attachmentContentTypeArray.length,
-                                      attachmentContentTypeArray,
+       messenger.detachAllAttachments(attachmentContentTypeArray,
                                       attachmentUrlArray,
                                       attachmentDisplayNameArray,
                                       attachmentMessageUriArray,
