@@ -5,24 +5,18 @@
 // @flow
 
 import type { ThunkArgs } from "./types";
-import type { Worker, Grip } from "../types";
+import type { Grip, URL } from "../types";
 
 /**
  * @memberof actions/toolbox
  * @static
  */
-export function openLink(url: string) {
+export function openLink(url: URL) {
   return async function({ panel }: ThunkArgs) {
     return panel.openLink(url);
   };
 }
 
-export function openWorkerToolbox(worker: Worker) {
-  return async function({ getState, panel }: ThunkArgs) {
-    return panel.openWorkerToolbox(worker);
-  };
-}
-
 export function evaluateInConsole(inputString: string) {
   return async ({ panel }: ThunkArgs) => {
     return panel.openConsoleAndEvaluate(inputString);
@@ -35,6 +29,12 @@ export function openElementInInspectorCommand(grip: Grip) {
   };
 }
 
+export function openInspector(grip: Grip) {
+  return async ({ panel }: ThunkArgs) => {
+    return panel.openInspector();
+  };
+}
+
 export function highlightDomElement(grip: Grip) {
   return async ({ panel }: ThunkArgs) => {
     return panel.highlightDomElement(grip);
