@@ -4,20 +4,31 @@
 
 "use strict";
 
-const { Component, createFactory } = require("devtools/client/shared/vendor/react");
+const {
+  Component,
+  createFactory,
+} = require("devtools/client/shared/vendor/react");
 const PropTypes = require("devtools/client/shared/vendor/react-prop-types");
 const dom = require("devtools/client/shared/vendor/react-dom-factories");
-const { connect } = require("devtools/client/shared/redux/visibility-handler-connect");
+const {
+  connect,
+} = require("devtools/client/shared/redux/visibility-handler-connect");
 
 // Components
-loader.lazyGetter(this, "MonitorPanel", function () {
-  return createFactory(require("./MonitorPanel"));
+loader.lazyGetter(this, "MonitorPanel", function() {
+  return createFactory(
+    require("devtools/client/netmonitor/src/components/MonitorPanel")
+  );
 });
-loader.lazyGetter(this, "StatisticsPanel", function () {
-  return createFactory(require("./StatisticsPanel"));
+loader.lazyGetter(this, "StatisticsPanel", function() {
+  return createFactory(
+    require("devtools/client/netmonitor/src/components/StatisticsPanel")
+  );
 });
-loader.lazyGetter(this, "DropHarHandler", function () {
-  return createFactory(require("./DropHarHandler"));
+loader.lazyGetter(this, "DropHarHandler", function() {
+  return createFactory(
+    require("devtools/client/netmonitor/src/components/DropHarHandler")
+  );
 });
 
 const { div } = dom;
@@ -41,43 +52,50 @@ class App extends Component {
       sourceMapService: PropTypes.object,
       // True if the stats panel is opened.
       statisticsOpen: PropTypes.bool.isRequired,
+      // Document which settings menu will be injected to
+      toolboxDoc: PropTypes.object.isRequired,
     };
   }
 
   // Rendering
 
   render() {
-    let {
+    const {
       actions,
       connector,
       openLink,
       openSplitConsole,
       sourceMapService,
       statisticsOpen,
+      toolboxDoc,
     } = this.props;
 
-    return (
-      div({className: "network-monitor"},
-        !statisticsOpen ?
-          DropHarHandler({
-            actions,
-            openSplitConsole,
-          },
+    return div(
+      { className: "network-monitor" },
+      !statisticsOpen
+        ? DropHarHandler(
+            {
+              actions,
+              openSplitConsole,
+            },
             MonitorPanel({
+              actions,
               connector,
+              openSplitConsole,
               sourceMapService,
               openLink,
+              toolboxDoc,
             })
-          ) : StatisticsPanel({
-            connector
-          }),
-      )
+          )
+        : StatisticsPanel({
+            connector,
+          })
     );
   }
 }
 
 // Exports
 
-module.exports = connect(
-  (state) => ({ statisticsOpen: state.ui.statisticsOpen }),
-)(App);
+module.exports = connect(state => ({
+  statisticsOpen: state.ui.statisticsOpen,
+}))(App);
