@@ -7,23 +7,24 @@
  * Tests if requests display the correct status code and text in the UI.
  */
 
-add_task(async function () {
-  let { L10N } = require("devtools/client/netmonitor/src/utils/l10n");
+add_task(async function() {
+  const { L10N } = require("devtools/client/netmonitor/src/utils/l10n");
 
-  let { tab, monitor } = await initNetMonitor(STATUS_CODES_URL);
+  const { tab, monitor } = await initNetMonitor(STATUS_CODES_URL, {
+    requestCount: 1,
+  });
 
   info("Starting test... ");
 
-  let { document, store, windowRequire } = monitor.panelWin;
-  let Actions = windowRequire("devtools/client/netmonitor/src/actions/index");
-  let {
-    getDisplayedRequests,
-    getSortedRequests,
-  } = windowRequire("devtools/client/netmonitor/src/selectors/index");
+  const { document, store, windowRequire } = monitor.panelWin;
+  const Actions = windowRequire("devtools/client/netmonitor/src/actions/index");
+  const { getDisplayedRequests, getSortedRequests } = windowRequire(
+    "devtools/client/netmonitor/src/selectors/index"
+  );
 
   store.dispatch(Actions.batchEnable(false));
 
-  let requestItems = [];
+  const requestItems = [];
 
   const REQUEST_DATA = [
     {
@@ -37,13 +38,13 @@ add_task(async function () {
         type: "plain",
         fullMimeType: "text/plain; charset=utf-8",
         size: L10N.getFormatStrWithNumbers("networkMenu.sizeB", 0),
-        time: true
-      }
+        time: true,
+      },
     },
     {
       // request #1
       method: "GET",
-      uri: STATUS_CODES_SJS + "?sts=200#doh",
+      uri: STATUS_CODES_SJS + "?sts=200",
       correctUri: STATUS_CODES_SJS + "?sts=200",
       details: {
         status: 202,
@@ -51,8 +52,8 @@ add_task(async function () {
         type: "plain",
         fullMimeType: "text/plain; charset=utf-8",
         size: L10N.getFormatStrWithNumbers("networkMenu.sizeB", 22),
-        time: true
-      }
+        time: true,
+      },
     },
     {
       // request #2
@@ -65,8 +66,8 @@ add_task(async function () {
         type: "plain",
         fullMimeType: "text/plain; charset=utf-8",
         size: L10N.getFormatStrWithNumbers("networkMenu.sizeB", 22),
-        time: true
-      }
+        time: true,
+      },
     },
     {
       // request #3
@@ -79,8 +80,8 @@ add_task(async function () {
         type: "plain",
         fullMimeType: "text/plain; charset=utf-8",
         size: L10N.getFormatStrWithNumbers("networkMenu.sizeB", 22),
-        time: true
-      }
+        time: true,
+      },
     },
     {
       // request #4
@@ -93,21 +94,17 @@ add_task(async function () {
         type: "plain",
         fullMimeType: "text/plain; charset=utf-8",
         size: L10N.getFormatStrWithNumbers("networkMenu.sizeB", 22),
-        time: true
-      }
-    }
+        time: true,
+      },
+    },
   ];
 
-  let wait = waitForNetworkEvents(monitor, 5);
-  await ContentTask.spawn(tab.linkedBrowser, {}, async function () {
-    content.wrappedJSObject.performRequests();
-  });
-  await wait;
+  // Execute requests.
+  await performRequests(monitor, tab, 5);
 
   info("Performing tests");
   await verifyRequests();
   await testTab(0, testHeaders);
-  await testTab(2, testParams);
 
   return teardown(monitor);
 
@@ -116,18 +113,18 @@ add_task(async function () {
    * request list items to requestItems array.
    */
   async function verifyRequests() {
-    let requestListItems = document.querySelectorAll(".request-list-item");
-    for (let requestItem of requestListItems) {
+    const requestListItems = document.querySelectorAll(".request-list-item");
+    for (const requestItem of requestListItems) {
       requestItem.scrollIntoView();
-      let requestsListStatus = requestItem.querySelector(".requests-list-status");
+      const requestsListStatus = requestItem.querySelector(".status-code");
       EventUtils.sendMouseEvent({ type: "mouseover" }, requestsListStatus);
       await waitUntil(() => requestsListStatus.title);
     }
 
     info("Verifying requests contain correct information.");
     let index = 0;
-    for (let request of REQUEST_DATA) {
-      let item = getSortedRequests(store.getState()).get(index);
+    for (const request of REQUEST_DATA) {
+      const item = getSortedRequests(store.getState())[index];
       requestItems[index] = item;
 
       info("Verifying request #" + index);
@@ -157,7 +154,7 @@ add_task(async function () {
    */
   async function testTab(tabIdx, testFn) {
     let counter = 0;
-    for (let item of REQUEST_DATA) {
+    for (const item of REQUEST_DATA) {
       info("Testing tab #" + tabIdx + " to update with request #" + counter);
       await testFn(item, counter);
 
@@ -169,62 +166,46 @@ add_task(async function () {
    * A function that tests "Headers" panel contains correct information.
    */
   async function testHeaders(data, index) {
-    EventUtils.sendMouseEvent({ type: "mousedown" },
-      document.querySelectorAll(".request-list-item")[index]);
+    EventUtils.sendMouseEvent(
+      { type: "mousedown" },
+      document.querySelectorAll(".request-list-item")[index]
+    );
 
-    await waitUntil(() => document.querySelector(
-      "#headers-panel .tabpanel-summary-value.textbox-input"));
+    // wait till all the summary section is loaded
+    await waitUntil(() =>
+      document.querySelector("#headers-panel .tabpanel-summary-value")
+    );
+    const panel = document.querySelector("#headers-panel");
+    const {
+      method,
+      correctUri,
+      details: { status, statusText },
+    } = data;
 
-    let panel = document.querySelector("#headers-panel");
-    let summaryValues = panel.querySelectorAll(".tabpanel-summary-value.textbox-input");
-    let { method, correctUri, details: { status, statusText } } = data;
+    const statusCode = panel.querySelector(".status-code");
 
-    is(summaryValues[0].value, correctUri,
-      "The url summary value is incorrect.");
-    is(summaryValues[1].value, method, "The method summary value is incorrect.");
-    is(panel.querySelector(".requests-list-status-icon").dataset.code, status,
-      "The status summary code is incorrect.");
-    is(summaryValues[3].value, status + " " + statusText,
-      "The status summary value is incorrect.");
-  }
+    EventUtils.sendMouseEvent({ type: "mouseover" }, statusCode);
+    await waitUntil(() => statusCode.title);
 
-  /**
-   * A function that tests "Params" panel contains correct information.
-   */
-  function testParams(data, index) {
-    EventUtils.sendMouseEvent({ type: "mousedown" },
-      document.querySelectorAll(".request-list-item")[index]);
-    EventUtils.sendMouseEvent({ type: "click" },
-      document.querySelector("#params-tab"));
-
-    let panel = document.querySelector("#params-panel");
-    // Bug 1414981 - Request URL should not show #hash
-    let statusParamValue = data.uri.split("=").pop().split("#")[0];
-    let treeSections = panel.querySelectorAll(".tree-section");
-
-    is(treeSections.length, 1,
-      "There should be 1 param section displayed in this panel.");
-    is(panel.querySelectorAll("tr:not(.tree-section).treeRow").length, 1,
-      "There should be 1 param row displayed in this panel.");
-    is(panel.querySelectorAll(".empty-notice").length, 0,
-      "The empty notice should not be displayed in this panel.");
-
-    let labels = panel
-      .querySelectorAll("tr:not(.tree-section) .treeLabelCell .treeLabel");
-    let values = panel
-      .querySelectorAll("tr:not(.tree-section) .treeValueCell .objectBox");
-
-    is(treeSections[0].querySelector(".treeLabel").textContent,
-      L10N.getStr("paramsQueryString"),
-      "The params scope doesn't have the correct title.");
-
-    is(labels[0].textContent, "sts", "The param name was incorrect.");
-    is(values[0].textContent, statusParamValue, "The param value was incorrect.");
-
-    ok(panel.querySelector(".treeTable"),
-      "The request params tree view should be displayed.");
-    is(panel.querySelector(".editor-mount") === null,
-      true,
-      "The request post data editor should be hidden.");
+    is(
+      panel.querySelector(".url-preview .url").textContent,
+      correctUri,
+      "The url summary value is incorrect."
+    );
+    is(
+      panel.querySelectorAll(".treeLabel")[0].textContent,
+      method,
+      "The method value is incorrect."
+    );
+    is(
+      statusCode.dataset.code,
+      status,
+      "The status summary code is incorrect."
+    );
+    is(
+      statusCode.getAttribute("title"),
+      status + " " + statusText,
+      "The status summary value is incorrect."
+    );
   }
 });
