@@ -8,21 +8,16 @@
 const {
   CSSFilterEditorWidget,
 } = require("devtools/client/shared/widgets/FilterWidget");
-const {
-  getClientCssProperties,
-} = require("devtools/shared/fronts/css-properties");
 
 const TEST_URI = CHROME_URL_ROOT + "doc_filter-editor-01.html";
 
 add_task(async function() {
-  const [, , doc] = await createHost("bottom", TEST_URI);
-  const cssIsValid = getClientCssProperties().getValidityChecker(doc);
+  const { doc } = await createHost("bottom", TEST_URI);
 
   const container = doc.querySelector("#filter-container");
   const widget = new CSSFilterEditorWidget(
     container,
-    "blur(2px) contrast(200%)",
-    cssIsValid
+    "blur(2px) contrast(200%)"
   );
 
   info("Test removing filters with remove button");
