@@ -1,11 +1,13 @@
-/* vim: set ft=javascript ts=2 et sw=2 tw=80: */
 /* Any copyright is dedicated to the Public Domain.
    http://creativecommons.org/publicdomain/zero/1.0/ */
 
 "use strict";
 
-const {StorageFront} = require("devtools/shared/fronts/storage");
-Services.scriptloader.loadSubScript("chrome://mochitests/content/browser/devtools/server/tests/browser/storage-helpers.js", this);
+/* import-globals-from storage-helpers.js */
+Services.scriptloader.loadSubScript(
+  "chrome://mochitests/content/browser/devtools/server/tests/browser/storage-helpers.js",
+  this
+);
 
 const storeMap = {
   cookies: {
@@ -16,7 +18,7 @@ const storeMap = {
         expires: 2000000000000,
         path: "/browser",
         host: "test1.example.org",
-        isDomain: false,
+        hostOnly: true,
         isSecure: false,
       },
       {
@@ -25,7 +27,7 @@ const storeMap = {
         path: "/",
         host: ".example.org",
         expires: 0,
-        isDomain: true,
+        hostOnly: false,
         isSecure: false,
       },
       {
@@ -34,9 +36,9 @@ const storeMap = {
         expires: 2000000001000,
         path: "/",
         host: "test1.example.org",
-        isDomain: false,
+        hostOnly: true,
         isSecure: true,
-      }
+      },
     ],
 
     "http://sectest1.example.org": [
@@ -46,18 +48,18 @@ const storeMap = {
         path: "/",
         host: ".example.org",
         expires: 0,
-        isDomain: true,
+        hostOnly: false,
         isSecure: false,
       },
       {
         name: "sc1",
         value: "foobar",
-        path: "/browser/devtools/server/tests/browser/",
+        path: "/browser/devtools/server/tests/browser",
         host: "sectest1.example.org",
         expires: 0,
-        isDomain: false,
+        hostOnly: true,
         isSecure: false,
-      }
+      },
     ],
 
     "https://sectest1.example.org": [
@@ -67,7 +69,7 @@ const storeMap = {
         host: ".example.org",
         path: "/",
         expires: 0,
-        isDomain: true,
+        hostOnly: false,
         isSecure: true,
       },
       {
@@ -76,80 +78,82 @@ const storeMap = {
         path: "/",
         host: ".example.org",
         expires: 0,
-        isDomain: true,
+        hostOnly: false,
         isSecure: false,
       },
       {
         name: "sc1",
         value: "foobar",
-        path: "/browser/devtools/server/tests/browser/",
+        path: "/browser/devtools/server/tests/browser",
         host: "sectest1.example.org",
         expires: 0,
-        isDomain: false,
+        hostOnly: true,
         isSecure: false,
-      }
-    ]
+      },
+    ],
   },
   localStorage: {
     "http://test1.example.org": [
       {
         name: "ls1",
-        value: "foobar"
+        value: "foobar",
       },
       {
         name: "ls2",
-        value: "foobar-2"
-      }
+        value: "foobar-2",
+      },
     ],
     "http://sectest1.example.org": [
       {
         name: "iframe-u-ls1",
-        value: "foobar"
-      }
+        value: "foobar",
+      },
     ],
     "https://sectest1.example.org": [
       {
         name: "iframe-s-ls1",
-        value: "foobar"
-      }
-    ]
+        value: "foobar",
+      },
+    ],
   },
   sessionStorage: {
     "http://test1.example.org": [
       {
         name: "ss1",
-        value: "foobar-3"
-      }
+        value: "foobar-3",
+      },
     ],
     "http://sectest1.example.org": [
       {
         name: "iframe-u-ss1",
-        value: "foobar1"
+        value: "foobar1",
       },
       {
         name: "iframe-u-ss2",
-        value: "foobar2"
-      }
+        value: "foobar2",
+      },
     ],
     "https://sectest1.example.org": [
       {
         name: "iframe-s-ss1",
-        value: "foobar-2"
-      }
-    ]
-  }
+        value: "foobar-2",
+      },
+    ],
+  },
 };
 
 const IDBValues = {
   listStoresResponse: {
     "http://test1.example.org": [
-      ["idb1 (default)", "obj1"], ["idb1 (default)", "obj2"], ["idb2 (default)", "obj3"]
-    ],
-    "http://sectest1.example.org": [
+      ["idb1 (default)", "obj1"],
+      ["idb1 (default)", "obj2"],
+      ["idb2 (default)", "obj3"],
     ],
+    "http://sectest1.example.org": [],
     "https://sectest1.example.org": [
-      ["idb-s1 (default)", "obj-s1"], ["idb-s2 (default)", "obj-s2"]
-    ]
+      ["idb-s1 (default)", "obj-s1"],
+      ["idb-s2 (default)", "obj-s2"],
+    ],
   },
   dbDetails: {
     "http://test1.example.org": [
@@ -157,31 +161,30 @@ const IDBValues = {
         db: "idb1 (default)",
         origin: "http://test1.example.org",
         version: 1,
-        objectStores: 2
+        objectStores: 2,
       },
       {
         db: "idb2 (default)",
         origin: "http://test1.example.org",
         version: 1,
-        objectStores: 1
+        objectStores: 1,
       },
     ],
-    "http://sectest1.example.org": [
-    ],
+    "http://sectest1.example.org": [],
     "https://sectest1.example.org": [
       {
         db: "idb-s1 (default)",
         origin: "https://sectest1.example.org",
         version: 1,
-        objectStores: 1
+        objectStores: 1,
       },
       {
         db: "idb-s2 (default)",
         origin: "https://sectest1.example.org",
         version: 1,
-        objectStores: 1
+        objectStores: 1,
       },
-    ]
+    ],
   },
   objectStoreDetails: {
     "http://test1.example.org": {
@@ -194,23 +197,23 @@ const IDBValues = {
             {
               name: "name",
               keyPath: "name",
-              "unique": false,
+              unique: false,
               multiEntry: false,
             },
             {
               name: "email",
               keyPath: "email",
-              "unique": true,
+              unique: true,
               multiEntry: false,
             },
-          ]
+          ],
         },
         {
           objectStore: "obj2",
           keyPath: "id2",
           autoIncrement: false,
-          indexes: []
-        }
+          indexes: [],
+        },
       ],
       "idb2 (default)": [
         {
@@ -221,12 +224,12 @@ const IDBValues = {
             {
               name: "name2",
               keyPath: "name2",
-              "unique": true,
+              unique: true,
               multiEntry: false,
-            }
-          ]
+            },
+          ],
         },
-      ]
+      ],
     },
     "http://sectest1.example.org": {},
     "https://sectest1.example.org": {
@@ -235,7 +238,7 @@ const IDBValues = {
           objectStore: "obj-s1",
           keyPath: "id",
           autoIncrement: false,
-          indexes: []
+          indexes: [],
         },
       ],
       "idb-s2 (default)": [
@@ -247,14 +250,13 @@ const IDBValues = {
             {
               name: "name2",
               keyPath: "name2",
-              "unique": true,
+              unique: true,
               multiEntry: false,
-            }
-          ]
+            },
+          ],
         },
-      ]
-    }
-
+      ],
+    },
   },
   entries: {
     "http://test1.example.org": {
@@ -265,7 +267,7 @@ const IDBValues = {
             id: 1,
             name: "foo",
             email: "foo@bar.com",
-          }
+          },
         },
         {
           name: 2,
@@ -273,7 +275,7 @@ const IDBValues = {
             id: 2,
             name: "foo2",
             email: "foo2@bar.com",
-          }
+          },
         },
         {
           name: 3,
@@ -281,8 +283,8 @@ const IDBValues = {
             id: 3,
             name: "foo2",
             email: "foo3@bar.com",
-          }
-        }
+          },
+        },
       ],
       "idb1 (default)#obj2": [
         {
@@ -291,11 +293,11 @@ const IDBValues = {
             id2: 1,
             name: "foo",
             email: "foo@bar.com",
-            extra: "baz"
-          }
-        }
+            extra: "baz",
+          },
+        },
       ],
-      "idb2 (default)#obj3": []
+      "idb2 (default)#obj3": [],
     },
     "http://sectest1.example.org": {},
     "https://sectest1.example.org": {
@@ -306,7 +308,7 @@ const IDBValues = {
             id: 6,
             name: "foo",
             email: "foo@bar.com",
-          }
+          },
         },
         {
           name: 7,
@@ -314,8 +316,8 @@ const IDBValues = {
             id: 7,
             name: "foo2",
             email: "foo2@bar.com",
-          }
-        }
+          },
+        },
       ],
       "idb-s2 (default)#obj-s2": [
         {
@@ -324,11 +326,11 @@ const IDBValues = {
             id2: 13,
             name2: "foo",
             email: "foo@bar.com",
-          }
-        }
-      ]
-    }
-  }
+          },
+        },
+      ],
+    },
+  },
 };
 
 async function testStores(data) {
@@ -343,26 +345,32 @@ async function testStores(data) {
 }
 
 function testCookies(cookiesActor) {
-  is(Object.keys(cookiesActor.hosts).length, 3,
-                 "Correct number of host entries for cookies");
+  is(
+    Object.keys(cookiesActor.hosts).length,
+    3,
+    "Correct number of host entries for cookies"
+  );
   return testCookiesObjects(0, cookiesActor.hosts, cookiesActor);
 }
 
-var testCookiesObjects = async function (index, hosts, cookiesActor) {
-  let host = Object.keys(hosts)[index];
-  let matchItems = data => {
+var testCookiesObjects = async function(index, hosts, cookiesActor) {
+  const host = Object.keys(hosts)[index];
+  const matchItems = data => {
     let cookiesLength = 0;
-    for (let secureCookie of storeMap.cookies[host]) {
+    for (const secureCookie of storeMap.cookies[host]) {
       if (secureCookie.isSecure) {
         ++cookiesLength;
       }
     }
     // Any secure cookies did not get stored in the database.
-    is(data.total, storeMap.cookies[host].length - cookiesLength,
-       "Number of cookies in host " + host + " matches");
-    for (let item of data.data) {
+    is(
+      data.total,
+      storeMap.cookies[host].length - cookiesLength,
+      "Number of cookies in host " + host + " matches"
+    );
+    for (const item of data.data) {
       let found = false;
-      for (let toMatch of storeMap.cookies[host]) {
+      for (const toMatch of storeMap.cookies[host]) {
         if (item.name == toMatch.name) {
           found = true;
           ok(true, "Found cookie " + item.name + " in response");
@@ -371,7 +379,7 @@ var testCookiesObjects = async function (index, hosts, cookiesActor) {
           is(item.path, toMatch.path, "The path matches.");
           is(item.host, toMatch.host, "The host matches.");
           is(item.isSecure, toMatch.isSecure, "The isSecure value matches.");
-          is(item.isDomain, toMatch.isDomain, "The isDomain value matches.");
+          is(item.hostOnly, toMatch.hostOnly, "The hostOnly value matches.");
           break;
         }
       }
@@ -388,19 +396,25 @@ var testCookiesObjects = async function (index, hosts, cookiesActor) {
 };
 
 function testLocalStorage(localStorageActor) {
-  is(Object.keys(localStorageActor.hosts).length, 3,
-     "Correct number of host entries for local storage");
+  is(
+    Object.keys(localStorageActor.hosts).length,
+    3,
+    "Correct number of host entries for local storage"
+  );
   return testLocalStorageObjects(0, localStorageActor.hosts, localStorageActor);
 }
 
-var testLocalStorageObjects = async function (index, hosts, localStorageActor) {
-  let host = Object.keys(hosts)[index];
-  let matchItems = data => {
-    is(data.total, storeMap.localStorage[host].length,
-       "Number of local storage items in host " + host + " matches");
-    for (let item of data.data) {
+var testLocalStorageObjects = async function(index, hosts, localStorageActor) {
+  const host = Object.keys(hosts)[index];
+  const matchItems = data => {
+    is(
+      data.total,
+      storeMap.localStorage[host].length,
+      "Number of local storage items in host " + host + " matches"
+    );
+    for (const item of data.data) {
       let found = false;
-      for (let toMatch of storeMap.localStorage[host]) {
+      for (const toMatch of storeMap.localStorage[host]) {
         if (item.name == toMatch.name) {
           found = true;
           ok(true, "Found local storage item " + item.name + " in response");
@@ -408,7 +422,10 @@ var testLocalStorageObjects = async function (index, hosts, localStorageActor) {
           break;
         }
       }
-      ok(found, "local storage item " + item.name + " should exist in response");
+      ok(
+        found,
+        "local storage item " + item.name + " should exist in response"
+      );
     }
   };
 
@@ -421,20 +438,33 @@ var testLocalStorageObjects = async function (index, hosts, localStorageActor) {
 };
 
 function testSessionStorage(sessionStorageActor) {
-  is(Object.keys(sessionStorageActor.hosts).length, 3,
-     "Correct number of host entries for session storage");
-  return testSessionStorageObjects(0, sessionStorageActor.hosts,
-                                   sessionStorageActor);
+  is(
+    Object.keys(sessionStorageActor.hosts).length,
+    3,
+    "Correct number of host entries for session storage"
+  );
+  return testSessionStorageObjects(
+    0,
+    sessionStorageActor.hosts,
+    sessionStorageActor
+  );
 }
 
-var testSessionStorageObjects = async function (index, hosts, sessionStorageActor) {
-  let host = Object.keys(hosts)[index];
-  let matchItems = data => {
-    is(data.total, storeMap.sessionStorage[host].length,
-       "Number of session storage items in host " + host + " matches");
-    for (let item of data.data) {
+var testSessionStorageObjects = async function(
+  index,
+  hosts,
+  sessionStorageActor
+) {
+  const host = Object.keys(hosts)[index];
+  const matchItems = data => {
+    is(
+      data.total,
+      storeMap.sessionStorage[host].length,
+      "Number of session storage items in host " + host + " matches"
+    );
+    for (const item of data.data) {
       let found = false;
-      for (let toMatch of storeMap.sessionStorage[host]) {
+      for (const toMatch of storeMap.sessionStorage[host]) {
         if (item.name == toMatch.name) {
           found = true;
           ok(true, "Found session storage item " + item.name + " in response");
@@ -442,7 +472,10 @@ var testSessionStorageObjects = async function (index, hosts, sessionStorageActo
           break;
         }
       }
-      ok(found, "session storage item " + item.name + " should exist in response");
+      ok(
+        found,
+        "session storage item " + item.name + " should exist in response"
+      );
     }
   };
 
@@ -454,15 +487,23 @@ var testSessionStorageObjects = async function (index, hosts, sessionStorageActo
   await testSessionStorageObjects(++index, hosts, sessionStorageActor);
 };
 
-var testIndexedDB = async function (indexedDBActor) {
-  is(Object.keys(indexedDBActor.hosts).length, 3,
-     "Correct number of host entries for indexed db");
+var testIndexedDB = async function(indexedDBActor) {
+  // If this test is run with chrome debugging enabled we get an extra
+  // key for "chrome". We don't want the test to fail in this case, so
+  // ignore it.
+  delete indexedDBActor.hosts.chrome;
 
-  for (let host in indexedDBActor.hosts) {
-    for (let item of indexedDBActor.hosts[host]) {
-      let parsedItem = JSON.parse(item);
+  is(
+    Object.keys(indexedDBActor.hosts).length,
+    3,
+    "Correct number of host entries for indexed db"
+  );
+
+  for (const host in indexedDBActor.hosts) {
+    for (const item of indexedDBActor.hosts[host]) {
+      const parsedItem = JSON.parse(item);
       let found = false;
-      for (let toMatch of IDBValues.listStoresResponse[host]) {
+      for (const toMatch of IDBValues.listStoresResponse[host]) {
         if (toMatch[0] == parsedItem[0] && toMatch[1] == parsedItem[1]) {
           found = true;
           break;
@@ -477,21 +518,27 @@ var testIndexedDB = async function (indexedDBActor) {
   await testIDBEntries(0, indexedDBActor.hosts, indexedDBActor);
 };
 
-var testIndexedDBs = async function (index, hosts, indexedDBActor) {
-  let host = Object.keys(hosts)[index];
-  let matchItems = data => {
-    is(data.total, IDBValues.dbDetails[host].length,
-       "Number of indexed db in host " + host + " matches");
-    for (let item of data.data) {
+var testIndexedDBs = async function(index, hosts, indexedDBActor) {
+  const host = Object.keys(hosts)[index];
+  const matchItems = data => {
+    is(
+      data.total,
+      IDBValues.dbDetails[host].length,
+      "Number of indexed db in host " + host + " matches"
+    );
+    for (const item of data.data) {
       let found = false;
-      for (let toMatch of IDBValues.dbDetails[host]) {
+      for (const toMatch of IDBValues.dbDetails[host]) {
         if (item.uniqueKey == toMatch.db) {
           found = true;
           ok(true, "Found indexed db " + item.uniqueKey + " in response");
           is(item.origin, toMatch.origin, "The origin matches.");
           is(item.version, toMatch.version, "The version matches.");
-          is(item.objectStores, toMatch.objectStores,
-             "The number of object stores matches.");
+          is(
+            item.objectStores,
+            toMatch.objectStores,
+            "The number of object stores matches."
+          );
           break;
         }
       }
@@ -507,32 +554,49 @@ var testIndexedDBs = async function (index, hosts, indexedDBActor) {
   await testIndexedDBs(++index, hosts, indexedDBActor);
 };
 
-var testObjectStores = async function (ix, hosts, indexedDBActor) {
-  let host = Object.keys(hosts)[ix];
-  let matchItems = (data, db) => {
-    is(data.total, IDBValues.objectStoreDetails[host][db].length,
-       "Number of object stores in host " + host + " matches");
-    for (let item of data.data) {
+var testObjectStores = async function(ix, hosts, indexedDBActor) {
+  const host = Object.keys(hosts)[ix];
+  const matchItems = (data, db) => {
+    is(
+      data.total,
+      IDBValues.objectStoreDetails[host][db].length,
+      "Number of object stores in host " + host + " matches"
+    );
+    for (const item of data.data) {
       let found = false;
-      for (let toMatch of IDBValues.objectStoreDetails[host][db]) {
+      for (const toMatch of IDBValues.objectStoreDetails[host][db]) {
         if (item.objectStore == toMatch.objectStore) {
           found = true;
           ok(true, "Found object store " + item.objectStore + " in response");
           is(item.keyPath, toMatch.keyPath, "The keyPath matches.");
-          is(item.autoIncrement, toMatch.autoIncrement, "The autoIncrement matches.");
+          is(
+            item.autoIncrement,
+            toMatch.autoIncrement,
+            "The autoIncrement matches."
+          );
           item.indexes = JSON.parse(item.indexes);
-          is(item.indexes.length, toMatch.indexes.length, "Number of indexes match");
-          for (let index of item.indexes) {
+          is(
+            item.indexes.length,
+            toMatch.indexes.length,
+            "Number of indexes match"
+          );
+          for (const index of item.indexes) {
             let indexFound = false;
-            for (let toMatchIndex of toMatch.indexes) {
+            for (const toMatchIndex of toMatch.indexes) {
               if (toMatchIndex.name == index.name) {
                 indexFound = true;
                 ok(true, "Found index " + index.name);
-                is(index.keyPath, toMatchIndex.keyPath,
-                   "The keyPath of index matches.");
+                is(
+                  index.keyPath,
+                  toMatchIndex.keyPath,
+                  "The keyPath of index matches."
+                );
                 is(index.unique, toMatchIndex.unique, "The unique matches");
-                is(index.multiEntry, toMatchIndex.multiEntry,
-                   "The multiEntry matches");
+                is(
+                  index.multiEntry,
+                  toMatchIndex.multiEntry,
+                  "The multiEntry matches"
+                );
                 break;
               }
             }
@@ -545,12 +609,16 @@ var testObjectStores = async function (ix, hosts, indexedDBActor) {
     }
   };
 
-  ok(!!IDBValues.objectStoreDetails[host], "Host is present in the list : " + host);
-  for (let name of hosts[host]) {
-    let objName = JSON.parse(name).slice(0, 1);
-    matchItems((
-      await indexedDBActor.getStoreObjects(host, [JSON.stringify(objName)])
-    ), objName[0]);
+  ok(
+    !!IDBValues.objectStoreDetails[host],
+    "Host is present in the list : " + host
+  );
+  for (const name of hosts[host]) {
+    const objName = JSON.parse(name).slice(0, 1);
+    matchItems(
+      await indexedDBActor.getStoreObjects(host, [JSON.stringify(objName)]),
+      objName[0]
+    );
   }
   if (ix == Object.keys(hosts).length - 1) {
     return;
@@ -558,23 +626,32 @@ var testObjectStores = async function (ix, hosts, indexedDBActor) {
   await testObjectStores(++ix, hosts, indexedDBActor);
 };
 
-var testIDBEntries = async function (index, hosts, indexedDBActor) {
-  let host = Object.keys(hosts)[index];
-  let matchItems = (data, obj) => {
-    is(data.total, IDBValues.entries[host][obj].length,
-       "Number of items in object store " + obj + " matches");
-    for (let item of data.data) {
+var testIDBEntries = async function(index, hosts, indexedDBActor) {
+  const host = Object.keys(hosts)[index];
+  const matchItems = (data, obj) => {
+    is(
+      data.total,
+      IDBValues.entries[host][obj].length,
+      "Number of items in object store " + obj + " matches"
+    );
+    for (const item of data.data) {
       let found = false;
-      for (let toMatch of IDBValues.entries[host][obj]) {
+      for (const toMatch of IDBValues.entries[host][obj]) {
         if (item.name == toMatch.name) {
           found = true;
           ok(true, "Found indexed db item " + item.name + " in response");
-          let value = JSON.parse(item.value.str);
-          is(Object.keys(value).length, Object.keys(toMatch.value).length,
-             "Number of entries in the value matches");
-          for (let key in value) {
-            is(value[key], toMatch.value[key],
-               "value of " + key + " value key matches");
+          const value = JSON.parse(item.value.str);
+          is(
+            Object.keys(value).length,
+            Object.keys(toMatch.value).length,
+            "Number of entries in the value matches"
+          );
+          for (const key in value) {
+            is(
+              value[key],
+              toMatch.value[key],
+              "value of " + key + " value key matches"
+            );
           }
           break;
         }
@@ -584,11 +661,12 @@ var testIDBEntries = async function (index, hosts, indexedDBActor) {
   };
 
   ok(!!IDBValues.entries[host], "Host is present in the list : " + host);
-  for (let name of hosts[host]) {
-    let parsed = JSON.parse(name);
-    matchItems((
-      await indexedDBActor.getStoreObjects(host, [name])
-    ), parsed[0] + "#" + parsed[1]);
+  for (const name of hosts[host]) {
+    const parsed = JSON.parse(name);
+    matchItems(
+      await indexedDBActor.getStoreObjects(host, [name]),
+      parsed[0] + "#" + parsed[1]
+    );
   }
   if (index == Object.keys(hosts).length - 1) {
     return;
@@ -596,22 +674,24 @@ var testIDBEntries = async function (index, hosts, indexedDBActor) {
   await testObjectStores(++index, hosts, indexedDBActor);
 };
 
-add_task(async function () {
-  await openTabAndSetupStorage(MAIN_DOMAIN + "storage-listings.html");
+add_task(async function() {
+  await SpecialPowers.pushPrefEnv({
+    set: [["privacy.documentCookies.maxage", 0]],
+  });
 
-  initDebuggerServer();
-  let client = new DebuggerClient(DebuggerServer.connectPipe());
-  let form = await connectDebuggerClient(client);
-  let front = StorageFront(client, form);
-  let data = await front.listStores();
+  const { target, front } = await openTabAndSetupStorage(
+    MAIN_DOMAIN + "storage-listings.html"
+  );
+
+  const data = await front.listStores();
   await testStores(data);
 
   await clearStorage();
 
   // Forcing GC/CC to get rid of docshells and windows created by this test.
   forceCollections();
-  await client.close();
+  await target.destroy();
   forceCollections();
-  DebuggerServer.destroy();
+  DevToolsServer.destroy();
   forceCollections();
 });
