@@ -10,12 +10,11 @@
 #include "base/basictypes.h"
 #include "base/platform_thread.h"
 #include "nsTArray.h"
-#include "mozilla/dom/battery/Types.h"
-#include "mozilla/dom/network/Types.h"
-#include "mozilla/dom/power/Types.h"
-#include "mozilla/dom/ScreenOrientation.h"
 #include "mozilla/hal_sandbox/PHal.h"
+#include "mozilla/HalBatteryInformation.h"
+#include "mozilla/HalNetworkInformation.h"
 #include "mozilla/HalScreenConfiguration.h"
+#include "mozilla/HalWakeLockInformation.h"
 #include "mozilla/HalTypes.h"
 #include "mozilla/Types.h"
 
@@ -32,8 +31,8 @@
 class nsPIDOMWindowInner;
 
 #ifndef MOZ_HAL_NAMESPACE
-# define MOZ_HAL_NAMESPACE hal
-# define MOZ_DEFINED_HAL_NAMESPACE 1
+#  define MOZ_HAL_NAMESPACE hal
+#  define MOZ_DEFINED_HAL_NAMESPACE 1
 #endif
 
 namespace mozilla {
@@ -42,13 +41,21 @@ namespace hal {
 
 class WindowIdentifier;
 
-typedef Observer<int64_t> SystemClockChangeObserver;
-typedef Observer<SystemTimezoneChangeInformation> SystemTimezoneChangeObserver;
-
-} // namespace hal
+}  // namespace hal
 
 namespace MOZ_HAL_NAMESPACE {
 
+/**
+ * Initializes the HAL. This must be called before any other HAL function.
+ */
+void Init();
+
+/**
+ * Shuts down the HAL. Besides freeing all the used resources this will check
+ * that all observers have been properly deregistered and assert if not.
+ */
+void Shutdown();
+
 /**
  * Turn the default vibrator device on/off per the pattern specified
  * by |pattern|.  Each element in the pattern is the number of
@@ -64,10 +71,8 @@ namespace MOZ_HAL_NAMESPACE {
  * nsIDOMWindow* in place of the WindowIdentifier parameter.
  * The method with WindowIdentifier will be called automatically.
  */
-void Vibrate(const nsTArray<uint32_t>& pattern,
-             nsPIDOMWindowInner* aWindow);
-void Vibrate(const nsTArray<uint32_t>& pattern,
-             const hal::WindowIdentifier &id);
+void Vibrate(const nsTArray<uint32_t>& pattern, nsPIDOMWindowInner* aWindow);
+void Vibrate(const nsTArray<uint32_t>& pattern, hal::WindowIdentifier&& id);
 
 /**
  * Cancel a vibration started by the content window identified by
@@ -82,19 +87,21 @@ void Vibrate(const nsTArray<uint32_t>& pattern,
  * automatically.
  */
 void CancelVibrate(nsPIDOMWindowInner* aWindow);
-void CancelVibrate(const hal::WindowIdentifier &id);
+void CancelVibrate(hal::WindowIdentifier&& id);
 
-/**
- * Inform the battery backend there is a new battery observer.
- * @param aBatteryObserver The observer that should be added.
- */
-void RegisterBatteryObserver(BatteryObserver* aBatteryObserver);
+#define MOZ_DEFINE_HAL_OBSERVER(name_)                             \
+  /**                                                              \
+   * Inform the backend there is a new |name_| observer.           \
+   * @param aObserver The observer that should be added.           \
+   */                                                              \
+  void Register##name_##Observer(hal::name_##Observer* aObserver); \
+  /**                                                              \
+   * Inform the backend a |name_| observer unregistered.           \
+   * @param aObserver The observer that should be removed.         \
+   */                                                              \
+  void Unregister##name_##Observer(hal::name_##Observer* aObserver);
 
-/**
- * Inform the battery backend a battery observer unregistered.
- * @param aBatteryObserver The observer that should be removed.
- */
-void UnregisterBatteryObserver(BatteryObserver* aBatteryObserver);
+MOZ_DEFINE_HAL_OBSERVER(Battery);
 
 /**
  * Returns the current battery information.
@@ -114,20 +121,20 @@ void NotifyBatteryChange(const hal::BatteryInformation& aBatteryInfo);
  * sensor is avaiable.
  */
 void RegisterSensorObserver(hal::SensorType aSensor,
-                            hal::ISensorObserver *aObserver);
+                            hal::ISensorObserver* aObserver);
 
 /**
  * Unregister an observer for the sensor of given type.
  */
 void UnregisterSensorObserver(hal::SensorType aSensor,
-                              hal::ISensorObserver *aObserver);
+                              hal::ISensorObserver* aObserver);
 
 /**
  * Post a value generated by a sensor.
  *
  * This API is internal to hal; clients shouldn't call it directly.
  */
-void NotifySensorChange(const hal::SensorData &aSensorData);
+void NotifySensorChange(const hal::SensorData& aSensorData);
 
 /**
  * Enable sensor notifications from the backend
@@ -145,18 +152,7 @@ void EnableSensorNotifications(hal::SensorType aSensor);
  */
 void DisableSensorNotifications(hal::SensorType aSensor);
 
-
-/**
- * Inform the network backend there is a new network observer.
- * @param aNetworkObserver The observer that should be added.
- */
-void RegisterNetworkObserver(NetworkObserver* aNetworkObserver);
-
-/**
- * Inform the network backend a network observer unregistered.
- * @param aNetworkObserver The observer that should be removed.
- */
-void UnregisterNetworkObserver(NetworkObserver* aNetworkObserver);
+MOZ_DEFINE_HAL_OBSERVER(Network);
 
 /**
  * Returns the current network information.
@@ -169,53 +165,6 @@ void GetCurrentNetworkInformation(hal::NetworkInformation* aNetworkInfo);
  */
 void NotifyNetworkChange(const hal::NetworkInformation& aNetworkInfo);
 
-/**
- * Adjusting system clock.
- * @param aDeltaMilliseconds The difference compared with current system clock.
- */
-void AdjustSystemClock(int64_t aDeltaMilliseconds);
-
-/**
- * Register observer for system clock changed notification.
- * @param aObserver The observer that should be added.
- */
-void RegisterSystemClockChangeObserver(
-  hal::SystemClockChangeObserver* aObserver);
-
-/**
- * Unregister the observer for system clock changed.
- * @param aObserver The observer that should be removed.
- */
-void UnregisterSystemClockChangeObserver(
-  hal::SystemClockChangeObserver* aObserver);
-
-/**
- * Notify of a change in the system clock.
- * @param aClockDeltaMS
- */
-void NotifySystemClockChange(const int64_t& aClockDeltaMS);
-
-/**
- * Register observer for system timezone changed notification.
- * @param aObserver The observer that should be added.
- */
-void RegisterSystemTimezoneChangeObserver(
-  hal::SystemTimezoneChangeObserver* aObserver);
-
-/**
- * Unregister the observer for system timezone changed.
- * @param aObserver The observer that should be removed.
- */
-void UnregisterSystemTimezoneChangeObserver(
-  hal::SystemTimezoneChangeObserver* aObserver);
-
-/**
- * Notify of a change in the system timezone.
- * @param aSystemTimezoneChangeInfo
- */
-void NotifySystemTimezoneChange(
-  const hal::SystemTimezoneChangeInformation& aSystemTimezoneChangeInfo);
-
 /**
  * Enable wake lock notifications from the backend.
  *
@@ -230,17 +179,7 @@ void EnableWakeLockNotifications();
  */
 void DisableWakeLockNotifications();
 
-/**
- * Inform the wake lock backend there is a new wake lock observer.
- * @param aWakeLockObserver The observer that should be added.
- */
-void RegisterWakeLockObserver(WakeLockObserver* aObserver);
-
-/**
- * Inform the wake lock backend a wake lock observer unregistered.
- * @param aWakeLockObserver The observer that should be removed.
- */
-void UnregisterWakeLockObserver(WakeLockObserver* aObserver);
+MOZ_DEFINE_HAL_OBSERVER(WakeLock);
 
 /**
  * Adjust a wake lock's counts on behalf of a given process.
@@ -258,8 +197,7 @@ void UnregisterWakeLockObserver(WakeLockObserver* aObserver);
  *                      CONTENT_PROCESS_ID_MAIN: The root process
  *                      X: The process with ContentChild::GetID() == X
  */
-void ModifyWakeLock(const nsAString &aTopic,
-                    hal::WakeLockControl aLockAdjust,
+void ModifyWakeLock(const nsAString& aTopic, hal::WakeLockControl aLockAdjust,
                     hal::WakeLockControl aHiddenAdjust,
                     uint64_t aProcessID = hal::CONTENT_PROCESS_ID_UNKNOWN);
 
@@ -268,7 +206,8 @@ void ModifyWakeLock(const nsAString &aTopic,
  * @param aTopic        lock topic
  * @param aWakeLockInfo wake lock numbers
  */
-void GetWakeLockInfo(const nsAString &aTopic, hal::WakeLockInformation *aWakeLockInfo);
+void GetWakeLockInfo(const nsAString& aTopic,
+                     hal::WakeLockInformation* aWakeLockInfo);
 
 /**
  * Notify of a change in the wake lock state.
@@ -276,34 +215,27 @@ void GetWakeLockInfo(const nsAString &aTopic, hal::WakeLockInformation *aWakeLoc
  */
 void NotifyWakeLockChange(const hal::WakeLockInformation& aWakeLockInfo);
 
-/**
- * Inform the backend there is a new screen configuration observer.
- * @param aScreenConfigurationObserver The observer that should be added.
- */
-void RegisterScreenConfigurationObserver(hal::ScreenConfigurationObserver* aScreenConfigurationObserver);
-
-/**
- * Inform the backend a screen configuration observer unregistered.
- * @param aScreenConfigurationObserver The observer that should be removed.
- */
-void UnregisterScreenConfigurationObserver(hal::ScreenConfigurationObserver* aScreenConfigurationObserver);
+MOZ_DEFINE_HAL_OBSERVER(ScreenConfiguration);
 
 /**
  * Returns the current screen configuration.
  */
-void GetCurrentScreenConfiguration(hal::ScreenConfiguration* aScreenConfiguration);
+void GetCurrentScreenConfiguration(
+    hal::ScreenConfiguration* aScreenConfiguration);
 
 /**
  * Notify of a change in the screen configuration.
  * @param aScreenConfiguration The new screen orientation.
  */
-void NotifyScreenConfigurationChange(const hal::ScreenConfiguration& aScreenConfiguration);
+void NotifyScreenConfigurationChange(
+    const hal::ScreenConfiguration& aScreenConfiguration);
 
 /**
  * Lock the screen orientation to the specific orientation.
  * @return Whether the lock has been accepted.
  */
-MOZ_MUST_USE bool LockScreenOrientation(const dom::ScreenOrientationInternal& aOrientation);
+MOZ_MUST_USE bool LockScreenOrientation(
+    const hal::ScreenOrientation& aOrientation);
 
 /**
  * Unlock the screen orientation.
@@ -325,42 +257,12 @@ bool SetProcessPrioritySupported();
  */
 void SetProcessPriority(int aPid, hal::ProcessPriority aPriority);
 
-
-/**
- * Set the current thread's priority to appropriate platform-specific value for
- * given functionality. Instead of providing arbitrary priority numbers you
- * must specify a type of function like THREAD_PRIORITY_COMPOSITOR.
- */
-void SetCurrentThreadPriority(hal::ThreadPriority aThreadPriority);
-
-/**
- * Set a thread priority to appropriate platform-specific value for
- * given functionality. Instead of providing arbitrary priority numbers you
- * must specify a type of function like THREAD_PRIORITY_COMPOSITOR.
- */
-void SetThreadPriority(PlatformThreadId aThreadId,
-                       hal::ThreadPriority aThreadPriority);
-
-/**
- * Start monitoring disk space for low space situations.
- *
- * This API is currently only allowed to be used from the main process.
- */
-void StartDiskSpaceWatcher();
-
-/**
- * Stop monitoring disk space for low space situations.
- *
- * This API is currently only allowed to be used from the main process.
- */
-void StopDiskSpaceWatcher();
-
-} // namespace MOZ_HAL_NAMESPACE
-} // namespace mozilla
+}  // namespace MOZ_HAL_NAMESPACE
+}  // namespace mozilla
 
 #ifdef MOZ_DEFINED_HAL_NAMESPACE
-# undef MOZ_DEFINED_HAL_NAMESPACE
-# undef MOZ_HAL_NAMESPACE
+#  undef MOZ_DEFINED_HAL_NAMESPACE
+#  undef MOZ_HAL_NAMESPACE
 #endif
 
 #endif  // mozilla_Hal_h
