@@ -29,9 +29,8 @@
 #include <sys/resource.h>
 #include <sys/prctl.h>
 #include "sqlite3.h"
-#include "SQLiteBridge.h"
-#include "NSSBridge.h"
-#include "ElfLoader.h"
+#include "Linker.h"
+#include "BaseProfiler.h"
 #include "application.ini.h"
 
 #include "mozilla/arm.h"
@@ -99,7 +98,7 @@ JavaVM* sJavaVM;
 
 void abortThroughJava(const char* msg) {
   struct sigaction sigact = {};
-  if (SEGVHandler::__wrap_sigaction(SIGSEGV, nullptr, &sigact)) {
+  if (__wrap_sigaction(SIGSEGV, nullptr, &sigact)) {
     return;  // sigaction call failed.
   }
 
@@ -190,14 +189,42 @@ static void* dlopenLibrary(const char* libraryName) {
                        RTLD_GLOBAL | RTLD_LAZY);
 }
 
+static void EnsureBaseProfilerInitialized() {
+  // There is no single entry-point into C++ code on Android.
+  // Instead, GeckoThread and GeckoLibLoader call various functions to load
+  // libraries one-by-one.
+  // We want to capture all that library loading in the profiler, so we need to
+  // kick off the base profiler at the beginning of whichever function is called
+  // first.
+  // We currently assume that all these functions are called on the same thread.
+  static bool sInitialized = false;
+  if (sInitialized) {
+    return;
+  }
+
+#ifdef MOZ_GECKO_PROFILER
+  // The stack depth we observe here will be determined by the stack of
+  // whichever caller enters this code first. In practice this means that we may
+  // miss some root-most frames, which hopefully shouldn't ruin profiling.
+  int stackBase = 5;
+  mozilla::baseprofiler::profiler_init(&stackBase);
+#endif
+  sInitialized = true;
+}
+
 static mozglueresult loadGeckoLibs() {
   TimeStamp t0 = TimeStamp::Now();
   struct rusage usage1_thread, usage1;
   getrusage(RUSAGE_THREAD, &usage1_thread);
   getrusage(RUSAGE_SELF, &usage1);
 
-  gBootstrap = GetBootstrap(getUnpackedLibraryName("libxul.so").get(),
-                            LibLoadingStrategy::ReadAhead);
+  static const char* libxul = getenv("MOZ_ANDROID_LIBDIR_OVERRIDE");
+  if (libxul) {
+    gBootstrap = GetBootstrap(libxul, LibLoadingStrategy::ReadAhead);
+  } else {
+    gBootstrap = GetBootstrap(getUnpackedLibraryName("libxul.so").get(),
+                              LibLoadingStrategy::ReadAhead);
+  }
   if (!gBootstrap) {
     __android_log_print(ANDROID_LOG_ERROR, "GeckoLibLoad",
                         "Couldn't get a handle to libxul!");
@@ -246,7 +273,6 @@ static mozglueresult loadSQLiteLibs() {
   }
 #endif
 
-  setup_sqlite_functions(sqlite_handle);
   return SUCCESS;
 }
 
@@ -281,12 +307,14 @@ static mozglueresult loadNSSLibs() {
   }
 #endif
 
-  return setup_nss_functions(nss_handle, nspr_handle, plc_handle);
+  return SUCCESS;
 }
 
 extern "C" APKOPEN_EXPORT void MOZ_JNICALL
 Java_org_mozilla_gecko_mozglue_GeckoLoader_loadGeckoLibsNative(
     JNIEnv* jenv, jclass jGeckoAppShellClass) {
+  EnsureBaseProfilerInitialized();
+
   jenv->GetJavaVM(&sJavaVM);
 
   int res = loadGeckoLibs();
@@ -298,6 +326,8 @@ Java_org_mozilla_gecko_mozglue_GeckoLoader_loadGeckoLibsNative(
 extern "C" APKOPEN_EXPORT void MOZ_JNICALL
 Java_org_mozilla_gecko_mozglue_GeckoLoader_loadSQLiteLibsNative(
     JNIEnv* jenv, jclass jGeckoAppShellClass) {
+  EnsureBaseProfilerInitialized();
+
   __android_log_print(ANDROID_LOG_ERROR, "GeckoLibLoad", "Load sqlite start\n");
   mozglueresult rv = loadSQLiteLibs();
   if (rv != SUCCESS) {
@@ -309,6 +339,8 @@ Java_org_mozilla_gecko_mozglue_GeckoLoader_loadSQLiteLibsNative(
 extern "C" APKOPEN_EXPORT void MOZ_JNICALL
 Java_org_mozilla_gecko_mozglue_GeckoLoader_loadNSSLibsNative(
     JNIEnv* jenv, jclass jGeckoAppShellClass) {
+  EnsureBaseProfilerInitialized();
+
   __android_log_print(ANDROID_LOG_ERROR, "GeckoLibLoad", "Load nss start\n");
   mozglueresult rv = loadNSSLibs();
   if (rv != SUCCESS) {
@@ -359,6 +391,8 @@ Java_org_mozilla_gecko_mozglue_GeckoLoader_nativeRun(JNIEnv* jenv, jclass jc,
                                                      int prefsFd, int prefMapFd,
                                                      int ipcFd, int crashFd,
                                                      int crashAnnotationFd) {
+  EnsureBaseProfilerInitialized();
+
   int argc = 0;
   char** argv = CreateArgvFromObjectArray(jenv, jargs, &argc);
 
@@ -368,9 +402,13 @@ Java_org_mozilla_gecko_mozglue_GeckoLoader_nativeRun(JNIEnv* jenv, jclass jc,
       return;
     }
 
+#ifdef MOZ_LINKER
     ElfLoader::Singleton.ExpectShutdown(false);
+#endif
     gBootstrap->GeckoStart(jenv, argv, argc, sAppData);
+#ifdef MOZ_LINKER
     ElfLoader::Singleton.ExpectShutdown(true);
+#endif
   } else {
     gBootstrap->XRE_SetAndroidChildFds(
         jenv, {prefsFd, prefMapFd, ipcFd, crashFd, crashAnnotationFd});
@@ -391,6 +429,8 @@ Java_org_mozilla_gecko_mozglue_GeckoLoader_nativeRun(JNIEnv* jenv, jclass jc,
 
 extern "C" APKOPEN_EXPORT mozglueresult ChildProcessInit(int argc,
                                                          char* argv[]) {
+  EnsureBaseProfilerInitialized();
+
   if (loadNSSLibs() != SUCCESS) {
     return FAILURE;
   }
