@@ -8,12 +8,21 @@
 #define FuzzyLayer_h__
 
 #include "prerror.h"
+#include "nsError.h"
+#include "nsIFile.h"
 
 namespace mozilla {
 namespace net {
 
 nsresult AttachFuzzyIOLayer(PRFileDesc* fd);
 
+extern Atomic<bool> gFuzzingConnClosed;
+bool signalNetworkFuzzingDone();
+
+void addNetworkFuzzingBuffer(const uint8_t* data, size_t size,
+                             bool readFirst = false,
+                             bool useIsOptional = false);
+
 }  // namespace net
 }  // namespace mozilla
 
