@@ -5,17 +5,22 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsPACMan.h"
-#include "nsThreadUtils.h"
-#include "nsIAuthPrompt.h"
-#include "nsIPromptFactory.h"
-#include "nsIHttpChannel.h"
-#include "nsIPrefService.h"
-#include "nsIPrefBranch.h"
-#include "nsNetUtil.h"
-#include "nsIAsyncVerifyRedirectCallback.h"
-#include "nsISystemProxySettings.h"
-#include "nsContentUtils.h"
+
 #include "mozilla/Preferences.h"
+#include "nsContentUtils.h"
+#include "nsIAsyncVerifyRedirectCallback.h"
+#include "nsIAuthPrompt.h"
+#include "nsIDHCPClient.h"
+#include "nsIHttpChannel.h"
+#include "nsIPrefBranch.h"
+#include "nsIPromptFactory.h"
+#include "nsIProtocolProxyService.h"
+#include "nsISystemProxySettings.h"
+#include "nsNetUtil.h"
+#include "nsThreadUtils.h"
+#include "mozilla/Result.h"
+#include "mozilla/ResultExtensions.h"
+#include "mozilla/Telemetry.h"
 
 //-----------------------------------------------------------------------------
 
@@ -26,6 +31,12 @@ LazyLogModule gProxyLog("proxy");
 
 #undef LOG
 #define LOG(args) MOZ_LOG(gProxyLog, LogLevel::Debug, args)
+#define MOZ_WPAD_URL "http://wpad/wpad.dat"
+#define MOZ_DHCP_WPAD_OPTION 252
+
+// These pointers are declared in nsProtocolProxyService.cpp
+extern const char kProxyType_HTTPS[];
+extern const char kProxyType_DIRECT[];
 
 // The PAC thread does evaluations of both PAC files and
 // nsISystemProxySettings because they can both block the calling thread and we
@@ -33,9 +44,7 @@ LazyLogModule gProxyLog("proxy");
 
 // Check to see if the underlying request was not an error page in the case of
 // a HTTP request.  For other types of channels, just return true.
-static bool
-HttpRequestSucceeded(nsIStreamLoader *loader)
-{
+static bool HttpRequestSucceeded(nsIStreamLoader* loader) {
   nsCOMPtr<nsIRequest> request;
   loader->GetRequest(getter_AddRefs(request));
 
@@ -53,9 +62,7 @@ HttpRequestSucceeded(nsIStreamLoader *loader)
 // Read preference setting of extra JavaScript context heap size.
 // PrefService tends to be run on main thread, where ProxyAutoConfig runs on
 // ProxyResolution thread, so it's read here and passed to ProxyAutoConfig.
-static uint32_t
-GetExtraJSContextHeapSize()
-{
+static uint32_t GetExtraJSContextHeapSize() {
   MOZ_ASSERT(NS_IsMainThread());
 
   static int32_t extraSize = -1;
@@ -64,8 +71,9 @@ GetExtraJSContextHeapSize()
     nsCOMPtr<nsIPrefBranch> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
     int32_t value;
 
-    if (prefs && NS_SUCCEEDED(prefs->GetIntPref(
-        "network.proxy.autoconfig_extra_jscontext_heap_size", &value))) {
+    if (prefs &&
+        NS_SUCCEEDED(prefs->GetIntPref(
+            "network.proxy.autoconfig_extra_jscontext_heap_size", &value))) {
       LOG(("autoconfig_extra_jscontext_heap_size: %d\n", value));
 
       extraSize = value;
@@ -75,6 +83,24 @@ GetExtraJSContextHeapSize()
   return extraSize < 0 ? 0 : extraSize;
 }
 
+// Read network proxy type from preference
+// Used to verify that the preference is WPAD in nsPACMan::ConfigureWPAD
+nsresult GetNetworkProxyTypeFromPref(int32_t* type) {
+  *type = 0;
+  nsCOMPtr<nsIPrefBranch> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
+
+  if (!prefs) {
+    LOG(("Failed to get a preference service object"));
+    return NS_ERROR_FACTORY_NOT_REGISTERED;
+  }
+  nsresult rv = prefs->GetIntPref("network.proxy.type", type);
+  if (NS_FAILED(rv)) {
+    LOG(("Failed to retrieve network.proxy.type from prefs"));
+    return rv;
+  }
+  LOG(("network.proxy.type pref retrieved: %d\n", *type));
+  return NS_OK;
+}
 
 //-----------------------------------------------------------------------------
 
@@ -82,38 +108,28 @@ GetExtraJSContextHeapSize()
 // nsPACManCallback::OnQueryComplete on the Main thread when its completion is
 // discovered on the pac thread
 
-class ExecuteCallback final : public Runnable
-{
-public:
+class ExecuteCallback final : public Runnable {
+ public:
   ExecuteCallback(nsPACManCallback* aCallback, nsresult status)
-    : Runnable("net::ExecuteCallback")
-    , mCallback(aCallback)
-    , mStatus(status)
-  {
-  }
+      : Runnable("net::ExecuteCallback"),
+        mCallback(aCallback),
+        mStatus(status) {}
 
-  void SetPACString(const nsCString &pacString)
-  {
-    mPACString = pacString;
-  }
+  void SetPACString(const nsACString& pacString) { mPACString = pacString; }
 
-  void SetPACURL(const nsCString &pacURL)
-  {
-    mPACURL = pacURL;
-  }
+  void SetPACURL(const nsACString& pacURL) { mPACURL = pacURL; }
 
-  NS_IMETHOD Run() override
-  {
+  NS_IMETHOD Run() override {
     mCallback->OnQueryComplete(mStatus, mPACString, mPACURL);
     mCallback = nullptr;
     return NS_OK;
   }
 
-private:
+ private:
   RefPtr<nsPACManCallback> mCallback;
-  nsresult                   mStatus;
-  nsCString                  mPACString;
-  nsCString                  mPACURL;
+  nsresult mStatus;
+  nsCString mPACString;
+  nsCString mPACURL;
 };
 
 //-----------------------------------------------------------------------------
@@ -122,39 +138,29 @@ private:
 // acts as a proxy to do that, as the PACMan is reference counted
 // and might be destroyed on either thread
 
-class ShutdownThread final : public Runnable
-{
-public:
+class ShutdownThread final : public Runnable {
+ public:
   explicit ShutdownThread(nsIThread* thread)
-    : Runnable("net::ShutdownThread")
-    , mThread(thread)
-  {
-  }
+      : Runnable("net::ShutdownThread"), mThread(thread) {}
 
-  NS_IMETHOD Run() override
-  {
+  NS_IMETHOD Run() override {
     MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
     mThread->Shutdown();
     return NS_OK;
   }
 
-private:
+ private:
   nsCOMPtr<nsIThread> mThread;
 };
 
 // Dispatch this to wait until the PAC thread shuts down.
 
-class WaitForThreadShutdown final : public Runnable
-{
-public:
+class WaitForThreadShutdown final : public Runnable {
+ public:
   explicit WaitForThreadShutdown(nsPACMan* aPACMan)
-    : Runnable("net::WaitForThreadShutdown")
-    , mPACMan(aPACMan)
-  {
-  }
+      : Runnable("net::WaitForThreadShutdown"), mPACMan(aPACMan) {}
 
-  NS_IMETHOD Run() override
-  {
+  NS_IMETHOD Run() override {
     MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
     if (mPACMan->mPACThread) {
       mPACMan->mPACThread->Shutdown();
@@ -163,7 +169,7 @@ public:
     return NS_OK;
   }
 
-private:
+ private:
   RefPtr<nsPACMan> mPACMan;
 };
 
@@ -173,68 +179,85 @@ private:
 // the javascript PAC file has been installed (perhaps unsuccessfully)
 // and that there is no reason to queue executions anymore
 
-class PACLoadComplete final : public Runnable
-{
-public:
+class PACLoadComplete final : public Runnable {
+ public:
   explicit PACLoadComplete(nsPACMan* aPACMan)
-    : Runnable("net::PACLoadComplete")
-    , mPACMan(aPACMan)
-  {
-  }
+      : Runnable("net::PACLoadComplete"), mPACMan(aPACMan) {}
 
-  NS_IMETHOD Run() override
-  {
+  NS_IMETHOD Run() override {
     MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
     mPACMan->mLoader = nullptr;
     mPACMan->PostProcessPendingQ();
     return NS_OK;
   }
 
-private:
-    RefPtr<nsPACMan> mPACMan;
+ private:
+  RefPtr<nsPACMan> mPACMan;
+};
+
+//-----------------------------------------------------------------------------
+
+// ConfigureWPADComplete allows the PAC thread to tell the main thread that
+// the URL for the PAC file has been found
+class ConfigureWPADComplete final : public Runnable {
+ public:
+  ConfigureWPADComplete(nsPACMan* aPACMan, const nsACString& aPACURISpec)
+      : Runnable("net::ConfigureWPADComplete"),
+        mPACMan(aPACMan),
+        mPACURISpec(aPACURISpec) {}
+
+  NS_IMETHOD Run() override {
+    MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
+    mPACMan->AssignPACURISpec(mPACURISpec);
+    mPACMan->ContinueLoadingAfterPACUriKnown();
+    return NS_OK;
+  }
+
+ private:
+  RefPtr<nsPACMan> mPACMan;
+  nsCString mPACURISpec;
 };
 
 //-----------------------------------------------------------------------------
 
 // ExecutePACThreadAction is used to proxy actions from the main
-// thread onto the PAC thread. There are 3 options: process the queue,
-// cancel the queue, and setup the javascript context with a new PAC file
+// thread onto the PAC thread. There are 4 options: process the queue,
+// cancel the queue, query DHCP for the PAC option
+// and setup the javascript context with a new PAC file
 
-class ExecutePACThreadAction final : public Runnable
-{
-public:
+class ExecutePACThreadAction final : public Runnable {
+ public:
   // by default we just process the queue
   explicit ExecutePACThreadAction(nsPACMan* aPACMan)
-    : Runnable("net::ExecutePACThreadAction")
-    , mPACMan(aPACMan)
-    , mCancel(false)
-    , mCancelStatus(NS_OK)
-    , mSetupPAC(false)
-    , mExtraHeapSize(0)
-  { }
+      : Runnable("net::ExecutePACThreadAction"),
+        mPACMan(aPACMan),
+        mCancel(false),
+        mCancelStatus(NS_OK),
+        mSetupPAC(false),
+        mExtraHeapSize(0),
+        mConfigureWPAD(false),
+        mShutdown(false) {}
 
-  void CancelQueue (nsresult status)
-  {
+  void CancelQueue(nsresult status, bool aShutdown) {
     mCancel = true;
     mCancelStatus = status;
+    mShutdown = aShutdown;
   }
 
-  void SetupPAC (const char *text,
-                 uint32_t datalen,
-                 nsCString &pacURI,
-                 uint32_t extraHeapSize)
-  {
+  void SetupPAC(const char* data, uint32_t dataLen, const nsACString& pacURI,
+                uint32_t extraHeapSize) {
     mSetupPAC = true;
-    mSetupPACData.Assign(text, datalen);
+    mSetupPACData.Assign(data, dataLen);
     mSetupPACURI = pacURI;
     mExtraHeapSize = extraHeapSize;
   }
 
-  NS_IMETHOD Run() override
-  {
+  void ConfigureWPAD() { mConfigureWPAD = true; }
+
+  NS_IMETHOD Run() override {
     MOZ_ASSERT(!NS_IsMainThread(), "wrong thread");
     if (mCancel) {
-      mPACMan->CancelPendingQ(mCancelStatus);
+      mPACMan->CancelPendingQ(mCancelStatus, mShutdown);
       mCancel = false;
       return NS_OK;
     }
@@ -243,55 +266,61 @@ public:
       mSetupPAC = false;
 
       nsCOMPtr<nsIEventTarget> target = mPACMan->GetNeckoTarget();
-      mPACMan->mPAC.Init(mSetupPACURI,
-                         mSetupPACData,
-                         mPACMan->mIncludePath,
-                         mExtraHeapSize,
-                         target);
+      mPACMan->mPAC.Init(mSetupPACURI, mSetupPACData, mPACMan->mIncludePath,
+                         mExtraHeapSize, target);
 
       RefPtr<PACLoadComplete> runnable = new PACLoadComplete(mPACMan);
       mPACMan->Dispatch(runnable.forget());
       return NS_OK;
     }
 
+    if (mConfigureWPAD) {
+      nsAutoCString spec;
+      mConfigureWPAD = false;
+      mPACMan->ConfigureWPAD(spec);
+      RefPtr<ConfigureWPADComplete> runnable =
+          new ConfigureWPADComplete(mPACMan, spec);
+      mPACMan->Dispatch(runnable.forget());
+      return NS_OK;
+    }
+
     mPACMan->ProcessPendingQ();
     return NS_OK;
   }
 
-private:
+ private:
   RefPtr<nsPACMan> mPACMan;
 
-  bool      mCancel;
-  nsresult  mCancelStatus;
+  bool mCancel;
+  nsresult mCancelStatus;
 
-  bool                 mSetupPAC;
-  uint32_t             mExtraHeapSize;
-  nsCString            mSetupPACData;
-  nsCString            mSetupPACURI;
+  bool mSetupPAC;
+  uint32_t mExtraHeapSize;
+  nsCString mSetupPACData;
+  nsCString mSetupPACURI;
+  bool mConfigureWPAD;
+  bool mShutdown;
 };
 
 //-----------------------------------------------------------------------------
 
-PendingPACQuery::PendingPACQuery(nsPACMan* pacMan,
-                                 nsIURI* uri,
-                                 nsPACManCallback* callback,
+PendingPACQuery::PendingPACQuery(nsPACMan* pacMan, nsIURI* uri,
+                                 nsPACManCallback* callback, uint32_t flags,
                                  bool mainThreadResponse)
-  : Runnable("net::PendingPACQuery")
-  , mPACMan(pacMan)
-  , mCallback(callback)
-  , mOnMainThreadOnly(mainThreadResponse)
-{
+    : Runnable("net::PendingPACQuery"),
+      mPort(0),
+      mFlags(flags),
+      mPACMan(pacMan),
+      mCallback(callback),
+      mOnMainThreadOnly(mainThreadResponse) {
   uri->GetAsciiSpec(mSpec);
   uri->GetAsciiHost(mHost);
   uri->GetScheme(mScheme);
   uri->GetPort(&mPort);
 }
 
-void
-PendingPACQuery::Complete(nsresult status, const nsCString &pacString)
-{
-  if (!mCallback)
-    return;
+void PendingPACQuery::Complete(nsresult status, const nsACString& pacString) {
+  if (!mCallback) return;
   RefPtr<ExecuteCallback> runnable = new ExecuteCallback(mCallback, status);
   runnable->SetPACString(pacString);
   if (mOnMainThreadOnly)
@@ -300,11 +329,8 @@ PendingPACQuery::Complete(nsresult status, const nsCString &pacString)
     runnable->Run();
 }
 
-void
-PendingPACQuery::UseAlternatePACFile(const nsCString &pacURL)
-{
-  if (!mCallback)
-    return;
+void PendingPACQuery::UseAlternatePACFile(const nsACString& pacURL) {
+  if (!mCallback) return;
 
   RefPtr<ExecuteCallback> runnable = new ExecuteCallback(mCallback, NS_OK);
   runnable->SetPACURL(pacURL);
@@ -315,8 +341,7 @@ PendingPACQuery::UseAlternatePACFile(const nsCString &pacURL)
 }
 
 NS_IMETHODIMP
-PendingPACQuery::Run()
-{
+PendingPACQuery::Run() {
   MOZ_ASSERT(!NS_IsMainThread(), "wrong thread");
   mPACMan->PostQuery(this);
   return NS_OK;
@@ -325,20 +350,22 @@ PendingPACQuery::Run()
 //-----------------------------------------------------------------------------
 
 static bool sThreadLocalSetup = false;
-static uint32_t sThreadLocalIndex = 0xdeadbeef; // out of range
+static uint32_t sThreadLocalIndex = 0xdeadbeef;  // out of range
 
-static const char *kPACIncludePath =
-  "network.proxy.autoconfig_url.include_path";
+static const char* kPACIncludePath =
+    "network.proxy.autoconfig_url.include_path";
 
-nsPACMan::nsPACMan(nsIEventTarget *mainThreadEventTarget)
-  : NeckoTargetHolder(mainThreadEventTarget)
-  , mLoadPending(false)
-  , mShutdown(false)
-  , mLoadFailureCount(0)
-  , mInProgress(false)
-{
+nsPACMan::nsPACMan(nsIEventTarget* mainThreadEventTarget)
+    : NeckoTargetHolder(mainThreadEventTarget),
+      mLoadPending(false),
+      mShutdown(false),
+      mLoadFailureCount(0),
+      mInProgress(false),
+      mAutoDetect(false),
+      mWPADOverDHCPEnabled(false),
+      mProxyConfigType(0) {
   MOZ_ASSERT(NS_IsMainThread(), "pacman must be created on main thread");
-  if (!sThreadLocalSetup){
+  if (!sThreadLocalSetup) {
     sThreadLocalSetup = true;
     PR_NewThreadPrivateIndex(&sThreadLocalIndex, nullptr);
   }
@@ -346,15 +373,14 @@ nsPACMan::nsPACMan(nsIEventTarget *mainThreadEventTarget)
   mIncludePath = Preferences::GetBool(kPACIncludePath, false);
 }
 
-nsPACMan::~nsPACMan()
-{
+nsPACMan::~nsPACMan() {
   MOZ_ASSERT(mShutdown, "Shutdown must be called before dtor.");
 
   if (mPACThread) {
     if (NS_IsMainThread()) {
       mPACThread->Shutdown();
-    }
-    else {
+      mPACThread = nullptr;
+    } else {
       RefPtr<ShutdownThread> runnable = new ShutdownThread(mPACThread);
       Dispatch(runnable.forget());
     }
@@ -364,42 +390,64 @@ nsPACMan::~nsPACMan()
   NS_ASSERTION(mPendingQ.isEmpty(), "pac man not shutdown properly");
 }
 
-void
-nsPACMan::Shutdown()
-{
+void nsPACMan::Shutdown() {
   MOZ_ASSERT(NS_IsMainThread(), "pacman must be shutdown on main thread");
   if (mShutdown) {
     return;
   }
-  mShutdown = true;
+
   CancelExistingLoad();
 
-  MOZ_ASSERT(mPACThread, "mPAC requires mPACThread to shutdown");
-  PostCancelPendingQ(NS_ERROR_ABORT);
+  if (mPACThread) {
+    PostCancelPendingQ(NS_ERROR_ABORT, /*aShutdown =*/true);
 
-  RefPtr<WaitForThreadShutdown> runnable = new WaitForThreadShutdown(this);
-  Dispatch(runnable.forget());
+    // Shutdown is initiated from an observer. We don't want to block the
+    // observer service on thread shutdown so we post a shutdown runnable that
+    // will run after we return instead.
+    RefPtr<WaitForThreadShutdown> runnable = new WaitForThreadShutdown(this);
+    Dispatch(runnable.forget());
+  }
+
+  mShutdown = true;
 }
 
-nsresult
-nsPACMan::AsyncGetProxyForURI(nsIURI *uri,
-                              nsPACManCallback *callback,
-                              bool mainThreadResponse)
-{
+nsresult nsPACMan::DispatchToPAC(already_AddRefed<nsIRunnable> aEvent,
+                                 bool aSync) {
   MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
-  if (mShutdown)
+
+  nsCOMPtr<nsIRunnable> e(aEvent);
+
+  if (mShutdown) {
     return NS_ERROR_NOT_AVAILABLE;
+  }
+
+  // Lazily create the PAC thread. This method is main-thread only so we don't
+  // have to worry about threading issues here.
+  if (!mPACThread) {
+    MOZ_TRY(NS_NewNamedThread("ProxyResolution", getter_AddRefs(mPACThread)));
+  }
+
+  return mPACThread->Dispatch(
+      e.forget(),
+      aSync ? nsIEventTarget::DISPATCH_SYNC : nsIEventTarget::DISPATCH_NORMAL);
+}
+
+nsresult nsPACMan::AsyncGetProxyForURI(nsIURI* uri, nsPACManCallback* callback,
+                                       uint32_t flags,
+                                       bool mainThreadResponse) {
+  MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
+  if (mShutdown) return NS_ERROR_NOT_AVAILABLE;
 
   // Maybe Reload PAC
   if (!mPACURISpec.IsEmpty() && !mScheduledReload.IsNull() &&
       TimeStamp::Now() > mScheduledReload) {
     LOG(("nsPACMan::AsyncGetProxyForURI reload as scheduled\n"));
 
-    LoadPACFromURI(EmptyCString());
+    LoadPACFromURI(mAutoDetect ? EmptyCString() : mPACURISpec, false);
   }
 
   RefPtr<PendingPACQuery> query =
-    new PendingPACQuery(this, uri, callback, mainThreadResponse);
+      new PendingPACQuery(this, uri, callback, flags, mainThreadResponse);
 
   if (IsPACURI(uri)) {
     // deal with this directly instead of queueing it
@@ -407,12 +455,10 @@ nsPACMan::AsyncGetProxyForURI(nsIURI *uri,
     return NS_OK;
   }
 
-  return mPACThread->Dispatch(query, nsIEventTarget::DISPATCH_NORMAL);
+  return DispatchToPAC(query.forget());
 }
 
-nsresult
-nsPACMan::PostQuery(PendingPACQuery *query)
-{
+nsresult nsPACMan::PostQuery(PendingPACQuery* query) {
   MOZ_ASSERT(!NS_IsMainThread(), "wrong thread");
 
   if (mShutdown) {
@@ -427,17 +473,49 @@ nsPACMan::PostQuery(PendingPACQuery *query)
   return NS_OK;
 }
 
-nsresult
-nsPACMan::LoadPACFromURI(const nsCString &spec)
-{
+nsresult nsPACMan::LoadPACFromURI(const nsACString& aSpec) {
+  return LoadPACFromURI(aSpec, true);
+}
+
+nsresult nsPACMan::LoadPACFromURI(const nsACString& aSpec,
+                                  bool aResetLoadFailureCount) {
   NS_ENSURE_STATE(!mShutdown);
-  NS_ENSURE_ARG(!spec.IsEmpty() || !mPACURISpec.IsEmpty());
 
   nsCOMPtr<nsIStreamLoader> loader =
       do_CreateInstance(NS_STREAMLOADER_CONTRACTID);
   NS_ENSURE_STATE(loader);
 
-  LOG(("nsPACMan::LoadPACFromURI %s\n", spec.get()));
+  LOG(("nsPACMan::LoadPACFromURI aSpec: %s, aResetLoadFailureCount: %s\n",
+       aSpec.BeginReading(), aResetLoadFailureCount ? "true" : "false"));
+
+  CancelExistingLoad();
+
+  mLoader = loader;
+  mPACURIRedirectSpec.Truncate();
+  mNormalPACURISpec.Truncate();  // set at load time
+  if (aResetLoadFailureCount) {
+    mLoadFailureCount = 0;
+  }
+  mAutoDetect = aSpec.IsEmpty();
+  mPACURISpec.Assign(aSpec);
+
+  // reset to Null
+  mScheduledReload = TimeStamp();
+
+  // if we're on the main thread here so we can get hold of prefs,
+  // we check that we have WPAD preffed on if we're auto-detecting
+  if (mAutoDetect && NS_IsMainThread()) {
+    nsresult rv = GetNetworkProxyTypeFromPref(&mProxyConfigType);
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
+    if (mProxyConfigType != nsIProtocolProxyService::PROXYCONFIG_WPAD) {
+      LOG(
+          ("LoadPACFromURI - Aborting WPAD autodetection because the pref "
+           "doesn't match anymore"));
+      return NS_BINDING_ABORTED;
+    }
+  }
   // Since we might get called from nsProtocolProxyService::Init, we need to
   // post an event back to the main thread before we try to use the IO service.
   //
@@ -445,34 +523,76 @@ nsPACMan::LoadPACFromURI(const nsCString &spec)
   // queries the enter between now and when we actually load the PAC file.
 
   if (!mLoadPending) {
-    nsCOMPtr<nsIRunnable> runnable =
-      NewRunnableMethod("nsPACMan::StartLoading", this, &nsPACMan::StartLoading);
-    nsresult rv = NS_IsMainThread()
-      ? Dispatch(runnable.forget())
-      : GetCurrentThreadEventTarget()->Dispatch(runnable.forget());
-    if (NS_FAILED(rv))
-      return rv;
+    nsCOMPtr<nsIRunnable> runnable = NewRunnableMethod(
+        "nsPACMan::StartLoading", this, &nsPACMan::StartLoading);
+    nsresult rv =
+        NS_IsMainThread()
+            ? Dispatch(runnable.forget())
+            : GetCurrentThreadEventTarget()->Dispatch(runnable.forget());
+    if (NS_FAILED(rv)) return rv;
     mLoadPending = true;
   }
 
-  CancelExistingLoad();
-
-  mLoader = loader;
-  if (!spec.IsEmpty()) {
-    mPACURISpec = spec;
-    mPACURIRedirectSpec.Truncate();
-    mNormalPACURISpec.Truncate(); // set at load time
-    mLoadFailureCount = 0;  // reset
-  }
-
-  // reset to Null
-  mScheduledReload = TimeStamp();
   return NS_OK;
 }
 
-void
-nsPACMan::StartLoading()
-{
+nsresult nsPACMan::GetPACFromDHCP(nsACString& aSpec) {
+  MOZ_ASSERT(!NS_IsMainThread(), "wrong thread");
+  if (!mDHCPClient) {
+    LOG(
+        ("nsPACMan::GetPACFromDHCP DHCP option %d query failed because there "
+         "is no DHCP client available\n",
+         MOZ_DHCP_WPAD_OPTION));
+    return NS_ERROR_NOT_IMPLEMENTED;
+  }
+  nsresult rv;
+  rv = mDHCPClient->GetOption(MOZ_DHCP_WPAD_OPTION, aSpec);
+  if (NS_FAILED(rv)) {
+    LOG((
+        "nsPACMan::GetPACFromDHCP DHCP option %d query failed with result %d\n",
+        MOZ_DHCP_WPAD_OPTION, (uint32_t)rv));
+  } else {
+    LOG(
+        ("nsPACMan::GetPACFromDHCP DHCP option %d query succeeded, finding PAC "
+         "URL %s\n",
+         MOZ_DHCP_WPAD_OPTION, aSpec.BeginReading()));
+  }
+  return rv;
+}
+
+nsresult nsPACMan::ConfigureWPAD(nsACString& aSpec) {
+  MOZ_ASSERT(!NS_IsMainThread(), "wrong thread");
+
+  if (mProxyConfigType != nsIProtocolProxyService::PROXYCONFIG_WPAD) {
+    LOG(
+        ("ConfigureWPAD - Aborting WPAD autodetection because the pref "
+         "doesn't match anymore"));
+    return NS_BINDING_ABORTED;
+  }
+
+  aSpec.Truncate();
+  if (mWPADOverDHCPEnabled) {
+    GetPACFromDHCP(aSpec);
+  }
+
+  if (aSpec.IsEmpty()) {
+    // We diverge from the WPAD spec here in that we don't walk the
+    // hosts's FQDN, stripping components until we hit a TLD.  Doing so
+    // is dangerous in the face of an incomplete list of TLDs, and TLDs
+    // get added over time.  We could consider doing only a single
+    // substitution of the first component, if that proves to help
+    // compatibility.
+    aSpec.AssignLiteral(MOZ_WPAD_URL);
+  }
+  return NS_OK;
+}
+
+void nsPACMan::AssignPACURISpec(const nsACString& aSpec) {
+  MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
+  mPACURISpec.Assign(aSpec);
+}
+
+void nsPACMan::StartLoading() {
   MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
   mLoadPending = false;
 
@@ -482,6 +602,31 @@ nsPACMan::StartLoading()
     return;
   }
 
+  if (mAutoDetect) {
+    nsresult rv = GetNetworkProxyTypeFromPref(&mProxyConfigType);
+    if (NS_FAILED(rv)) {
+      NS_WARNING(
+          "Could not retrieve Network Proxy Type pref when auto-detecting "
+          "proxy. Halting.");
+      return;
+    }
+    RefPtr<ExecutePACThreadAction> wpadConfigurer =
+        new ExecutePACThreadAction(this);
+    wpadConfigurer->ConfigureWPAD();
+    DispatchToPAC(wpadConfigurer.forget());
+  } else {
+    ContinueLoadingAfterPACUriKnown();
+  }
+}
+
+void nsPACMan::ContinueLoadingAfterPACUriKnown() {
+  MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
+
+  // CancelExistingLoad was called...
+  if (!mLoader) {
+    PostCancelPendingQ(NS_ERROR_ABORT);
+    return;
+  }
   if (NS_SUCCEEDED(mLoader->Init(this, nullptr))) {
     // Always hit the origin server when loading PAC.
     nsCOMPtr<nsIIOService> ios = do_GetIOService();
@@ -492,29 +637,38 @@ nsPACMan::StartLoading()
 
       // NOTE: This results in GetProxyForURI being called
       if (pacURI) {
+        if (pacURI->SchemeIs("ftp")) {
+          Telemetry::AccumulateCategorical(
+              Telemetry::LABELS_NETWORK_PAC_URL_SCHEME::ftp);
+        } else {
+          Telemetry::AccumulateCategorical(
+              Telemetry::LABELS_NETWORK_PAC_URL_SCHEME::other);
+        }
+
         nsresult rv = pacURI->GetSpec(mNormalPACURISpec);
         MOZ_RELEASE_ASSERT(NS_SUCCEEDED(rv));
-        NS_NewChannel(getter_AddRefs(channel),
-                      pacURI,
+        NS_NewChannel(getter_AddRefs(channel), pacURI,
                       nsContentUtils::GetSystemPrincipal(),
                       nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
                       nsIContentPolicy::TYPE_OTHER,
-                      nullptr, // PerformanceStorage,
-                      nullptr, // aLoadGroup
-                      nullptr, // aCallbacks
-                      nsIRequest::LOAD_NORMAL,
-                      ios);
-      }
-      else {
+                      nullptr,  // nsICookieJarSettings
+                      nullptr,  // PerformanceStorage
+                      nullptr,  // aLoadGroup
+                      nullptr,  // aCallbacks
+                      nsIRequest::LOAD_NORMAL, ios);
+      } else {
         LOG(("nsPACMan::StartLoading Failed pacspec uri conversion %s\n",
              mPACURISpec.get()));
       }
 
       if (channel) {
+        // allow deprecated HTTP request from SystemPrincipal
+        nsCOMPtr<nsILoadInfo> loadInfo = channel->LoadInfo();
+        loadInfo->SetAllowDeprecatedSystemRequests(true);
+
         channel->SetLoadFlags(nsIRequest::LOAD_BYPASS_CACHE);
         channel->SetNotificationCallbacks(this);
-        if (NS_SUCCEEDED(channel->AsyncOpen2(mLoader)))
-          return;
+        if (NS_SUCCEEDED(channel->AsyncOpen(mLoader))) return;
       }
     }
   }
@@ -523,10 +677,7 @@ nsPACMan::StartLoading()
   PostCancelPendingQ(NS_ERROR_UNEXPECTED);
 }
 
-
-void
-nsPACMan::OnLoadFailure()
-{
+void nsPACMan::OnLoadFailure() {
   int32_t minInterval = 5;    // 5 seconds
   int32_t maxInterval = 300;  // 5 minutes
 
@@ -539,55 +690,41 @@ nsPACMan::OnLoadFailure()
   }
 
   int32_t interval = minInterval << mLoadFailureCount++;  // seconds
-  if (!interval || interval > maxInterval)
-    interval = maxInterval;
+  if (!interval || interval > maxInterval) interval = maxInterval;
 
   mScheduledReload = TimeStamp::Now() + TimeDuration::FromSeconds(interval);
 
-  LOG(("OnLoadFailure: retry in %d seconds (%d fails)\n",
-       interval, mLoadFailureCount));
+  LOG(("OnLoadFailure: retry in %d seconds (%d fails)\n", interval,
+       mLoadFailureCount));
 
   // while we wait for the retry queued members should try direct
   // even if that means fast failure.
   PostCancelPendingQ(NS_ERROR_NOT_AVAILABLE);
 }
 
-void
-nsPACMan::CancelExistingLoad()
-{
+void nsPACMan::CancelExistingLoad() {
   if (mLoader) {
     nsCOMPtr<nsIRequest> request;
     mLoader->GetRequest(getter_AddRefs(request));
-    if (request)
-      request->Cancel(NS_ERROR_ABORT);
+    if (request) request->Cancel(NS_ERROR_ABORT);
     mLoader = nullptr;
   }
 }
 
-void
-nsPACMan::PostProcessPendingQ()
-{
+void nsPACMan::PostProcessPendingQ() {
   MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
-  RefPtr<ExecutePACThreadAction> pending =
-    new ExecutePACThreadAction(this);
-  if (mPACThread)
-    mPACThread->Dispatch(pending, nsIEventTarget::DISPATCH_NORMAL);
+  RefPtr<ExecutePACThreadAction> pending = new ExecutePACThreadAction(this);
+  DispatchToPAC(pending.forget());
 }
 
-void
-nsPACMan::PostCancelPendingQ(nsresult status)
-{
+void nsPACMan::PostCancelPendingQ(nsresult status, bool aShutdown) {
   MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
-  RefPtr<ExecutePACThreadAction> pending =
-    new ExecutePACThreadAction(this);
-  pending->CancelQueue(status);
-  if (mPACThread)
-    mPACThread->Dispatch(pending, nsIEventTarget::DISPATCH_NORMAL);
+  RefPtr<ExecutePACThreadAction> pending = new ExecutePACThreadAction(this);
+  pending->CancelQueue(status, aShutdown);
+  DispatchToPAC(pending.forget());
 }
 
-void
-nsPACMan::CancelPendingQ(nsresult status)
-{
+void nsPACMan::CancelPendingQ(nsresult status, bool aShutdown) {
   MOZ_ASSERT(!NS_IsMainThread(), "wrong thread");
   RefPtr<PendingPACQuery> query;
 
@@ -596,15 +733,13 @@ nsPACMan::CancelPendingQ(nsresult status)
     query->Complete(status, EmptyCString());
   }
 
-  if (mShutdown)
-    mPAC.Shutdown();
+  if (aShutdown) mPAC.Shutdown();
 }
 
-void
-nsPACMan::ProcessPendingQ()
-{
+void nsPACMan::ProcessPendingQ() {
   MOZ_ASSERT(!NS_IsMainThread(), "wrong thread");
-  while (ProcessPending());
+  while (ProcessPending())
+    ;
 
   if (mShutdown) {
     mPAC.Shutdown();
@@ -615,16 +750,12 @@ nsPACMan::ProcessPendingQ()
 }
 
 // returns true if progress was made by shortening the queue
-bool
-nsPACMan::ProcessPending()
-{
-  if (mPendingQ.isEmpty())
-    return false;
+bool nsPACMan::ProcessPending() {
+  if (mPendingQ.isEmpty()) return false;
 
   // queue during normal load, but if we are retrying a failed load then
   // fast fail the queries
-  if (mInProgress || (IsLoading() && !mLoadFailureCount))
-    return false;
+  if (mInProgress || (IsLoading() && !mLoadFailureCount)) return false;
 
   RefPtr<PendingPACQuery> query(dont_AddRef(mPendingQ.popFirst()));
 
@@ -641,8 +772,7 @@ nsPACMan::ProcessPending()
   // first we need to consider the system proxy changing the pac url
   if (mSystemProxySettings &&
       NS_SUCCEEDED(mSystemProxySettings->GetPACURI(PACURI)) &&
-      !PACURI.IsEmpty() &&
-      !PACURI.Equals(mPACURISpec)) {
+      !PACURI.IsEmpty() && !PACURI.Equals(mPACURISpec)) {
     query->UseAlternatePACFile(PACURI);
     LOG(("Use PAC from system settings: %s\n", PACURI.get()));
     completed = true;
@@ -651,10 +781,21 @@ nsPACMan::ProcessPending()
   // now try the system proxy settings for this particular url if
   // PAC was not specified
   if (!completed && mSystemProxySettings && PACURI.IsEmpty() &&
-      NS_SUCCEEDED(mSystemProxySettings->
-                   GetProxyForURI(query->mSpec, query->mScheme,
-                                  query->mHost, query->mPort,
-                                  pacString))) {
+      NS_SUCCEEDED(mSystemProxySettings->GetProxyForURI(
+          query->mSpec, query->mScheme, query->mHost, query->mPort,
+          pacString))) {
+    if (query->mFlags & nsIProtocolProxyService::RESOLVE_PREFER_SOCKS_PROXY &&
+        query->mFlags & nsIProtocolProxyService::RESOLVE_PREFER_HTTPS_PROXY) {
+      if (StringBeginsWith(pacString, nsDependentCString(kProxyType_DIRECT),
+                           nsCaseInsensitiveUTF8StringComparator)) {
+        // DIRECT indicates that system proxy settings are not configured to use
+        // SOCKS proxy. Try https proxy as a secondary preferrable proxy. This
+        // is mainly for websocket whose precedence is SOCKS > HTTPS > DIRECT.
+        NS_SUCCEEDED(mSystemProxySettings->GetProxyForURI(
+            query->mSpec, nsDependentCString(kProxyType_HTTPS), query->mHost,
+            query->mPort, pacString));
+      }
+    }
     LOG(("Use proxy from system settings: %s\n", pacString.get()));
     query->Complete(NS_OK, pacString);
     completed = true;
@@ -662,8 +803,8 @@ nsPACMan::ProcessPending()
 
   // the systemproxysettings didn't complete the resolution. try via PAC
   if (!completed) {
-    nsresult status = mPAC.GetProxyForURI(query->mSpec, query->mHost,
-                                          pacString);
+    nsresult status =
+        mPAC.GetProxyForURI(query->mSpec, query->mHost, pacString);
     LOG(("Use proxy from PAC: %s\n", pacString.get()));
     query->Complete(status, pacString);
   }
@@ -672,16 +813,13 @@ nsPACMan::ProcessPending()
   return true;
 }
 
-NS_IMPL_ISUPPORTS(nsPACMan, nsIStreamLoaderObserver,
-                  nsIInterfaceRequestor, nsIChannelEventSink)
+NS_IMPL_ISUPPORTS(nsPACMan, nsIStreamLoaderObserver, nsIInterfaceRequestor,
+                  nsIChannelEventSink)
 
 NS_IMETHODIMP
-nsPACMan::OnStreamComplete(nsIStreamLoader *loader,
-                           nsISupports *context,
-                           nsresult status,
-                           uint32_t dataLen,
-                           const uint8_t *data)
-{
+nsPACMan::OnStreamComplete(nsIStreamLoader* loader, nsISupports* context,
+                           nsresult status, uint32_t dataLen,
+                           const uint8_t* data) {
   MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
   if (mLoader != loader) {
     // If this happens, then it means that LoadPACFromURI was called more
@@ -689,8 +827,7 @@ nsPACMan::OnStreamComplete(nsIStreamLoader *loader,
     // should be NS_ERROR_ABORT, and if so, then we know that we can and
     // should delay any processing.
     LOG(("OnStreamComplete: called more than once\n"));
-    if (status == NS_ERROR_ABORT)
-      return NS_OK;
+    if (status == NS_ERROR_ABORT) return NS_OK;
   }
 
   LOG(("OnStreamComplete: entry\n"));
@@ -705,26 +842,18 @@ nsPACMan::OnStreamComplete(nsIStreamLoader *loader,
       if (channel) {
         nsCOMPtr<nsIURI> uri;
         channel->GetURI(getter_AddRefs(uri));
-        if (uri)
-          uri->GetAsciiSpec(pacURI);
+        if (uri) uri->GetAsciiSpec(pacURI);
       }
     }
 
-    // We assume that the PAC text is ASCII (or ISO-Latin-1).  We've had this
-    // assumption forever, and some real-world PAC scripts actually have some
-    // non-ASCII text in comment blocks (see bug 296163).
-    const char *text = (const char *) data;
-
-    // we have succeeded in loading the pac file using a bunch of interfaces that
-    // are main thread only, unfortunately we have to initialize the instance of
-    // the PAC evaluator (NS_PROXYAUTOCONFIG_CONTRACTID) on the pac thread, because
-    // that is where it will be used.
-
-    RefPtr<ExecutePACThreadAction> pending =
-      new ExecutePACThreadAction(this);
-    pending->SetupPAC(text, dataLen, pacURI, GetExtraJSContextHeapSize());
-    if (mPACThread)
-      mPACThread->Dispatch(pending, nsIEventTarget::DISPATCH_NORMAL);
+    // We succeeded in loading the pac file using a bunch of interfaces that are
+    // main thread only.  Unfortunately, we have to initialize the instance of
+    // the PAC evaluator (NS_PROXYAUTOCONFIG_CONTRACTID) on the PAC thread,
+    // because that's where it will be used.
+    RefPtr<ExecutePACThreadAction> pending = new ExecutePACThreadAction(this);
+    pending->SetupPAC(reinterpret_cast<const char*>(data), dataLen, pacURI,
+                      GetExtraJSContextHeapSize());
+    DispatchToPAC(pending.forget());
 
     LOG(("OnStreamComplete: process the PAC contents\n"));
 
@@ -747,19 +876,24 @@ nsPACMan::OnStreamComplete(nsIStreamLoader *loader,
 }
 
 NS_IMETHODIMP
-nsPACMan::GetInterface(const nsIID &iid, void **result)
-{
+nsPACMan::GetInterface(const nsIID& iid, void** result) {
   // In case loading the PAC file requires authentication.
   if (iid.Equals(NS_GET_IID(nsIAuthPrompt))) {
-    nsCOMPtr<nsIPromptFactory> promptFac = do_GetService("@mozilla.org/prompter;1");
-    NS_ENSURE_TRUE(promptFac, NS_ERROR_FAILURE);
-    return promptFac->GetPrompt(nullptr, iid, reinterpret_cast<void**>(result));
+    nsCOMPtr<nsIPromptFactory> promptFac =
+        do_GetService("@mozilla.org/prompter;1");
+    NS_ENSURE_TRUE(promptFac, NS_ERROR_NO_INTERFACE);
+    nsresult rv =
+        promptFac->GetPrompt(nullptr, iid, reinterpret_cast<void**>(result));
+    if (NS_FAILED(rv)) {
+      return NS_ERROR_NO_INTERFACE;
+    }
+    return NS_OK;
   }
 
   // In case loading the PAC file results in a redirect.
   if (iid.Equals(NS_GET_IID(nsIChannelEventSink))) {
     NS_ADDREF_THIS();
-    *result = static_cast<nsIChannelEventSink *>(this);
+    *result = static_cast<nsIChannelEventSink*>(this);
     return NS_OK;
   }
 
@@ -767,20 +901,17 @@ nsPACMan::GetInterface(const nsIID &iid, void **result)
 }
 
 NS_IMETHODIMP
-nsPACMan::AsyncOnChannelRedirect(nsIChannel *oldChannel, nsIChannel *newChannel,
+nsPACMan::AsyncOnChannelRedirect(nsIChannel* oldChannel, nsIChannel* newChannel,
                                  uint32_t flags,
-                                 nsIAsyncVerifyRedirectCallback *callback)
-{
+                                 nsIAsyncVerifyRedirectCallback* callback) {
   MOZ_ASSERT(NS_IsMainThread(), "wrong thread");
 
   nsresult rv = NS_OK;
   nsCOMPtr<nsIURI> pacURI;
-  if (NS_FAILED((rv = newChannel->GetURI(getter_AddRefs(pacURI)))))
-      return rv;
+  if (NS_FAILED((rv = newChannel->GetURI(getter_AddRefs(pacURI))))) return rv;
 
   rv = pacURI->GetSpec(mPACURIRedirectSpec);
-  if (NS_FAILED(rv))
-      return rv;
+  if (NS_FAILED(rv)) return rv;
 
   LOG(("nsPACMan redirect from original %s to redirected %s\n",
        mPACURISpec.get(), mPACURIRedirectSpec.get()));
@@ -795,16 +926,11 @@ nsPACMan::AsyncOnChannelRedirect(nsIChannel *oldChannel, nsIChannel *newChannel,
   return NS_OK;
 }
 
-nsresult
-nsPACMan::Init(nsISystemProxySettings *systemProxySettings)
-{
+nsresult nsPACMan::Init(nsISystemProxySettings* systemProxySettings) {
   mSystemProxySettings = systemProxySettings;
-
-  nsresult rv =
-    NS_NewNamedThread("ProxyResolution", getter_AddRefs(mPACThread));
-
-  return rv;
+  mDHCPClient = do_GetService(NS_DHCPCLIENT_CONTRACTID);
+  return NS_OK;
 }
 
-} // namespace net
-} // namespace mozilla
+}  // namespace net
+}  // namespace mozilla
