@@ -15,7 +15,7 @@ namespace net {
 
 #define kMinBufSize 512
 
-CacheFileChunkBuffer::CacheFileChunkBuffer(CacheFileChunk *aChunk)
+CacheFileChunkBuffer::CacheFileChunkBuffer(CacheFileChunk* aChunk)
     : mChunk(aChunk),
       mBuf(nullptr),
       mBufSize(0),
@@ -32,14 +32,14 @@ CacheFileChunkBuffer::~CacheFileChunkBuffer() {
   }
 }
 
-void CacheFileChunkBuffer::CopyFrom(CacheFileChunkBuffer *aOther) {
+void CacheFileChunkBuffer::CopyFrom(CacheFileChunkBuffer* aOther) {
   MOZ_RELEASE_ASSERT(mBufSize >= aOther->mDataSize);
   mDataSize = aOther->mDataSize;
   memcpy(mBuf, aOther->mBuf, mDataSize);
 }
 
 nsresult CacheFileChunkBuffer::FillInvalidRanges(
-    CacheFileChunkBuffer *aOther, CacheFileUtils::ValidityMap *aMap) {
+    CacheFileChunkBuffer* aOther, CacheFileUtils::ValidityMap* aMap) {
   nsresult rv;
 
   rv = EnsureBufSize(aOther->mDataSize);
@@ -71,7 +71,7 @@ nsresult CacheFileChunkBuffer::FillInvalidRanges(
   return NS_OK;
 }
 
-MOZ_MUST_USE nsresult CacheFileChunkBuffer::EnsureBufSize(uint32_t aBufSize) {
+[[nodiscard]] nsresult CacheFileChunkBuffer::EnsureBufSize(uint32_t aBufSize) {
   AssertOwnsLock();
 
   if (mBufSize >= aBufSize) {
@@ -95,7 +95,7 @@ MOZ_MUST_USE nsresult CacheFileChunkBuffer::EnsureBufSize(uint32_t aBufSize) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
-  char *newBuf = static_cast<char *>(realloc(mBuf, aBufSize));
+  char* newBuf = static_cast<char*>(realloc(mBuf, aBufSize));
   if (!newBuf) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
@@ -167,7 +167,7 @@ uint32_t CacheFileChunkHandle::Offset() {
   return mBuf->mChunk->Index() * kChunkSize;
 }
 
-CacheFileChunkReadHandle::CacheFileChunkReadHandle(CacheFileChunkBuffer *aBuf) {
+CacheFileChunkReadHandle::CacheFileChunkReadHandle(CacheFileChunkBuffer* aBuf) {
   mBuf = aBuf;
   mBuf->mReadHandlesCount++;
 }
@@ -176,10 +176,10 @@ CacheFileChunkReadHandle::~CacheFileChunkReadHandle() {
   mBuf->RemoveReadHandle();
 }
 
-const char *CacheFileChunkReadHandle::Buf() { return mBuf->mBuf; }
+const char* CacheFileChunkReadHandle::Buf() { return mBuf->mBuf; }
 
 CacheFileChunkWriteHandle::CacheFileChunkWriteHandle(
-    CacheFileChunkBuffer *aBuf) {
+    CacheFileChunkBuffer* aBuf) {
   mBuf = aBuf;
   if (mBuf) {
     MOZ_ASSERT(!mBuf->mWriteHandleExists);
@@ -193,7 +193,7 @@ CacheFileChunkWriteHandle::~CacheFileChunkWriteHandle() {
   }
 }
 
-char *CacheFileChunkWriteHandle::Buf() { return mBuf ? mBuf->mBuf : nullptr; }
+char* CacheFileChunkWriteHandle::Buf() { return mBuf ? mBuf->mBuf : nullptr; }
 
 void CacheFileChunkWriteHandle::UpdateDataSize(uint32_t aOffset,
                                                uint32_t aLen) {
@@ -210,8 +210,8 @@ void CacheFileChunkWriteHandle::UpdateDataSize(uint32_t aOffset,
 
 class NotifyUpdateListenerEvent : public Runnable {
  public:
-  NotifyUpdateListenerEvent(CacheFileChunkListener *aCallback,
-                            CacheFileChunk *aChunk)
+  NotifyUpdateListenerEvent(CacheFileChunkListener* aCallback,
+                            CacheFileChunk* aChunk)
       : Runnable("net::NotifyUpdateListenerEvent"),
         mCallback(aCallback),
         mChunk(aChunk) {
@@ -258,7 +258,7 @@ CacheFileChunk::Release() {
     return count;
   }
 
-  NS_PRECONDITION(0 != mRefCnt, "dup release");
+  MOZ_ASSERT(0 != mRefCnt, "dup release");
   count = --mRefCnt;
   NS_LOG_RELEASE(this, count, "CacheFileChunk");
 
@@ -290,7 +290,7 @@ NS_INTERFACE_MAP_BEGIN(CacheFileChunk)
   NS_INTERFACE_MAP_ENTRY(nsISupports)
 NS_INTERFACE_MAP_END
 
-CacheFileChunk::CacheFileChunk(CacheFile *aFile, uint32_t aIndex,
+CacheFileChunk::CacheFileChunk(CacheFile* aFile, uint32_t aIndex,
                                bool aInitByWriter)
     : CacheMemoryConsumer(aFile->mOpenAsMemoryOnly ? MEMORY_ONLY : DONT_REPORT),
       mIndex(aIndex),
@@ -331,9 +331,9 @@ void CacheFileChunk::InitNew() {
   mState = READY;
 }
 
-nsresult CacheFileChunk::Read(CacheFileHandle *aHandle, uint32_t aLen,
+nsresult CacheFileChunk::Read(CacheFileHandle* aHandle, uint32_t aLen,
                               CacheHash::Hash16_t aHash,
-                              CacheFileChunkListener *aCallback) {
+                              CacheFileChunkListener* aCallback) {
   AssertOwnsLock();
 
   LOG(("CacheFileChunk::Read() [this=%p, handle=%p, len=%d, listener=%p]", this,
@@ -378,8 +378,8 @@ nsresult CacheFileChunk::Read(CacheFileHandle *aHandle, uint32_t aLen,
   return rv;
 }
 
-nsresult CacheFileChunk::Write(CacheFileHandle *aHandle,
-                               CacheFileChunkListener *aCallback) {
+nsresult CacheFileChunk::Write(CacheFileHandle* aHandle,
+                               CacheFileChunkListener* aCallback) {
   AssertOwnsLock();
 
   LOG(("CacheFileChunk::Write() [this=%p, handle=%p, listener=%p]", this,
@@ -395,7 +395,7 @@ nsresult CacheFileChunk::Write(CacheFileHandle *aHandle,
   nsresult rv;
 
   mState = WRITING;
-  mWritingStateHandle = new CacheFileChunkReadHandle(mBuf);
+  mWritingStateHandle = MakeUnique<CacheFileChunkReadHandle>(mBuf);
 
   rv = CacheFileIOManager::Write(
       aHandle, mIndex * kChunkSize, mWritingStateHandle->Buf(),
@@ -411,7 +411,7 @@ nsresult CacheFileChunk::Write(CacheFileHandle *aHandle,
   return rv;
 }
 
-void CacheFileChunk::WaitForUpdate(CacheFileChunkListener *aCallback) {
+void CacheFileChunk::WaitForUpdate(CacheFileChunkListener* aCallback) {
   AssertOwnsLock();
 
   LOG(("CacheFileChunk::WaitForUpdate() [this=%p, listener=%p]", this,
@@ -426,7 +426,7 @@ void CacheFileChunk::WaitForUpdate(CacheFileChunkListener *aCallback) {
   }
 #endif
 
-  ChunkListenerItem *item = new ChunkListenerItem();
+  ChunkListenerItem* item = new ChunkListenerItem();
   item->mTarget = CacheFileIOManager::IOTarget();
   if (!item->mTarget) {
     LOG(
@@ -441,7 +441,7 @@ void CacheFileChunk::WaitForUpdate(CacheFileChunkListener *aCallback) {
   mUpdateListeners.AppendElement(item);
 }
 
-nsresult CacheFileChunk::CancelWait(CacheFileChunkListener *aCallback) {
+void CacheFileChunk::CancelWait(CacheFileChunkListener* aCallback) {
   AssertOwnsLock();
 
   LOG(("CacheFileChunk::CancelWait() [this=%p, listener=%p]", this, aCallback));
@@ -450,7 +450,7 @@ nsresult CacheFileChunk::CancelWait(CacheFileChunkListener *aCallback) {
 
   uint32_t i;
   for (i = 0; i < mUpdateListeners.Length(); i++) {
-    ChunkListenerItem *item = mUpdateListeners[i];
+    ChunkListenerItem* item = mUpdateListeners[i];
 
     if (item->mCallback == aCallback) {
       mUpdateListeners.RemoveElementAt(i);
@@ -464,8 +464,6 @@ nsresult CacheFileChunk::CancelWait(CacheFileChunkListener *aCallback) {
     MOZ_ASSERT(mUpdateListeners[i]->mCallback != aCallback);
   }
 #endif
-
-  return NS_OK;
 }
 
 nsresult CacheFileChunk::NotifyUpdateListeners() {
@@ -479,7 +477,7 @@ nsresult CacheFileChunk::NotifyUpdateListeners() {
 
   rv = NS_OK;
   for (uint32_t i = 0; i < mUpdateListeners.Length(); i++) {
-    ChunkListenerItem *item = mUpdateListeners[i];
+    ChunkListenerItem* item = mUpdateListeners[i];
 
     LOG(
         ("CacheFileChunk::NotifyUpdateListeners() - Notifying listener %p "
@@ -550,7 +548,7 @@ void CacheFileChunk::UpdateDataSize(uint32_t aOffset, uint32_t aLen) {
   mValidityMap.Log();
 }
 
-nsresult CacheFileChunk::Truncate(uint32_t aOffset) {
+void CacheFileChunk::Truncate(uint32_t aOffset) {
   MOZ_RELEASE_ASSERT(mState == READY || mState == WRITING || mState == READING);
 
   if (mState == READING) {
@@ -558,17 +556,16 @@ nsresult CacheFileChunk::Truncate(uint32_t aOffset) {
   }
 
   mBuf->SetDataSize(aOffset);
-  return NS_OK;
 }
 
-nsresult CacheFileChunk::OnFileOpened(CacheFileHandle *aHandle,
+nsresult CacheFileChunk::OnFileOpened(CacheFileHandle* aHandle,
                                       nsresult aResult) {
   MOZ_CRASH("CacheFileChunk::OnFileOpened should not be called!");
   return NS_ERROR_UNEXPECTED;
 }
 
-nsresult CacheFileChunk::OnDataWritten(CacheFileHandle *aHandle,
-                                       const char *aBuf, nsresult aResult) {
+nsresult CacheFileChunk::OnDataWritten(CacheFileHandle* aHandle,
+                                       const char* aBuf, nsresult aResult) {
   LOG((
       "CacheFileChunk::OnDataWritten() [this=%p, handle=%p, result=0x%08" PRIx32
       "]",
@@ -597,7 +594,7 @@ nsresult CacheFileChunk::OnDataWritten(CacheFileHandle *aHandle,
   return NS_OK;
 }
 
-nsresult CacheFileChunk::OnDataRead(CacheFileHandle *aHandle, char *aBuf,
+nsresult CacheFileChunk::OnDataRead(CacheFileHandle* aHandle, char* aBuf,
                                     nsresult aResult) {
   LOG(("CacheFileChunk::OnDataRead() [this=%p, handle=%p, result=0x%08" PRIx32
        "]",
@@ -661,18 +658,18 @@ nsresult CacheFileChunk::OnDataRead(CacheFileHandle *aHandle, char *aBuf,
   return NS_OK;
 }
 
-nsresult CacheFileChunk::OnFileDoomed(CacheFileHandle *aHandle,
+nsresult CacheFileChunk::OnFileDoomed(CacheFileHandle* aHandle,
                                       nsresult aResult) {
   MOZ_CRASH("CacheFileChunk::OnFileDoomed should not be called!");
   return NS_ERROR_UNEXPECTED;
 }
 
-nsresult CacheFileChunk::OnEOFSet(CacheFileHandle *aHandle, nsresult aResult) {
+nsresult CacheFileChunk::OnEOFSet(CacheFileHandle* aHandle, nsresult aResult) {
   MOZ_CRASH("CacheFileChunk::OnEOFSet should not be called!");
   return NS_ERROR_UNEXPECTED;
 }
 
-nsresult CacheFileChunk::OnFileRenamed(CacheFileHandle *aHandle,
+nsresult CacheFileChunk::OnFileRenamed(CacheFileHandle* aHandle,
                                        nsresult aResult) {
   MOZ_CRASH("CacheFileChunk::OnFileRenamed should not be called!");
   return NS_ERROR_UNEXPECTED;
@@ -690,11 +687,7 @@ bool CacheFileChunk::IsDirty() const {
   return mIsDirty;
 }
 
-nsresult CacheFileChunk::GetStatus() {
-  AssertOwnsLock();
-
-  return mStatus;
-}
+nsresult CacheFileChunk::GetStatus() { return mStatus; }
 
 void CacheFileChunk::SetError(nsresult aStatus) {
   LOG(("CacheFileChunk::SetError() [this=%p, status=0x%08" PRIx32 "]", this,
@@ -796,12 +789,17 @@ bool CacheFileChunk::CanAllocate(uint32_t aSize) const {
 
   LOG(("CacheFileChunk::CanAllocate() [this=%p, size=%u]", this, aSize));
 
-  uint32_t limit = CacheObserver::MaxDiskChunksMemoryUsage(mIsPriority);
+  int64_t limit = CacheObserver::MaxDiskChunksMemoryUsage(mIsPriority);
   if (limit == 0) {
     return true;
   }
 
-  uint32_t usage = ChunksMemoryUsage();
+  limit <<= 10;
+  if (limit > UINT32_MAX) {
+    limit = UINT32_MAX;
+  }
+
+  int64_t usage = ChunksMemoryUsage();
   if (usage + aSize > limit) {
     LOG(("CacheFileChunk::CanAllocate() - Returning false. [this=%p]", this));
     return false;
@@ -831,7 +829,7 @@ void CacheFileChunk::BuffersAllocationChanged(uint32_t aFreed,
        static_cast<uint32_t>(ChunksMemoryUsage()), this));
 }
 
-mozilla::Atomic<uint32_t, ReleaseAcquire> &CacheFileChunk::ChunksMemoryUsage()
+mozilla::Atomic<uint32_t, ReleaseAcquire>& CacheFileChunk::ChunksMemoryUsage()
     const {
   static mozilla::Atomic<uint32_t, ReleaseAcquire> chunksMemoryUsage(0);
   static mozilla::Atomic<uint32_t, ReleaseAcquire> prioChunksMemoryUsage(0);
