@@ -10,12 +10,12 @@
 #include "prnetdb.h"
 #include "nsPrintfCString.h"
 
-#define LOGSHA1(x)                                          \
-  PR_htonl((reinterpret_cast<const uint32_t *>(x))[0]),     \
-      PR_htonl((reinterpret_cast<const uint32_t *>(x))[1]), \
-      PR_htonl((reinterpret_cast<const uint32_t *>(x))[2]), \
-      PR_htonl((reinterpret_cast<const uint32_t *>(x))[3]), \
-      PR_htonl((reinterpret_cast<const uint32_t *>(x))[4])
+#define LOGSHA1(x)                                         \
+  PR_htonl((reinterpret_cast<const uint32_t*>(x))[0]),     \
+      PR_htonl((reinterpret_cast<const uint32_t*>(x))[1]), \
+      PR_htonl((reinterpret_cast<const uint32_t*>(x))[2]), \
+      PR_htonl((reinterpret_cast<const uint32_t*>(x))[3]), \
+      PR_htonl((reinterpret_cast<const uint32_t*>(x))[4])
 
 #define SHA1STRING(x) \
   (nsPrintfCString("%08x%08x%08x%08x%08x", LOGSHA1(x)).get())
@@ -33,19 +33,19 @@ class CacheHash : public nsISupports {
   typedef uint16_t Hash16_t;
   typedef uint32_t Hash32_t;
 
-  static Hash32_t Hash(const char *aData, uint32_t aSize,
+  static Hash32_t Hash(const char* aData, uint32_t aSize,
                        uint32_t aInitval = 0);
-  static Hash16_t Hash16(const char *aData, uint32_t aSize,
+  static Hash16_t Hash16(const char* aData, uint32_t aSize,
                          uint32_t aInitval = 0);
 
   explicit CacheHash(uint32_t aInitval = 0);
 
-  void Update(const char *aData, uint32_t aLen);
+  void Update(const char* aData, uint32_t aLen);
   Hash32_t GetHash();
   Hash16_t GetHash16();
 
  private:
-  virtual ~CacheHash() {}
+  virtual ~CacheHash() = default;
 
   void Feed(uint32_t aVal, uint8_t aLen = 4);
 
@@ -59,7 +59,7 @@ class CacheHash : public nsISupports {
 
 typedef uint64_t OriginAttrsHash;
 
-OriginAttrsHash GetOriginAttrsHash(const mozilla::OriginAttributes &aOA);
+OriginAttrsHash GetOriginAttrsHash(const mozilla::OriginAttributes& aOA);
 
 }  // namespace net
 }  // namespace mozilla
