@@ -12,12 +12,15 @@
 #include "mozilla/Attributes.h"
 #include "mozilla/Mutex.h"
 #include "DNSRequestChild.h"
+#include "DNSRequestParent.h"
 #include "nsHashKeys.h"
 #include "nsClassHashtable.h"
 
 namespace mozilla {
 namespace net {
 
+class TRRServiceParent;
+
 class ChildDNSService final : public nsPIDNSService, public nsIObserver {
  public:
   // AsyncResolve (and CancelAsyncResolve) can be called off-main
@@ -30,34 +33,34 @@ class ChildDNSService final : public nsPIDNSService, public nsIObserver {
 
   static already_AddRefed<ChildDNSService> GetSingleton();
 
-  void NotifyRequestDone(DNSRequestChild* aDnsRequest);
-
-  bool GetOffline() const;
+  void NotifyRequestDone(DNSRequestSender* aDnsRequest);
 
  private:
   virtual ~ChildDNSService() = default;
 
   void MOZ_ALWAYS_INLINE GetDNSRecordHashKey(
-      const nsACString& aHost, uint16_t aType,
+      const nsACString& aHost, const nsACString& aTrrServer, uint16_t aType,
       const OriginAttributes& aOriginAttributes, uint32_t aFlags,
-      nsIDNSListener* aListener, nsACString& aHashKey);
-  nsresult AsyncResolveInternal(const nsACString& hostname, uint16_t type,
+      uintptr_t aListenerAddr, nsACString& aHashKey);
+  nsresult AsyncResolveInternal(const nsACString& hostname,
+                                const nsACString& aTrrServer, uint16_t type,
                                 uint32_t flags, nsIDNSListener* listener,
                                 nsIEventTarget* target_,
                                 const OriginAttributes& aOriginAttributes,
                                 nsICancelable** result);
   nsresult CancelAsyncResolveInternal(
-      const nsACString& aHostname, uint16_t aType, uint32_t aFlags,
-      nsIDNSListener* aListener, nsresult aReason,
+      const nsACString& aHostname, const nsACString& aTrrServer, uint16_t aType,
+      uint32_t aFlags, nsIDNSListener* aListener, nsresult aReason,
       const OriginAttributes& aOriginAttributes);
 
   bool mFirstTime;
   bool mDisablePrefetch;
 
   // We need to remember pending dns requests to be able to cancel them.
-  nsClassHashtable<nsCStringHashKey, nsTArray<RefPtr<DNSRequestChild>>>
+  nsClassHashtable<nsCStringHashKey, nsTArray<RefPtr<DNSRequestSender>>>
       mPendingRequests;
   Mutex mPendingRequestsLock;
+  RefPtr<TRRServiceParent> mTRRServiceParent;
 };
 
 }  // namespace net
