@@ -9,11 +9,17 @@
 
 #include "nsError.h"
 #include "nscore.h"
+#include "nsINativeDNSResolverOverride.h"
+#include "nsHashKeys.h"
+#include "nsDataHashtable.h"
+#include "mozilla/RWLock.h"
+#include "nsTArray.h"
+#include "prio.h"
 
 #if defined(XP_WIN)
-#define DNSQUERY_AVAILABLE 1
+#  define DNSQUERY_AVAILABLE 1
 #else
-#undef DNSQUERY_AVAILABLE
+#  undef DNSQUERY_AVAILABLE
 #endif
 
 namespace mozilla {
@@ -36,9 +42,8 @@ class AddrInfo;
  * @param aGetTtl[in] If true, the TTL will be retrieved if DNS provides the
  *     answers..
  */
-nsresult GetAddrInfo(const char* aHost, uint16_t aAddressFamily,
-                     uint16_t aFlags, const char* aNetworkInterface,
-                     AddrInfo** aAddrInfo, bool aGetTtl);
+nsresult GetAddrInfo(const nsACString& aHost, uint16_t aAddressFamily,
+                     uint16_t aFlags, AddrInfo** aAddrInfo, bool aGetTtl);
 
 /**
  * Initialize the GetAddrInfo module.
@@ -57,6 +62,25 @@ nsresult GetAddrInfoInit();
  */
 nsresult GetAddrInfoShutdown();
 
+class NativeDNSResolverOverride : public nsINativeDNSResolverOverride {
+  NS_DECL_THREADSAFE_ISUPPORTS
+  NS_DECL_NSINATIVEDNSRESOLVEROVERRIDE
+ public:
+  NativeDNSResolverOverride() : mLock("NativeDNSResolverOverride") {}
+
+  static already_AddRefed<nsINativeDNSResolverOverride> GetSingleton();
+
+ private:
+  virtual ~NativeDNSResolverOverride() = default;
+  mozilla::RWLock mLock;
+
+  nsDataHashtable<nsCStringHashKey, nsTArray<PRNetAddr>> mOverrides;
+  nsDataHashtable<nsCStringHashKey, nsCString> mCnames;
+
+  friend bool FindAddrOverride(const nsACString& aHost, uint16_t aAddressFamily,
+                               uint16_t aFlags, AddrInfo** aAddrInfo);
+};
+
 }  // namespace net
 }  // namespace mozilla
 
