@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -19,23 +19,22 @@
 #include "nsInputStreamPump.h"
 #include "CacheFileUtils.h"
 #include <algorithm>
-#include "nsIPipe.h"
 
 using namespace mozilla::net;
 
 #define HEXDUMP_MAX_ROWS 16
 
-static void HexDump(uint32_t *state, const char *buf, int32_t n,
-                    nsCString &result) {
+static void HexDump(uint32_t* state, const char* buf, int32_t n,
+                    nsCString& result) {
   char temp[16];
 
-  const unsigned char *p;
+  const unsigned char* p;
   while (n) {
     SprintfLiteral(temp, "%08x:  ", *state);
     result.Append(temp);
     *state += HEXDUMP_MAX_ROWS;
 
-    p = (const unsigned char *)buf;
+    p = (const unsigned char*)buf;
 
     int32_t i, row_max = std::min(HEXDUMP_MAX_ROWS, n);
 
@@ -49,7 +48,7 @@ static void HexDump(uint32_t *state, const char *buf, int32_t n,
     }
 
     // print ASCII glyphs if possible:
-    p = (const unsigned char *)buf;
+    p = (const unsigned char*)buf;
     for (i = 0; i < row_max; ++i, ++p) {
       switch (*p) {
         case '<':
@@ -89,8 +88,8 @@ NS_IMPL_ISUPPORTS(nsAboutCacheEntry::Channel, nsICacheEntryOpenCallback,
 // nsAboutCacheEntry::nsIAboutModule
 
 NS_IMETHODIMP
-nsAboutCacheEntry::NewChannel(nsIURI *uri, nsILoadInfo *aLoadInfo,
-                              nsIChannel **result) {
+nsAboutCacheEntry::NewChannel(nsIURI* uri, nsILoadInfo* aLoadInfo,
+                              nsIChannel** result) {
   NS_ENSURE_ARG_POINTER(uri);
   nsresult rv;
 
@@ -104,7 +103,7 @@ nsAboutCacheEntry::NewChannel(nsIURI *uri, nsILoadInfo *aLoadInfo,
 }
 
 NS_IMETHODIMP
-nsAboutCacheEntry::GetURIFlags(nsIURI *aURI, uint32_t *result) {
+nsAboutCacheEntry::GetURIFlags(nsIURI* aURI, uint32_t* result) {
   *result = nsIAboutModule::HIDE_FROM_ABOUTABOUT |
             nsIAboutModule::URI_SAFE_FOR_UNTRUSTED_CONTENT;
   return NS_OK;
@@ -113,7 +112,7 @@ nsAboutCacheEntry::GetURIFlags(nsIURI *aURI, uint32_t *result) {
 //-----------------------------------------------------------------------------
 // nsAboutCacheEntry::Channel
 
-nsresult nsAboutCacheEntry::Channel::Init(nsIURI *uri, nsILoadInfo *aLoadInfo) {
+nsresult nsAboutCacheEntry::Channel::Init(nsIURI* uri, nsILoadInfo* aLoadInfo) {
   nsresult rv;
 
   nsCOMPtr<nsIInputStream> stream;
@@ -128,8 +127,8 @@ nsresult nsAboutCacheEntry::Channel::Init(nsIURI *uri, nsILoadInfo *aLoadInfo) {
   return NS_OK;
 }
 
-nsresult nsAboutCacheEntry::Channel::GetContentStream(nsIURI *uri,
-                                                      nsIInputStream **result) {
+nsresult nsAboutCacheEntry::Channel::GetContentStream(nsIURI* uri,
+                                                      nsIInputStream** result) {
   nsresult rv;
 
   // Init: (block size, maximum length)
@@ -143,6 +142,8 @@ nsresult nsAboutCacheEntry::Channel::GetContentStream(nsIURI *uri,
       "<!DOCTYPE html>\n"
       "<html>\n"
       "<head>\n"
+      "  <meta http-equiv=\"Content-Security-Policy\" content=\"default-src "
+      "chrome:; object-src 'none'\" />\n"
       "  <title>Cache entry information</title>\n"
       "  <link rel=\"stylesheet\" "
       "href=\"chrome://global/skin/about.css\" type=\"text/css\"/>\n"
@@ -163,7 +164,7 @@ nsresult nsAboutCacheEntry::Channel::GetContentStream(nsIURI *uri,
   return NS_OK;
 }
 
-nsresult nsAboutCacheEntry::Channel::OpenCacheEntry(nsIURI *uri) {
+nsresult nsAboutCacheEntry::Channel::OpenCacheEntry(nsIURI* uri) {
   nsresult rv;
 
   rv = ParseURI(uri, mStorageName, getter_AddRefs(mLoadInfo), mEnhanceId,
@@ -190,11 +191,11 @@ nsresult nsAboutCacheEntry::Channel::OpenCacheEntry() {
   return NS_OK;
 }
 
-nsresult nsAboutCacheEntry::Channel::ParseURI(nsIURI *uri,
-                                              nsACString &storageName,
-                                              nsILoadContextInfo **loadInfo,
-                                              nsCString &enahnceID,
-                                              nsIURI **cacheUri) {
+nsresult nsAboutCacheEntry::Channel::ParseURI(nsIURI* uri,
+                                              nsACString& storageName,
+                                              nsILoadContextInfo** loadInfo,
+                                              nsCString& enahnceID,
+                                              nsIURI** cacheUri) {
   //
   // about:cache-entry?storage=[string]&contenxt=[string]&eid=[string]&uri=[string]
   //
@@ -259,15 +260,15 @@ nsresult nsAboutCacheEntry::Channel::ParseURI(nsIURI *uri,
 
 NS_IMETHODIMP
 nsAboutCacheEntry::Channel::OnCacheEntryCheck(
-    nsICacheEntry *aEntry, nsIApplicationCache *aApplicationCache,
-    uint32_t *result) {
+    nsICacheEntry* aEntry, nsIApplicationCache* aApplicationCache,
+    uint32_t* result) {
   *result = nsICacheEntryOpenCallback::ENTRY_WANTED;
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsAboutCacheEntry::Channel::OnCacheEntryAvailable(
-    nsICacheEntry *entry, bool isNew, nsIApplicationCache *aApplicationCache,
+    nsICacheEntry* entry, bool isNew, nsIApplicationCache* aApplicationCache,
     nsresult status) {
   nsresult rv;
 
@@ -287,9 +288,9 @@ nsAboutCacheEntry::Channel::OnCacheEntryAvailable(
   return NS_OK;
 }
 
-  //-----------------------------------------------------------------------------
-  // Print-out helper methods
-  //-----------------------------------------------------------------------------
+//-----------------------------------------------------------------------------
+// Print-out helper methods
+//-----------------------------------------------------------------------------
 
 #define APPEND_ROW(label, value) \
   PR_BEGIN_MACRO                 \
@@ -307,9 +308,12 @@ nsAboutCacheEntry::Channel::OnCacheEntryAvailable(
   PR_END_MACRO
 
 nsresult nsAboutCacheEntry::Channel::WriteCacheEntryDescription(
-    nsICacheEntry *entry) {
+    nsICacheEntry* entry) {
   nsresult rv;
-  nsCString buffer;
+  // This method appears to run in a situation where the run-time stack
+  // should have plenty of space, so allocating a large string on the
+  // stack is OK.
+  nsAutoCStringN<4097> buffer;
   uint32_t n;
 
   nsAutoCString str;
@@ -317,7 +321,6 @@ nsresult nsAboutCacheEntry::Channel::WriteCacheEntryDescription(
   rv = entry->GetKey(str);
   if (NS_FAILED(rv)) return rv;
 
-  buffer.SetCapacity(4096);
   buffer.AssignLiteral(
       "<table>\n"
       "  <tr>\n"
@@ -326,19 +329,15 @@ nsresult nsAboutCacheEntry::Channel::WriteCacheEntryDescription(
 
   // Test if the key is actually a URI
   nsCOMPtr<nsIURI> uri;
-  bool isJS = false;
-  bool isData = false;
-
   rv = NS_NewURI(getter_AddRefs(uri), str);
-  // javascript: and data: URLs should not be linkified
-  // since clicking them can cause scripts to run - bug 162584
-  if (NS_SUCCEEDED(rv)) {
-    uri->SchemeIs("javascript", &isJS);
-    uri->SchemeIs("data", &isData);
-  }
+
   nsAutoCString escapedStr;
   nsAppendEscapedHTML(str, escapedStr);
-  if (NS_SUCCEEDED(rv) && !(isJS || isData)) {
+
+  // javascript: and data: URLs should not be linkified
+  // since clicking them can cause scripts to run - bug 162584
+  if (NS_SUCCEEDED(rv) &&
+      !(uri->SchemeIs("javascript") || uri->SchemeIs("data"))) {
     buffer.AppendLiteral("<a href=\"");
     buffer.Append(escapedStr);
     buffer.AppendLiteral("\">");
@@ -478,8 +477,8 @@ nsresult nsAboutCacheEntry::Channel::WriteCacheEntryUnavailable() {
 //-----------------------------------------------------------------------------
 
 NS_IMETHODIMP
-nsAboutCacheEntry::Channel::OnMetaDataElement(char const *key,
-                                              char const *value) {
+nsAboutCacheEntry::Channel::OnMetaDataElement(char const* key,
+                                              char const* value) {
   mBuffer->AppendLiteral(
       "  <tr>\n"
       "    <th>");
@@ -500,8 +499,7 @@ nsAboutCacheEntry::Channel::OnMetaDataElement(char const *key,
 //-----------------------------------------------------------------------------
 
 NS_IMETHODIMP
-nsAboutCacheEntry::Channel::OnStartRequest(nsIRequest *request,
-                                           nsISupports *ctx) {
+nsAboutCacheEntry::Channel::OnStartRequest(nsIRequest* request) {
   mHexDumpState = 0;
 
   NS_NAMED_LITERAL_CSTRING(buffer, "<hr/>\n<pre>");
@@ -510,20 +508,20 @@ nsAboutCacheEntry::Channel::OnStartRequest(nsIRequest *request,
 }
 
 NS_IMETHODIMP
-nsAboutCacheEntry::Channel::OnDataAvailable(nsIRequest *request,
-                                            nsISupports *ctx,
-                                            nsIInputStream *aInputStream,
+nsAboutCacheEntry::Channel::OnDataAvailable(nsIRequest* request,
+                                            nsIInputStream* aInputStream,
                                             uint64_t aOffset, uint32_t aCount) {
   uint32_t n;
   return aInputStream->ReadSegments(&nsAboutCacheEntry::Channel::PrintCacheData,
                                     this, aCount, &n);
 }
 
-/* static */ nsresult nsAboutCacheEntry::Channel::PrintCacheData(
-    nsIInputStream *aInStream, void *aClosure, const char *aFromSegment,
-    uint32_t aToOffset, uint32_t aCount, uint32_t *aWriteCount) {
-  nsAboutCacheEntry::Channel *a =
-      static_cast<nsAboutCacheEntry::Channel *>(aClosure);
+/* static */
+nsresult nsAboutCacheEntry::Channel::PrintCacheData(
+    nsIInputStream* aInStream, void* aClosure, const char* aFromSegment,
+    uint32_t aToOffset, uint32_t aCount, uint32_t* aWriteCount) {
+  nsAboutCacheEntry::Channel* a =
+      static_cast<nsAboutCacheEntry::Channel*>(aClosure);
 
   nsCString buffer;
   HexDump(&a->mHexDumpState, aFromSegment, aCount, buffer);
@@ -537,7 +535,7 @@ nsAboutCacheEntry::Channel::OnDataAvailable(nsIRequest *request,
 }
 
 NS_IMETHODIMP
-nsAboutCacheEntry::Channel::OnStopRequest(nsIRequest *request, nsISupports *ctx,
+nsAboutCacheEntry::Channel::OnStopRequest(nsIRequest* request,
                                           nsresult result) {
   NS_NAMED_LITERAL_CSTRING(buffer, "</pre>\n");
   uint32_t n;
