@@ -18,23 +18,23 @@ namespace net {
 
 class ASpdySession : public nsAHttpTransaction {
  public:
-  ASpdySession();
-  virtual ~ASpdySession();
+  ASpdySession() = default;
+  virtual ~ASpdySession() = default;
 
-  virtual MOZ_MUST_USE bool AddStream(nsAHttpTransaction *, int32_t, bool,
-                                      nsIInterfaceRequestor *) = 0;
+  [[nodiscard]] virtual bool AddStream(nsAHttpTransaction*, int32_t, bool, bool,
+                                       nsIInterfaceRequestor*) = 0;
   virtual bool CanReuse() = 0;
   virtual bool RoomForMoreStreams() = 0;
   virtual PRIntervalTime IdleTime() = 0;
   virtual uint32_t ReadTimeoutTick(PRIntervalTime now) = 0;
   virtual void DontReuse() = 0;
-  virtual uint32_t SpdyVersion() = 0;
+  virtual enum SpdyVersion SpdyVersion() = 0;
 
-  static ASpdySession *NewSpdySession(uint32_t version, nsISocketTransport *,
-                                      bool);
+  static ASpdySession* NewSpdySession(net::SpdyVersion version,
+                                      nsISocketTransport*, bool);
 
-  virtual bool TestJoinConnection(const nsACString &hostname, int32_t port) = 0;
-  virtual bool JoinConnection(const nsACString &hostname, int32_t port) = 0;
+  virtual bool TestJoinConnection(const nsACString& hostname, int32_t port) = 0;
+  virtual bool JoinConnection(const nsACString& hostname, int32_t port) = 0;
 
   // MaybeReTunnel() is called by the connection manager when it cannot
   // dispatch a tunneled transaction. That might be because the tunnels it
@@ -43,9 +43,9 @@ class ASpdySession : public nsAHttpTransaction {
   //
   // return true if the session takes back ownership of the transaction from
   // the connection manager.
-  virtual bool MaybeReTunnel(nsAHttpTransaction *) = 0;
+  virtual bool MaybeReTunnel(nsAHttpTransaction*) = 0;
 
-  virtual void PrintDiagnostics(nsCString &log) = 0;
+  virtual void PrintDiagnostics(nsCString& log) = 0;
 
   bool ResponseTimeoutEnabled() const final { return true; }
 
@@ -63,7 +63,7 @@ class ASpdySession : public nsAHttpTransaction {
   // soft errors are errors that terminate a stream without terminating the
   // connection. In general non-network errors are stream errors as well
   // as network specific items like cancels.
-  bool SoftStreamError(nsresult code) {
+  static bool SoftStreamError(nsresult code) {
     if (NS_SUCCEEDED(code) || code == NS_BASE_STREAM_WOULD_BLOCK) {
       return false;
     }
@@ -85,9 +85,12 @@ class ASpdySession : public nsAHttpTransaction {
             code == NS_BINDING_RETARGETED ||
             code == NS_ERROR_CORRUPTED_CONTENT);
   }
+
+  virtual void SetCleanShutdown(bool) = 0;
+  virtual bool CanAcceptWebsocket() = 0;
 };
 
-typedef bool (*ALPNCallback)(nsISupports *);  // nsISSLSocketControl is typical
+typedef bool (*ALPNCallback)(nsISupports*);  // nsISSLSocketControl is typical
 
 // this is essentially a single instantiation as a member of nsHttpHandler.
 // It could be all static except using static ctors of XPCOM objects is a
@@ -95,19 +98,19 @@ typedef bool (*ALPNCallback)(nsISupports *);  // nsISSLSocketControl is typical
 class SpdyInformation {
  public:
   SpdyInformation();
-  ~SpdyInformation() {}
+  ~SpdyInformation() = default;
 
   static const uint32_t kCount = 1;
 
   // determine the index (0..kCount-1) of the spdy information that
   // correlates to the npn string. NS_FAILED() if no match is found.
-  MOZ_MUST_USE nsresult GetNPNIndex(const nsACString &npnString,
-                                    uint32_t *result) const;
+  [[nodiscard]] nsresult GetNPNIndex(const nsACString& npnString,
+                                     uint32_t* result) const;
 
   // determine if a version of the protocol is enabled for index < kCount
   bool ProtocolEnabled(uint32_t index) const;
 
-  uint8_t Version[kCount];          // telemetry enum e.g. SPDY_VERSION_31
+  SpdyVersion Version[kCount];      // telemetry enum e.g. SPDY_VERSION_31
   nsCString VersionString[kCount];  // npn string e.g. "spdy/3.1"
 
   // the ALPNCallback function allows the protocol stack to decide whether or
