@@ -16,7 +16,6 @@
 #include "mozilla/TimeStamp.h"
 #include "mozilla/UniquePtr.h"
 #include "nsHttpRequestHead.h"
-#include "nsILoadGroup.h"
 #include "nsIRequestContext.h"
 #include "nsString.h"
 #include "PSpdyPush.h"
@@ -28,32 +27,32 @@ class Http2PushTransactionBuffer;
 
 class Http2PushedStream final : public Http2Stream {
  public:
-  Http2PushedStream(Http2PushTransactionBuffer *aTransaction,
-                    Http2Session *aSession, Http2Stream *aAssociatedStream,
+  Http2PushedStream(Http2PushTransactionBuffer* aTransaction,
+                    Http2Session* aSession, Http2Stream* aAssociatedStream,
                     uint32_t aID,
                     uint64_t aCurrentForegroundTabOuterContentWindowId);
-  virtual ~Http2PushedStream() {}
+  virtual ~Http2PushedStream() = default;
 
   bool GetPushComplete();
 
   // The consumer stream is the synthetic pull stream hooked up to this push
-  virtual Http2Stream *GetConsumerStream() override { return mConsumerStream; };
+  virtual Http2Stream* GetConsumerStream() override { return mConsumerStream; };
 
-  void SetConsumerStream(Http2Stream *aStream);
-  MOZ_MUST_USE bool GetHashKey(nsCString &key);
+  void SetConsumerStream(Http2Stream* aStream);
+  [[nodiscard]] bool GetHashKey(nsCString& key);
 
   // override of Http2Stream
-  MOZ_MUST_USE nsresult ReadSegments(nsAHttpSegmentReader *, uint32_t,
-                                     uint32_t *) override;
-  MOZ_MUST_USE nsresult WriteSegments(nsAHttpSegmentWriter *, uint32_t,
-                                      uint32_t *) override;
+  [[nodiscard]] nsresult ReadSegments(nsAHttpSegmentReader*, uint32_t,
+                                      uint32_t*) override;
+  [[nodiscard]] nsresult WriteSegments(nsAHttpSegmentWriter*, uint32_t,
+                                       uint32_t*) override;
   void AdjustInitialWindow() override;
 
-  nsIRequestContext *RequestContext() override { return mRequestContext; };
-  void ConnectPushedStream(Http2Stream *consumer);
+  nsIRequestContext* RequestContext() override { return mRequestContext; };
+  void ConnectPushedStream(Http2Stream* consumer);
 
-  MOZ_MUST_USE bool TryOnPush();
-  static MOZ_MUST_USE bool TestOnPush(Http2Stream *consumer);
+  [[nodiscard]] bool TryOnPush();
+  [[nodiscard]] static bool TestOnPush(Http2Stream* consumer);
 
   virtual bool DeferCleanup(nsresult status) override;
   void SetDeferCleanupOnSuccess(bool val) { mDeferCleanupOnSuccess = val; }
@@ -64,25 +63,27 @@ class Http2PushedStream final : public Http2Stream {
     mOnPushFailed = true;
   }
 
-  MOZ_MUST_USE nsresult GetBufferedData(char *buf, uint32_t count,
-                                        uint32_t *countWritten);
+  [[nodiscard]] nsresult GetBufferedData(char* buf, uint32_t count,
+                                         uint32_t* countWritten);
 
   // overload of Http2Stream
   virtual bool HasSink() override { return !!mConsumerStream; }
   virtual void SetPushComplete() override { mPushCompleted = true; }
+  virtual void TopLevelOuterContentWindowIdChanged(uint64_t) override;
 
-  nsCString &GetRequestString() { return mRequestString; }
+  nsCString& GetRequestString() { return mRequestString; }
+  nsCString& GetResourceUrl() { return mResourceUrl; }
 
  private:
-  Http2Stream
-      *mConsumerStream;  // paired request stream that consumes from
-                         // real http/2 one.. null until a match is made.
+  Http2Stream*
+      mConsumerStream;  // paired request stream that consumes from
+                        // real http/2 one.. null until a match is made.
 
   nsCOMPtr<nsIRequestContext> mRequestContext;
 
-  nsAHttpTransaction *mAssociatedTransaction;
+  nsAHttpTransaction* mAssociatedTransaction;
 
-  Http2PushTransactionBuffer *mBufferedPush;
+  Http2PushTransactionBuffer* mBufferedPush;
   mozilla::TimeStamp mLastRead;
 
   nsCString mHashKey;
@@ -99,6 +100,9 @@ class Http2PushedStream final : public Http2Stream {
   bool mDeferCleanupOnPush;
   bool mOnPushFailed;
   nsCString mRequestString;
+  nsCString mResourceUrl;
+
+  uint32_t mDefaultPriorityDependency;
 };
 
 class Http2PushTransactionBuffer final : public nsAHttpTransaction {
@@ -108,9 +112,9 @@ class Http2PushTransactionBuffer final : public nsAHttpTransaction {
 
   Http2PushTransactionBuffer();
 
-  MOZ_MUST_USE nsresult GetBufferedData(char *buf, uint32_t count,
-                                        uint32_t *countWritten);
-  void SetPushStream(Http2PushedStream *stream) { mPushStream = stream; }
+  [[nodiscard]] nsresult GetBufferedData(char* buf, uint32_t count,
+                                         uint32_t* countWritten);
+  void SetPushStream(Http2PushedStream* stream) { mPushStream = stream; }
 
  private:
   virtual ~Http2PushTransactionBuffer();
@@ -119,8 +123,8 @@ class Http2PushTransactionBuffer final : public nsAHttpTransaction {
   const static uint32_t kDefaultBufferSize = 4096;
 
   nsresult mStatus;
-  nsHttpRequestHead *mRequestHead;
-  Http2PushedStream *mPushStream;
+  nsHttpRequestHead* mRequestHead;
+  Http2PushedStream* mPushStream;
   bool mIsDone;
 
   UniquePtr<char[]> mBufferedHTTP1;
@@ -129,6 +133,28 @@ class Http2PushTransactionBuffer final : public nsAHttpTransaction {
   uint32_t mBufferedHTTP1Consumed;
 };
 
+class Http2PushedStreamWrapper : public nsISupports {
+ public:
+  NS_DECL_THREADSAFE_ISUPPORTS
+  bool DispatchRelease();
+
+  explicit Http2PushedStreamWrapper(Http2PushedStream* aPushStream);
+
+  nsCString& GetRequestString() { return mRequestString; }
+  nsCString& GetResourceUrl() { return mResourceUrl; }
+  Http2PushedStream* GetStream();
+  void OnPushFailed();
+  uint32_t StreamID() { return mStreamID; }
+
+ private:
+  virtual ~Http2PushedStreamWrapper();
+
+  nsCString mRequestString;
+  nsCString mResourceUrl;
+  uint32_t mStreamID;
+  WeakPtr<Http2Stream> mStream;
+};
+
 }  // namespace net
 }  // namespace mozilla
 
