@@ -6,6 +6,7 @@
 #define nsHttpNTLMAuth_h__
 
 #include "nsIHttpAuthenticator.h"
+#include "mozilla/StaticPtr.h"
 
 namespace mozilla {
 namespace net {
@@ -15,14 +16,18 @@ class nsHttpNTLMAuth : public nsIHttpAuthenticator {
   NS_DECL_ISUPPORTS
   NS_DECL_NSIHTTPAUTHENTICATOR
 
-  nsHttpNTLMAuth() {}
+  nsHttpNTLMAuth() : mUseNative(false) {}
+
+  static already_AddRefed<nsIHttpAuthenticator> GetOrCreate();
 
  private:
-  virtual ~nsHttpNTLMAuth() {}
+  virtual ~nsHttpNTLMAuth() = default;
 
   // This flag indicates whether we are using the native NTLM implementation
   // or the internal one.
   bool mUseNative;
+
+  static StaticRefPtr<nsHttpNTLMAuth> gSingleton;
 };
 
 }  // namespace net
