@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -8,6 +8,7 @@
 
 #include "nsHttp.h"
 #include "nsAHttpTransaction.h"
+#include "HttpTransactionShell.h"
 #include "nsAHttpConnection.h"
 #include "EventTokenBucket.h"
 #include "nsCOMPtr.h"
@@ -16,8 +17,8 @@
 #include "TimingStruct.h"
 #include "Http2Push.h"
 #include "mozilla/net/DNS.h"
+#include "mozilla/net/NeckoChannelParams.h"
 #include "ARefBase.h"
-#include "AlternateServices.h"
 
 //-----------------------------------------------------------------------------
 
@@ -34,6 +35,8 @@ class nsHttpChunkedDecoder;
 class nsHttpHeaderArray;
 class nsHttpRequestHead;
 class nsHttpResponseHead;
+class NullHttpTransaction;
+class SpdyConnectTransaction;
 
 //-----------------------------------------------------------------------------
 // nsHttpTransaction represents a single HTTP transaction.  It is thread-safe,
@@ -41,6 +44,7 @@ class nsHttpResponseHead;
 //-----------------------------------------------------------------------------
 
 class nsHttpTransaction final : public nsAHttpTransaction,
+                                public HttpTransactionShell,
                                 public ATokenBucketEvent,
                                 public nsIInputStreamCallback,
                                 public nsIOutputStreamCallback,
@@ -48,85 +52,33 @@ class nsHttpTransaction final : public nsAHttpTransaction,
  public:
   NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSAHTTPTRANSACTION
+  NS_DECL_HTTPTRANSACTIONSHELL
   NS_DECL_NSIINPUTSTREAMCALLBACK
   NS_DECL_NSIOUTPUTSTREAMCALLBACK
 
   nsHttpTransaction();
 
-  //
-  // called to initialize the transaction
-  //
-  // @param caps
-  //        the transaction capabilities (see nsHttp.h)
-  // @param connInfo
-  //        the connection type for this transaction.
-  // @param reqHeaders
-  //        the request header struct
-  // @param reqBody
-  //        the request body (POST or PUT data stream)
-  // @param reqBodyIncludesHeaders
-  //        fun stuff to support NPAPI plugins.
-  // @param target
-  //        the dispatch target were notifications should be sent.
-  // @param callbacks
-  //        the notification callbacks to be given to PSM.
-  // @param topLevelOuterContentWindowId
-  //        indicate the top level outer content window in which
-  //        this transaction is being loaded.
-  // @param responseBody
-  //        the input stream that will contain the response data.  async
-  //        wait on this input stream for data.  on first notification,
-  //        headers should be available (check transaction status).
-  //
-  MOZ_MUST_USE nsresult Init(uint32_t caps, nsHttpConnectionInfo *connInfo,
-                             nsHttpRequestHead *reqHeaders,
-                             nsIInputStream *reqBody, uint64_t reqContentLength,
-                             bool reqBodyIncludesHeaders,
-                             nsIEventTarget *consumerTarget,
-                             nsIInterfaceRequestor *callbacks,
-                             nsITransportEventSink *eventsink,
-                             uint64_t topLevelOuterContentWindowId,
-                             nsIAsyncInputStream **responseBody);
-
   void OnActivated() override;
 
   // attributes
-  nsHttpResponseHead *ResponseHead() {
+  nsHttpResponseHead* ResponseHead() {
     return mHaveAllHeaders ? mResponseHead : nullptr;
   }
-  nsISupports *SecurityInfo() { return mSecurityInfo; }
 
-  nsIEventTarget *ConsumerTarget() { return mConsumerTarget; }
-  nsISupports *HttpChannel() { return mChannel; }
-
-  void SetSecurityCallbacks(nsIInterfaceRequestor *aCallbacks);
-
-  // Called to take ownership of the response headers; the transaction
-  // will drop any reference to the response headers after this call.
-  nsHttpResponseHead *TakeResponseHead();
-
-  // Called to take ownership of the trailer headers.
-  // Returning null if there is no trailer.
-  nsHttpHeaderArray *TakeResponseTrailers();
-
-  // Provides a thread safe reference of the connection
-  // nsHttpTransaction::Connection should only be used on the socket thread
-  already_AddRefed<nsAHttpConnection> GetConnectionReference();
+  nsIEventTarget* ConsumerTarget() { return mConsumerTarget; }
 
   // Called to set/find out if the transaction generated a complete response.
-  bool ResponseIsComplete() { return mResponseIsComplete; }
   void SetResponseIsComplete() { mResponseIsComplete = true; }
 
-  bool ProxyConnectFailed() { return mProxyConnectFailed; }
-
   void EnableKeepAlive() { mCaps |= NS_HTTP_ALLOW_KEEPALIVE; }
   void MakeSticky() { mCaps |= NS_HTTP_STICKY_CONNECTION; }
+  void MakeNonSticky() override { mCaps &= ~NS_HTTP_STICKY_CONNECTION; }
 
   // SetPriority() may only be used by the connection manager.
   void SetPriority(int32_t priority) { mPriority = priority; }
   int32_t Priority() { return mPriority; }
 
-  void PrintDiagnostics(nsCString &log);
+  void PrintDiagnostics(nsCString& log);
 
   // Sets mPendingTime to the current time stamp or to a null time stamp (if now
   // is false)
@@ -136,59 +88,42 @@ class nsHttpTransaction final : public nsAHttpTransaction,
   const TimeStamp GetPendingTime() { return mPendingTime; }
 
   // overload of nsAHttpTransaction::RequestContext()
-  nsIRequestContext *RequestContext() override { return mRequestContext.get(); }
-  void SetRequestContext(nsIRequestContext *aRequestContext);
+  nsIRequestContext* RequestContext() override { return mRequestContext.get(); }
   void DispatchedAsBlocking();
   void RemoveDispatchedAsBlocking();
 
-  nsHttpTransaction *QueryHttpTransaction() override { return this; }
+  void DisableSpdy() override;
 
-  Http2PushedStream *GetPushedStream() { return mPushedStream; }
-  Http2PushedStream *TakePushedStream() {
-    Http2PushedStream *r = mPushedStream;
-    mPushedStream = nullptr;
-    return r;
+  nsHttpTransaction* QueryHttpTransaction() override { return this; }
+
+  already_AddRefed<Http2PushedStreamWrapper> GetPushedStream() {
+    return do_AddRef(mPushedStream);
   }
-  void SetPushedStream(Http2PushedStream *push) { mPushedStream = push; }
+  already_AddRefed<Http2PushedStreamWrapper> TakePushedStream() {
+    return mPushedStream.forget();
+  }
+
   uint32_t InitialRwin() const { return mInitialRwin; };
   bool ChannelPipeFull() { return mWaitingOnPipeOut; }
 
   // Locked methods to get and set timing info
-  const TimingStruct Timings();
   void BootstrapTimings(TimingStruct times);
-  void SetDomainLookupStart(mozilla::TimeStamp timeStamp,
-                            bool onlyIfNull = false);
-  void SetDomainLookupEnd(mozilla::TimeStamp timeStamp,
-                          bool onlyIfNull = false);
   void SetConnectStart(mozilla::TimeStamp timeStamp, bool onlyIfNull = false);
   void SetConnectEnd(mozilla::TimeStamp timeStamp, bool onlyIfNull = false);
   void SetRequestStart(mozilla::TimeStamp timeStamp, bool onlyIfNull = false);
   void SetResponseStart(mozilla::TimeStamp timeStamp, bool onlyIfNull = false);
   void SetResponseEnd(mozilla::TimeStamp timeStamp, bool onlyIfNull = false);
 
-  mozilla::TimeStamp GetDomainLookupStart();
-  mozilla::TimeStamp GetDomainLookupEnd();
-  mozilla::TimeStamp GetConnectStart();
-  mozilla::TimeStamp GetTcpConnectEnd();
-  mozilla::TimeStamp GetSecureConnectionStart();
-
-  mozilla::TimeStamp GetConnectEnd();
-  mozilla::TimeStamp GetRequestStart();
-  mozilla::TimeStamp GetResponseStart();
-  mozilla::TimeStamp GetResponseEnd();
-
-  int64_t GetTransferSize() { return mTransferSize; }
-
-  MOZ_MUST_USE bool Do0RTT() override;
-  MOZ_MUST_USE nsresult Finish0RTT(bool aRestart,
-                                   bool aAlpnChanged /* ignored */) override;
+  [[nodiscard]] bool Do0RTT() override;
+  [[nodiscard]] nsresult Finish0RTT(bool aRestart,
+                                    bool aAlpnChanged /* ignored */) override;
 
   // After Finish0RTT early data may have failed but the caller did not request
   // restart - this indicates that state for dev tools
   void Refused0RTT();
 
-  MOZ_MUST_USE bool CanDo0RTT() override;
-  MOZ_MUST_USE nsresult RestartOnFastOpenError() override;
+  [[nodiscard]] bool CanDo0RTT() override;
+  [[nodiscard]] nsresult RestartOnFastOpenError() override;
 
   uint64_t TopLevelOuterContentWindowId() override {
     return mTopLevelOuterContentWindowId;
@@ -196,37 +131,46 @@ class nsHttpTransaction final : public nsAHttpTransaction,
 
   void SetFastOpenStatus(uint8_t aStatus) override;
 
-  void SetHttpTrailers(nsCString &aTrailers);
+  void SetHttpTrailers(nsCString& aTrailers);
+
+  bool IsWebsocketUpgrade();
+  void SetH2WSTransaction(SpdyConnectTransaction*);
+
+  void OnProxyConnectComplete(int32_t aResponseCode) override;
+  void SetFlat407Headers(const nsACString& aHeaders);
+
+  // This is only called by Http2PushedStream::TryOnPush when a new pushed
+  // stream is available. The newly added stream will be taken by another
+  // transaction.
+  void OnPush(Http2PushedStreamWrapper* aStream);
 
  private:
   friend class DeleteHttpTransaction;
   virtual ~nsHttpTransaction();
 
-  MOZ_MUST_USE nsresult Restart();
-  char *LocateHttpStart(char *buf, uint32_t len, bool aAllowPartialMatch);
-  MOZ_MUST_USE nsresult ParseLine(nsACString &line);
-  MOZ_MUST_USE nsresult ParseLineSegment(char *seg, uint32_t len);
-  MOZ_MUST_USE nsresult ParseHead(char *, uint32_t count, uint32_t *countRead);
-  MOZ_MUST_USE nsresult HandleContentStart();
-  MOZ_MUST_USE nsresult HandleContent(char *, uint32_t count,
-                                      uint32_t *contentRead,
-                                      uint32_t *contentRemaining);
-  MOZ_MUST_USE nsresult ProcessData(char *, uint32_t, uint32_t *);
+  [[nodiscard]] nsresult Restart();
+  char* LocateHttpStart(char* buf, uint32_t len, bool aAllowPartialMatch);
+  [[nodiscard]] nsresult ParseLine(nsACString& line);
+  [[nodiscard]] nsresult ParseLineSegment(char* seg, uint32_t len);
+  [[nodiscard]] nsresult ParseHead(char*, uint32_t count, uint32_t* countRead);
+  [[nodiscard]] nsresult HandleContentStart();
+  [[nodiscard]] nsresult HandleContent(char*, uint32_t count,
+                                       uint32_t* contentRead,
+                                       uint32_t* contentRemaining);
+  [[nodiscard]] nsresult ProcessData(char*, uint32_t, uint32_t*);
   void DeleteSelfOnConsumerThread();
   void ReleaseBlockingTransaction();
 
-  static MOZ_MUST_USE nsresult ReadRequestSegment(nsIInputStream *, void *,
-                                                  const char *, uint32_t,
-                                                  uint32_t, uint32_t *);
-  static MOZ_MUST_USE nsresult WritePipeSegment(nsIOutputStream *, void *,
-                                                char *, uint32_t, uint32_t,
-                                                uint32_t *);
+  [[nodiscard]] static nsresult ReadRequestSegment(nsIInputStream*, void*,
+                                                   const char*, uint32_t,
+                                                   uint32_t, uint32_t*);
+  [[nodiscard]] static nsresult WritePipeSegment(nsIOutputStream*, void*, char*,
+                                                 uint32_t, uint32_t, uint32_t*);
 
   bool TimingEnabled() const { return mCaps & NS_HTTP_TIMING_ENABLED; }
 
   bool ResponseTimeoutEnabled() const final;
 
-  void DisableSpdy() override;
   void ReuseConnectionOnRestartOK(bool reuseOk) override {
     mReuseOnRestart = reuseOk;
   }
@@ -237,18 +181,23 @@ class nsHttpTransaction final : public nsAHttpTransaction,
   // is set in mCaps. We need the sticky flag be set early to keep the
   // connection from very start of the authentication process.
   void CheckForStickyAuthScheme();
-  void CheckForStickyAuthSchemeAt(nsHttpAtom const &header);
+  void CheckForStickyAuthSchemeAt(nsHttpAtom const& header);
 
   // Called from WriteSegments.  Checks for conditions whether to throttle
   // reading the content.  When this returns true, WriteSegments returns
   // WOULD_BLOCK.
   bool ShouldThrottle();
 
+  void NotifyTransactionObserver(nsresult reason);
+
+  already_AddRefed<Http2PushedStreamWrapper> TakePushedStreamById(
+      uint32_t aStreamId);
+
  private:
   class UpdateSecurityCallbacks : public Runnable {
    public:
-    UpdateSecurityCallbacks(nsHttpTransaction *aTrans,
-                            nsIInterfaceRequestor *aCallbacks)
+    UpdateSecurityCallbacks(nsHttpTransaction* aTrans,
+                            nsIInterfaceRequestor* aCallbacks)
         : Runnable("net::nsHttpTransaction::UpdateSecurityCallbacks"),
           mTrans(aTrans),
           mCallbacks(aCallbacks) {}
@@ -274,7 +223,7 @@ class nsHttpTransaction final : public nsAHttpTransaction,
   nsCOMPtr<nsIAsyncOutputStream> mPipeOut;
   nsCOMPtr<nsIRequestContext> mRequestContext;
 
-  nsCOMPtr<nsISupports> mChannel;
+  uint64_t mChannelId;
   nsCOMPtr<nsIHttpActivityObserver> mActivityDistributor;
 
   nsCString mReqHeaderBuf;  // flattened request headers
@@ -283,11 +232,11 @@ class nsHttpTransaction final : public nsAHttpTransaction,
 
   RefPtr<nsAHttpConnection> mConnection;
   RefPtr<nsHttpConnectionInfo> mConnInfo;
-  nsHttpRequestHead *mRequestHead;    // weak ref
-  nsHttpResponseHead *mResponseHead;  // owning pointer
+  nsHttpRequestHead* mRequestHead;    // weak ref
+  nsHttpResponseHead* mResponseHead;  // owning pointer
 
-  nsAHttpSegmentReader *mReader;
-  nsAHttpSegmentWriter *mWriter;
+  nsAHttpSegmentReader* mReader;
+  nsAHttpSegmentWriter* mWriter;
 
   nsCString mLineBuf;  // may contain a partial line
 
@@ -302,10 +251,10 @@ class nsHttpTransaction final : public nsAHttpTransaction,
   // so far been skipped.
   uint32_t mInvalidResponseBytesRead;
 
-  Http2PushedStream *mPushedStream;
+  RefPtr<Http2PushedStreamWrapper> mPushedStream;
   uint32_t mInitialRwin;
 
-  nsHttpChunkedDecoder *mChunkedDecoder;
+  nsHttpChunkedDecoder* mChunkedDecoder;
 
   TimingStruct mTimings;
 
@@ -317,8 +266,9 @@ class nsHttpTransaction final : public nsAHttpTransaction,
       mRestartCount;  // the number of times this transaction has been restarted
   uint32_t mCaps;
 
-  nsHttpVersion mHttpVersion;
+  HttpVersion mHttpVersion;
   uint16_t mHttpResponseCode;
+  nsCString mFlat407Headers;
 
   uint32_t mCurrentHttpResponseHeaderSize;
 
@@ -376,6 +326,8 @@ class nsHttpTransaction final : public nsAHttpTransaction,
   bool mDeferredSendProgress;
   bool mWaitingOnPipeOut;
 
+  bool mDoNotRemoveAltSvc;
+
   // mClosed           := transaction has been explicitly closed
   // mTransactionDone  := transaction ran to completion or was interrupted
   // mResponseComplete := transaction ran to completion
@@ -385,9 +337,8 @@ class nsHttpTransaction final : public nsAHttpTransaction,
   bool mReportedResponseHeader;
 
   // protected by nsHttp::GetLock()
-  nsHttpResponseHead *mForTakeResponseHead;
   bool mResponseHeadTaken;
-  nsAutoPtr<nsHttpHeaderArray> mForTakeResponseTrailers;
+  UniquePtr<nsHttpHeaderArray> mForTakeResponseTrailers;
   bool mResponseTrailersTaken;
 
   // The time when the transaction was submitted to the Connection Manager
@@ -430,7 +381,6 @@ class nsHttpTransaction final : public nsAHttpTransaction,
   nsCOMPtr<nsICancelable> mTokenBucketCancel;
 
  public:
-  void SetClassOfService(uint32_t cos);
   uint32_t ClassOfService() { return mClassOfService; }
 
  private:
@@ -444,27 +394,16 @@ class nsHttpTransaction final : public nsAHttpTransaction,
   // existing tunnel instead of triggering creation of a new one.
   // The tunnel provider is used for ASpdySession::MaybeReTunnel() checks.
 
-  void SetTunnelProvider(ASpdySession *provider) { mTunnelProvider = provider; }
-  ASpdySession *TunnelProvider() { return mTunnelProvider; }
-  nsIInterfaceRequestor *SecurityCallbacks() { return mCallbacks; }
+  void SetTunnelProvider(ASpdySession* provider) { mTunnelProvider = provider; }
+  ASpdySession* TunnelProvider() { return mTunnelProvider; }
+  nsIInterfaceRequestor* SecurityCallbacks() { return mCallbacks; }
 
  private:
   RefPtr<ASpdySession> mTunnelProvider;
-
- public:
-  void SetTransactionObserver(TransactionObserver *arg) {
-    mTransactionObserver = arg;
-  }
-
- private:
-  RefPtr<TransactionObserver> mTransactionObserver;
-
- public:
-  void GetNetworkAddresses(NetAddr &self, NetAddr &peer);
-
- private:
+  TransactionObserverFunc mTransactionObserver;
   NetAddr mSelfAddr;
   NetAddr mPeerAddr;
+  bool mResolvedByTRR;
 
   bool m0RTTInProgress;
   bool mDoNotTryEarlyData;
@@ -476,6 +415,17 @@ class nsHttpTransaction final : public nsAHttpTransaction,
   } mEarlyDataDisposition;
 
   uint8_t mFastOpenStatus;
+
+  // H2 websocket support
+  RefPtr<SpdyConnectTransaction> mH2WSTransaction;
+
+  HttpTrafficCategory mTrafficCategory;
+  bool mThroughCaptivePortal;
+  Atomic<int32_t> mProxyConnectResponseCode;
+
+  OnPushCallback mOnPushCallback;
+  nsDataHashtable<nsUint32HashKey, RefPtr<Http2PushedStreamWrapper>>
+      mIDToStreamMap;
 };
 
 }  // namespace net
