@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -13,17 +13,14 @@
 #include "nsStringStream.h"
 #include "nsIFile.h"
 #include "nsIFileURL.h"
-#include "nsIURIMutator.h"
 #include "nsEscape.h"
 #include "nsIDirIndex.h"
 #include "nsURLHelper.h"
-#include "nsIPrefService.h"
-#include "nsIPrefBranch.h"
-#include "nsIPrefLocalizedString.h"
 #include "nsIStringBundle.h"
-#include "nsITextToSubURI.h"
+#include "nsDirIndexParser.h"
 #include "nsNativeCharsetUtils.h"
 #include "nsString.h"
+#include "nsContentUtils.h"
 #include <algorithm>
 #include "nsIChannel.h"
 #include "mozilla/Unused.h"
@@ -50,6 +47,8 @@ static void AppendNonAsciiToNCR(const nsAString& in, nsCString& out) {
   }
 }
 
+nsIndexedToHTML::nsIndexedToHTML() : mExpectAbsLoc(false) {}
+
 nsresult nsIndexedToHTML::Create(nsISupports* aOuter, REFNSIID aIID,
                                  void** aResult) {
   nsresult rv;
@@ -92,14 +91,20 @@ nsIndexedToHTML::AsyncConvertData(const char* aFromType, const char* aToType,
 }
 
 NS_IMETHODIMP
-nsIndexedToHTML::OnStartRequest(nsIRequest* request, nsISupports* aContext) {
+nsIndexedToHTML::GetConvertedType(const nsACString& aFromType,
+                                  nsIChannel* aChannel, nsACString& aToType) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsIndexedToHTML::OnStartRequest(nsIRequest* request) {
   nsCString buffer;
-  nsresult rv = DoOnStartRequest(request, aContext, buffer);
+  nsresult rv = DoOnStartRequest(request, nullptr, buffer);
   if (NS_FAILED(rv)) {
     request->Cancel(rv);
   }
 
-  rv = mListener->OnStartRequest(request, aContext);
+  rv = mListener->OnStartRequest(request);
   if (NS_FAILED(rv)) return rv;
 
   // The request may have been canceled, and if that happens, we want to
@@ -109,7 +114,7 @@ nsIndexedToHTML::OnStartRequest(nsIRequest* request, nsISupports* aContext) {
 
   // Push our buffer to the listener.
 
-  rv = SendToListener(request, aContext, buffer);
+  rv = SendToListener(request, nullptr, buffer);
   return rv;
 }
 
@@ -123,26 +128,22 @@ nsresult nsIndexedToHTML::DoOnStartRequest(nsIRequest* request,
   rv = channel->GetOriginalURI(getter_AddRefs(uri));
   if (NS_FAILED(rv)) return rv;
 
-  bool isResource = false;
-  rv = uri->SchemeIs("resource", &isResource);
-  if (NS_FAILED(rv)) return rv;
-
   // We use the original URI for the title and parent link when it's a
   // resource:// url, instead of the jar:file:// url it resolves to.
-  if (!isResource) {
+  if (!uri->SchemeIs("resource")) {
     rv = channel->GetURI(getter_AddRefs(uri));
     if (NS_FAILED(rv)) return rv;
   }
 
   channel->SetContentType(NS_LITERAL_CSTRING("text/html"));
 
-  mParser = do_CreateInstance("@mozilla.org/dirIndexParser;1", &rv);
-  if (NS_FAILED(rv)) return rv;
+  mParser = nsDirIndexParser::CreateInstance();
+  if (!mParser) return NS_ERROR_FAILURE;
 
   rv = mParser->SetListener(this);
   if (NS_FAILED(rv)) return rv;
 
-  rv = mParser->OnStartRequest(request, aContext);
+  rv = mParser->OnStartRequest(request);
   if (NS_FAILED(rv)) return rv;
 
   nsAutoCString baseUri, titleUri;
@@ -171,9 +172,7 @@ nsresult nsIndexedToHTML::DoOnStartRequest(nsIRequest* request,
   // would muck up the XUL display
   // - bbaetz
 
-  bool isScheme = false;
-  bool isSchemeFile = false;
-  if (NS_SUCCEEDED(uri->SchemeIs("ftp", &isScheme)) && isScheme) {
+  if (uri->SchemeIs("ftp")) {
     // strip out the password here, so it doesn't show in the page title
     // This is done by the 300: line generation in ftp, but we don't use
     // that - see above
@@ -196,8 +195,7 @@ nsresult nsIndexedToHTML::DoOnStartRequest(nsIRequest* request,
       rv = uri->Resolve(NS_LITERAL_CSTRING(".."), parentStr);
       if (NS_FAILED(rv)) return rv;
     }
-  } else if (NS_SUCCEEDED(uri->SchemeIs("file", &isSchemeFile)) &&
-             isSchemeFile) {
+  } else if (uri->SchemeIs("file")) {
     nsCOMPtr<nsIFileURL> fileUrl = do_QueryInterface(uri);
     nsCOMPtr<nsIFile> file;
     rv = fileUrl->GetFile(getter_AddRefs(file));
@@ -221,7 +219,7 @@ nsresult nsIndexedToHTML::DoOnStartRequest(nsIRequest* request,
     // Directory index will be always encoded in UTF-8 if this is file url
     buffer.AppendLiteral("<meta charset=\"UTF-8\">\n");
 
-  } else if (NS_SUCCEEDED(uri->SchemeIs("jar", &isScheme)) && isScheme) {
+  } else if (uri->SchemeIs("jar")) {
     nsAutoCString path;
     rv = uri->GetPathQueryRef(path);
     if (NS_FAILED(rv)) return rv;
@@ -386,7 +384,7 @@ nsresult nsIndexedToHTML::DoOnStartRequest(nsIRequest* request,
       "    headCells[i].addEventListener(\"click\", rowAction(i), true);\n"
       "  }\n"
       "  if (gUI_showHidden) {\n"
-      "    gRows = Array.slice(gTBody.rows);\n"
+      "    gRows = Array.from(gTBody.rows);\n"
       "    hiddenObjects = gRows.some(row => row.className == "
       "\"hidden-object\");\n"
       "  }\n"
@@ -418,7 +416,7 @@ nsresult nsIndexedToHTML::DoOnStartRequest(nsIRequest* request,
       "}\n"
       "function orderBy(column) {\n"
       "  if (!gRows)\n"
-      "    gRows = Array.slice(gTBody.rows);\n"
+      "    gRows = Array.from(gTBody.rows);\n"
       "  var order;\n"
       "  if (gOrderBy == column) {\n"
       "    order = gTable.getAttribute(\"order\") == \"asc\" ? \"desc\" : "
@@ -447,7 +445,7 @@ nsresult nsIndexedToHTML::DoOnStartRequest(nsIRequest* request,
       "}\n"
       "</script>\n");
 
-  buffer.AppendLiteral("<link rel=\"icon\" type=\"image/png\" href=\"");
+  buffer.AppendLiteral(R"(<link rel="icon" type="image/png" href=")");
   nsCOMPtr<nsIURI> innerUri = NS_GetInnermostURI(uri);
   if (!innerUri) return NS_ERROR_UNEXPECTED;
   nsCOMPtr<nsIFileURL> fileURL(do_QueryInterface(innerUri));
@@ -512,7 +510,7 @@ nsresult nsIndexedToHTML::DoOnStartRequest(nsIRequest* request,
   // 1. file URL may be encoded in platform charset for backward compatibility
   // 2. query part may not be encoded in UTF-8 (see bug 261929)
   // so try the platform's default if this is file url
-  if (NS_FAILED(rv) && isSchemeFile && !NS_IsNativeUTF8()) {
+  if (NS_FAILED(rv) && uri->SchemeIs("file") && !NS_IsNativeUTF8()) {
     auto encoding = mozilla::dom::FallbackEncoding::FromLocale();
     nsAutoCString charset;
     encoding->Name(charset);
@@ -522,13 +520,11 @@ nsresult nsIndexedToHTML::DoOnStartRequest(nsIRequest* request,
 
   nsCString htmlEscSpecUtf8;
   nsAppendEscapedHTML(NS_ConvertUTF16toUTF8(unEscapeSpec), htmlEscSpecUtf8);
-  NS_ConvertUTF8toUTF16 htmlEscSpec(htmlEscSpecUtf8);
+  AutoTArray<nsString, 1> formatTitle;
+  CopyUTF8toUTF16(htmlEscSpecUtf8, *formatTitle.AppendElement());
 
   nsAutoString title;
-  const char16_t* formatTitle[] = {htmlEscSpec.get()};
-
-  rv = mBundle->FormatStringFromName(
-      "DirTitle", formatTitle, sizeof(formatTitle) / sizeof(char16_t*), title);
+  rv = mBundle->FormatStringFromName("DirTitle", formatTitle, title);
   if (NS_FAILED(rv)) return rv;
 
   // we want to convert string bundle to NCR
@@ -551,7 +547,7 @@ nsresult nsIndexedToHTML::DoOnStartRequest(nsIRequest* request,
     // will prematurely close the string.  Go ahead an
     // add a base href, but only do so if we're not
     // dealing with a resource URI.
-    if (!isResource) {
+    if (!uri->SchemeIs("resource")) {
       buffer.AppendLiteral("<base href=\"");
       nsAppendEscapedHTML(baseUri, buffer);
       buffer.AppendLiteral("\" />\n");
@@ -568,14 +564,6 @@ nsresult nsIndexedToHTML::DoOnStartRequest(nsIRequest* request,
   buffer.AppendLiteral("</head>\n<body dir=\"");
   buffer.Append(direction);
   buffer.AppendLiteral("\">\n<h1>");
-
-  const char16_t* formatHeading[] = {htmlEscSpec.get()};
-
-  rv = mBundle->FormatStringFromName("DirTitle", formatHeading,
-                                     sizeof(formatHeading) / sizeof(char16_t*),
-                                     title);
-  if (NS_FAILED(rv)) return rv;
-
   AppendNonAsciiToNCR(title, buffer);
   buffer.AppendLiteral("</h1>\n");
 
@@ -584,14 +572,14 @@ nsresult nsIndexedToHTML::DoOnStartRequest(nsIRequest* request,
     rv = mBundle->GetStringFromName("DirGoUp", parentText);
     if (NS_FAILED(rv)) return rv;
 
-    buffer.AppendLiteral("<p id=\"UI_goUp\"><a class=\"up\" href=\"");
+    buffer.AppendLiteral(R"(<p id="UI_goUp"><a class="up" href=")");
     nsAppendEscapedHTML(parentStr, buffer);
     buffer.AppendLiteral("\">");
     AppendNonAsciiToNCR(parentText, buffer);
     buffer.AppendLiteral("</a></p>\n");
   }
 
-  if (isSchemeFile) {
+  if (uri->SchemeIs("file")) {
     nsAutoString showHiddenText;
     rv = mBundle->GetStringFromName("ShowHidden", showHiddenText);
     if (NS_FAILED(rv)) return rv;
@@ -638,19 +626,18 @@ nsresult nsIndexedToHTML::DoOnStartRequest(nsIRequest* request,
 }
 
 NS_IMETHODIMP
-nsIndexedToHTML::OnStopRequest(nsIRequest* request, nsISupports* aContext,
-                               nsresult aStatus) {
+nsIndexedToHTML::OnStopRequest(nsIRequest* request, nsresult aStatus) {
   if (NS_SUCCEEDED(aStatus)) {
     nsCString buffer;
     buffer.AssignLiteral("</tbody></table></body></html>\n");
 
-    aStatus = SendToListener(request, aContext, buffer);
+    aStatus = SendToListener(request, nullptr, buffer);
   }
 
-  mParser->OnStopRequest(request, aContext, aStatus);
+  mParser->OnStopRequest(request, aStatus);
   mParser = nullptr;
 
-  return mListener->OnStopRequest(request, aContext, aStatus);
+  return mListener->OnStopRequest(request, aStatus);
 }
 
 nsresult nsIndexedToHTML::SendToListener(nsIRequest* aRequest,
@@ -659,15 +646,31 @@ nsresult nsIndexedToHTML::SendToListener(nsIRequest* aRequest,
   nsCOMPtr<nsIInputStream> inputData;
   nsresult rv = NS_NewCStringInputStream(getter_AddRefs(inputData), aBuffer);
   NS_ENSURE_SUCCESS(rv, rv);
-  return mListener->OnDataAvailable(aRequest, aContext, inputData, 0,
-                                    aBuffer.Length());
+  return mListener->OnDataAvailable(aRequest, inputData, 0, aBuffer.Length());
 }
 
 NS_IMETHODIMP
-nsIndexedToHTML::OnDataAvailable(nsIRequest* aRequest, nsISupports* aCtxt,
-                                 nsIInputStream* aInput, uint64_t aOffset,
-                                 uint32_t aCount) {
-  return mParser->OnDataAvailable(aRequest, aCtxt, aInput, aOffset, aCount);
+nsIndexedToHTML::OnDataAvailable(nsIRequest* aRequest, nsIInputStream* aInput,
+                                 uint64_t aOffset, uint32_t aCount) {
+  return mParser->OnDataAvailable(aRequest, aInput, aOffset, aCount);
+}
+
+static nsresult FormatTime(const nsDateFormatSelector aDateFormatSelector,
+                           const nsTimeFormatSelector aTimeFormatSelector,
+                           const PRTime aPrTime, nsAString& aStringOut) {
+  // FormatPRExplodedTime will use GMT based formatted string (e.g. GMT+1)
+  // instead of local time zone name (e.g. CEST).
+  // To avoid this case when ResistFingerprinting is disabled, use
+  // |FormatPRTime| to show exact time zone name.
+  if (!nsContentUtils::ShouldResistFingerprinting()) {
+    return mozilla::DateTimeFormat::FormatPRTime(
+        aDateFormatSelector, aTimeFormatSelector, aPrTime, aStringOut);
+  }
+
+  PRExplodedTime prExplodedTime;
+  PR_ExplodeTime(aPrTime, PR_GMTParameters, &prExplodedTime);
+  return mozilla::DateTimeFormat::FormatPRExplodedTime(
+      aDateFormatSelector, aTimeFormatSelector, &prExplodedTime, aStringOut);
 }
 
 NS_IMETHODIMP
@@ -682,7 +685,7 @@ nsIndexedToHTML::OnIndexAvailable(nsIRequest* aRequest, nsISupports* aCtxt,
   // We don't know the file's character set yet, so retrieve the raw bytes
   // which will be decoded by the HTML parser.
   nsCString loc;
-  aIndex->GetLocation(getter_Copies(loc));
+  aIndex->GetLocation(loc);
 
   // Adjust the length in case unescaping shortened the string.
   loc.Truncate(nsUnescapeCount(loc.BeginWriting()));
@@ -715,7 +718,7 @@ nsIndexedToHTML::OnIndexAvailable(nsIRequest* aRequest, nsISupports* aCtxt,
   pushBuffer.Append(escaped);
 
   pushBuffer.AppendLiteral(
-      "\"><table class=\"ellipsis\"><tbody><tr><td><a class=\"");
+      R"("><table class="ellipsis"><tbody><tr><td><a class=")");
   switch (type) {
     case nsIDirIndex::TYPE_DIRECTORY:
       pushBuffer.AppendLiteral("dir");
@@ -816,12 +819,10 @@ nsIndexedToHTML::OnIndexAvailable(nsIRequest* aRequest, nsISupports* aCtxt,
     pushBuffer.AppendInt(static_cast<int64_t>(t));
     pushBuffer.AppendLiteral("\">");
     nsAutoString formatted;
-    mozilla::DateTimeFormat::FormatPRTime(kDateFormatShort, kTimeFormatNone, t,
-                                          formatted);
+    FormatTime(kDateFormatShort, kTimeFormatNone, t, formatted);
     AppendNonAsciiToNCR(formatted, pushBuffer);
     pushBuffer.AppendLiteral("</td>\n <td>");
-    mozilla::DateTimeFormat::FormatPRTime(kDateFormatNone, kTimeFormatSeconds,
-                                          t, formatted);
+    FormatTime(kDateFormatNone, kTimeFormatSeconds, t, formatted);
     // use NCR to show date in any doc charset
     AppendNonAsciiToNCR(formatted, pushBuffer);
   }
@@ -858,7 +859,3 @@ void nsIndexedToHTML::FormatSizeString(int64_t inSize,
     outSizeString.AppendLiteral(" KB");
   }
 }
-
-nsIndexedToHTML::nsIndexedToHTML() {}
-
-nsIndexedToHTML::~nsIndexedToHTML() {}
