@@ -20,68 +20,50 @@
 
 "use strict";
 
-ChromeUtils.import("resource://gre/modules/NetUtil.jsm");
-
-var ios = Cc["@mozilla.org/network/io-service;1"]
-            .getService(Ci.nsIIOService);
-var pps = Cc["@mozilla.org/network/protocol-proxy-service;1"]
-            .getService();
-var prefs = Cc["@mozilla.org/preferences-service;1"]
-                     .getService(Ci.nsIPrefBranch);
+var ios = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
+var pps = Cc["@mozilla.org/network/protocol-proxy-service;1"].getService();
+var prefs = Cc["@mozilla.org/preferences-service;1"].getService(
+  Ci.nsIPrefBranch
+);
 
 /**
  * Test nsIProtocolHandler that allows proxying, but doesn't allow HTTP
  * proxying.
  */
-function TestProtocolHandler() {
-}
+function TestProtocolHandler() {}
 TestProtocolHandler.prototype = {
-  QueryInterface: function(iid) {
-    if (iid.equals(Ci.nsIProtocolHandler) ||
-        iid.equals(Ci.nsISupports))
-      return this;
-    throw Cr.NS_ERROR_NO_INTERFACE;
-  },
+  QueryInterface: ChromeUtils.generateQI([Ci.nsIProtocolHandler]),
   scheme: "moz-test",
   defaultPort: -1,
-  protocolFlags: Ci.nsIProtocolHandler.URI_NOAUTH |
-                 Ci.nsIProtocolHandler.URI_NORELATIVE |
-                 Ci.nsIProtocolHandler.ALLOWS_PROXY |
-                 Ci.nsIProtocolHandler.URI_DANGEROUS_TO_LOAD,
-  newURI: function(spec, originCharset, baseURI) {
-    return Cc["@mozilla.org/network/simple-uri-mutator;1"]
-             .createInstance(Ci.nsIURIMutator)
-             .setSpec(spec)
-             .finalize();
+  protocolFlags:
+    Ci.nsIProtocolHandler.URI_NOAUTH |
+    Ci.nsIProtocolHandler.URI_NORELATIVE |
+    Ci.nsIProtocolHandler.ALLOWS_PROXY |
+    Ci.nsIProtocolHandler.URI_DANGEROUS_TO_LOAD,
+  newChannel(uri, aLoadInfo) {
+    throw Components.Exception("", Cr.NS_ERROR_NOT_IMPLEMENTED);
   },
-  newChannel2: function(uri, aLoadInfo) {
-    throw Cr.NS_ERROR_NOT_IMPLEMENTED;
-  },
-  newChannel: function(uri) {
-    throw Cr.NS_ERROR_NOT_IMPLEMENTED;
-  },
-  allowPort: function(port, scheme) {
+  allowPort(port, scheme) {
     return true;
-  }
+  },
 };
 
-function TestProtocolHandlerFactory() {
-}
+function TestProtocolHandlerFactory() {}
 TestProtocolHandlerFactory.prototype = {
-  createInstance: function(delegate, iid) {
+  createInstance(delegate, iid) {
     return new TestProtocolHandler().QueryInterface(iid);
   },
-  lockFactory: function(lock) {
-  }
+  lockFactory(lock) {},
 };
 
 function register_test_protocol_handler() {
-  var reg = Components.manager.QueryInterface(
-      Ci.nsIComponentRegistrar);
-  reg.registerFactory(Components.ID("{4ea7dd3a-8cae-499c-9f18-e1de773ca25b}"),
-                      "TestProtocolHandler",
-                      "@mozilla.org/network/protocol;1?name=moz-test",
-                      new TestProtocolHandlerFactory());
+  var reg = Components.manager.QueryInterface(Ci.nsIComponentRegistrar);
+  reg.registerFactory(
+    Components.ID("{4ea7dd3a-8cae-499c-9f18-e1de773ca25b}"),
+    "TestProtocolHandler",
+    "@mozilla.org/network/protocol;1?name=moz-test",
+    new TestProtocolHandlerFactory()
+  );
 }
 
 function check_proxy(pi, type, host, port, flags, timeout, hasNext) {
@@ -89,14 +71,17 @@ function check_proxy(pi, type, host, port, flags, timeout, hasNext) {
   Assert.equal(pi.type, type);
   Assert.equal(pi.host, host);
   Assert.equal(pi.port, port);
-  if (flags != -1)
+  if (flags != -1) {
     Assert.equal(pi.flags, flags);
-  if (timeout != -1)
+  }
+  if (timeout != -1) {
     Assert.equal(pi.failoverTimeout, timeout);
-  if (hasNext)
+  }
+  if (hasNext) {
     Assert.notEqual(pi.failoverProxy, null);
-  else
+  } else {
     Assert.equal(pi.failoverProxy, null);
+  }
 }
 
 function TestFilter(type, host, port, flags, timeout) {
@@ -112,76 +97,80 @@ TestFilter.prototype = {
   _port: -1,
   _flags: 0,
   _timeout: 0,
-  QueryInterface: function(iid) {
-    if (iid.equals(Ci.nsIProtocolProxyFilter) ||
-        iid.equals(Ci.nsISupports))
-      return this;
-    throw Cr.NS_ERROR_NO_INTERFACE;
-  },
-  applyFilter: function(pps, uri, pi, cb) {
-    var pi_tail = pps.newProxyInfo(this._type, this._host, this._port,
-                                   this._flags, this._timeout, null);
-    if (pi)
+  QueryInterface: ChromeUtils.generateQI([Ci.nsIProtocolProxyFilter]),
+  applyFilter(uri, pi, cb) {
+    var pi_tail = pps.newProxyInfo(
+      this._type,
+      this._host,
+      this._port,
+      "",
+      "",
+      this._flags,
+      this._timeout,
+      null
+    );
+    if (pi) {
       pi.failoverProxy = pi_tail;
-    else
+    } else {
       pi = pi_tail;
+    }
     cb.onProxyFilterResult(pi);
-  }
+  },
 };
 
 function BasicFilter() {}
 BasicFilter.prototype = {
-  QueryInterface: function(iid) {
-    if (iid.equals(Ci.nsIProtocolProxyFilter) ||
-        iid.equals(Ci.nsISupports))
-      return this;
-    throw Cr.NS_ERROR_NO_INTERFACE;
-  },
-  applyFilter: function(pps, uri, pi, cb) {
+  QueryInterface: ChromeUtils.generateQI([Ci.nsIProtocolProxyFilter]),
+  applyFilter(uri, pi, cb) {
     cb.onProxyFilterResult(
-      pps.newProxyInfo("http", "localhost", 8080, 0, 10,
-      pps.newProxyInfo("direct", "", -1, 0, 0, null))
+      pps.newProxyInfo(
+        "http",
+        "localhost",
+        8080,
+        "",
+        "",
+        0,
+        10,
+        pps.newProxyInfo("direct", "", -1, "", "", 0, 0, null)
+      )
     );
-  }
+  },
 };
 
 function BasicChannelFilter() {}
 BasicChannelFilter.prototype = {
-  QueryInterface: function(iid) {
-    if (iid.equals(Ci.nsIProtocolProxyChannelFilter) ||
-        iid.equals(Ci.nsISupports))
-      return this;
-    throw Cr.NS_ERROR_NO_INTERFACE;
-  },
-  applyFilter: function(pps, channel, pi, cb) {
+  QueryInterface: ChromeUtils.generateQI([Ci.nsIProtocolProxyChannelFilter]),
+  applyFilter(channel, pi, cb) {
     cb.onProxyFilterResult(
-      pps.newProxyInfo("http", channel.URI.host, 7777, 0, 10,
-      pps.newProxyInfo("direct", "", -1, 0, 0, null))
+      pps.newProxyInfo(
+        "http",
+        channel.URI.host,
+        7777,
+        "",
+        "",
+        0,
+        10,
+        pps.newProxyInfo("direct", "", -1, "", "", 0, 0, null)
+      )
     );
-  }
+  },
 };
 
-function resolveCallback() { }
+function resolveCallback() {}
 resolveCallback.prototype = {
   nextFunction: null,
 
-  QueryInterface : function (iid) {
-    const interfaces = [Ci.nsIProtocolProxyCallback,
-                        Ci.nsISupports];
-    if (!interfaces.some( function(v) { return iid.equals(v) } ))
-      throw Cr.NS_ERROR_NO_INTERFACE;
-    return this;
-  },
+  QueryInterface: ChromeUtils.generateQI([Ci.nsIProtocolProxyCallback]),
 
-  onProxyAvailable : function (req, channel, pi, status) {
+  onProxyAvailable(req, channel, pi, status) {
     this.nextFunction(pi);
-  }
+  },
 };
 
 function run_filter_test() {
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
 
   // Verify initial state
@@ -207,13 +196,12 @@ function filter_test0_1(pi) {
   cb.nextFunction = filter_test0_2;
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   var req = pps.asyncResolve(channel, 0, cb);
 }
 
-function filter_test0_2(pi)
-{
+function filter_test0_2(pi) {
   check_proxy(pi, "http", "localhost", 8080, 0, 10, true);
   check_proxy(pi.failoverProxy, "direct", "", -1, 0, 0, false);
 
@@ -223,13 +211,12 @@ function filter_test0_2(pi)
   cb.nextFunction = filter_test0_3;
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   var req = pps.asyncResolve(channel, 0, cb);
 }
 
-function filter_test0_3(pi)
-{
+function filter_test0_3(pi) {
   check_proxy(pi, "http", "localhost", 8080, 0, 10, true);
   check_proxy(pi.failoverProxy, "direct", "", -1, 0, 0, false);
 
@@ -241,15 +228,14 @@ function filter_test0_3(pi)
   cb.nextFunction = filter_test0_4;
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   var req = pps.asyncResolve(channel, 0, cb);
 }
 
 var filter03;
 
-function filter_test0_4(pi)
-{
+function filter_test0_4(pi) {
   Assert.equal(pi, null);
   filter03 = new BasicChannelFilter();
   pps.registerChannelFilter(filter03, 10);
@@ -257,13 +243,12 @@ function filter_test0_4(pi)
   cb.nextFunction = filter_test0_5;
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   var req = pps.asyncResolve(channel, 0, cb);
 }
 
-function filter_test0_5(pi)
-{
+function filter_test0_5(pi) {
   pps.unregisterChannelFilter(filter03);
   check_proxy(pi, "http", "www.mozilla.org", 7777, 0, 10, true);
   check_proxy(pi.failoverProxy, "direct", "", -1, 0, 0, false);
@@ -293,8 +278,7 @@ function filter_test_uri0_1(pi) {
   pps.asyncResolve(uri, 0, cb);
 }
 
-function filter_test_uri0_2(pi)
-{
+function filter_test_uri0_2(pi) {
   check_proxy(pi, "http", "localhost", 8080, 0, 10, true);
   check_proxy(pi.failoverProxy, "direct", "", -1, 0, 0, false);
 
@@ -306,8 +290,7 @@ function filter_test_uri0_2(pi)
   pps.asyncResolve(uri, 0, cb);
 }
 
-function filter_test_uri0_3(pi)
-{
+function filter_test_uri0_3(pi) {
   check_proxy(pi, "http", "localhost", 8080, 0, 10, true);
   check_proxy(pi.failoverProxy, "direct", "", -1, 0, 0, false);
 
@@ -321,8 +304,7 @@ function filter_test_uri0_3(pi)
   pps.asyncResolve(uri, 0, cb);
 }
 
-function filter_test_uri0_4(pi)
-{
+function filter_test_uri0_4(pi) {
   Assert.equal(pi, null);
   run_filter_test2();
 }
@@ -342,7 +324,7 @@ function run_filter_test2() {
   cb.nextFunction = filter_test1_1;
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   var req = pps.asyncResolve(channel, 0, cb);
 }
@@ -357,7 +339,7 @@ function filter_test1_1(pi) {
   cb.nextFunction = filter_test1_2;
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   var req = pps.asyncResolve(channel, 0, cb);
 }
@@ -373,7 +355,7 @@ function filter_test1_2(pi) {
   cb.nextFunction = filter_test1_3;
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   var req = pps.asyncResolve(channel, 0, cb);
 }
@@ -388,7 +370,7 @@ var filter_3_1;
 function run_filter_test3() {
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   // Push a filter and verify the results asynchronously
 
@@ -409,7 +391,7 @@ function filter_test3_1(pi) {
 function run_pref_test() {
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   // Verify 'direct' setting
 
@@ -420,8 +402,7 @@ function run_pref_test() {
   var req = pps.asyncResolve(channel, 0, cb);
 }
 
-function pref_test1_1(pi)
-{
+function pref_test1_1(pi) {
   Assert.equal(pi, null);
 
   // Verify 'manual' setting
@@ -431,13 +412,12 @@ function pref_test1_1(pi)
   cb.nextFunction = pref_test1_2;
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   var req = pps.asyncResolve(channel, 0, cb);
 }
 
-function pref_test1_2(pi)
-{
+function pref_test1_2(pi) {
   // nothing yet configured
   Assert.equal(pi, null);
 
@@ -449,13 +429,12 @@ function pref_test1_2(pi)
   cb.nextFunction = pref_test1_3;
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   var req = pps.asyncResolve(channel, 0, cb);
 }
 
-function pref_test1_3(pi)
-{
+function pref_test1_3(pi) {
   check_proxy(pi, "http", "foopy", 8080, 0, -1, false);
 
   prefs.setCharPref("network.proxy.http", "");
@@ -469,19 +448,17 @@ function pref_test1_3(pi)
   cb.nextFunction = pref_test1_4;
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   var req = pps.asyncResolve(channel, 0, cb);
 }
 
-function pref_test1_4(pi)
-{
+function pref_test1_4(pi) {
   check_proxy(pi, "socks", "barbar", 1203, 0, -1, false);
   run_pac_test();
 }
 
-function protocol_handler_test_1(pi)
-{
+function protocol_handler_test_1(pi) {
   Assert.equal(pi, null);
   prefs.setCharPref("network.proxy.autoconfig_url", "");
   prefs.setIntPref("network.proxy.type", 0);
@@ -494,16 +471,14 @@ function TestResolveCallback(type, nexttest) {
   this.nexttest = nexttest;
 }
 TestResolveCallback.prototype = {
-  QueryInterface:
-  function TestResolveCallback_QueryInterface(iid) {
-    if (iid.equals(Ci.nsIProtocolProxyCallback) ||
-        iid.equals(Ci.nsISupports))
-      return this;
-    throw Cr.NS_ERROR_NO_INTERFACE;
-  },
+  QueryInterface: ChromeUtils.generateQI([Ci.nsIProtocolProxyCallback]),
 
-  onProxyAvailable:
-  function TestResolveCallback_onProxyAvailable(req, channel, pi, status) {
+  onProxyAvailable: function TestResolveCallback_onProxyAvailable(
+    req,
+    channel,
+    pi,
+    status
+  ) {
     dump("*** channelURI=" + channel.URI.spec + ", status=" + status + "\n");
 
     if (this.type == null) {
@@ -518,35 +493,41 @@ TestResolveCallback.prototype = {
     }
 
     this.nexttest();
-  }
+  },
 };
 
 var originalTLSProxy;
 
 function run_pac_test() {
-  var pac = 'data:text/plain,' +
-            'function FindProxyForURL(url, host) {' +
-            '  return "PROXY foopy:8080; DIRECT";' +
-            '}';
+  var pac =
+    "data:text/plain," +
+    "function FindProxyForURL(url, host) {" +
+    '  return "PROXY foopy:8080; DIRECT";' +
+    "}";
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   // Configure PAC
 
   prefs.setIntPref("network.proxy.type", 2);
   prefs.setCharPref("network.proxy.autoconfig_url", pac);
-  var req = pps.asyncResolve(channel, 0, new TestResolveCallback("http", run_pac2_test));
+  var req = pps.asyncResolve(
+    channel,
+    0,
+    new TestResolveCallback("http", run_pac2_test)
+  );
 }
 
 function run_pac2_test() {
-  var pac = 'data:text/plain,' +
-            'function FindProxyForURL(url, host) {' +
-            '  return "HTTPS foopy:8080; DIRECT";' +
-            '}';
+  var pac =
+    "data:text/plain," +
+    "function FindProxyForURL(url, host) {" +
+    '  return "HTTPS foopy:8080; DIRECT";' +
+    "}";
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   // Configure PAC
   originalTLSProxy = prefs.getBoolPref("network.proxy.proxy_over_tls");
@@ -554,23 +535,32 @@ function run_pac2_test() {
   prefs.setCharPref("network.proxy.autoconfig_url", pac);
   prefs.setBoolPref("network.proxy.proxy_over_tls", true);
 
-  var req = pps.asyncResolve(channel, 0, new TestResolveCallback("https", run_pac3_test));
+  var req = pps.asyncResolve(
+    channel,
+    0,
+    new TestResolveCallback("https", run_pac3_test)
+  );
 }
 
 function run_pac3_test() {
-  var pac = 'data:text/plain,' +
-            'function FindProxyForURL(url, host) {' +
-            '  return "HTTPS foopy:8080; DIRECT";' +
-            '}';
+  var pac =
+    "data:text/plain," +
+    "function FindProxyForURL(url, host) {" +
+    '  return "HTTPS foopy:8080; DIRECT";' +
+    "}";
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   // Configure PAC
   prefs.setCharPref("network.proxy.autoconfig_url", pac);
   prefs.setBoolPref("network.proxy.proxy_over_tls", false);
 
-  var req = pps.asyncResolve(channel, 0, new TestResolveCallback(null, run_pac4_test));
+  var req = pps.asyncResolve(
+    channel,
+    0,
+    new TestResolveCallback(null, run_pac4_test)
+  );
 }
 
 function run_pac4_test() {
@@ -584,23 +574,96 @@ function run_pac4_test() {
     ["THU", "SAT", "THU", "SUN"], // for Fri
     ["FRI", "SAT", "FRI", "SUN"], // for Sat
   ];
-  let today = (new Date()).getDay();
-  var pac = 'data:text/plain,' +
-            'function FindProxyForURL(url, host) {' +
-            '  if (weekdayRange("' + wRange[today][0] + '", "' + wRange[today][1] + '") &&' +
-            '      weekdayRange("' + wRange[today][2] + '", "' + wRange[today][3] + '")) {' +
-            '    return "PROXY foopy:8080; DIRECT";' +
-            '  }' +
-            '}';
+  let today = new Date().getDay();
+  var pac =
+    "data:text/plain," +
+    "function FindProxyForURL(url, host) {" +
+    '  if (weekdayRange("' +
+    wRange[today][0] +
+    '", "' +
+    wRange[today][1] +
+    '") &&' +
+    '      weekdayRange("' +
+    wRange[today][2] +
+    '", "' +
+    wRange[today][3] +
+    '")) {' +
+    '    return "PROXY foopy:8080; DIRECT";' +
+    "  }" +
+    "}";
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   // Configure PAC
 
   prefs.setIntPref("network.proxy.type", 2);
   prefs.setCharPref("network.proxy.autoconfig_url", pac);
-  var req = pps.asyncResolve(channel, 0, new TestResolveCallback("http", finish_pac_test));
+  var req = pps.asyncResolve(
+    channel,
+    0,
+    new TestResolveCallback("http", run_utf8_pac_test)
+  );
+}
+
+function run_utf8_pac_test() {
+  var pac =
+    "data:text/plain;charset=UTF-8," +
+    "function FindProxyForURL(url, host) {" +
+    "  /*" +
+    "   U+00A9 COPYRIGHT SIGN: %C2%A9," +
+    "   U+0B87 TAMIL LETTER I: %E0%AE%87," +
+    "   U+10398 UGARITIC LETTER THANNA: %F0%90%8E%98 " +
+    "  */" +
+    '  var multiBytes = "%C2%A9 %E0%AE%87 %F0%90%8E%98"; ' +
+    "  /* 6 UTF-16 units above if PAC script run as UTF-8; 11 units if run as Latin-1 */ " +
+    "  return multiBytes.length === 6 " +
+    '         ? "PROXY foopy:8080; DIRECT" ' +
+    '         : "PROXY epicfail-utf8:12345; DIRECT";' +
+    "}";
+
+  var channel = NetUtil.newChannel({
+    uri: "http://www.mozilla.org/",
+    loadUsingSystemPrincipal: true,
+  });
+
+  // Configure PAC
+  prefs.setIntPref("network.proxy.type", 2);
+  prefs.setCharPref("network.proxy.autoconfig_url", pac);
+
+  var req = pps.asyncResolve(
+    channel,
+    0,
+    new TestResolveCallback("http", run_latin1_pac_test)
+  );
+}
+
+function run_latin1_pac_test() {
+  var pac =
+    "data:text/plain," +
+    "function FindProxyForURL(url, host) {" +
+    "  /* A too-long encoding of U+0000, so not valid UTF-8 */ " +
+    '  var multiBytes = "%C0%80"; ' +
+    "  /* 2 UTF-16 units because interpreted as Latin-1 */ " +
+    "  return multiBytes.length === 2 " +
+    '         ? "PROXY foopy:8080; DIRECT" ' +
+    '         : "PROXY epicfail-latin1:12345; DIRECT";' +
+    "}";
+
+  var channel = NetUtil.newChannel({
+    uri: "http://www.mozilla.org/",
+    loadUsingSystemPrincipal: true,
+  });
+
+  // Configure PAC
+  prefs.setIntPref("network.proxy.type", 2);
+  prefs.setCharPref("network.proxy.autoconfig_url", pac);
+
+  var req = pps.asyncResolve(
+    channel,
+    0,
+    new TestResolveCallback("http", finish_pac_test)
+  );
 }
 
 function finish_pac_test() {
@@ -608,19 +671,16 @@ function finish_pac_test() {
   run_pac_cancel_test();
 }
 
-function TestResolveCancelationCallback() {
-}
+function TestResolveCancelationCallback() {}
 TestResolveCancelationCallback.prototype = {
-  QueryInterface:
-  function TestResolveCallback_QueryInterface(iid) {
-    if (iid.equals(Ci.nsIProtocolProxyCallback) ||
-        iid.equals(Ci.nsISupports))
-      return this;
-    throw Cr.NS_ERROR_NO_INTERFACE;
-  },
+  QueryInterface: ChromeUtils.generateQI([Ci.nsIProtocolProxyCallback]),
 
-  onProxyAvailable:
-  function TestResolveCancelationCallback_onProxyAvailable(req, channel, pi, status) {
+  onProxyAvailable: function TestResolveCancelationCallback_onProxyAvailable(
+    req,
+    channel,
+    pi,
+    status
+  ) {
     dump("*** channelURI=" + channel.URI.spec + ", status=" + status + "\n");
 
     Assert.notEqual(req, null);
@@ -632,19 +692,20 @@ TestResolveCancelationCallback.prototype = {
     prefs.setIntPref("network.proxy.type", 0);
 
     run_proxy_host_filters_test();
-  }
+  },
 };
 
 function run_pac_cancel_test() {
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   // Configure PAC
-  var pac = 'data:text/plain,' +
-            'function FindProxyForURL(url, host) {' +
-            '  return "PROXY foopy:8080; DIRECT";' +
-            '}';
+  var pac =
+    "data:text/plain," +
+    "function FindProxyForURL(url, host) {" +
+    '  return "PROXY foopy:8080; DIRECT";' +
+    "}";
   prefs.setIntPref("network.proxy.type", 2);
   prefs.setCharPref("network.proxy.autoconfig_url", pac);
 
@@ -663,33 +724,35 @@ function check_host_filters(hl, shouldBe, nextFX) {
   bShouldBeFiltered = shouldBe;
   hostNextFX = nextFX;
 
-  if (hostList.length > hostIDX)
+  if (hostList.length > hostIDX) {
     check_host_filter(hostIDX);
+  }
 }
 
-function check_host_filters_cb()
-{
+function check_host_filters_cb() {
   hostIDX++;
-  if (hostList.length > hostIDX)
+  if (hostList.length > hostIDX) {
     check_host_filter(hostIDX);
-  else
+  } else {
     hostNextFX();
+  }
 }
 
 function check_host_filter(i) {
   var uri;
-  dump("*** uri=" + hostList[i] + " bShouldBeFiltered=" + bShouldBeFiltered + "\n");
-    var channel = NetUtil.newChannel({
+  dump(
+    "*** uri=" + hostList[i] + " bShouldBeFiltered=" + bShouldBeFiltered + "\n"
+  );
+  var channel = NetUtil.newChannel({
     uri: hostList[i],
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   var cb = new resolveCallback();
   cb.nextFunction = host_filter_cb;
   var req = pps.asyncResolve(channel, 0, cb);
 }
 
-function host_filter_cb(proxy)
-{
+function host_filter_cb(proxy) {
   if (bShouldBeFiltered) {
     Assert.equal(proxy, null);
   } else {
@@ -701,7 +764,6 @@ function host_filter_cb(proxy)
   check_host_filters_cb();
 }
 
-
 // Verify that hists in the host filter list are not proxied
 // refers to "network.proxy.no_proxies_on"
 
@@ -719,51 +781,60 @@ function run_proxy_host_filters_test() {
   prefs.setIntPref("network.proxy.http_port", 8080);
 
   // Setup host filter list string for "no_proxies_on"
-  hostFilterList = "www.mozilla.org, www.google.com, www.apple.com, "
-                       + ".domain, .domain2.org"
+  hostFilterList =
+    "www.mozilla.org, www.google.com, www.apple.com, " +
+    ".domain, .domain2.org";
   prefs.setCharPref("network.proxy.no_proxies_on", hostFilterList);
-  Assert.equal(prefs.getCharPref("network.proxy.no_proxies_on"), hostFilterList);
+  Assert.equal(
+    prefs.getCharPref("network.proxy.no_proxies_on"),
+    hostFilterList
+  );
 
   var rv;
   // Check the hosts that should be filtered out
-  uriStrFilterList = [ "http://www.mozilla.org/",
-                           "http://www.google.com/",
-                           "http://www.apple.com/",
-                           "http://somehost.domain/",
-                           "http://someotherhost.domain/",
-                           "http://somehost.domain2.org/",
-                           "http://somehost.subdomain.domain2.org/" ];
+  uriStrFilterList = [
+    "http://www.mozilla.org/",
+    "http://www.google.com/",
+    "http://www.apple.com/",
+    "http://somehost.domain/",
+    "http://someotherhost.domain/",
+    "http://somehost.domain2.org/",
+    "http://somehost.subdomain.domain2.org/",
+  ];
   check_host_filters(uriStrFilterList, true, host_filters_1);
 }
 
-function host_filters_1()
-{
+function host_filters_1() {
   // Check the hosts that should be proxied
-  uriStrUseProxyList = [ "http://www.mozilla.com/",
-                             "http://mail.google.com/",
-                             "http://somehost.domain.co.uk/",
-                             "http://somelocalhost/" ];
+  uriStrUseProxyList = [
+    "http://www.mozilla.com/",
+    "http://mail.google.com/",
+    "http://somehost.domain.co.uk/",
+    "http://somelocalhost/",
+  ];
   check_host_filters(uriStrUseProxyList, false, host_filters_2);
 }
 
-function host_filters_2()
-{
+function host_filters_2() {
   // Set no_proxies_on to include local hosts
-  prefs.setCharPref("network.proxy.no_proxies_on", hostFilterList + ", <local>");
-  Assert.equal(prefs.getCharPref("network.proxy.no_proxies_on"),
-               hostFilterList + ", <local>");
+  prefs.setCharPref(
+    "network.proxy.no_proxies_on",
+    hostFilterList + ", <local>"
+  );
+  Assert.equal(
+    prefs.getCharPref("network.proxy.no_proxies_on"),
+    hostFilterList + ", <local>"
+  );
   // Amend lists - move local domain to filtered list
   uriStrFilterList.push(uriStrUseProxyList.pop());
   check_host_filters(uriStrFilterList, true, host_filters_3);
 }
 
-function host_filters_3()
-{
+function host_filters_3() {
   check_host_filters(uriStrUseProxyList, false, host_filters_4);
 }
 
-function host_filters_4()
-{
+function host_filters_4() {
   // Cleanup
   prefs.setCharPref("network.proxy.no_proxies_on", "");
   Assert.equal(prefs.getCharPref("network.proxy.no_proxies_on"), "");
@@ -771,24 +842,24 @@ function host_filters_4()
   run_myipaddress_test();
 }
 
-function run_myipaddress_test()
-{
+function run_myipaddress_test() {
   // This test makes sure myIpAddress() comes up with some valid
   // IP address other than localhost. The DUT must be configured with
   // an Internet route for this to work - though no Internet traffic
   // should be created.
 
-  var pac = 'data:text/plain,' +
-            'var pacUseMultihomedDNS = true;\n' +
-            'function FindProxyForURL(url, host) {' +
-            ' return "PROXY " + myIpAddress() + ":1234";' +
-            '}';
+  var pac =
+    "data:text/plain," +
+    "var pacUseMultihomedDNS = true;\n" +
+    "function FindProxyForURL(url, host) {" +
+    ' return "PROXY " + myIpAddress() + ":1234";' +
+    "}";
 
   // no traffic to this IP is ever sent, it is just a public IP that
   // does not require DNS to determine a route.
   var channel = NetUtil.newChannel({
     uri: "http://192.0.43.10/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   prefs.setIntPref("network.proxy.type", 2);
   prefs.setCharPref("network.proxy.autoconfig_url", pac);
@@ -798,8 +869,7 @@ function run_myipaddress_test()
   var req = pps.asyncResolve(channel, 0, cb);
 }
 
-function myipaddress_callback(pi)
-{
+function myipaddress_callback(pi) {
   Assert.notEqual(pi, null);
   Assert.equal(pi.type, "http");
   Assert.equal(pi.port, 1234);
@@ -812,21 +882,21 @@ function myipaddress_callback(pi)
   run_myipaddress_test_2();
 }
 
-function run_myipaddress_test_2()
-{
+function run_myipaddress_test_2() {
   // test that myIPAddress() can be used outside of the scope of
   // FindProxyForURL(). bug 829646.
 
-  var pac = 'data:text/plain,' +
-            'var pacUseMultihomedDNS = true;\n' +
-            'var myaddr = myIpAddress(); ' +
-            'function FindProxyForURL(url, host) {' +
-            ' return "PROXY " + myaddr + ":5678";' +
-            '}';
+  var pac =
+    "data:text/plain," +
+    "var pacUseMultihomedDNS = true;\n" +
+    "var myaddr = myIpAddress(); " +
+    "function FindProxyForURL(url, host) {" +
+    ' return "PROXY " + myaddr + ":5678";' +
+    "}";
 
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   prefs.setIntPref("network.proxy.type", 2);
   prefs.setCharPref("network.proxy.autoconfig_url", pac);
@@ -836,8 +906,7 @@ function run_myipaddress_test_2()
   var req = pps.asyncResolve(channel, 0, cb);
 }
 
-function myipaddress2_callback(pi)
-{
+function myipaddress2_callback(pi) {
   Assert.notEqual(pi, null);
   Assert.equal(pi.type, "http");
   Assert.equal(pi.port, 5678);
@@ -850,15 +919,14 @@ function myipaddress2_callback(pi)
   run_failed_script_test();
 }
 
-function run_failed_script_test()
-{
+function run_failed_script_test() {
   // test to make sure we go direct with invalid PAC
-  var pac = 'data:text/plain,' +
-            '\nfor(;\n';
+  // eslint-disable-next-line no-useless-concat
+  var pac = "data:text/plain," + "\nfor(;\n";
 
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   prefs.setIntPref("network.proxy.type", 2);
   prefs.setCharPref("network.proxy.autoconfig_url", pac);
@@ -870,8 +938,7 @@ function run_failed_script_test()
 
 var directFilter;
 
-function failed_script_callback(pi)
-{
+function failed_script_callback(pi) {
   // we should go direct
   Assert.equal(pi, null);
 
@@ -887,18 +954,18 @@ function failed_script_callback(pi)
 
   var chan = NetUtil.newChannel({
     uri: "http://127.0.0.1:7247",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
-  chan.asyncOpen2(directFilterListener);
+  chan.asyncOpen(directFilterListener);
 }
 
 var directFilterListener = {
-  onModifyRequestCalled : false,
+  onModifyRequestCalled: false,
 
-  onStartRequest: function test_onStart(request, ctx) {  },
-  onDataAvailable: function test_OnData() { },
+  onStartRequest: function test_onStart(request) {},
+  onDataAvailable: function test_OnData() {},
 
-  onStopRequest: function test_onStop(request, ctx, status) {
+  onStopRequest: function test_onStop(request, status) {
     // check on the PI from the channel itself
     request.QueryInterface(Ci.nsIProxiedChannel);
     check_proxy(request.proxyInfo, "http", "127.0.0.1", 7246, 0, 0, false);
@@ -913,30 +980,32 @@ var directFilterListener = {
     run_isresolvable_test();
   },
 
-   observe: function(subject, topic, data) {
-     if (topic === "http-on-modify-request" &&
-         subject instanceof Ci.nsIHttpChannel &&
-         subject instanceof Ci.nsIProxiedChannel) {
-       check_proxy(subject.proxyInfo, "http", "127.0.0.1", 7246, 0, 0, false);
-       this.onModifyRequestCalled = true;
-     }
-   }
+  observe(subject, topic, data) {
+    if (
+      topic === "http-on-modify-request" &&
+      subject instanceof Ci.nsIHttpChannel &&
+      subject instanceof Ci.nsIProxiedChannel
+    ) {
+      check_proxy(subject.proxyInfo, "http", "127.0.0.1", 7246, 0, 0, false);
+      this.onModifyRequestCalled = true;
+    }
+  },
 };
 
-function run_isresolvable_test()
-{
+function run_isresolvable_test() {
   // test a non resolvable host in the pac file
 
-  var pac = 'data:text/plain,' +
-            'function FindProxyForURL(url, host) {' +
-            ' if (isResolvable("nonexistant.lan.onion"))' +
-            '   return "DIRECT";' +
-            ' return "PROXY 127.0.0.1:1234";' +
-            '}';
+  var pac =
+    "data:text/plain," +
+    "function FindProxyForURL(url, host) {" +
+    ' if (isResolvable("nonexistant.lan.onion"))' +
+    '   return "DIRECT";' +
+    ' return "PROXY 127.0.0.1:1234";' +
+    "}";
 
   var channel = NetUtil.newChannel({
     uri: "http://www.mozilla.org/",
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   });
   prefs.setIntPref("network.proxy.type", 2);
   prefs.setCharPref("network.proxy.autoconfig_url", pac);
@@ -946,13 +1015,41 @@ function run_isresolvable_test()
   var req = pps.asyncResolve(channel, 0, cb);
 }
 
-function isresolvable_callback(pi)
-{
+function isresolvable_callback(pi) {
   Assert.notEqual(pi, null);
   Assert.equal(pi.type, "http");
   Assert.equal(pi.port, 1234);
   Assert.equal(pi.host, "127.0.0.1");
 
+  run_localhost_pac();
+}
+
+function run_localhost_pac() {
+  // test localhost in the pac file
+
+  var pac =
+    "data:text/plain," +
+    "function FindProxyForURL(url, host) {" +
+    ' return "PROXY totallycrazy:1234";' +
+    "}";
+
+  // Use default filter list string for "no_proxies_on" ("localhost, 127.0.0.1")
+  prefs.clearUserPref("network.proxy.no_proxies_on");
+  var channel = NetUtil.newChannel({
+    uri: "http://localhost/",
+    loadUsingSystemPrincipal: true,
+  });
+  prefs.setIntPref("network.proxy.type", 2);
+  prefs.setCharPref("network.proxy.autoconfig_url", pac);
+
+  var cb = new resolveCallback();
+  cb.nextFunction = localhost_callback;
+  var req = pps.asyncResolve(channel, 0, cb);
+}
+
+function localhost_callback(pi) {
+  Assert.equal(pi, null); // no proxy!
+
   prefs.setIntPref("network.proxy.type", 0);
   do_test_finished();
 }
