@@ -1,4 +1,5 @@
 // Exercise Unix domain sockets.
+"use strict";
 
 var CC = Components.Constructor;
 
@@ -87,7 +88,7 @@ function test_echo() {
   info("creating socket: " + socketName.path);
   let server = new UnixServerSocket(socketName, allPermissions, -1);
   server.asyncListen({
-    onSocketAccepted: function(aServ, aTransport) {
+    onSocketAccepted(aServ, aTransport) {
       info("called test_echo's onSocketAccepted");
       log += "a";
 
@@ -133,7 +134,7 @@ function test_echo() {
       );
     },
 
-    onStopListening: function(aServ, aStatus) {
+    onStopListening(aServ, aStatus) {
       info("called test_echo's onStopListening");
       log += "s";
 
@@ -528,10 +529,7 @@ function test_keep_when_offline() {
 
   // Create a listening socket.
   let listener = new UnixServerSocket(socketName, allPermissions, -1);
-  listener.asyncListen({
-    onSocketAccepted: onAccepted,
-    onStopListening: onStopListening,
-  });
+  listener.asyncListen({ onSocketAccepted: onAccepted, onStopListening });
 
   // Connect a client socket to the listening socket.
   let client = socketTransportService.createUnixDomainTransport(socketName);
