@@ -4,11 +4,11 @@
 
 #include "nsHtml5Highlighter.h"
 #include "nsDebug.h"
-#include "nsHtml5Tokenizer.h"
 #include "nsHtml5AttributeName.h"
+#include "nsHtml5Tokenizer.h"
+#include "nsHtml5ViewSourceUtils.h"
 #include "nsString.h"
 #include "nsThreadUtils.h"
-#include "nsHtml5ViewSourceUtils.h"
 
 #include "mozilla/Attributes.h"
 #include "mozilla/Preferences.h"
@@ -19,80 +19,74 @@ using namespace mozilla;
 // the size of 16 tokens on cnn.com.
 #define NS_HTML5_HIGHLIGHTER_PRE_BREAK_THRESHOLD 1300
 
-char16_t nsHtml5Highlighter::sComment[] =
-  { 'c', 'o', 'm', 'm', 'e', 'n', 't', 0 };
+char16_t nsHtml5Highlighter::sComment[] = {'c', 'o', 'm', 'm',
+                                           'e', 'n', 't', 0};
 
-char16_t nsHtml5Highlighter::sCdata[] =
-  { 'c', 'd', 'a', 't', 'a', 0 };
+char16_t nsHtml5Highlighter::sCdata[] = {'c', 'd', 'a', 't', 'a', 0};
 
-char16_t nsHtml5Highlighter::sEntity[] =
-  { 'e', 'n', 't', 'i', 't', 'y', 0 };
+char16_t nsHtml5Highlighter::sEntity[] = {'e', 'n', 't', 'i', 't', 'y', 0};
 
-char16_t nsHtml5Highlighter::sEndTag[] =
-  { 'e', 'n', 'd', '-', 't', 'a', 'g', 0 };
+char16_t nsHtml5Highlighter::sEndTag[] = {'e', 'n', 'd', '-', 't', 'a', 'g', 0};
 
-char16_t nsHtml5Highlighter::sStartTag[] =
-  { 's', 't', 'a', 'r', 't', '-', 't', 'a', 'g', 0 };
+char16_t nsHtml5Highlighter::sStartTag[] = {'s', 't', 'a', 'r', 't',
+                                            '-', 't', 'a', 'g', 0};
 
-char16_t nsHtml5Highlighter::sAttributeName[] =
-  { 'a', 't', 't', 'r', 'i', 'b', 'u', 't', 'e', '-', 'n', 'a', 'm', 'e', 0 };
+char16_t nsHtml5Highlighter::sAttributeName[] = {
+    'a', 't', 't', 'r', 'i', 'b', 'u', 't', 'e', '-', 'n', 'a', 'm', 'e', 0};
 
-char16_t nsHtml5Highlighter::sAttributeValue[] =
-  { 'a', 't', 't', 'r', 'i', 'b', 'u', 't', 'e', '-',
-    'v', 'a', 'l', 'u', 'e', 0 };
+char16_t nsHtml5Highlighter::sAttributeValue[] = {'a', 't', 't', 'r', 'i', 'b',
+                                                  'u', 't', 'e', '-', 'v', 'a',
+                                                  'l', 'u', 'e', 0};
 
-char16_t nsHtml5Highlighter::sDoctype[] =
-  { 'd', 'o', 'c', 't', 'y', 'p', 'e', 0 };
+char16_t nsHtml5Highlighter::sDoctype[] = {'d', 'o', 'c', 't',
+                                           'y', 'p', 'e', 0};
 
-char16_t nsHtml5Highlighter::sPi[] =
-  { 'p', 'i', 0 };
+char16_t nsHtml5Highlighter::sPi[] = {'p', 'i', 0};
 
 nsHtml5Highlighter::nsHtml5Highlighter(nsAHtml5TreeOpSink* aOpSink)
-  : mState(nsHtml5Tokenizer::DATA)
-  , mCStart(INT32_MAX)
-  , mPos(0)
-  , mLineNumber(1)
-  , mInlinesOpen(0)
-  , mInCharacters(false)
-  , mBuffer(nullptr)
-  , mOpSink(aOpSink)
-  , mCurrentRun(nullptr)
-  , mAmpersand(nullptr)
-  , mSlash(nullptr)
-  , mHandles(
-      MakeUnique<nsIContent* []>(NS_HTML5_HIGHLIGHTER_HANDLE_ARRAY_LENGTH))
-  , mHandlesUsed(0)
-  , mSeenBase(false)
-{
+    : mState(nsHtml5Tokenizer::DATA),
+      mCStart(INT32_MAX),
+      mPos(0),
+      mLineNumber(1),
+      mInlinesOpen(0),
+      mInCharacters(false),
+      mBuffer(nullptr),
+      mOpSink(aOpSink),
+      mCurrentRun(nullptr),
+      mAmpersand(nullptr),
+      mSlash(nullptr),
+      mHandles(
+          MakeUnique<nsIContent*[]>(NS_HTML5_HIGHLIGHTER_HANDLE_ARRAY_LENGTH)),
+      mHandlesUsed(0),
+      mSeenBase(false) {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
 }
 
-nsHtml5Highlighter::~nsHtml5Highlighter()
-{
+nsHtml5Highlighter::~nsHtml5Highlighter() {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
 }
 
-void
-nsHtml5Highlighter::Start(const nsAutoString& aTitle)
-{
+void nsHtml5Highlighter::Start(const nsAutoString& aTitle) {
   // Doctype
-  mOpQueue.AppendElement()->Init(nsGkAtoms::html, EmptyString(), EmptyString());
+  opAppendDoctypeToDocument operation(nsGkAtoms::html, EmptyString(),
+                                      EmptyString());
+  mOpQueue.AppendElement()->Init(mozilla::AsVariant(operation));
 
-  mOpQueue.AppendElement()->Init(STANDARDS_MODE);
+  mOpQueue.AppendElement()->Init(mozilla::AsVariant(STANDARDS_MODE));
 
   // <html> uses NS_NewHTMLSharedElement creator
   nsIContent** root =
-    CreateElement(nsGkAtoms::html, nullptr, nullptr, NS_NewHTMLSharedElement);
-  mOpQueue.AppendElement()->Init(eTreeOpAppendToDocument, root);
+      CreateElement(nsGkAtoms::html, nullptr, nullptr, NS_NewHTMLSharedElement);
+  opAppendToDocument appendOp(root);
+  mOpQueue.AppendElement()->Init(mozilla::AsVariant(appendOp));
   mStack.AppendElement(root);
 
   // <head> uses NS_NewHTMLSharedElement creator
   Push(nsGkAtoms::head, nullptr, NS_NewHTMLSharedElement);
 
-  Push(nsGkAtoms::meta,
-       nsHtml5ViewSourceUtils::NewMetaViewportAttributes(),
+  Push(nsGkAtoms::meta, nsHtml5ViewSourceUtils::NewMetaViewportAttributes(),
        NS_NewHTMLMetaElement);
-  Pop(); // meta
+  Pop();  // meta
 
   Push(nsGkAtoms::title, nullptr, NS_NewHTMLTitleElement);
   // XUL will add the "Source of: " prefix.
@@ -101,20 +95,19 @@ nsHtml5Highlighter::Start(const nsAutoString& aTitle)
     length = INT32_MAX;
   }
   AppendCharacters(aTitle.BeginReading(), 0, (int32_t)length);
-  Pop(); // title
+  Pop();  // title
 
-  Push(nsGkAtoms::link,
-       nsHtml5ViewSourceUtils::NewLinkAttributes(),
+  Push(nsGkAtoms::link, nsHtml5ViewSourceUtils::NewLinkAttributes(),
        NS_NewHTMLLinkElement);
 
-  mOpQueue.AppendElement()->Init(eTreeOpUpdateStyleSheet, CurrentNode());
+  opUpdateStyleSheet updateOp(CurrentNode());
+  mOpQueue.AppendElement()->Init(mozilla::AsVariant(updateOp));
 
-  Pop(); // link
+  Pop();  // link
 
-  Pop(); // head
+  Pop();  // head
 
-  Push(nsGkAtoms::body,
-       nsHtml5ViewSourceUtils::NewBodyAttributes(),
+  Push(nsGkAtoms::body, nsHtml5ViewSourceUtils::NewBodyAttributes(),
        NS_NewHTMLBodyElement);
 
   nsHtml5HtmlAttributes* preAttrs = new nsHtml5HtmlAttributes(0);
@@ -124,12 +117,11 @@ nsHtml5Highlighter::Start(const nsAutoString& aTitle)
 
   StartCharacters();
 
-  mOpQueue.AppendElement()->Init(eTreeOpStartLayout);
+  mOpQueue.AppendElement()->Init(mozilla::AsVariant(opStartLayout()));
 }
 
-int32_t
-nsHtml5Highlighter::Transition(int32_t aState, bool aReconsume, int32_t aPos)
-{
+int32_t nsHtml5Highlighter::Transition(int32_t aState, bool aReconsume,
+                                       int32_t aPos) {
   mPos = aPos;
   switch (mState) {
     case nsHtml5Tokenizer::SCRIPT_DATA:
@@ -153,7 +145,7 @@ nsHtml5Highlighter::Transition(int32_t aState, bool aReconsume, int32_t aPos)
           StartSpan(sStartTag);
           break;
         case nsHtml5Tokenizer::DATA:
-          FinishTag(); // DATA
+          FinishTag();  // DATA
           break;
         case nsHtml5Tokenizer::PROCESSING_INSTRUCTION:
           AddClass(sPi);
@@ -163,11 +155,11 @@ nsHtml5Highlighter::Transition(int32_t aState, bool aReconsume, int32_t aPos)
     case nsHtml5Tokenizer::TAG_NAME:
       switch (aState) {
         case nsHtml5Tokenizer::BEFORE_ATTRIBUTE_NAME:
-          EndSpanOrA(); // nsHtml5Tokenizer::TAG_NAME
+          EndSpanOrA();  // nsHtml5Tokenizer::TAG_NAME
           break;
         case nsHtml5Tokenizer::SELF_CLOSING_START_TAG:
-          EndSpanOrA(); // nsHtml5Tokenizer::TAG_NAME
-          StartSpan(); // for highlighting the slash
+          EndSpanOrA();  // nsHtml5Tokenizer::TAG_NAME
+          StartSpan();   // for highlighting the slash
           mSlash = CurrentNode();
           break;
         default:
@@ -181,7 +173,7 @@ nsHtml5Highlighter::Transition(int32_t aState, bool aReconsume, int32_t aPos)
           StartSpan(sAttributeName);
           break;
         case nsHtml5Tokenizer::SELF_CLOSING_START_TAG:
-          StartSpan(); // for highlighting the slash
+          StartSpan();  // for highlighting the slash
           mSlash = CurrentNode();
           break;
         default:
@@ -193,11 +185,11 @@ nsHtml5Highlighter::Transition(int32_t aState, bool aReconsume, int32_t aPos)
       switch (aState) {
         case nsHtml5Tokenizer::AFTER_ATTRIBUTE_NAME:
         case nsHtml5Tokenizer::BEFORE_ATTRIBUTE_VALUE:
-          EndSpanOrA(); // nsHtml5Tokenizer::BEFORE_ATTRIBUTE_NAME
+          EndSpanOrA();  // nsHtml5Tokenizer::BEFORE_ATTRIBUTE_NAME
           break;
         case nsHtml5Tokenizer::SELF_CLOSING_START_TAG:
-          EndSpanOrA(); // nsHtml5Tokenizer::BEFORE_ATTRIBUTE_NAME
-          StartSpan(); // for highlighting the slash
+          EndSpanOrA();  // nsHtml5Tokenizer::BEFORE_ATTRIBUTE_NAME
+          StartSpan();   // for highlighting the slash
           mSlash = CurrentNode();
           break;
         default:
@@ -228,11 +220,11 @@ nsHtml5Highlighter::Transition(int32_t aState, bool aReconsume, int32_t aPos)
           break;
         case nsHtml5Tokenizer::CONSUME_CHARACTER_REFERENCE:
           StartSpan();
-          StartSpan(); // for ampersand itself
+          StartSpan();  // for ampersand itself
           mAmpersand = CurrentNode();
           break;
         default:
-          NS_NOTREACHED("Impossible transition.");
+          MOZ_ASSERT_UNREACHABLE("Impossible transition.");
           break;
       }
       break;
@@ -241,7 +233,7 @@ nsHtml5Highlighter::Transition(int32_t aState, bool aReconsume, int32_t aPos)
         case nsHtml5Tokenizer::BEFORE_ATTRIBUTE_NAME:
           break;
         case nsHtml5Tokenizer::SELF_CLOSING_START_TAG:
-          StartSpan(); // for highlighting the slash
+          StartSpan();  // for highlighting the slash
           mSlash = CurrentNode();
           break;
         default:
@@ -250,7 +242,7 @@ nsHtml5Highlighter::Transition(int32_t aState, bool aReconsume, int32_t aPos)
       }
       break;
     case nsHtml5Tokenizer::SELF_CLOSING_START_TAG:
-      EndSpanOrA(); // end the slash highlight
+      EndSpanOrA();  // end the slash highlight
       switch (aState) {
         case nsHtml5Tokenizer::BEFORE_ATTRIBUTE_NAME:
           break;
@@ -266,7 +258,7 @@ nsHtml5Highlighter::Transition(int32_t aState, bool aReconsume, int32_t aPos)
           break;
         case nsHtml5Tokenizer::CONSUME_CHARACTER_REFERENCE:
           StartSpan();
-          StartSpan(); // for ampersand itself
+          StartSpan();  // for ampersand itself
           mAmpersand = CurrentNode();
           break;
         default:
@@ -277,7 +269,7 @@ nsHtml5Highlighter::Transition(int32_t aState, bool aReconsume, int32_t aPos)
     case nsHtml5Tokenizer::AFTER_ATTRIBUTE_NAME:
       switch (aState) {
         case nsHtml5Tokenizer::SELF_CLOSING_START_TAG:
-          StartSpan(); // for highlighting the slash
+          StartSpan();  // for highlighting the slash
           mSlash = CurrentNode();
           break;
         case nsHtml5Tokenizer::BEFORE_ATTRIBUTE_VALUE:
@@ -290,8 +282,8 @@ nsHtml5Highlighter::Transition(int32_t aState, bool aReconsume, int32_t aPos)
           break;
       }
       break;
-      // most comment states are omitted, because they don't matter to
-      // highlighting
+    // most comment states are omitted, because they don't matter to
+    // highlighting
     case nsHtml5Tokenizer::COMMENT_START:
     case nsHtml5Tokenizer::COMMENT_END:
     case nsHtml5Tokenizer::COMMENT_END_BANG:
@@ -312,7 +304,7 @@ nsHtml5Highlighter::Transition(int32_t aState, bool aReconsume, int32_t aPos)
       }
       break;
     case nsHtml5Tokenizer::CONSUME_CHARACTER_REFERENCE:
-      EndSpanOrA(); // the span for the ampersand
+      EndSpanOrA();  // the span for the ampersand
       switch (aState) {
         case nsHtml5Tokenizer::CONSUME_NCR:
         case nsHtml5Tokenizer::CHARACTER_REFERENCE_HILO_LOOKUP:
@@ -362,7 +354,7 @@ nsHtml5Highlighter::Transition(int32_t aState, bool aReconsume, int32_t aPos)
     case nsHtml5Tokenizer::RAWTEXT_RCDATA_LESS_THAN_SIGN:
       if (aState == nsHtml5Tokenizer::NON_DATA_END_TAG_NAME) {
         FlushCurrent();
-        StartSpan(); // don't know if it is "end-tag" yet :-(
+        StartSpan();  // don't know if it is "end-tag" yet :-(
         break;
       }
       EndSpanOrA();
@@ -377,10 +369,10 @@ nsHtml5Highlighter::Transition(int32_t aState, bool aReconsume, int32_t aPos)
         case nsHtml5Tokenizer::SELF_CLOSING_START_TAG:
           AddClass(sEndTag);
           EndSpanOrA();
-          StartSpan(); // for highlighting the slash
+          StartSpan();  // for highlighting the slash
           mSlash = CurrentNode();
           break;
-        case nsHtml5Tokenizer::DATA: // yes, as a result of emitting the token
+        case nsHtml5Tokenizer::DATA:  // yes, as a result of emitting the token
           AddClass(sEndTag);
           FinishTag();
           break;
@@ -393,7 +385,7 @@ nsHtml5Highlighter::Transition(int32_t aState, bool aReconsume, int32_t aPos)
     case nsHtml5Tokenizer::SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN:
       if (aState == nsHtml5Tokenizer::NON_DATA_END_TAG_NAME) {
         FlushCurrent();
-        StartSpan(); // don't know if it is "end-tag" yet :-(
+        StartSpan();  // don't know if it is "end-tag" yet :-(
         break;
       }
       FinishTag();
@@ -440,9 +432,7 @@ nsHtml5Highlighter::Transition(int32_t aState, bool aReconsume, int32_t aPos)
   return aState;
 }
 
-void
-nsHtml5Highlighter::End()
-{
+void nsHtml5Highlighter::End() {
   switch (mState) {
     case nsHtml5Tokenizer::COMMENT_END:
     case nsHtml5Tokenizer::COMMENT_END_BANG:
@@ -480,64 +470,50 @@ nsHtml5Highlighter::End()
   }
   nsHtml5TreeOperation* treeOp = mOpQueue.AppendElement();
   NS_ASSERTION(treeOp, "Tree op allocation failed.");
-  treeOp->Init(eTreeOpStreamEnded);
+  treeOp->Init(mozilla::AsVariant(opStreamEnded()));
   FlushOps();
 }
 
-void
-nsHtml5Highlighter::SetBuffer(nsHtml5UTF16Buffer* aBuffer)
-{
-  NS_PRECONDITION(!mBuffer, "Old buffer still here!");
+void nsHtml5Highlighter::SetBuffer(nsHtml5UTF16Buffer* aBuffer) {
+  MOZ_ASSERT(!mBuffer, "Old buffer still here!");
   mBuffer = aBuffer;
   mCStart = aBuffer->getStart();
 }
 
-void
-nsHtml5Highlighter::DropBuffer(int32_t aPos)
-{
-  NS_PRECONDITION(mBuffer, "No buffer to drop!");
+void nsHtml5Highlighter::DropBuffer(int32_t aPos) {
+  MOZ_ASSERT(mBuffer, "No buffer to drop!");
   mPos = aPos;
   FlushChars();
   mBuffer = nullptr;
 }
 
-void
-nsHtml5Highlighter::StartSpan()
-{
+void nsHtml5Highlighter::StartSpan() {
   FlushChars();
   Push(nsGkAtoms::span, nullptr, NS_NewHTMLSpanElement);
   ++mInlinesOpen;
 }
 
-void
-nsHtml5Highlighter::StartSpan(const char16_t* aClass)
-{
+void nsHtml5Highlighter::StartSpan(const char16_t* aClass) {
   StartSpan();
   AddClass(aClass);
 }
 
-void
-nsHtml5Highlighter::EndSpanOrA()
-{
+void nsHtml5Highlighter::EndSpanOrA() {
   FlushChars();
   Pop();
   --mInlinesOpen;
 }
 
-void
-nsHtml5Highlighter::StartCharacters()
-{
-  NS_PRECONDITION(!mInCharacters, "Already in characters!");
+void nsHtml5Highlighter::StartCharacters() {
+  MOZ_ASSERT(!mInCharacters, "Already in characters!");
   FlushChars();
   Push(nsGkAtoms::span, nullptr, NS_NewHTMLSpanElement);
   mCurrentRun = CurrentNode();
   mInCharacters = true;
 }
 
-void
-nsHtml5Highlighter::EndCharactersAndStartMarkupRun()
-{
-  NS_PRECONDITION(mInCharacters, "Not in characters!");
+void nsHtml5Highlighter::EndCharactersAndStartMarkupRun() {
+  MOZ_ASSERT(mInCharacters, "Not in characters!");
   FlushChars();
   Pop();
   mInCharacters = false;
@@ -546,30 +522,24 @@ nsHtml5Highlighter::EndCharactersAndStartMarkupRun()
   mCurrentRun = CurrentNode();
 }
 
-void
-nsHtml5Highlighter::StartA()
-{
+void nsHtml5Highlighter::StartA() {
   FlushChars();
   Push(nsGkAtoms::a, nullptr, NS_NewHTMLAnchorElement);
   AddClass(sAttributeValue);
   ++mInlinesOpen;
 }
 
-void
-nsHtml5Highlighter::FinishTag()
-{
+void nsHtml5Highlighter::FinishTag() {
   while (mInlinesOpen > 1) {
     EndSpanOrA();
   }
-  FlushCurrent(); // >
-  EndSpanOrA(); // DATA
+  FlushCurrent();  // >
+  EndSpanOrA();    // DATA
   NS_ASSERTION(!mInlinesOpen, "mInlinesOpen got out of sync!");
   StartCharacters();
 }
 
-void
-nsHtml5Highlighter::FlushChars()
-{
+void nsHtml5Highlighter::FlushChars() {
   if (mCStart < mPos) {
     char16_t* buf = mBuffer->getBuffer();
     int32_t i = mCStart;
@@ -583,7 +553,7 @@ nsHtml5Highlighter::FlushChars()
           // the input data, because there are no reparses in the View Source
           // case, so we won't need the original data in the buffer anymore.
           buf[i] = '\n';
-          MOZ_FALLTHROUGH;
+          [[fallthrough]];
         case '\n': {
           ++i;
           if (mCStart < i) {
@@ -595,7 +565,8 @@ nsHtml5Highlighter::FlushChars()
           Push(nsGkAtoms::span, nullptr, NS_NewHTMLSpanElement);
           nsHtml5TreeOperation* treeOp = mOpQueue.AppendElement();
           NS_ASSERTION(treeOp, "Tree op allocation failed.");
-          treeOp->InitAddLineNumberId(CurrentNode(), mLineNumber);
+          opAddLineNumberId operation(CurrentNode(), mLineNumber);
+          treeOp->Init(mozilla::AsVariant(operation));
           Pop();
           break;
         }
@@ -612,16 +583,12 @@ nsHtml5Highlighter::FlushChars()
   }
 }
 
-void
-nsHtml5Highlighter::FlushCurrent()
-{
+void nsHtml5Highlighter::FlushCurrent() {
   mPos++;
   FlushChars();
 }
 
-bool
-nsHtml5Highlighter::FlushOps()
-{
+bool nsHtml5Highlighter::FlushOps() {
   bool hasOps = !mOpQueue.IsEmpty();
   if (hasOps) {
     mOpSink->MoveOpsFrom(mOpQueue);
@@ -629,10 +596,8 @@ nsHtml5Highlighter::FlushOps()
   return hasOps;
 }
 
-void
-nsHtml5Highlighter::MaybeLinkifyAttributeValue(nsHtml5AttributeName* aName,
-                                               nsHtml5String aValue)
-{
+void nsHtml5Highlighter::MaybeLinkifyAttributeValue(nsHtml5AttributeName* aName,
+                                                    nsHtml5String aValue) {
   if (!(nsHtml5AttributeName::ATTR_HREF == aName ||
         nsHtml5AttributeName::ATTR_SRC == aName ||
         nsHtml5AttributeName::ATTR_ACTION == aName ||
@@ -646,18 +611,15 @@ nsHtml5Highlighter::MaybeLinkifyAttributeValue(nsHtml5AttributeName* aName,
   AddViewSourceHref(aValue);
 }
 
-void
-nsHtml5Highlighter::CompletedNamedCharacterReference()
-{
+void nsHtml5Highlighter::CompletedNamedCharacterReference() {
   AddClass(sEntity);
 }
 
-nsIContent**
-nsHtml5Highlighter::AllocateContentHandle()
-{
+nsIContent** nsHtml5Highlighter::AllocateContentHandle() {
   if (mHandlesUsed == NS_HTML5_HIGHLIGHTER_HANDLE_ARRAY_LENGTH) {
-    mOldHandles.AppendElement(Move(mHandles));
-    mHandles = MakeUnique<nsIContent*[]>(NS_HTML5_HIGHLIGHTER_HANDLE_ARRAY_LENGTH);
+    mOldHandles.AppendElement(std::move(mHandles));
+    mHandles =
+        MakeUnique<nsIContent*[]>(NS_HTML5_HIGHLIGHTER_HANDLE_ARRAY_LENGTH);
     mHandlesUsed = 0;
   }
 #ifdef DEBUG
@@ -666,93 +628,67 @@ nsHtml5Highlighter::AllocateContentHandle()
   return &mHandles[mHandlesUsed++];
 }
 
-nsIContent**
-nsHtml5Highlighter::CreateElement(
-  nsAtom* aName,
-  nsHtml5HtmlAttributes* aAttributes,
-  nsIContent** aIntendedParent,
-  mozilla::dom::HTMLContentCreatorFunction aCreator)
-{
-  NS_PRECONDITION(aName, "Got null name.");
-  nsHtml5ContentCreatorFunction creator;
-  creator.html = aCreator;
+nsIContent** nsHtml5Highlighter::CreateElement(
+    nsAtom* aName, nsHtml5HtmlAttributes* aAttributes,
+    nsIContent** aIntendedParent,
+    mozilla::dom::HTMLContentCreatorFunction aCreator) {
+  MOZ_ASSERT(aName, "Got null name.");
   nsIContent** content = AllocateContentHandle();
-  mOpQueue.AppendElement()->Init(kNameSpaceID_XHTML,
-                                 aName,
-                                 aAttributes,
-                                 content,
-                                 aIntendedParent,
-                                 true,
-                                 creator);
+  opCreateHTMLElement opeation(content, aName, aAttributes, aCreator,
+                               aIntendedParent,
+                               mozilla::dom::FROM_PARSER_NETWORK);
+  mOpQueue.AppendElement()->Init(mozilla::AsVariant(opeation));
   return content;
 }
 
-nsIContent**
-nsHtml5Highlighter::CurrentNode()
-{
-  NS_PRECONDITION(mStack.Length() >= 1, "Must have something on stack.");
+nsIContent** nsHtml5Highlighter::CurrentNode() {
+  MOZ_ASSERT(mStack.Length() >= 1, "Must have something on stack.");
   return mStack[mStack.Length() - 1];
 }
 
-void
-nsHtml5Highlighter::Push(nsAtom* aName,
-                         nsHtml5HtmlAttributes* aAttributes,
-                         mozilla::dom::HTMLContentCreatorFunction aCreator)
-{
-  NS_PRECONDITION(mStack.Length() >= 1, "Pushing without root.");
-  nsIContent** elt = CreateElement(
-    aName, aAttributes, CurrentNode(), aCreator); // Don't inline below!
-  mOpQueue.AppendElement()->Init(eTreeOpAppend, elt, CurrentNode());
+void nsHtml5Highlighter::Push(
+    nsAtom* aName, nsHtml5HtmlAttributes* aAttributes,
+    mozilla::dom::HTMLContentCreatorFunction aCreator) {
+  MOZ_ASSERT(mStack.Length() >= 1, "Pushing without root.");
+  nsIContent** elt = CreateElement(aName, aAttributes, CurrentNode(),
+                                   aCreator);  // Don't inline below!
+  opAppend operation(elt, CurrentNode());
+  mOpQueue.AppendElement()->Init(mozilla::AsVariant(operation));
   mStack.AppendElement(elt);
 }
 
-void
-nsHtml5Highlighter::Pop()
-{
-  NS_PRECONDITION(mStack.Length() >= 2, "Popping when stack too short.");
-  mStack.RemoveElementAt(mStack.Length() - 1);
+void nsHtml5Highlighter::Pop() {
+  MOZ_ASSERT(mStack.Length() >= 2, "Popping when stack too short.");
+  mStack.RemoveLastElement();
 }
 
-void
-nsHtml5Highlighter::AppendCharacters(const char16_t* aBuffer,
-                                     int32_t aStart,
-                                     int32_t aLength)
-{
-  NS_PRECONDITION(aBuffer, "Null buffer");
+void nsHtml5Highlighter::AppendCharacters(const char16_t* aBuffer,
+                                          int32_t aStart, int32_t aLength) {
+  MOZ_ASSERT(aBuffer, "Null buffer");
 
   char16_t* bufferCopy = new char16_t[aLength];
   memcpy(bufferCopy, aBuffer + aStart, aLength * sizeof(char16_t));
 
-  mOpQueue.AppendElement()->Init(eTreeOpAppendText,
-                                 bufferCopy,
-                                 aLength,
-                                 CurrentNode());
+  opAppendText operation(CurrentNode(), bufferCopy, aLength);
+  mOpQueue.AppendElement()->Init(mozilla::AsVariant(operation));
 }
 
-
-void
-nsHtml5Highlighter::AddClass(const char16_t* aClass)
-{
-  mOpQueue.AppendElement()->InitAddClass(CurrentNode(), aClass);
+void nsHtml5Highlighter::AddClass(const char16_t* aClass) {
+  opAddClass operation(CurrentNode(), (char16_t*)aClass);
+  mOpQueue.AppendElement()->Init(mozilla::AsVariant(operation));
 }
 
-void
-nsHtml5Highlighter::AddViewSourceHref(nsHtml5String aValue)
-{
+void nsHtml5Highlighter::AddViewSourceHref(nsHtml5String aValue) {
   char16_t* bufferCopy = new char16_t[aValue.Length() + 1];
   aValue.CopyToBuffer(bufferCopy);
   bufferCopy[aValue.Length()] = 0;
 
-  mOpQueue.AppendElement()->Init(eTreeOpAddViewSourceHref,
-                                 bufferCopy,
-                                 aValue.Length(),
-                                 CurrentNode());
+  opAddViewSourceHref operation(CurrentNode(), bufferCopy, aValue.Length());
+  mOpQueue.AppendElement()->Init(mozilla::AsVariant(operation));
 }
 
-void
-nsHtml5Highlighter::AddBase(nsHtml5String aValue)
-{
-  if(mSeenBase) {
+void nsHtml5Highlighter::AddBase(nsHtml5String aValue) {
+  if (mSeenBase) {
     return;
   }
   mSeenBase = true;
@@ -760,63 +696,55 @@ nsHtml5Highlighter::AddBase(nsHtml5String aValue)
   aValue.CopyToBuffer(bufferCopy);
   bufferCopy[aValue.Length()] = 0;
 
-  mOpQueue.AppendElement()->Init(eTreeOpAddViewSourceBase,
-                                 bufferCopy,
-                                 aValue.Length());
+  opAddViewSourceBase operation(bufferCopy, aValue.Length());
+  mOpQueue.AppendElement()->Init(mozilla::AsVariant(operation));
 }
 
-void
-nsHtml5Highlighter::AddErrorToCurrentNode(const char* aMsgId)
-{
+void nsHtml5Highlighter::AddErrorToCurrentNode(const char* aMsgId) {
   nsHtml5TreeOperation* treeOp = mOpQueue.AppendElement();
   NS_ASSERTION(treeOp, "Tree op allocation failed.");
-  treeOp->Init(CurrentNode(), aMsgId);
+  opAddErrorType operation(CurrentNode(), (char*)aMsgId);
+  treeOp->Init(mozilla::AsVariant(operation));
 }
 
-void
-nsHtml5Highlighter::AddErrorToCurrentRun(const char* aMsgId)
-{
-  NS_PRECONDITION(mCurrentRun, "Adding error to run without one!");
+void nsHtml5Highlighter::AddErrorToCurrentRun(const char* aMsgId) {
+  MOZ_ASSERT(mCurrentRun, "Adding error to run without one!");
   nsHtml5TreeOperation* treeOp = mOpQueue.AppendElement();
   NS_ASSERTION(treeOp, "Tree op allocation failed.");
-  treeOp->Init(mCurrentRun, aMsgId);
+  opAddErrorType operation(mCurrentRun, (char*)aMsgId);
+  treeOp->Init(mozilla::AsVariant(operation));
 }
 
-void
-nsHtml5Highlighter::AddErrorToCurrentRun(const char* aMsgId,
-                                         nsAtom* aName)
-{
-  NS_PRECONDITION(mCurrentRun, "Adding error to run without one!");
+void nsHtml5Highlighter::AddErrorToCurrentRun(const char* aMsgId,
+                                              nsAtom* aName) {
+  MOZ_ASSERT(mCurrentRun, "Adding error to run without one!");
   nsHtml5TreeOperation* treeOp = mOpQueue.AppendElement();
   NS_ASSERTION(treeOp, "Tree op allocation failed.");
-  treeOp->Init(mCurrentRun, aMsgId, aName);
+  opAddErrorType operation(mCurrentRun, (char*)aMsgId, aName);
+  treeOp->Init(mozilla::AsVariant(operation));
 }
 
-void
-nsHtml5Highlighter::AddErrorToCurrentRun(const char* aMsgId,
-                                         nsAtom* aName,
-                                         nsAtom* aOther)
-{
-  NS_PRECONDITION(mCurrentRun, "Adding error to run without one!");
+void nsHtml5Highlighter::AddErrorToCurrentRun(const char* aMsgId, nsAtom* aName,
+                                              nsAtom* aOther) {
+  MOZ_ASSERT(mCurrentRun, "Adding error to run without one!");
   nsHtml5TreeOperation* treeOp = mOpQueue.AppendElement();
   NS_ASSERTION(treeOp, "Tree op allocation failed.");
-  treeOp->Init(mCurrentRun, aMsgId, aName, aOther);
+  opAddErrorType operation(mCurrentRun, (char*)aMsgId, aName, aOther);
+  treeOp->Init(mozilla::AsVariant(operation));
 }
 
-void
-nsHtml5Highlighter::AddErrorToCurrentAmpersand(const char* aMsgId)
-{
-  NS_PRECONDITION(mAmpersand, "Adding error to ampersand without one!");
+void nsHtml5Highlighter::AddErrorToCurrentAmpersand(const char* aMsgId) {
+  MOZ_ASSERT(mAmpersand, "Adding error to ampersand without one!");
   nsHtml5TreeOperation* treeOp = mOpQueue.AppendElement();
   NS_ASSERTION(treeOp, "Tree op allocation failed.");
-  treeOp->Init(mAmpersand, aMsgId);
+  opAddErrorType operation(mAmpersand, (char*)aMsgId);
+  treeOp->Init(mozilla::AsVariant(operation));
 }
 
-void
-nsHtml5Highlighter::AddErrorToCurrentSlash(const char* aMsgId)
-{
-  NS_PRECONDITION(mSlash, "Adding error to slash without one!");
+void nsHtml5Highlighter::AddErrorToCurrentSlash(const char* aMsgId) {
+  MOZ_ASSERT(mSlash, "Adding error to slash without one!");
   nsHtml5TreeOperation* treeOp = mOpQueue.AppendElement();
   NS_ASSERTION(treeOp, "Tree op allocation failed.");
-  treeOp->Init(mSlash, aMsgId);
+  opAddErrorType operation(mSlash, (char*)aMsgId);
+  treeOp->Init(mozilla::AsVariant(operation));
 }
