@@ -1,5 +1,5 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim: set sw=2 ts=2 et tw=79: */
+/* vim: set sw=2 ts=2 et tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -10,9 +10,7 @@
 #include "nsCRT.h"
 #include "nsScanner.h"
 #include "plstr.h"
-#include "nsIStringStream.h"
 #include "nsIChannel.h"
-#include "nsICachingChannel.h"
 #include "nsIInputStream.h"
 #include "CNavDTD.h"
 #include "prenv.h"
@@ -22,9 +20,6 @@
 #include "nsReadableUtils.h"
 #include "nsCOMPtr.h"
 #include "nsExpatDriver.h"
-#include "nsIServiceManager.h"
-#include "nsICategoryManager.h"
-#include "nsISupportsPrimitives.h"
 #include "nsIFragmentContentSink.h"
 #include "nsStreamUtils.h"
 #include "nsHTMLTokenizer.h"
@@ -43,10 +38,10 @@
 
 using namespace mozilla;
 
-#define NS_PARSER_FLAG_OBSERVERS_ENABLED      0x00000004
+#define NS_PARSER_FLAG_OBSERVERS_ENABLED 0x00000004
 #define NS_PARSER_FLAG_PENDING_CONTINUE_EVENT 0x00000008
-#define NS_PARSER_FLAG_FLUSH_TOKENS           0x00000020
-#define NS_PARSER_FLAG_CAN_TOKENIZE           0x00000040
+#define NS_PARSER_FLAG_FLUSH_TOKENS 0x00000020
+#define NS_PARSER_FLAG_CAN_TOKENIZE 0x00000040
 
 //-------------- Begin ParseContinue Event Definition ------------------------
 /*
@@ -103,19 +98,14 @@ document.write.
 For more details @see bugzilla bug 76722
 */
 
-
-class nsParserContinueEvent : public Runnable
-{
-public:
+class nsParserContinueEvent : public Runnable {
+ public:
   RefPtr<nsParser> mParser;
 
   explicit nsParserContinueEvent(nsParser* aParser)
-    : mozilla::Runnable("nsParserContinueEvent")
-    , mParser(aParser)
-  {}
+      : mozilla::Runnable("nsParserContinueEvent"), mParser(aParser) {}
 
-  NS_IMETHOD Run() override
-  {
+  NS_IMETHOD Run() override {
     mParser->HandleParserContinueEvent(this);
     return NS_OK;
   }
@@ -127,24 +117,17 @@ public:
  *  default constructor
  */
 nsParser::nsParser()
-  : mCharset(WINDOWS_1252_ENCODING)
-{
+    : mParserContext(nullptr), mCharset(WINDOWS_1252_ENCODING) {
   Initialize(true);
 }
 
-nsParser::~nsParser()
-{
-  Cleanup();
-}
+nsParser::~nsParser() { Cleanup(); }
 
-void
-nsParser::Initialize(bool aConstructor)
-{
+void nsParser::Initialize(bool aConstructor) {
   if (aConstructor) {
     // Raw pointer
     mParserContext = 0;
-  }
-  else {
+  } else {
     // nsCOMPtrs
     mObserver = nullptr;
     mUnusedInput.Truncate();
@@ -157,16 +140,13 @@ nsParser::Initialize(bool aConstructor)
   mStreamStatus = NS_OK;
   mCommand = eViewNormal;
   mBlocked = 0;
-  mFlags = NS_PARSER_FLAG_OBSERVERS_ENABLED |
-           NS_PARSER_FLAG_CAN_TOKENIZE;
+  mFlags = NS_PARSER_FLAG_OBSERVERS_ENABLED | NS_PARSER_FLAG_CAN_TOKENIZE;
 
   mProcessingNetworkData = false;
   mIsAboutBlank = false;
 }
 
-void
-nsParser::Cleanup()
-{
+void nsParser::Cleanup() {
 #ifdef DEBUG
   if (mParserContext && mParserContext->mPrevContext) {
     NS_WARNING("Extra parser contexts still on the parser stack");
@@ -174,7 +154,7 @@ nsParser::Cleanup()
 #endif
 
   while (mParserContext) {
-    CParserContext *pc = mParserContext->mPrevContext;
+    CParserContext* pc = mParserContext->mPrevContext;
     delete mParserContext;
     mParserContext = pc;
   }
@@ -191,13 +171,14 @@ NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsParser)
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mDTD)
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mSink)
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mObserver)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK_WEAK_REFERENCE
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsParser)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mDTD)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mSink)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mObserver)
-  CParserContext *pc = tmp->mParserContext;
+  CParserContext* pc = tmp->mParserContext;
   while (pc) {
     cb.NoteXPCOMChild(pc->mTokenizer);
     pc = pc->mPrevContext;
@@ -219,9 +200,7 @@ NS_INTERFACE_MAP_END
 // and the parser has been interrupted by the content sink
 // because the processing of tokens took too long.
 
-nsresult
-nsParser::PostContinueEvent()
-{
+nsresult nsParser::PostContinueEvent() {
   if (!(mFlags & NS_PARSER_FLAG_PENDING_CONTINUE_EVENT)) {
     // If this flag isn't set, then there shouldn't be a live continue event!
     NS_ASSERTION(!mContinueEvent, "bad");
@@ -230,20 +209,17 @@ nsParser::PostContinueEvent()
     // broken when the event fires.
     nsCOMPtr<nsIRunnable> event = new nsParserContinueEvent(this);
     if (NS_FAILED(NS_DispatchToCurrentThread(event))) {
-        NS_WARNING("failed to dispatch parser continuation event");
+      NS_WARNING("failed to dispatch parser continuation event");
     } else {
-        mFlags |= NS_PARSER_FLAG_PENDING_CONTINUE_EVENT;
-        mContinueEvent = event;
+      mFlags |= NS_PARSER_FLAG_PENDING_CONTINUE_EVENT;
+      mContinueEvent = event;
     }
   }
   return NS_OK;
 }
 
 NS_IMETHODIMP_(void)
-nsParser::GetCommand(nsCString& aCommand)
-{
-  aCommand = mCommandStr;
-}
+nsParser::GetCommand(nsCString& aCommand) { aCommand = mCommandStr; }
 
 /**
  *  Call this method once you've created a parser, and want to instruct it
@@ -253,8 +229,7 @@ nsParser::GetCommand(nsCString& aCommand)
  *  @param   aCommand the command string to set
  */
 NS_IMETHODIMP_(void)
-nsParser::SetCommand(const char* aCommand)
-{
+nsParser::SetCommand(const char* aCommand) {
   mCommandStr.Assign(aCommand);
   if (mCommandStr.EqualsLiteral("view-source")) {
     mCommand = eViewSource;
@@ -273,8 +248,7 @@ nsParser::SetCommand(const char* aCommand)
  *  @param   aParserCommand the command to set
  */
 NS_IMETHODIMP_(void)
-nsParser::SetCommand(eParserCommands aParserCommand)
-{
+nsParser::SetCommand(eParserCommands aParserCommand) {
   mCommand = aParserCommand;
 }
 
@@ -285,20 +259,16 @@ nsParser::SetCommand(eParserCommands aParserCommand)
  *  @param   aCharset- the charset of a document
  *  @param   aCharsetSource- the source of the charset
  */
-void
-nsParser::SetDocumentCharset(NotNull<const Encoding*> aCharset,
-                             int32_t aCharsetSource)
-{
+void nsParser::SetDocumentCharset(NotNull<const Encoding*> aCharset,
+                                  int32_t aCharsetSource) {
   mCharset = aCharset;
   mCharsetSource = aCharsetSource;
   if (mParserContext && mParserContext->mScanner) {
-     mParserContext->mScanner->SetDocumentCharset(aCharset, aCharsetSource);
+    mParserContext->mScanner->SetDocumentCharset(aCharset, aCharsetSource);
   }
 }
 
-void
-nsParser::SetSinkCharset(NotNull<const Encoding*> aCharset)
-{
+void nsParser::SetSinkCharset(NotNull<const Encoding*> aCharset) {
   if (mSink) {
     mSink->SetDocumentCharset(aCharset);
   }
@@ -311,9 +281,8 @@ nsParser::SetSinkCharset(NotNull<const Encoding*> aCharset)
  *  @param   nsIContentSink interface for node receiver
  */
 NS_IMETHODIMP_(void)
-nsParser::SetContentSink(nsIContentSink* aSink)
-{
-  NS_PRECONDITION(aSink, "sink cannot be null!");
+nsParser::SetContentSink(nsIContentSink* aSink) {
+  MOZ_ASSERT(aSink, "sink cannot be null!");
   mSink = aSink;
 
   if (mSink) {
@@ -330,14 +299,9 @@ nsParser::SetContentSink(nsIContentSink* aSink)
  * @return  current sink
  */
 NS_IMETHODIMP_(nsIContentSink*)
-nsParser::GetContentSink()
-{
-  return mSink;
-}
+nsParser::GetContentSink() { return mSink; }
 
-static nsIDTD*
-FindSuitableDTD(CParserContext& aParserContext)
-{
+static nsIDTD* FindSuitableDTD(CParserContext& aParserContext) {
   // We always find a DTD.
   aParserContext.mAutoDetectStatus = ePrimaryDetect;
 
@@ -353,13 +317,13 @@ FindSuitableDTD(CParserContext& aParserContext)
   }
 
   // If we're here, then we'd better be parsing XML.
-  NS_ASSERTION(aParserContext.mDocType == eXML, "What are you trying to send me, here?");
+  NS_ASSERTION(aParserContext.mDocType == eXML,
+               "What are you trying to send me, here?");
   return new nsExpatDriver();
 }
 
 NS_IMETHODIMP
-nsParser::CancelParsingEvents()
-{
+nsParser::CancelParsingEvents() {
   if (mFlags & NS_PARSER_FLAG_PENDING_CONTINUE_EVENT) {
     NS_ASSERTION(mContinueEvent, "mContinueEvent is null");
     // Revoke the pending continue parsing event
@@ -415,13 +379,14 @@ nsParser::CancelParsingEvents()
  * to mWhatever, nsCaller wants to guarantee that mWhatever->DoThatThing() will
  * be called regardless of how nsSomething::DoThatThing behaves, &c.
  */
-#define PREFER_LATTER_ERROR_CODE(EXPR1, EXPR2, RV) {                          \
-  nsresult RV##__temp = EXPR1;                                                \
-  RV = EXPR2;                                                                 \
-  if (NS_FAILED(RV)) {                                                        \
-    RV = RV##__temp;                                                          \
-  }                                                                           \
-}
+#define PREFER_LATTER_ERROR_CODE(EXPR1, EXPR2, RV) \
+  {                                                \
+    nsresult RV##__temp = EXPR1;                   \
+    RV = EXPR2;                                    \
+    if (NS_FAILED(RV)) {                           \
+      RV = RV##__temp;                             \
+    }                                              \
+  }
 
 /**
  * This gets called just prior to the model actually
@@ -430,14 +395,10 @@ nsParser::CancelParsingEvents()
  * can delay until the last moment the resolution of
  * which DTD to use (unless of course we're assigned one).
  */
-nsresult
-nsParser::WillBuildModel(nsString& aFilename)
-{
-  if (!mParserContext)
-    return NS_ERROR_HTMLPARSER_INVALIDPARSERCONTEXT;
+nsresult nsParser::WillBuildModel(nsString& aFilename) {
+  if (!mParserContext) return NS_ERROR_HTMLPARSER_INVALIDPARSERCONTEXT;
 
-  if (eUnknownDetect != mParserContext->mAutoDetectStatus)
-    return NS_OK;
+  if (eUnknownDetect != mParserContext->mAutoDetectStatus) return NS_OK;
 
   if (eDTDMode_unknown == mParserContext->mDTDMode ||
       eDTDMode_autodetect == mParserContext->mDTDMode) {
@@ -448,7 +409,7 @@ nsParser::WillBuildModel(nsString& aFilename)
       mParserContext->mDTDMode = eDTDMode_full_standards;
       mParserContext->mDocType = eXML;
     }
-  } // else XML fragment with nested parser context
+  }  // else XML fragment with nested parser context
 
   NS_ASSERTION(!mDTD || !mParserContext->mPrevContext,
                "Clobbering DTD for non-root parser context!");
@@ -476,9 +437,7 @@ nsParser::WillBuildModel(nsString& aFilename)
  * Note that the parser may have been called recursively, so we
  * have to check for a prev. context before closing out the DTD/sink.
  */
-nsresult
-nsParser::DidBuildModel(nsresult anErrorCode)
-{
+nsresult nsParser::DidBuildModel(nsresult anErrorCode) {
   nsresult result = anErrorCode;
 
   if (IsComplete()) {
@@ -487,8 +446,8 @@ nsParser::DidBuildModel(nsresult anErrorCode)
       // In that case we don't want it to run deferred scripts.
       bool terminated = mInternalState == NS_ERROR_HTMLPARSER_STOPPARSING;
       if (mDTD && mSink) {
-        nsresult dtdResult =  mDTD->DidBuildModel(anErrorCode),
-                sinkResult = mSink->DidBuildModel(terminated);
+        nsresult dtdResult = mDTD->DidBuildModel(anErrorCode),
+                 sinkResult = mSink->DidBuildModel(terminated);
         // nsIDTD::DidBuildModel used to be responsible for calling
         // nsIContentSink::DidBuildModel, but that obligation isn't expressible
         // in the nsIDTD interface itself, so it's sounder and simpler to give
@@ -499,7 +458,7 @@ nsParser::DidBuildModel(nsresult anErrorCode)
         result = NS_FAILED(sinkResult) ? sinkResult : dtdResult;
       }
 
-      //Ref. to bug 61462.
+      // Ref. to bug 61462.
       mParserContext->mRequest = nullptr;
     }
   }
@@ -513,9 +472,7 @@ nsParser::DidBuildModel(nsresult anErrorCode)
  *
  * @param   ptr to new context
  */
-void
-nsParser::PushContext(CParserContext& aContext)
-{
+void nsParser::PushContext(CParserContext& aContext) {
   NS_ASSERTION(aContext.mPrevContext == mParserContext,
                "Trying to push a context whose previous context differs from "
                "the current parser context.");
@@ -529,16 +486,15 @@ nsParser::PushContext(CParserContext& aContext)
  * @update	gess7/22/98
  * @return  prev. context
  */
-CParserContext*
-nsParser::PopContext()
-{
+CParserContext* nsParser::PopContext() {
   CParserContext* oldContext = mParserContext;
   if (oldContext) {
     mParserContext = oldContext->mPrevContext;
     if (mParserContext) {
       // If the old context was blocked, propagate the blocked state
       // back to the new one. Also, propagate the stream listener state
-      // but don't override onStop state to guarantee the call to DidBuildModel().
+      // but don't override onStop state to guarantee the call to
+      // DidBuildModel().
       if (mParserContext->mStreamListenerState != eOnStop) {
         mParserContext->mStreamListenerState = oldContext->mStreamListenerState;
       }
@@ -555,11 +511,7 @@ nsParser::PopContext()
  *  @param   aState determines whether we parse/tokenize or just cache.
  *  @return  current state
  */
-void
-nsParser::SetUnusedInput(nsString& aBuffer)
-{
-  mUnusedInput = aBuffer;
-}
+void nsParser::SetUnusedInput(nsString& aBuffer) { mUnusedInput = aBuffer; }
 
 /**
  *  Call this when you want to *force* the parser to terminate the
@@ -567,8 +519,7 @@ nsParser::SetUnusedInput(nsString& aBuffer)
  *  you can't resume without restarting altogether.
  */
 NS_IMETHODIMP
-nsParser::Terminate(void)
-{
+nsParser::Terminate(void) {
   // We should only call DidBuildModel once, so don't do anything if this is
   // the second time that Terminate has been called.
   if (mInternalState == NS_ERROR_HTMLPARSER_STOPPARSING) {
@@ -585,7 +536,8 @@ nsParser::Terminate(void)
   // @see bug 108049
   // If NS_PARSER_FLAG_PENDING_CONTINUE_EVENT is set then CancelParsingEvents
   // will reset it so DidBuildModel will call DidBuildModel on the DTD. Note:
-  // The IsComplete() call inside of DidBuildModel looks at the pendingContinueEvents flag.
+  // The IsComplete() call inside of DidBuildModel looks at the
+  // pendingContinueEvents flag.
   CancelParsingEvents();
 
   // If we got interrupted in the middle of a document.write, then we might
@@ -594,7 +546,7 @@ nsParser::Terminate(void)
   // our sink's DidBuildModel and break the reference cycle, causing a leak.
   // Since we're getting terminated, we manually clean up our context stack.
   while (mParserContext && mParserContext->mPrevContext) {
-    CParserContext *prev = mParserContext->mPrevContext;
+    CParserContext* prev = mParserContext->mPrevContext;
     delete mParserContext;
     mParserContext = prev;
   }
@@ -613,8 +565,7 @@ nsParser::Terminate(void)
 }
 
 NS_IMETHODIMP
-nsParser::ContinueInterruptedParsing()
-{
+nsParser::ContinueInterruptedParsing() {
   // If there are scripts executing, then the content sink is jumping the gun
   // (probably due to a synchronous XMLHttpRequest) and will re-enable us
   // later, see bug 460706.
@@ -626,7 +577,7 @@ nsParser::ContinueInterruptedParsing()
   // that we might start closing things down when the parser
   // is reenabled. To make sure that we're not deleted across
   // the reenabling process, hold a reference to ourselves.
-  nsresult result=NS_OK;
+  nsresult result = NS_OK;
   nsCOMPtr<nsIParser> kungFuDeathGrip(this);
   nsCOMPtr<nsIContentSink> sinkDeathGrip(mSink);
 
@@ -636,18 +587,18 @@ nsParser::ContinueInterruptedParsing()
   }
 #endif
 
-  bool isFinalChunk = mParserContext &&
-                        mParserContext->mStreamListenerState == eOnStop;
+  bool isFinalChunk =
+      mParserContext && mParserContext->mStreamListenerState == eOnStop;
 
   mProcessingNetworkData = true;
   if (sinkDeathGrip) {
     sinkDeathGrip->WillParse();
   }
-  result = ResumeParse(true, isFinalChunk); // Ref. bug 57999
+  result = ResumeParse(true, isFinalChunk);  // Ref. bug 57999
   mProcessingNetworkData = false;
 
   if (result != NS_OK) {
-    result=mInternalState;
+    result = mInternalState;
   }
 
   return result;
@@ -660,10 +611,7 @@ nsParser::ContinueInterruptedParsing()
  *  page, or any number of extension content scripts).
  */
 NS_IMETHODIMP_(void)
-nsParser::BlockParser()
-{
-  mBlocked++;
-}
+nsParser::BlockParser() { mBlocked++; }
 
 /**
  *  Open up the parser for tokenization, building up content
@@ -672,8 +620,7 @@ nsParser::BlockParser()
  *  the parsing engine.
  */
 NS_IMETHODIMP_(void)
-nsParser::UnblockParser()
-{
+nsParser::UnblockParser() {
   MOZ_DIAGNOSTIC_ASSERT(mBlocked > 0);
   if (MOZ_LIKELY(mBlocked > 0)) {
     mBlocked--;
@@ -681,8 +628,7 @@ nsParser::UnblockParser()
 }
 
 NS_IMETHODIMP_(void)
-nsParser::ContinueInterruptedParsingAsync()
-{
+nsParser::ContinueInterruptedParsingAsync() {
   MOZ_ASSERT(mSink);
   if (MOZ_LIKELY(mSink)) {
     mSink->ContinueInterruptedParsingAsync();
@@ -693,26 +639,19 @@ nsParser::ContinueInterruptedParsingAsync()
  * Call this to query whether the parser is enabled or not.
  */
 NS_IMETHODIMP_(bool)
-nsParser::IsParserEnabled()
-{
-  return !mBlocked;
-}
+nsParser::IsParserEnabled() { return !mBlocked; }
 
 /**
  * Call this to query whether the parser thinks it's done with parsing.
  */
 NS_IMETHODIMP_(bool)
-nsParser::IsComplete()
-{
+nsParser::IsComplete() {
   return !(mFlags & NS_PARSER_FLAG_PENDING_CONTINUE_EVENT);
 }
 
-
-void nsParser::HandleParserContinueEvent(nsParserContinueEvent *ev)
-{
+void nsParser::HandleParserContinueEvent(nsParserContinueEvent* ev) {
   // Ignore any revoked continue events...
-  if (mContinueEvent != ev)
-    return;
+  if (mContinueEvent != ev) return;
 
   mFlags &= ~NS_PARSER_FLAG_PENDING_CONTINUE_EVENT;
   mContinueEvent = nullptr;
@@ -722,32 +661,17 @@ void nsParser::HandleParserContinueEvent(nsParserContinueEvent *ev)
   ContinueInterruptedParsing();
 }
 
-bool
-nsParser::IsInsertionPointDefined()
-{
-  return false;
-}
+bool nsParser::IsInsertionPointDefined() { return false; }
 
-void
-nsParser::PushDefinedInsertionPoint()
-{
-}
+void nsParser::IncrementScriptNestingLevel() {}
 
-void
-nsParser::PopDefinedInsertionPoint()
-{
-}
+void nsParser::DecrementScriptNestingLevel() {}
 
-void
-nsParser::MarkAsNotScriptCreated(const char* aCommand)
-{
-}
+bool nsParser::HasNonzeroScriptNestingLevel() const { return false; }
 
-bool
-nsParser::IsScriptCreated()
-{
-  return false;
-}
+void nsParser::MarkAsNotScriptCreated(const char* aCommand) {}
+
+bool nsParser::IsScriptCreated() { return false; }
 
 /**
  *  This is the main controlling routine in the parsing process.
@@ -757,13 +681,9 @@ nsParser::IsScriptCreated()
  *  of this method.
  */
 NS_IMETHODIMP
-nsParser::Parse(nsIURI* aURL,
-                nsIRequestObserver* aListener,
-                void* aKey,
-                nsDTDMode aMode)
-{
-
-  NS_PRECONDITION(aURL, "Error: Null URL given");
+nsParser::Parse(nsIURI* aURL, nsIRequestObserver* aListener, void* aKey,
+                nsDTDMode aMode) {
+  MOZ_ASSERT(aURL, "Error: Null URL given");
 
   nsresult result = NS_ERROR_HTMLPARSER_BADURL;
   mObserver = aListener;
@@ -798,11 +718,8 @@ nsParser::Parse(nsIURI* aURL,
  *
  * @param   aSourceBuffer contains a string-full of real content
  */
-nsresult
-nsParser::Parse(const nsAString& aSourceBuffer,
-                void* aKey,
-                bool aLastCall)
-{
+nsresult nsParser::Parse(const nsAString& aSourceBuffer, void* aKey,
+                         bool aLastCall) {
   nsresult result = NS_OK;
 
   // Don't bother if we're never going to parse this.
@@ -852,13 +769,13 @@ nsParser::Parse(const nsAString& aSourceBuffer,
         }
       }
 
-      pc = new CParserContext(mParserContext, theScanner, aKey, mCommand,
-                              0, theStatus, aLastCall);
+      pc = new CParserContext(mParserContext, theScanner, aKey, mCommand, 0,
+                              theStatus, aLastCall);
       NS_ENSURE_TRUE(pc, NS_ERROR_OUT_OF_MEMORY);
 
       PushContext(*pc);
 
-      pc->mMultipart = !aLastCall; // By default
+      pc->mMultipart = !aLastCall;  // By default
       if (pc->mPrevContext) {
         pc->mMultipart |= pc->mPrevContext->mMultipart;
       }
@@ -877,7 +794,7 @@ nsParser::Parse(const nsAString& aSourceBuffer,
       }
       // end fix for 40143
 
-      pc->mContextType=CParserContext::eCTString;
+      pc->mContextType = CParserContext::eCTString;
       pc->SetMimeType(NS_LITERAL_CSTRING("application/xml"));
       pc->mDTDMode = eDTDMode_full_standards;
 
@@ -889,8 +806,8 @@ nsParser::Parse(const nsAString& aSourceBuffer,
     } else {
       pc->mScanner->Append(aSourceBuffer);
       if (!pc->mPrevContext) {
-        // Set stream listener state to eOnStop, on the final context - Fix 68160,
-        // to guarantee DidBuildModel() call - Fix 36148
+        // Set stream listener state to eOnStop, on the final context - Fix
+        // 68160, to guarantee DidBuildModel() call - Fix 36148
         if (aLastCall) {
           pc->mStreamListenerState = eOnStop;
           pc->mScanner->SetIncremental(false);
@@ -911,10 +828,9 @@ nsParser::Parse(const nsAString& aSourceBuffer,
 
 NS_IMETHODIMP
 nsParser::ParseFragment(const nsAString& aSourceBuffer,
-                        nsTArray<nsString>& aTagStack)
-{
+                        nsTArray<nsString>& aTagStack) {
   nsresult result = NS_OK;
-  nsAutoString  theContext;
+  nsAutoString theContext;
   uint32_t theCount = aTagStack.Length();
   uint32_t theIndex = 0;
 
@@ -935,9 +851,7 @@ nsParser::ParseFragment(const nsAString& aSourceBuffer,
 
   // First, parse the context to build up the DTD's tag stack. Note that we
   // pass false for the aLastCall parameter.
-  result = Parse(theContext,
-                 (void*)&theContext,
-                 false);
+  result = Parse(theContext, (void*)&theContext, false);
   if (NS_FAILED(result)) {
     mFlags |= NS_PARSER_FLAG_OBSERVERS_ENABLED;
     return result;
@@ -957,22 +871,18 @@ nsParser::ParseFragment(const nsAString& aSourceBuffer,
   // the end tags.  However, if tagStack is empty, it's the last call
   // for XML as well.
   if (theCount == 0) {
-    result = Parse(aSourceBuffer,
-                   &theContext,
-                   true);
+    result = Parse(aSourceBuffer, &theContext, true);
     fragSink->DidBuildContent();
   } else {
     // Add an end tag chunk, so expat will read the whole source buffer,
     // and not worry about ']]' etc.
-    result = Parse(aSourceBuffer + NS_LITERAL_STRING("</"),
-                   &theContext,
-                   false);
+    result = Parse(aSourceBuffer + NS_LITERAL_STRING("</"), &theContext, false);
     fragSink->DidBuildContent();
 
     if (NS_SUCCEEDED(result)) {
       nsAutoString endContext;
       for (theIndex = 0; theIndex < theCount; theIndex++) {
-         // we already added an end tag chunk above
+        // we already added an end tag chunk above
         if (theIndex > 0) {
           endContext.AppendLiteral("</");
         }
@@ -983,15 +893,13 @@ nsParser::ParseFragment(const nsAString& aSourceBuffer,
         if (endOfTag == -1) {
           endContext.Append(thisTag);
         } else {
-          endContext.Append(Substring(thisTag,0,endOfTag));
+          endContext.Append(Substring(thisTag, 0, endOfTag));
         }
 
         endContext.Append('>');
       }
 
-      result = Parse(endContext,
-                     &theContext,
-                     true);
+      result = Parse(endContext, &theContext, true);
     }
   }
 
@@ -1022,14 +930,11 @@ nsParser::ParseFragment(const nsAString& aSourceBuffer,
  *  @param   aIsFinalChunk : tells us when the last chunk of data is provided.
  *  @return  error code -- 0 if ok, non-zero if error.
  */
-nsresult
-nsParser::ResumeParse(bool allowIteration, bool aIsFinalChunk,
-                      bool aCanInterrupt)
-{
+nsresult nsParser::ResumeParse(bool allowIteration, bool aIsFinalChunk,
+                               bool aCanInterrupt) {
   nsresult result = NS_OK;
 
   if (!mBlocked && mInternalState != NS_ERROR_HTMLPARSER_STOPPARSING) {
-
     result = WillBuildModel(mParserContext->mScanner->GetFilename());
     if (NS_FAILED(result)) {
       mFlags &= ~NS_PARSER_FLAG_CAN_TOKENIZE;
@@ -1053,8 +958,8 @@ nsParser::ResumeParse(bool allowIteration, bool aIsFinalChunk,
         // Only allow parsing to be interrupted in the subsequent call to
         // build model.
         nsresult theTokenizerResult = (mFlags & NS_PARSER_FLAG_CAN_TOKENIZE)
-                                      ? Tokenize(aIsFinalChunk)
-                                      : NS_OK;
+                                          ? Tokenize(aIsFinalChunk)
+                                          : NS_OK;
         result = BuildModel();
 
         if (result == NS_ERROR_HTMLPARSER_INTERRUPTED && aIsFinalChunk) {
@@ -1092,7 +997,7 @@ nsParser::ResumeParse(bool allowIteration, bool aIsFinalChunk,
              theTokenizerResult == NS_ERROR_HTMLPARSER_EOF) ||
             result == NS_ERROR_HTMLPARSER_INTERRUPTED) {
           bool theContextIsStringBased =
-            CParserContext::eCTString == mParserContext->mContextType;
+              CParserContext::eCTString == mParserContext->mContextType;
 
           if (mParserContext->mStreamListenerState == eOnStop ||
               !mParserContext->mMultipart || theContextIsStringBased) {
@@ -1140,9 +1045,7 @@ nsParser::ResumeParse(bool allowIteration, bool aIsFinalChunk,
  *  This is where we loop over the tokens created in the
  *  tokenization phase, and try to make sense out of them.
  */
-nsresult
-nsParser::BuildModel()
-{
+nsresult nsParser::BuildModel() {
   nsITokenizer* theTokenizer = nullptr;
 
   nsresult result = NS_OK;
@@ -1164,14 +1067,13 @@ nsParser::BuildModel()
   These methods are used to talk to the netlib system...
  *******************************************************************/
 
-nsresult
-nsParser::OnStartRequest(nsIRequest *request, nsISupports* aContext)
-{
-  NS_PRECONDITION(eNone == mParserContext->mStreamListenerState,
-                  "Parser's nsIStreamListener API was not setup "
-                  "correctly in constructor.");
+nsresult nsParser::OnStartRequest(nsIRequest* request) {
+  MOZ_ASSERT(eNone == mParserContext->mStreamListenerState,
+             "Parser's nsIStreamListener API was not setup "
+             "correctly in constructor.");
+
   if (mObserver) {
-    mObserver->OnStartRequest(request, aContext);
+    mObserver->OnStartRequest(request);
   }
   mParserContext->mStreamListenerState = eOnStart;
   mParserContext->mAutoDetectStatus = eUnknownDetect;
@@ -1196,26 +1098,20 @@ nsParser::OnStartRequest(nsIRequest *request, nsISupports* aContext)
   return rv;
 }
 
-static bool
-ExtractCharsetFromXmlDeclaration(const unsigned char* aBytes, int32_t aLen,
-                                 nsCString& oCharset)
-{
+static bool ExtractCharsetFromXmlDeclaration(const unsigned char* aBytes,
+                                             int32_t aLen,
+                                             nsCString& oCharset) {
   // This code is rather pointless to have. Might as well reuse expat as
   // seen in nsHtml5StreamParser. -- hsivonen
   oCharset.Truncate();
-  if ((aLen >= 5) &&
-      ('<' == aBytes[0]) &&
-      ('?' == aBytes[1]) &&
-      ('x' == aBytes[2]) &&
-      ('m' == aBytes[3]) &&
-      ('l' == aBytes[4])) {
+  if ((aLen >= 5) && ('<' == aBytes[0]) && ('?' == aBytes[1]) &&
+      ('x' == aBytes[2]) && ('m' == aBytes[3]) && ('l' == aBytes[4])) {
     int32_t i;
     bool versionFound = false, encodingFound = false;
     for (i = 6; i < aLen && !encodingFound; ++i) {
       // end of XML declaration?
-      if ((((char*) aBytes)[i] == '?') &&
-          ((i + 1) < aLen) &&
-          (((char*) aBytes)[i + 1] == '>')) {
+      if ((((char*)aBytes)[i] == '?') && ((i + 1) < aLen) &&
+          (((char*)aBytes)[i + 1] == '>')) {
         break;
       }
       // Version is required.
@@ -1225,13 +1121,12 @@ ExtractCharsetFromXmlDeclaration(const unsigned char* aBytes, int32_t aLen,
         // foolproof, but fast.
         // The shortest string allowed before this is  (strlen==13):
         // <?xml version
-        if ((((char*) aBytes)[i] == 'n') &&
-            (i >= 12) &&
-            (0 == PL_strncmp("versio", (char*) (aBytes + i - 6), 6))) {
+        if ((((char*)aBytes)[i] == 'n') && (i >= 12) &&
+            (0 == PL_strncmp("versio", (char*)(aBytes + i - 6), 6))) {
           // Fast forward through version
           char q = 0;
           for (++i; i < aLen; ++i) {
-            char qi = ((char*) aBytes)[i];
+            char qi = ((char*)aBytes)[i];
             if (qi == '\'' || qi == '"') {
               if (q && q == qi) {
                 //  ending quote
@@ -1251,19 +1146,20 @@ ExtractCharsetFromXmlDeclaration(const unsigned char* aBytes, int32_t aLen,
         // foolproof, but fast.
         // The shortest allowed string before this (strlen==26):
         // <?xml version="1" encoding
-        if ((((char*) aBytes)[i] == 'g') && (i >= 25) && (0 == PL_strncmp(
-            "encodin", (char*) (aBytes + i - 7), 7))) {
+        if ((((char*)aBytes)[i] == 'g') && (i >= 25) &&
+            (0 == PL_strncmp("encodin", (char*)(aBytes + i - 7), 7))) {
           int32_t encStart = 0;
           char q = 0;
           for (++i; i < aLen; ++i) {
-            char qi = ((char*) aBytes)[i];
+            char qi = ((char*)aBytes)[i];
             if (qi == '\'' || qi == '"') {
               if (q && q == qi) {
                 int32_t count = i - encStart;
                 // encoding value is invalid if it is UTF-16
-                if (count > 0 && PL_strncasecmp("UTF-16",
-                    (char*) (aBytes + encStart), count)) {
-                  oCharset.Assign((char*) (aBytes + encStart), count);
+                if (count > 0 &&
+                    PL_strncasecmp("UTF-16", (char*)(aBytes + encStart),
+                                   count)) {
+                  oCharset.Assign((char*)(aBytes + encStart), count);
                 }
                 encodingFound = true;
                 break;
@@ -1274,28 +1170,22 @@ ExtractCharsetFromXmlDeclaration(const unsigned char* aBytes, int32_t aLen,
             }
           }
         }
-      } // if (!versionFound)
-    } // for
+      }  // if (!versionFound)
+    }    // for
   }
   return !oCharset.IsEmpty();
 }
 
-inline char
-GetNextChar(nsACString::const_iterator& aStart,
-            nsACString::const_iterator& aEnd)
-{
+inline char GetNextChar(nsACString::const_iterator& aStart,
+                        nsACString::const_iterator& aEnd) {
   NS_ASSERTION(aStart != aEnd, "end of buffer");
   return (++aStart != aEnd) ? *aStart : '\0';
 }
 
-static nsresult
-NoOpParserWriteFunc(nsIInputStream* in,
-                void* closure,
-                const char* fromRawSegment,
-                uint32_t toOffset,
-                uint32_t count,
-                uint32_t *writeCount)
-{
+static nsresult NoOpParserWriteFunc(nsIInputStream* in, void* closure,
+                                    const char* fromRawSegment,
+                                    uint32_t toOffset, uint32_t count,
+                                    uint32_t* writeCount) {
   *writeCount = count;
   return NS_OK;
 }
@@ -1313,18 +1203,13 @@ typedef struct {
  * of data in the underlying stream or pipe. Using ReadSegments
  * allows us to avoid copying data to read out of the stream.
  */
-static nsresult
-ParserWriteFunc(nsIInputStream* in,
-                void* closure,
-                const char* fromRawSegment,
-                uint32_t toOffset,
-                uint32_t count,
-                uint32_t *writeCount)
-{
+static nsresult ParserWriteFunc(nsIInputStream* in, void* closure,
+                                const char* fromRawSegment, uint32_t toOffset,
+                                uint32_t count, uint32_t* writeCount) {
   nsresult result;
   ParserWriteStruct* pws = static_cast<ParserWriteStruct*>(closure);
   const unsigned char* buf =
-    reinterpret_cast<const unsigned char*> (fromRawSegment);
+      reinterpret_cast<const unsigned char*>(fromRawSegment);
   uint32_t theNumRead = count;
 
   if (!pws) {
@@ -1362,7 +1247,6 @@ ParserWriteFunc(nsIInputStream* in,
 
     pws->mParser->SetDocumentCharset(preferred, source);
     pws->mParser->SetSinkCharset(preferred);
-
   }
 
   result = pws->mScanner->Append(fromRawSegment, theNumRead);
@@ -1373,16 +1257,14 @@ ParserWriteFunc(nsIInputStream* in,
   return result;
 }
 
-nsresult
-nsParser::OnDataAvailable(nsIRequest *request, nsISupports* aContext,
-                          nsIInputStream *pIStream, uint64_t sourceOffset,
-                          uint32_t aLength)
-{
-  NS_PRECONDITION((eOnStart == mParserContext->mStreamListenerState ||
-                   eOnDataAvail == mParserContext->mStreamListenerState),
-            "Error: OnStartRequest() must be called before OnDataAvailable()");
-  NS_PRECONDITION(NS_InputStreamIsBuffered(pIStream),
-                  "Must have a buffered input stream");
+nsresult nsParser::OnDataAvailable(nsIRequest* request,
+                                   nsIInputStream* pIStream,
+                                   uint64_t sourceOffset, uint32_t aLength) {
+  MOZ_ASSERT((eOnStart == mParserContext->mStreamListenerState ||
+              eOnDataAvail == mParserContext->mStreamListenerState),
+             "Error: OnStartRequest() must be called before OnDataAvailable()");
+  MOZ_ASSERT(NS_InputStreamIsBuffered(pIStream),
+             "Must have a buffered input stream");
 
   nsresult rv = NS_OK;
 
@@ -1391,14 +1273,12 @@ nsParser::OnDataAvailable(nsIRequest *request, nsISupports* aContext,
     // ... but if an extension tries to feed us data for about:blank in a
     // release build, silently ignore the data.
     uint32_t totalRead;
-    rv = pIStream->ReadSegments(NoOpParserWriteFunc,
-                                nullptr,
-                                aLength,
+    rv = pIStream->ReadSegments(NoOpParserWriteFunc, nullptr, aLength,
                                 &totalRead);
     return rv;
   }
 
-  CParserContext *theContext = mParserContext;
+  CParserContext* theContext = mParserContext;
 
   while (theContext && theContext->mRequest != request) {
     theContext = theContext->mPrevContext;
@@ -1419,7 +1299,7 @@ nsParser::OnDataAvailable(nsIRequest *request, nsISupports* aContext,
     ParserWriteStruct pws;
     pws.mNeedCharsetCheck = true;
     pws.mParser = this;
-    pws.mScanner = theContext->mScanner;
+    pws.mScanner = theContext->mScanner.get();
     pws.mRequest = request;
 
     rv = pIStream->ReadSegments(ParserWriteFunc, &pws, aLength, &totalRead);
@@ -1448,13 +1328,10 @@ nsParser::OnDataAvailable(nsIRequest *request, nsISupports* aContext,
  *  This is called by the networking library once the last block of data
  *  has been collected from the net.
  */
-nsresult
-nsParser::OnStopRequest(nsIRequest *request, nsISupports* aContext,
-                        nsresult status)
-{
+nsresult nsParser::OnStopRequest(nsIRequest* request, nsresult status) {
   nsresult rv = NS_OK;
 
-  CParserContext *pc = mParserContext;
+  CParserContext* pc = mParserContext;
   while (pc) {
     if (pc->mRequest == request) {
       pc->mStreamListenerState = eOnStop;
@@ -1479,30 +1356,25 @@ nsParser::OnStopRequest(nsIRequest *request, nsISupports* aContext,
   // If the parser isn't enabled, we don't finish parsing till
   // it is reenabled.
 
-
   // XXX Should we wait to notify our observers as well if the
   // parser isn't yet enabled?
   if (mObserver) {
-    mObserver->OnStopRequest(request, aContext, status);
+    mObserver->OnStopRequest(request, status);
   }
 
   return rv;
 }
 
-
 /*******************************************************************
   Here come the tokenization methods...
  *******************************************************************/
 
-
 /**
  *  Part of the code sandwich, this gets called right before
  *  the tokenization process begins. The main reason for
  *  this call is to allow the delegate to do initialization.
  */
-bool
-nsParser::WillTokenize(bool aIsFinalChunk)
-{
+bool nsParser::WillTokenize(bool aIsFinalChunk) {
   if (!mParserContext) {
     return true;
   }
@@ -1513,14 +1385,12 @@ nsParser::WillTokenize(bool aIsFinalChunk)
   return NS_SUCCEEDED(theTokenizer->WillTokenize(aIsFinalChunk));
 }
 
-
 /**
  * This is the primary control routine to consume tokens.
  * It iteratively consumes tokens until an error occurs or
  * you run out of data.
  */
-nsresult nsParser::Tokenize(bool aIsFinalChunk)
-{
+nsresult nsParser::Tokenize(bool aIsFinalChunk) {
   nsITokenizer* theTokenizer;
 
   nsresult result = NS_ERROR_NOT_AVAILABLE;
@@ -1536,8 +1406,8 @@ nsresult nsParser::Tokenize(bool aIsFinalChunk)
     WillTokenize(aIsFinalChunk);
     while (NS_SUCCEEDED(result)) {
       mParserContext->mScanner->Mark();
-      result = theTokenizer->ConsumeToken(*mParserContext->mScanner,
-                                          flushTokens);
+      result =
+          theTokenizer->ConsumeToken(*mParserContext->mScanner, flushTokens);
       if (NS_FAILED(result)) {
         mParserContext->mScanner->RewindToMark();
         if (NS_ERROR_HTMLPARSER_EOF == result) {
@@ -1549,8 +1419,8 @@ nsresult nsParser::Tokenize(bool aIsFinalChunk)
           break;
         }
       } else if (flushTokens && (mFlags & NS_PARSER_FLAG_OBSERVERS_ENABLED)) {
-        // I added the extra test of NS_PARSER_FLAG_OBSERVERS_ENABLED to fix Bug# 23931.
-        // Flush tokens on seeing </SCRIPT> -- Ref: Bug# 22485 --
+        // I added the extra test of NS_PARSER_FLAG_OBSERVERS_ENABLED to fix
+        // Bug# 23931. Flush tokens on seeing </SCRIPT> -- Ref: Bug# 22485 --
         // Also remember to update the marked position.
         mFlags |= NS_PARSER_FLAG_FLUSH_TOKENS;
         mParserContext->mScanner->Mark();
@@ -1575,8 +1445,7 @@ nsresult nsParser::Tokenize(bool aIsFinalChunk)
  * @return NS_OK if successful
  */
 NS_IMETHODIMP
-nsParser::GetChannel(nsIChannel** aChannel)
-{
+nsParser::GetChannel(nsIChannel** aChannel) {
   nsresult result = NS_ERROR_NOT_AVAILABLE;
   if (mParserContext && mParserContext->mRequest) {
     result = CallQueryInterface(mParserContext->mRequest, aChannel);
@@ -1588,8 +1457,7 @@ nsParser::GetChannel(nsIChannel** aChannel)
  * Get the DTD associated with this parser
  */
 NS_IMETHODIMP
-nsParser::GetDTD(nsIDTD** aDTD)
-{
+nsParser::GetDTD(nsIDTD** aDTD) {
   if (mParserContext) {
     NS_IF_ADDREF(*aDTD = mDTD);
   }
@@ -1600,8 +1468,4 @@ nsParser::GetDTD(nsIDTD** aDTD)
 /**
  * Get this as nsIStreamListener
  */
-nsIStreamListener*
-nsParser::GetStreamListener()
-{
-  return this;
-}
+nsIStreamListener* nsParser::GetStreamListener() { return this; }
