@@ -2,8 +2,6 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-import binascii
-
 
 def _file_byte_generator(filename):
     with open(filename, "rb") as f:
@@ -19,7 +17,8 @@ def _file_byte_generator(filename):
 
 
 def _create_header(array_name, cert_bytes):
-    hexified = ["0x" + binascii.hexlify(byte) for byte in cert_bytes]
+    hexified = ["0x%02x" % byte for byte in cert_bytes]
+
     substs = {'array_name': array_name, 'bytes': ', '.join(hexified)}
     return "const uint8_t %(array_name)s[] = {\n%(bytes)s\n};\n" % substs
 
@@ -34,7 +33,6 @@ array_names = [
     'addonsPublicRoot',
     'addonsPublicIntermediate',
     'addonsStageRoot',
-    'privilegedPackageRoot',
 ]
 
 for n in array_names:
