@@ -18,6 +18,7 @@
 #include "mozilla/Logging.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/Sprintf.h"
+#include "mozilla/StaticPrefs_network.h"
 #include "mozilla/Telemetry.h"
 #include "nsCOMPtr.h"
 #include "nsComponentManagerUtils.h"
@@ -31,73 +32,69 @@
 #include "pk11pub.h"
 #include "prsystem.h"
 
-static bool sNTLMv1Forced = false;
 static mozilla::LazyLogModule sNTLMLog("NTLM");
 
 #define LOG(x) MOZ_LOG(sNTLMLog, mozilla::LogLevel::Debug, x)
 #define LOG_ENABLED() MOZ_LOG_TEST(sNTLMLog, mozilla::LogLevel::Debug)
 
-static void des_makekey(const uint8_t *raw, uint8_t *key);
-static void des_encrypt(const uint8_t *key, const uint8_t *src, uint8_t *hash);
+static void des_makekey(const uint8_t* raw, uint8_t* key);
+static void des_encrypt(const uint8_t* key, const uint8_t* src, uint8_t* hash);
 
 //-----------------------------------------------------------------------------
 // this file contains a cross-platform NTLM authentication implementation. it
 // is based on documentation from: http://davenport.sourceforge.net/ntlm.html
 //-----------------------------------------------------------------------------
 
-#define NTLM_NegotiateUnicode               0x00000001
-#define NTLM_NegotiateOEM                   0x00000002
-#define NTLM_RequestTarget                  0x00000004
-#define NTLM_Unknown1                       0x00000008
-#define NTLM_NegotiateSign                  0x00000010
-#define NTLM_NegotiateSeal                  0x00000020
-#define NTLM_NegotiateDatagramStyle         0x00000040
-#define NTLM_NegotiateLanManagerKey         0x00000080
-#define NTLM_NegotiateNetware               0x00000100
-#define NTLM_NegotiateNTLMKey               0x00000200
-#define NTLM_Unknown2                       0x00000400
-#define NTLM_Unknown3                       0x00000800
-#define NTLM_NegotiateDomainSupplied        0x00001000
-#define NTLM_NegotiateWorkstationSupplied   0x00002000
-#define NTLM_NegotiateLocalCall             0x00004000
-#define NTLM_NegotiateAlwaysSign            0x00008000
-#define NTLM_TargetTypeDomain               0x00010000
-#define NTLM_TargetTypeServer               0x00020000
-#define NTLM_TargetTypeShare                0x00040000
-#define NTLM_NegotiateNTLM2Key              0x00080000
-#define NTLM_RequestInitResponse            0x00100000
-#define NTLM_RequestAcceptResponse          0x00200000
-#define NTLM_RequestNonNTSessionKey         0x00400000
-#define NTLM_NegotiateTargetInfo            0x00800000
-#define NTLM_Unknown4                       0x01000000
-#define NTLM_Unknown5                       0x02000000
-#define NTLM_Unknown6                       0x04000000
-#define NTLM_Unknown7                       0x08000000
-#define NTLM_Unknown8                       0x10000000
-#define NTLM_Negotiate128                   0x20000000
-#define NTLM_NegotiateKeyExchange           0x40000000
-#define NTLM_Negotiate56                    0x80000000
+#define NTLM_NegotiateUnicode 0x00000001
+#define NTLM_NegotiateOEM 0x00000002
+#define NTLM_RequestTarget 0x00000004
+#define NTLM_Unknown1 0x00000008
+#define NTLM_NegotiateSign 0x00000010
+#define NTLM_NegotiateSeal 0x00000020
+#define NTLM_NegotiateDatagramStyle 0x00000040
+#define NTLM_NegotiateLanManagerKey 0x00000080
+#define NTLM_NegotiateNetware 0x00000100
+#define NTLM_NegotiateNTLMKey 0x00000200
+#define NTLM_Unknown2 0x00000400
+#define NTLM_Unknown3 0x00000800
+#define NTLM_NegotiateDomainSupplied 0x00001000
+#define NTLM_NegotiateWorkstationSupplied 0x00002000
+#define NTLM_NegotiateLocalCall 0x00004000
+#define NTLM_NegotiateAlwaysSign 0x00008000
+#define NTLM_TargetTypeDomain 0x00010000
+#define NTLM_TargetTypeServer 0x00020000
+#define NTLM_TargetTypeShare 0x00040000
+#define NTLM_NegotiateNTLM2Key 0x00080000
+#define NTLM_RequestInitResponse 0x00100000
+#define NTLM_RequestAcceptResponse 0x00200000
+#define NTLM_RequestNonNTSessionKey 0x00400000
+#define NTLM_NegotiateTargetInfo 0x00800000
+#define NTLM_Unknown4 0x01000000
+#define NTLM_Unknown5 0x02000000
+#define NTLM_Unknown6 0x04000000
+#define NTLM_Unknown7 0x08000000
+#define NTLM_Unknown8 0x10000000
+#define NTLM_Negotiate128 0x20000000
+#define NTLM_NegotiateKeyExchange 0x40000000
+#define NTLM_Negotiate56 0x80000000
 
 // we send these flags with our type 1 message
-#define NTLM_TYPE1_FLAGS      \
-  (NTLM_NegotiateUnicode |    \
-   NTLM_NegotiateOEM |        \
-   NTLM_RequestTarget |       \
-   NTLM_NegotiateNTLMKey |    \
-   NTLM_NegotiateAlwaysSign | \
-   NTLM_NegotiateNTLM2Key)
+#define NTLM_TYPE1_FLAGS                                            \
+  (NTLM_NegotiateUnicode | NTLM_NegotiateOEM | NTLM_RequestTarget | \
+   NTLM_NegotiateNTLMKey | NTLM_NegotiateAlwaysSign | NTLM_NegotiateNTLM2Key)
 
 static const char NTLM_SIGNATURE[] = "NTLMSSP";
-static const char NTLM_TYPE1_MARKER[] = { 0x01, 0x00, 0x00, 0x00 };
-static const char NTLM_TYPE2_MARKER[] = { 0x02, 0x00, 0x00, 0x00 };
-static const char NTLM_TYPE3_MARKER[] = { 0x03, 0x00, 0x00, 0x00 };
+static const char NTLM_TYPE1_MARKER[] = {0x01, 0x00, 0x00, 0x00};
+static const char NTLM_TYPE2_MARKER[] = {0x02, 0x00, 0x00, 0x00};
+static const char NTLM_TYPE3_MARKER[] = {0x03, 0x00, 0x00, 0x00};
 
 #define NTLM_TYPE1_HEADER_LEN 32
 #define NTLM_TYPE2_HEADER_LEN 48
 #define NTLM_TYPE3_HEADER_LEN 64
 
 /**
- * We don't actually send a LM response, but we still have to send something in this spot
+ * We don't actually send a LM response, but we still have to send something in
+ * this spot
  */
 #define LM_RESP_LEN 24
 
@@ -114,13 +111,11 @@ static const char NTLM_TYPE3_MARKER[] = { 0x03, 0x00, 0x00, 0x00 };
 /**
  * Prints a description of flags to the NSPR Log, if enabled.
  */
-static void LogFlags(uint32_t flags)
-{
-  if (!LOG_ENABLED())
-    return;
-#define TEST(_flag) \
-  if (flags & NTLM_ ## _flag) \
-    PR_LogPrint("    0x%08x (" # _flag ")\n", NTLM_ ## _flag)
+static void LogFlags(uint32_t flags) {
+  if (!LOG_ENABLED()) return;
+#define TEST(_flag)         \
+  if (flags & NTLM_##_flag) \
+  PR_LogPrint("    0x%08x (" #_flag ")\n", NTLM_##_flag)
 
   TEST(NegotiateUnicode);
   TEST(NegotiateOEM);
@@ -164,38 +159,30 @@ static void LogFlags(uint32_t flags)
  * @param buf the data to print
  * @param bufLen length of the data
  */
-static void
-LogBuf(const char *tag, const uint8_t *buf, uint32_t bufLen)
-{
+static void LogBuf(const char* tag, const uint8_t* buf, uint32_t bufLen) {
   int i;
 
-  if (!LOG_ENABLED())
-    return;
+  if (!LOG_ENABLED()) return;
 
   PR_LogPrint("%s =\n", tag);
   char line[80];
-  while (bufLen > 0)
-  {
+  while (bufLen > 0) {
     int count = bufLen;
-    if (count > 8)
-      count = 8;
+    if (count > 8) count = 8;
 
     strcpy(line, "    ");
-    for (i=0; i<count; ++i)
-    {
+    for (i = 0; i < count; ++i) {
       int len = strlen(line);
       snprintf(line + len, sizeof(line) - len, "0x%02x ", int(buf[i]));
     }
-    for (; i<8; ++i)
-    {
+    for (; i < 8; ++i) {
       int len = strlen(line);
       snprintf(line + len, sizeof(line) - len, "     ");
     }
 
     int len = strlen(line);
     snprintf(line + len, sizeof(line) - len, "   ");
-    for (i=0; i<count; ++i)
-    {
+    for (i = 0; i < count; ++i) {
       len = strlen(line);
       if (isprint(buf[i]))
         snprintf(line + len, sizeof(line) - len, "%c", buf[i]);
@@ -215,9 +202,7 @@ LogBuf(const char *tag, const uint8_t *buf, uint32_t bufLen)
  * @param token The token to print
  * @param tokenLen length of the data in token
  */
-static void
-LogToken(const char* name, const void* token, uint32_t tokenLen)
-{
+static void LogToken(const char* name, const void* token, uint32_t tokenLen) {
   if (!LOG_ENABLED()) {
     return;
   }
@@ -235,19 +220,15 @@ LogToken(const char* name, const void* token, uint32_t tokenLen)
 //-----------------------------------------------------------------------------
 
 // byte order swapping
-#define SWAP16(x) ((((x) & 0xff) << 8) | (((x) >> 8) & 0xff))
-#define SWAP32(x) ((SWAP16((x) & 0xffff) << 16) | (SWAP16((x) >> 16)))
+#define SWAP16(x) ((((x)&0xff) << 8) | (((x) >> 8) & 0xff))
+#define SWAP32(x) ((SWAP16((x)&0xffff) << 16) | (SWAP16((x) >> 16)))
 
-static void *
-WriteBytes(void *buf, const void *data, uint32_t dataLen)
-{
+static void* WriteBytes(void* buf, const void* data, uint32_t dataLen) {
   memcpy(buf, data, dataLen);
-  return (uint8_t *) buf + dataLen;
+  return (uint8_t*)buf + dataLen;
 }
 
-static void *
-WriteDWORD(void *buf, uint32_t dword)
-{
+static void* WriteDWORD(void* buf, uint32_t dword) {
 #ifdef IS_BIG_ENDIAN
   // NTLM uses little endian on the wire
   dword = SWAP32(dword);
@@ -255,9 +236,7 @@ WriteDWORD(void *buf, uint32_t dword)
   return WriteBytes(buf, &dword, sizeof(dword));
 }
 
-static void *
-WriteSecBuf(void *buf, uint16_t length, uint32_t offset)
-{
+static void* WriteSecBuf(void* buf, uint16_t length, uint32_t offset) {
 #ifdef IS_BIG_ENDIAN
   length = SWAP16(length);
   offset = SWAP32(offset);
@@ -276,14 +255,10 @@ WriteSecBuf(void *buf, uint16_t length, uint32_t offset)
  * to pass the same buffer as both input and output, which is a handy way to
  * convert the unicode buffer to little-endian on big-endian platforms.
  */
-static void *
-WriteUnicodeLE(void *buf, const char16_t *str, uint32_t strLen)
-{
+static void* WriteUnicodeLE(void* buf, const char16_t* str, uint32_t strLen) {
   // convert input string from BE to LE
-  uint8_t *cursor = (uint8_t *) buf,
-          *input  = (uint8_t *) str;
-  for (uint32_t i=0; i<strLen; ++i, input+=2, cursor+=2)
-  {
+  uint8_t *cursor = (uint8_t*)buf, *input = (uint8_t*)str;
+  for (uint32_t i = 0; i < strLen; ++i, input += 2, cursor += 2) {
     // allow for the case where |buf == str|
     uint8_t temp = input[0];
     cursor[0] = input[1];
@@ -293,38 +268,24 @@ WriteUnicodeLE(void *buf, const char16_t *str, uint32_t strLen)
 }
 #endif
 
-static uint16_t
-ReadUint16(const uint8_t *&buf)
-{
-  uint16_t x = ((uint16_t) buf[0]) | ((uint16_t) buf[1] << 8);
+static uint16_t ReadUint16(const uint8_t*& buf) {
+  uint16_t x = ((uint16_t)buf[0]) | ((uint16_t)buf[1] << 8);
   buf += sizeof(x);
   return x;
 }
 
-static uint32_t
-ReadUint32(const uint8_t *&buf)
-{
-  uint32_t x = ( (uint32_t) buf[0])        |
-               (((uint32_t) buf[1]) << 8)  |
-               (((uint32_t) buf[2]) << 16) |
-               (((uint32_t) buf[3]) << 24);
+static uint32_t ReadUint32(const uint8_t*& buf) {
+  uint32_t x = ((uint32_t)buf[0]) | (((uint32_t)buf[1]) << 8) |
+               (((uint32_t)buf[2]) << 16) | (((uint32_t)buf[3]) << 24);
   buf += sizeof(x);
   return x;
 }
 
 //-----------------------------------------------------------------------------
 
-static void
-ZapBuf(void *buf, size_t bufLen)
-{
-  memset(buf, 0, bufLen);
-}
+static void ZapBuf(void* buf, size_t bufLen) { memset(buf, 0, bufLen); }
 
-static void
-ZapString(nsString &s)
-{
-  ZapBuf(s.BeginWriting(), s.Length() * 2);
-}
+static void ZapString(nsString& s) { ZapBuf(s.BeginWriting(), s.Length() * 2); }
 
 /**
  * NTLM_Hash computes the NTLM hash of the given password.
@@ -334,17 +295,15 @@ ZapString(nsString &s)
  * @param hash
  *        16-byte result buffer
  */
-static void
-NTLM_Hash(const nsString &password, unsigned char *hash)
-{
+static void NTLM_Hash(const nsString& password, unsigned char* hash) {
   uint32_t len = password.Length();
-  uint8_t *passbuf;
+  uint8_t* passbuf;
 
 #ifdef IS_BIG_ENDIAN
-  passbuf = (uint8_t *) malloc(len * 2);
+  passbuf = (uint8_t*)malloc(len * 2);
   WriteUnicodeLE(passbuf, password.get(), len);
 #else
-  passbuf = (uint8_t *) password.get();
+  passbuf = (uint8_t*)password.get();
 #endif
 
   md4sum(passbuf, len * 2, hash);
@@ -368,16 +327,15 @@ NTLM_Hash(const nsString &password, unsigned char *hash)
  * @param response
  *        24-byte buffer to contain the LM response upon return
  */
-static void
-LM_Response(const uint8_t *hash, const uint8_t *challenge, uint8_t *response)
-{
+static void LM_Response(const uint8_t* hash, const uint8_t* challenge,
+                        uint8_t* response) {
   uint8_t keybytes[21], k1[8], k2[8], k3[8];
 
   memcpy(keybytes, hash, 16);
   ZapBuf(keybytes + 16, 5);
 
-  des_makekey(keybytes     , k1);
-  des_makekey(keybytes +  7, k2);
+  des_makekey(keybytes, k1);
+  des_makekey(keybytes + 7, k2);
   des_makekey(keybytes + 14, k3);
 
   des_encrypt(k1, challenge, response);
@@ -387,21 +345,17 @@ LM_Response(const uint8_t *hash, const uint8_t *challenge, uint8_t *response)
 
 //-----------------------------------------------------------------------------
 
-static nsresult
-GenerateType1Msg(void **outBuf, uint32_t *outLen)
-{
+static nsresult GenerateType1Msg(void** outBuf, uint32_t* outLen) {
   //
   // verify that bufLen is sufficient
   //
   *outLen = NTLM_TYPE1_HEADER_LEN;
   *outBuf = moz_xmalloc(*outLen);
-  if (!*outBuf)
-    return NS_ERROR_OUT_OF_MEMORY;
 
   //
   // write out type 1 msg
   //
-  void *cursor = *outBuf;
+  void* cursor = *outBuf;
 
   // 0 : signature
   cursor = WriteBytes(cursor, NTLM_SIGNATURE, sizeof(NTLM_SIGNATURE));
@@ -429,19 +383,18 @@ GenerateType1Msg(void **outBuf, uint32_t *outLen)
   return NS_OK;
 }
 
-struct Type2Msg
-{
-  uint32_t    flags;                    // NTLM_Xxx bitwise combination
-  uint8_t     challenge[NTLM_CHAL_LEN]; // 8 byte challenge
-  const uint8_t *target;                // target string (type depends on flags)
-  uint32_t    targetLen;                // target length in bytes
-  const uint8_t *targetInfo;            // target Attribute-Value pairs (DNS domain, et al)
-  uint32_t    targetInfoLen;            // target AV pairs length in bytes
+struct Type2Msg {
+  uint32_t flags;                    // NTLM_Xxx bitwise combination
+  uint8_t challenge[NTLM_CHAL_LEN];  // 8 byte challenge
+  const uint8_t* target;             // target string (type depends on flags)
+  uint32_t targetLen;                // target length in bytes
+  const uint8_t*
+      targetInfo;          // target Attribute-Value pairs (DNS domain, et al)
+  uint32_t targetInfoLen;  // target AV pairs length in bytes
 };
 
-static nsresult
-ParseType2Msg(const void *inBuf, uint32_t inLen, Type2Msg *msg)
-{
+static nsresult ParseType2Msg(const void* inBuf, uint32_t inLen,
+                              Type2Msg* msg) {
   // make sure inBuf is long enough to contain a meaningful type2 msg.
   //
   // 0  NTLMSSP Signature
@@ -452,8 +405,7 @@ ParseType2Msg(const void *inBuf, uint32_t inLen, Type2Msg *msg)
   // 32 targetInfo
   // 48 start of optional data blocks
   //
-  if (inLen < NTLM_TYPE2_HEADER_LEN)
-    return NS_ERROR_UNEXPECTED;
+  if (inLen < NTLM_TYPE2_HEADER_LEN) return NS_ERROR_UNEXPECTED;
 
   auto cursor = static_cast<const uint8_t*>(inBuf);
 
@@ -528,29 +480,25 @@ ParseType2Msg(const void *inBuf, uint32_t inLen, Type2Msg *msg)
   return NS_OK;
 }
 
-static nsresult
-GenerateType3Msg(const nsString &domain,
-                 const nsString &username,
-                 const nsString &password,
-                 const void     *inBuf,
-                 uint32_t        inLen,
-                 void          **outBuf,
-                 uint32_t       *outLen)
-{
+static nsresult GenerateType3Msg(const nsString& domain,
+                                 const nsString& username,
+                                 const nsString& password, const void* inBuf,
+                                 uint32_t inLen, void** outBuf,
+                                 uint32_t* outLen) {
   // inBuf contains Type-2 msg (the challenge) from server
   MOZ_ASSERT(NS_IsMainThread());
   nsresult rv;
   Type2Msg msg;
 
   rv = ParseType2Msg(inBuf, inLen, &msg);
-  if (NS_FAILED(rv))
-    return rv;
+  if (NS_FAILED(rv)) return rv;
 
   bool unicode = (msg.flags & NTLM_NegotiateUnicode);
 
   // There is no negotiation for NTLMv2, so we just do it unless we are forced
   // by explict user configuration to use the older DES-based cryptography.
-  bool ntlmv2 = (sNTLMv1Forced == false);
+  bool ntlmv2 =
+      mozilla::StaticPrefs::network_auth_force_generic_ntlm_v1() == false;
 
   // temporary buffers for unicode strings
 #ifdef IS_BIG_ENDIAN
@@ -571,8 +519,7 @@ GenerateType3Msg(const nsString &domain,
   //
   // get domain name
   //
-  if (unicode)
-  {
+  if (unicode) {
 #ifdef IS_BIG_ENDIAN
     ucsDomainBuf = domain;
     domainPtr = ucsDomainBuf.get();
@@ -584,9 +531,7 @@ GenerateType3Msg(const nsString &domain,
     domainPtr = domain.get();
     domainLen = domain.Length() * 2;
 #endif
-  }
-  else
-  {
+  } else {
     NS_CopyUnicodeToNative(domain, oemDomainBuf);
     domainPtr = oemDomainBuf.get();
     domainLen = oemDomainBuf.Length();
@@ -595,22 +540,18 @@ GenerateType3Msg(const nsString &domain,
   //
   // get user name
   //
-  if (unicode)
-  {
+  if (unicode) {
 #ifdef IS_BIG_ENDIAN
     ucsUserBuf = username;
     userPtr = ucsUserBuf.get();
     userLen = ucsUserBuf.Length() * 2;
     WriteUnicodeLE(const_cast<void*>(userPtr),
-                   static_cast<const char16_t*>(userPtr),
-                   ucsUserBuf.Length());
+                   static_cast<const char16_t*>(userPtr), ucsUserBuf.Length());
 #else
     userPtr = username.get();
     userLen = username.Length() * 2;
 #endif
-  }
-  else
-  {
+  } else {
     NS_CopyUnicodeToNative(username, oemUserBuf);
     userPtr = oemUserBuf.get();
     userLen = oemUserBuf.Length();
@@ -626,19 +567,15 @@ GenerateType3Msg(const nsString &domain,
     return rv;
   }
 
-  if (unicode)
-  {
+  if (unicode) {
     ucsHostBuf = NS_ConvertUTF8toUTF16(hostBuf);
     hostPtr = ucsHostBuf.get();
     hostLen = ucsHostBuf.Length() * 2;
 #ifdef IS_BIG_ENDIAN
     WriteUnicodeLE(const_cast<void*>(hostPtr),
-                   static_cast<const char16_t*>(hostPtr),
-                   ucsHostBuf.Length());
+                   static_cast<const char16_t*>(hostPtr), ucsHostBuf.Length());
 #endif
-  }
-  else
-  {
+  } else {
     hostPtr = hostBuf.get();
     hostLen = hostBuf.Length();
   }
@@ -665,8 +602,8 @@ GenerateType3Msg(const nsString &domain,
     // temporary buffers for unicode strings
     nsAutoString ucsDomainUpperBuf;
     nsAutoString ucsUserUpperBuf;
-    const void *domainUpperPtr;
-    const void *userUpperPtr;
+    const void* domainUpperPtr;
+    const void* userUpperPtr;
     uint32_t domainUpperLen;
     uint32_t userUpperLen;
 
@@ -694,7 +631,8 @@ GenerateType3Msg(const nsString &domain,
 
     NTLM_Hash(password, ntlmHash);
     ntlmHashStr = nsAutoCString(
-      mozilla::BitwiseCast<const char*, const uint8_t*>(ntlmHash), NTLM_HASH_LEN);
+        mozilla::BitwiseCast<const char*, const uint8_t*>(ntlmHash),
+        NTLM_HASH_LEN);
 
     nsCOMPtr<nsIKeyObjectFactory> keyFactory =
         do_CreateInstance(NS_KEYMODULEOBJECTFACTORY_CONTRACTID, &rv);
@@ -709,7 +647,8 @@ GenerateType3Msg(const nsString &domain,
       return rv;
     }
 
-    rv = keyFactory->KeyFromString(nsIKeyObject::HMAC, ntlmHashStr, getter_AddRefs(ntlmKey));
+    rv = keyFactory->KeyFromString(nsIKeyObject::HMAC, ntlmHashStr,
+                                   getter_AddRefs(ntlmKey));
     if (NS_FAILED(rv)) {
       return rv;
     }
@@ -723,7 +662,8 @@ GenerateType3Msg(const nsString &domain,
     if (NS_FAILED(rv)) {
       return rv;
     }
-    rv = hasher->Update(static_cast<const uint8_t*>(userUpperPtr), userUpperLen);
+    rv =
+        hasher->Update(static_cast<const uint8_t*>(userUpperPtr), userUpperLen);
     if (NS_FAILED(rv)) {
       return rv;
     }
@@ -747,7 +687,8 @@ GenerateType3Msg(const nsString &domain,
     }
 
     // Prepare the LMv2 response
-    rv = keyFactory->KeyFromString(nsIKeyObject::HMAC, ntlmv2HashStr, getter_AddRefs(ntlmv2Key));
+    rv = keyFactory->KeyFromString(nsIKeyObject::HMAC, ntlmv2HashStr,
+                                   getter_AddRefs(ntlmv2Key));
     if (NS_FAILED(rv)) {
       return rv;
     }
@@ -780,8 +721,8 @@ GenerateType3Msg(const nsString &domain,
 
     time_t unix_time;
     uint64_t nt_time = time(&unix_time);
-    nt_time += 11644473600LL;    // Number of seconds betwen 1601 and 1970
-    nt_time *= 1000 * 1000 * 10; // Convert seconds to 100 ns units
+    nt_time += 11644473600LL;     // Number of seconds betwen 1601 and 1970
+    nt_time *= 1000 * 1000 * 10;  // Convert seconds to 100 ns units
 
     ntlmv2_blob1[0] = 1;
     ntlmv2_blob1[1] = 1;
@@ -850,7 +791,7 @@ GenerateType3Msg(const nsString &domain,
     }
 
     auto sessionHash = mozilla::BitwiseCast<const uint8_t*, const char*>(
-      sessionHashString.get());
+        sessionHashString.get());
 
     LogBuf("NTLM2 effective key: ", sessionHash, 8);
 
@@ -878,14 +819,11 @@ GenerateType3Msg(const nsString &domain,
   }
   *outBuf = moz_xmalloc(totalLen.value());
   *outLen = totalLen.value();
-  if (!*outBuf) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
 
   //
   // finally, we assemble the Type-3 msg :-)
   //
-  void *cursor = *outBuf;
+  void* cursor = *outBuf;
   mozilla::CheckedInt<uint32_t> offset;
 
   // 0 : signature
@@ -970,21 +908,9 @@ GenerateType3Msg(const nsString &domain,
 
 NS_IMPL_ISUPPORTS(nsNTLMAuthModule, nsIAuthModule)
 
-nsNTLMAuthModule::~nsNTLMAuthModule()
-{
-  ZapString(mPassword);
-}
-
-nsresult
-nsNTLMAuthModule::InitTest()
-{
-  static bool prefObserved = false;
-  if (!prefObserved) {
-    mozilla::Preferences::AddBoolVarCache(
-      &sNTLMv1Forced, "network.auth.force-generic-ntlm-v1", sNTLMv1Forced);
-    prefObserved = true;
-  }
+nsNTLMAuthModule::~nsNTLMAuthModule() { ZapString(mPassword); }
 
+nsresult nsNTLMAuthModule::InitTest() {
   // disable NTLM authentication when FIPS mode is enabled.
   return PK11_IsFIPS() ? NS_ERROR_NOT_AVAILABLE : NS_OK;
 }
@@ -992,10 +918,9 @@ nsNTLMAuthModule::InitTest()
 NS_IMETHODIMP
 nsNTLMAuthModule::Init(const char* /*serviceName*/, uint32_t serviceFlags,
                        const char16_t* domain, const char16_t* username,
-                       const char16_t* password)
-{
+                       const char16_t* password) {
   MOZ_ASSERT((serviceFlags & ~nsIAuthModule::REQ_PROXY_AUTH) ==
-               nsIAuthModule::REQ_DEFAULT,
+                 nsIAuthModule::REQ_DEFAULT,
              "Unexpected service flags");
 
   mDomain = domain;
@@ -1005,23 +930,19 @@ nsNTLMAuthModule::Init(const char* /*serviceName*/, uint32_t serviceFlags,
 
   static bool sTelemetrySent = false;
   if (!sTelemetrySent) {
-      mozilla::Telemetry::Accumulate(
-          mozilla::Telemetry::NTLM_MODULE_USED_2,
-          serviceFlags & nsIAuthModule::REQ_PROXY_AUTH
-              ? NTLM_MODULE_GENERIC_PROXY
-              : NTLM_MODULE_GENERIC_DIRECT);
-      sTelemetrySent = true;
+    mozilla::Telemetry::Accumulate(mozilla::Telemetry::NTLM_MODULE_USED_2,
+                                   serviceFlags & nsIAuthModule::REQ_PROXY_AUTH
+                                       ? NTLM_MODULE_GENERIC_PROXY
+                                       : NTLM_MODULE_GENERIC_DIRECT);
+    sTelemetrySent = true;
   }
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsNTLMAuthModule::GetNextToken(const void *inToken,
-                               uint32_t    inTokenLen,
-                               void      **outToken,
-                               uint32_t   *outTokenLen)
-{
+nsNTLMAuthModule::GetNextToken(const void* inToken, uint32_t inTokenLen,
+                               void** outToken, uint32_t* outTokenLen) {
   nsresult rv;
 
   // disable NTLM authentication when FIPS mode is enabled.
@@ -1035,11 +956,12 @@ nsNTLMAuthModule::GetNextToken(const void *inToken,
     if (inToken) {
       LogToken("in-token", inToken, inTokenLen);
       // Now generate the NTLMSSP_AUTH (type 3)
-      rv = GenerateType3Msg(mDomain, mUsername, mPassword, inToken,
-			    inTokenLen, outToken, outTokenLen);
+      rv = GenerateType3Msg(mDomain, mUsername, mPassword, inToken, inTokenLen,
+                            outToken, outTokenLen);
     } else {
-      LOG(("NTLMSSP_NEGOTIATE already sent and presumably "
-	   "rejected by the server, refusing to send another"));
+      LOG(
+          ("NTLMSSP_NEGOTIATE already sent and presumably "
+           "rejected by the server, refusing to send another"));
       rv = NS_ERROR_UNEXPECTED;
     }
   } else {
@@ -1049,33 +971,26 @@ nsNTLMAuthModule::GetNextToken(const void *inToken,
     } else {
       rv = GenerateType1Msg(outToken, outTokenLen);
       if (NS_SUCCEEDED(rv)) {
-	mNTLMNegotiateSent = true;
+        mNTLMNegotiateSent = true;
       }
     }
   }
 
-  if (NS_SUCCEEDED(rv))
-    LogToken("out-token", *outToken, *outTokenLen);
+  if (NS_SUCCEEDED(rv)) LogToken("out-token", *outToken, *outTokenLen);
 
   return rv;
 }
 
 NS_IMETHODIMP
-nsNTLMAuthModule::Unwrap(const void *inToken,
-                        uint32_t    inTokenLen,
-                        void      **outToken,
-                        uint32_t   *outTokenLen)
-{
+nsNTLMAuthModule::Unwrap(const void* inToken, uint32_t inTokenLen,
+                         void** outToken, uint32_t* outTokenLen) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsNTLMAuthModule::Wrap(const void *inToken,
-                       uint32_t    inTokenLen,
-                       bool        confidential,
-                       void      **outToken,
-                       uint32_t   *outTokenLen)
-{
+nsNTLMAuthModule::Wrap(const void* inToken, uint32_t inTokenLen,
+                       bool confidential, void** outToken,
+                       uint32_t* outTokenLen) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
@@ -1083,12 +998,10 @@ nsNTLMAuthModule::Wrap(const void *inToken,
 // DES support code
 
 // set odd parity bit (in least significant bit position)
-static uint8_t
-des_setkeyparity(uint8_t x)
-{
-  if ((((x >> 7) ^ (x >> 6) ^ (x >> 5) ^
-        (x >> 4) ^ (x >> 3) ^ (x >> 2) ^
-        (x >> 1)) & 0x01) == 0)
+static uint8_t des_setkeyparity(uint8_t x) {
+  if ((((x >> 7) ^ (x >> 6) ^ (x >> 5) ^ (x >> 4) ^ (x >> 3) ^ (x >> 2) ^
+        (x >> 1)) &
+       0x01) == 0)
     x |= 0x01;
   else
     x &= 0xfe;
@@ -1096,9 +1009,7 @@ des_setkeyparity(uint8_t x)
 }
 
 // build 64-bit des key from 56-bit raw key
-static void
-des_makekey(const uint8_t *raw, uint8_t *key)
-{
+static void des_makekey(const uint8_t* raw, uint8_t* key) {
   key[0] = des_setkeyparity(raw[0]);
   key[1] = des_setkeyparity((raw[0] << 7) | (raw[1] >> 1));
   key[2] = des_setkeyparity((raw[1] << 6) | (raw[2] >> 2));
@@ -1110,65 +1021,57 @@ des_makekey(const uint8_t *raw, uint8_t *key)
 }
 
 // run des encryption algorithm (using NSS)
-static void
-des_encrypt(const uint8_t *key, const uint8_t *src, uint8_t *hash)
-{
+static void des_encrypt(const uint8_t* key, const uint8_t* src, uint8_t* hash) {
   CK_MECHANISM_TYPE cipherMech = CKM_DES_ECB;
-  PK11SymKey *symkey = nullptr;
-  PK11Context *ctxt = nullptr;
+  PK11SymKey* symkey = nullptr;
+  PK11Context* ctxt = nullptr;
   SECItem keyItem;
   mozilla::UniqueSECItem param;
   SECStatus rv;
   unsigned int n;
 
   mozilla::UniquePK11SlotInfo slot(PK11_GetBestSlot(cipherMech, nullptr));
-  if (!slot)
-  {
+  if (!slot) {
     NS_ERROR("no slot");
     goto done;
   }
 
   keyItem.data = const_cast<uint8_t*>(key);
   keyItem.len = 8;
-  symkey = PK11_ImportSymKey(slot.get(), cipherMech,
-                             PK11_OriginUnwrap, CKA_ENCRYPT,
-                             &keyItem, nullptr);
-  if (!symkey)
-  {
+  symkey = PK11_ImportSymKey(slot.get(), cipherMech, PK11_OriginUnwrap,
+                             CKA_ENCRYPT, &keyItem, nullptr);
+  if (!symkey) {
     NS_ERROR("no symkey");
     goto done;
   }
 
   // no initialization vector required
   param = mozilla::UniqueSECItem(PK11_ParamFromIV(cipherMech, nullptr));
-  if (!param)
-  {
+  if (!param) {
     NS_ERROR("no param");
     goto done;
   }
 
-  ctxt = PK11_CreateContextBySymKey(cipherMech, CKA_ENCRYPT,
-                                    symkey, param.get());
+  ctxt =
+      PK11_CreateContextBySymKey(cipherMech, CKA_ENCRYPT, symkey, param.get());
   if (!ctxt) {
     NS_ERROR("no context");
     goto done;
   }
 
-  rv = PK11_CipherOp(ctxt, hash, (int *) &n, 8, (uint8_t *) src, 8);
+  rv = PK11_CipherOp(ctxt, hash, (int*)&n, 8, (uint8_t*)src, 8);
   if (rv != SECSuccess) {
     NS_ERROR("des failure");
     goto done;
   }
 
-  rv = PK11_DigestFinal(ctxt, hash+8, &n, 0);
+  rv = PK11_DigestFinal(ctxt, hash + 8, &n, 0);
   if (rv != SECSuccess) {
     NS_ERROR("des failure");
     goto done;
   }
 
 done:
-  if (ctxt)
-    PK11_DestroyContext(ctxt, true);
-  if (symkey)
-    PK11_FreeSymKey(symkey);
+  if (ctxt) PK11_DestroyContext(ctxt, true);
+  if (symkey) PK11_FreeSymKey(symkey);
 }
