@@ -5,8 +5,9 @@
 // Tests that the cert download/import UI correctly identifies the cert being
 // downloaded, and allows the trust of the cert to be specified.
 
-const { MockRegistrar } =
-  ChromeUtils.import("resource://testing-common/MockRegistrar.jsm", {});
+const { MockRegistrar } = ChromeUtils.import(
+  "resource://testing-common/MockRegistrar.jsm"
+);
 
 /**
  * @typedef {TestCase}
@@ -24,12 +25,12 @@ const { MockRegistrar } =
  * @type TestCase[]
  */
 const TEST_CASES = [
-  { certFilename: "has-cn.pem",
-    expectedDisplayString: "Foo",
-    cert: null },
-  { certFilename: "has-empty-subject.pem",
+  { certFilename: "has-cn.pem", expectedDisplayString: "Foo", cert: null },
+  {
+    certFilename: "has-empty-subject.pem",
     expectedDisplayString: "Certificate Authority (unnamed)",
-    cert: null },
+    cert: null,
+  },
 ];
 
 /**
@@ -44,53 +45,36 @@ const TEST_CASES = [
  *            2. The return value nsIWritablePropertyBag2 passed to the dialog.
  */
 function openCertDownloadDialog(cert) {
-  let returnVals = Cc["@mozilla.org/hash-property-bag;1"]
-                     .createInstance(Ci.nsIWritablePropertyBag2);
-  let win = window.openDialog("chrome://pippki/content/downloadcert.xul", "",
-                              "", cert, returnVals);
+  let returnVals = Cc["@mozilla.org/hash-property-bag;1"].createInstance(
+    Ci.nsIWritablePropertyBag2
+  );
+  let win = window.openDialog(
+    "chrome://pippki/content/downloadcert.xhtml",
+    "",
+    "",
+    cert,
+    returnVals
+  );
   return new Promise((resolve, reject) => {
-    win.addEventListener("load", function() {
-      executeSoon(() => resolve([win, returnVals]));
-    }, {once: true});
+    win.addEventListener(
+      "load",
+      function() {
+        executeSoon(() => resolve([win, returnVals]));
+      },
+      { once: true }
+    );
   });
 }
 
-// Mock implementation of nsICertificateDialogs.
-const gCertificateDialogs = {
-  expectedCert: null,
-  viewCertCallCount: 0,
-  confirmDownloadCACert(ctx, cert, trust) {
-    Assert.ok(false, "confirmDownloadCACert() should not have been called");
-  },
-  setPKCS12FilePassword(ctx, password) {
-    Assert.ok(false, "setPKCS12FilePassword() should not have been called");
-  },
-  getPKCS12FilePassword(ctx, password) {
-    Assert.ok(false, "getPKCS12FilePassword() should not have been called");
-  },
-  viewCert(ctx, cert) {
-    this.viewCertCallCount++;
-    Assert.notEqual(cert, null, "Cert to view should not be null");
-    Assert.equal(cert, this.expectedCert,
-                 "Actual and expected cert should match");
-  },
-
-  QueryInterface: XPCOMUtils.generateQI([Ci.nsICertificateDialogs])
-};
-
 add_task(async function setup() {
   for (let testCase of TEST_CASES) {
     testCase.cert = await readCertificate(testCase.certFilename, ",,");
-    Assert.notEqual(testCase.cert, null,
-                    `'${testCase.certFilename}' should have been read`);
+    Assert.notEqual(
+      testCase.cert,
+      null,
+      `'${testCase.certFilename}' should have been read`
+    );
   }
-
-  let certificateDialogsCID =
-    MockRegistrar.register("@mozilla.org/nsCertificateDialogs;1",
-                           gCertificateDialogs);
-  registerCleanupFunction(() => {
-    MockRegistrar.unregister(certificateDialogsCID);
-  });
 });
 
 // Test that the trust header message corresponds to the provided cert, and that
@@ -101,17 +85,12 @@ add_task(async function testTrustHeaderAndViewCertButton() {
     let expectedTrustHeaderString =
       `Do you want to trust \u201C${testCase.expectedDisplayString}\u201D ` +
       "for the following purposes?";
-    Assert.equal(win.document.getElementById("trustHeader").textContent,
-                 expectedTrustHeaderString,
-                 "Actual and expected trust header text should match for " +
-                 `${testCase.certFilename}`);
-
-    gCertificateDialogs.viewCertCallCount = 0;
-    gCertificateDialogs.expectedCert = testCase.cert;
-    info("Pressing View Cert button");
-    win.document.getElementById("viewC-button").doCommand();
-    Assert.equal(gCertificateDialogs.viewCertCallCount, 1,
-                 "viewCert() should've been called once");
+    Assert.equal(
+      win.document.getElementById("trustHeader").textContent,
+      expectedTrustHeaderString,
+      "Actual and expected trust header text should match for " +
+        `${testCase.certFilename}`
+    );
 
     await BrowserTestUtils.closeWindow(win);
   }
@@ -126,12 +105,18 @@ add_task(async function testAcceptDialogReturnValues() {
   win.document.getElementById("download_cert").acceptDialog();
   await BrowserTestUtils.windowClosed(win);
 
-  Assert.ok(retVals.get("importConfirmed"),
-            "Return value should signal user chose to import the cert");
-  Assert.ok(retVals.get("trustForSSL"),
-            "Return value should signal SSL trust checkbox was checked");
-  Assert.ok(!retVals.get("trustForEmail"),
-            "Return value should signal E-mail trust checkbox was unchecked");
+  Assert.ok(
+    retVals.get("importConfirmed"),
+    "Return value should signal user chose to import the cert"
+  );
+  Assert.ok(
+    retVals.get("trustForSSL"),
+    "Return value should signal SSL trust checkbox was checked"
+  );
+  Assert.ok(
+    !retVals.get("trustForEmail"),
+    "Return value should signal E-mail trust checkbox was unchecked"
+  );
 });
 
 // Test that the right values are returned when the dialog is canceled.
@@ -141,6 +126,8 @@ add_task(async function testCancelDialogReturnValues() {
   win.document.getElementById("download_cert").cancelDialog();
   await BrowserTestUtils.windowClosed(win);
 
-  Assert.ok(!retVals.get("importConfirmed"),
-            "Return value should signal user chose not to import the cert");
+  Assert.ok(
+    !retVals.get("importConfirmed"),
+    "Return value should signal user chose not to import the cert"
+  );
 });
