@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -13,13 +13,14 @@ from base64 import b64encode
 from cbor2 import dumps
 from cbor2.types import CBORTag
 from hashlib import sha1, sha256
-import StringIO
 import argparse
+from io import StringIO
 import os
 import pycert
 import pycms
 import pykey
 import re
+import six
 import zipfile
 
 
@@ -109,7 +110,7 @@ def walkDirectory(directory):
     is the full path to the file. The second in each pair is
     the path to the file relative to the directory itself."""
     paths = []
-    for path, dirs, files in os.walk(directory):
+    for path, _dirs, files in os.walk(directory):
         for f in files:
             fullPath = os.path.join(path, f)
             internalPath = re.sub(r'^/', '', fullPath.replace(directory, ''))
@@ -124,7 +125,7 @@ def addManifestEntry(filename, hashes, contents, entries):
     of manifest entries."""
     entry = 'Name: %s\n' % filename
     for (hashFunc, name) in hashes:
-        base64hash = b64encode(hashFunc(contents).digest())
+        base64hash = b64encode(hashFunc(contents).digest()).decode("ascii")
         entry += '%s-Digest: %s\n' % (name, base64hash)
     entries.append(entry)
 
@@ -144,8 +145,8 @@ def getCert(subject, keyName, issuerName, ee, issuerKey=""):
             'extension:keyUsage:cRLSign,keyCertSign'
     if issuerKey:
         certSpecification += '\nissuerKey:%s' % issuerKey
-    certSpecificationStream = StringIO.StringIO()
-    print >>certSpecificationStream, certSpecification
+    certSpecificationStream = StringIO()
+    print(certSpecification, file=certSpecificationStream)
     certSpecificationStream.seek(0)
     return pycert.Certificate(certSpecificationStream)
 
@@ -202,7 +203,7 @@ def signZip(appDirectory, outputFile, issuerName, rootName, manifestHashes,
 
     with zipfile.ZipFile(outputFile, 'w', zipfile.ZIP_DEFLATED) as outZip:
         for (fullPath, internalPath) in walkDirectory(appDirectory):
-            with open(fullPath) as inputFile:
+            with open(fullPath, 'rb') as inputFile:
                 contents = inputFile.read()
             outZip.writestr(internalPath, contents)
 
@@ -212,6 +213,7 @@ def signZip(appDirectory, outputFile, issuerName, rootName, manifestHashes,
         if len(coseAlgorithms) > 0:
             coseManifest = '\n'.join(mfEntries)
             outZip.writestr('META-INF/cose.manifest', coseManifest)
+            coseManifest = six.ensure_binary(coseManifest)
             addManifestEntry('META-INF/cose.manifest', manifestHashes,
                              coseManifest, mfEntries)
             intermediates = []
@@ -221,9 +223,10 @@ def signZip(appDirectory, outputFile, issuerName, rootName, manifestHashes,
                 intermediate = getCert(coseIssuerName, 'default', rootName, False)
                 intermediate = intermediate.toDER()
                 intermediates.append(intermediate)
-            signatures = map(lambda coseAlgorithm:
-                             coseAlgorithmToSignatureParams(coseAlgorithm, coseIssuerName),
-                             coseAlgorithms)
+            signatures = [
+                coseAlgorithmToSignatureParams(coseAlgorithm, coseIssuerName)
+                for coseAlgorithm in coseAlgorithms
+            ]
             coseSignatureBytes = coseSig(coseManifest, intermediates, signatures)
             outZip.writestr('META-INF/cose.sig', coseSignatureBytes)
             addManifestEntry('META-INF/cose.sig', manifestHashes,
@@ -233,20 +236,21 @@ def signZip(appDirectory, outputFile, issuerName, rootName, manifestHashes,
             mfContents = '\n'.join(mfEntries)
             sfContents = 'Signature-Version: 1.0\n'
             for (hashFunc, name) in signatureHashes:
-                base64hash = b64encode(hashFunc(mfContents).digest())
+                hashed = hashFunc(six.ensure_binary(mfContents)).digest()
+                base64hash = b64encode(hashed).decode("ascii")
                 sfContents += '%s-Digest-Manifest: %s\n' % (name, base64hash)
 
             cmsSpecification = ''
             for name in pkcs7Hashes:
                 hashFunc, _ = hashNameToFunctionAndIdentifier(name)
-                cmsSpecification += '%s:%s\n' % (name,
-                                                 hashFunc(sfContents).hexdigest())
+                cmsSpecification += '%s:%s\n' % (
+                    name, hashFunc(six.ensure_binary(sfContents)).hexdigest())
             cmsSpecification += 'signer:\n' + \
                 'issuer:%s\n' % issuerName + \
                 'subject:xpcshell signed app test signer\n' + \
                 'extension:keyUsage:digitalSignature'
-            cmsSpecificationStream = StringIO.StringIO()
-            print >>cmsSpecificationStream, cmsSpecification
+            cmsSpecificationStream = StringIO()
+            print(cmsSpecification, file=cmsSpecificationStream)
             cmsSpecificationStream.seek(0)
             cms = pycms.CMS(cmsSpecificationStream)
             p7 = cms.toDER()
@@ -325,7 +329,7 @@ def main(outputFile, appPath, *args):
     if len(parsed.signature_hash) == 0:
         parsed.signature_hash.append('sha256')
     signZip(appPath, outputFile, parsed.issuer, parsed.root,
-            map(hashNameToFunctionAndIdentifier, parsed.manifest_hash),
-            map(hashNameToFunctionAndIdentifier, parsed.signature_hash),
+            [hashNameToFunctionAndIdentifier(h) for h in parsed.manifest_hash],
+            [hashNameToFunctionAndIdentifier(h) for h in parsed.signature_hash],
             parsed.pkcs7_hash, parsed.cose_sign, parsed.empty_signerInfos,
             int(parsed.pad_headers))
