@@ -7,9 +7,12 @@
 #include "SandboxFilterUtil.h"
 
 #ifndef ANDROID
-#include <linux/ipc.h>
+#  include <linux/ipc.h>
 #endif
 #include <linux/net.h>
+#include <sys/socket.h>
+#include <sys/syscall.h>
+#include <unistd.h>
 
 #include "mozilla/UniquePtr.h"
 #include "sandbox/linux/bpf_dsl/bpf_dsl.h"
@@ -17,13 +20,13 @@
 // Older kernel headers (mostly Android, but also some older desktop
 // distributions) are missing some or all of these:
 #ifndef SYS_ACCEPT4
-#define SYS_ACCEPT4  18
+#  define SYS_ACCEPT4 18
 #endif
 #ifndef SYS_RECVMMSG
-#define SYS_RECVMMSG 19
+#  define SYS_RECVMMSG 19
 #endif
 #ifndef SYS_SENDMMSG
-#define SYS_SENDMMSG 20
+#  define SYS_SENDMMSG 20
 #endif
 
 using namespace sandbox::bpf_dsl;
@@ -31,8 +34,8 @@ using namespace sandbox::bpf_dsl;
 
 namespace mozilla {
 
-sandbox::bpf_dsl::ResultExpr
-SandboxPolicyBase::EvaluateSyscall(int aSysno) const {
+sandbox::bpf_dsl::ResultExpr SandboxPolicyBase::EvaluateSyscall(
+    int aSysno) const {
   switch (aSysno) {
 #ifdef __NR_socketcall
     case __NR_socketcall: {
@@ -47,7 +50,7 @@ SandboxPolicyBase::EvaluateSyscall(int aSysno) const {
       }
       return acc->Default(InvalidSyscall());
     }
-#ifndef ANDROID
+#  ifndef ANDROID
     case __NR_ipc: {
       Arg<int> callAndVersion(0);
       auto call = callAndVersion & 0xFFFF;
@@ -61,11 +64,12 @@ SandboxPolicyBase::EvaluateSyscall(int aSysno) const {
       }
       return acc->Default(InvalidSyscall());
     }
-#endif // ANDROID
-#endif // __NR_socketcall
-#define DISPATCH_SOCKETCALL(sysnum, socketnum)                       \
-    case sysnum:                                                     \
-      return EvaluateSocketCall(socketnum, true).valueOr(InvalidSyscall())
+#  endif  // ANDROID
+#endif    // __NR_socketcall
+          // clang-format off
+#define DISPATCH_SOCKETCALL(sysnum, socketnum) \
+  case sysnum:                                 \
+    return EvaluateSocketCall(socketnum, true).valueOr(InvalidSyscall())
 #ifdef __NR_socket
       DISPATCH_SOCKETCALL(__NR_socket,      SYS_SOCKET);
       DISPATCH_SOCKETCALL(__NR_bind,        SYS_BIND);
@@ -80,7 +84,7 @@ SandboxPolicyBase::EvaluateSyscall(int aSysno) const {
 #ifdef __NR_send
       DISPATCH_SOCKETCALL(__NR_send,        SYS_SEND);
       DISPATCH_SOCKETCALL(__NR_recv,        SYS_RECV);
-#endif // __NR_send
+#endif  // __NR_send
       DISPATCH_SOCKETCALL(__NR_sendto,      SYS_SENDTO);
       DISPATCH_SOCKETCALL(__NR_recvfrom,    SYS_RECVFROM);
       DISPATCH_SOCKETCALL(__NR_shutdown,    SYS_SHUTDOWN);
@@ -91,13 +95,13 @@ SandboxPolicyBase::EvaluateSyscall(int aSysno) const {
       DISPATCH_SOCKETCALL(__NR_accept4,     SYS_ACCEPT4);
       DISPATCH_SOCKETCALL(__NR_recvmmsg,    SYS_RECVMMSG);
       DISPATCH_SOCKETCALL(__NR_sendmmsg,    SYS_SENDMMSG);
-#endif // __NR_socket
+#endif  // __NR_socket
 #undef DISPATCH_SOCKETCALL
 #ifndef __NR_socketcall
 #ifndef ANDROID
-#define DISPATCH_SYSVCALL(sysnum, ipcnum)         \
-    case sysnum:                                  \
-      return EvaluateIpcCall(ipcnum).valueOr(InvalidSyscall())
+#define DISPATCH_SYSVCALL(sysnum, ipcnum) \
+  case sysnum:                            \
+    return EvaluateIpcCall(ipcnum).valueOr(InvalidSyscall())
       DISPATCH_SYSVCALL(__NR_semop,       SEMOP);
       DISPATCH_SYSVCALL(__NR_semget,      SEMGET);
       DISPATCH_SYSVCALL(__NR_semctl,      SEMCTL);
@@ -111,11 +115,31 @@ SandboxPolicyBase::EvaluateSyscall(int aSysno) const {
       DISPATCH_SYSVCALL(__NR_shmget,      SHMGET);
       DISPATCH_SYSVCALL(__NR_shmctl,      SHMCTL);
 #undef DISPATCH_SYSVCALL
-#endif // ANDROID
-#endif // __NR_socketcall
-  default:
-    return InvalidSyscall();
+#endif  // ANDROID
+#endif  // __NR_socketcall
+          // clang-format on
+    default:
+      return InvalidSyscall();
   }
 }
 
+/* static */ bool SandboxPolicyBase::HasSeparateSocketCalls() {
+#ifdef __NR_socket
+  // If there's no socketcall, then obviously there are separate syscalls.
+#  ifdef __NR_socketcall
+  // This could be memoized, but currently it's called at most once
+  // per process.
+  int fd = syscall(__NR_socket, AF_LOCAL, SOCK_STREAM, 0);
+  if (fd < 0) {
+    MOZ_DIAGNOSTIC_ASSERT(errno == ENOSYS);
+    return false;
+  }
+  close(fd);
+#  endif  // __NR_socketcall
+  return true;
+#else   // ifndef __NR_socket
+  return false;
+#endif  // __NR_socket
 }
+
+}  // namespace mozilla
