@@ -10,25 +10,25 @@
 #include <time.h>
 
 #include "mozilla/Assertions.h"
-#include "mozilla/ModuleUtils.h"
+#include "mozilla/Components.h"
 #include "nsCOMPtr.h"
 #include "nsPrintfCString.h"
 #include "nsTArray.h"
 #include "nsXULAppAPI.h"
 
+using namespace mozilla;
+
 namespace mozilla {
 
-class SandboxReportWrapper final : public mozISandboxReport
-{
-public:
+class SandboxReportWrapper final : public mozISandboxReport {
+ public:
   NS_DECL_ISUPPORTS
   NS_DECL_MOZISANDBOXREPORT
 
   explicit SandboxReportWrapper(const SandboxReport& aReport)
-  : mReport(aReport)
-  { }
+      : mReport(aReport) {}
 
-private:
+ private:
   ~SandboxReportWrapper() = default;
   SandboxReport mReport;
 };
@@ -36,15 +36,13 @@ private:
 NS_IMPL_ISUPPORTS(SandboxReportWrapper, mozISandboxReport)
 
 /* readonly attribute uint64_t msecAgo; */
-NS_IMETHODIMP SandboxReportWrapper::GetMsecAgo(uint64_t* aMsec)
-{
-  struct timespec then = mReport.mTime, now = { 0, 0 };
+NS_IMETHODIMP SandboxReportWrapper::GetMsecAgo(uint64_t* aMsec) {
+  struct timespec then = mReport.mTime, now = {0, 0};
   clock_gettime(CLOCK_MONOTONIC_COARSE, &now);
 
-  const uint64_t now_msec =
-    uint64_t(now.tv_sec) * 1000 + now.tv_nsec / 1000000;
+  const uint64_t now_msec = uint64_t(now.tv_sec) * 1000 + now.tv_nsec / 1000000;
   const uint64_t then_msec =
-    uint64_t(then.tv_sec) * 1000 + then.tv_nsec / 1000000;
+      uint64_t(then.tv_sec) * 1000 + then.tv_nsec / 1000000;
   MOZ_DIAGNOSTIC_ASSERT(now_msec >= then_msec);
   if (now_msec >= then_msec) {
     *aMsec = now_msec - then_msec;
@@ -55,57 +53,57 @@ NS_IMETHODIMP SandboxReportWrapper::GetMsecAgo(uint64_t* aMsec)
 }
 
 /* readonly attribute int32_t pid; */
-NS_IMETHODIMP SandboxReportWrapper::GetPid(int32_t *aPid)
-{
+NS_IMETHODIMP SandboxReportWrapper::GetPid(int32_t* aPid) {
   *aPid = mReport.mPid;
   return NS_OK;
 }
 
 /* readonly attribute int32_t tid; */
-NS_IMETHODIMP SandboxReportWrapper::GetTid(int32_t *aTid)
-{
+NS_IMETHODIMP SandboxReportWrapper::GetTid(int32_t* aTid) {
   *aTid = mReport.mTid;
   return NS_OK;
 }
 
 /* readonly attribute ACString procType; */
-NS_IMETHODIMP SandboxReportWrapper::GetProcType(nsACString& aProcType)
-{
+NS_IMETHODIMP SandboxReportWrapper::GetProcType(nsACString& aProcType) {
   switch (mReport.mProcType) {
-  case SandboxReport::ProcType::CONTENT:
-    aProcType.AssignLiteral("content");
-    return NS_OK;
-  case SandboxReport::ProcType::FILE:
-    aProcType.AssignLiteral("file");
-    return NS_OK;
-  case SandboxReport::ProcType::MEDIA_PLUGIN:
-    aProcType.AssignLiteral("mediaPlugin");
-    return NS_OK;
-  default:
-    MOZ_ASSERT(false);
-    return NS_ERROR_UNEXPECTED;
+    case SandboxReport::ProcType::CONTENT:
+      aProcType.AssignLiteral("content");
+      return NS_OK;
+    case SandboxReport::ProcType::FILE:
+      aProcType.AssignLiteral("file");
+      return NS_OK;
+    case SandboxReport::ProcType::MEDIA_PLUGIN:
+      aProcType.AssignLiteral("mediaPlugin");
+      return NS_OK;
+    case SandboxReport::ProcType::RDD:
+      aProcType.AssignLiteral("dataDecoder");
+      return NS_OK;
+    case SandboxReport::ProcType::SOCKET_PROCESS:
+      aProcType.AssignLiteral("socketProcess");
+      return NS_OK;
+    default:
+      MOZ_ASSERT(false);
+      return NS_ERROR_UNEXPECTED;
   }
 }
 
 /* readonly attribute uint32_t syscall; */
-NS_IMETHODIMP SandboxReportWrapper::GetSyscall(uint32_t *aSyscall)
-{
+NS_IMETHODIMP SandboxReportWrapper::GetSyscall(uint32_t* aSyscall) {
   *aSyscall = static_cast<uint32_t>(mReport.mSyscall);
   MOZ_ASSERT(static_cast<SandboxReport::ULong>(*aSyscall) == mReport.mSyscall);
   return NS_OK;
 }
 
 /* readonly attribute uint32_t numArgs; */
-NS_IMETHODIMP SandboxReportWrapper::GetNumArgs(uint32_t *aNumArgs)
-{
+NS_IMETHODIMP SandboxReportWrapper::GetNumArgs(uint32_t* aNumArgs) {
   *aNumArgs = static_cast<uint32_t>(kSandboxSyscallArguments);
   return NS_OK;
 }
 
 /* ACString getArg (in uint32_t aIndex); */
 NS_IMETHODIMP SandboxReportWrapper::GetArg(uint32_t aIndex,
-					   nsACString& aRetval)
-{
+                                           nsACString& aRetval) {
   if (aIndex >= kSandboxSyscallArguments) {
     return NS_ERROR_INVALID_ARG;
   }
@@ -123,18 +121,15 @@ NS_IMETHODIMP SandboxReportWrapper::GetArg(uint32_t aIndex,
   return NS_OK;
 }
 
-class SandboxReportArray final : public mozISandboxReportArray
-{
-public:
+class SandboxReportArray final : public mozISandboxReportArray {
+ public:
   NS_DECL_ISUPPORTS
   NS_DECL_MOZISANDBOXREPORTARRAY
 
   explicit SandboxReportArray(SandboxReporter::Snapshot&& aSnap)
-  : mOffset(aSnap.mOffset)
-  , mArray(Move(aSnap.mReports))
-  { }
+      : mOffset(aSnap.mOffset), mArray(std::move(aSnap.mReports)) {}
 
-private:
+ private:
   ~SandboxReportArray() = default;
   uint64_t mOffset;
   nsTArray<SandboxReport> mArray;
@@ -143,81 +138,59 @@ private:
 NS_IMPL_ISUPPORTS(SandboxReportArray, mozISandboxReportArray)
 
 /* readonly attribute uint64_t begin; */
-NS_IMETHODIMP SandboxReportArray::GetBegin(uint64_t *aBegin)
-{
+NS_IMETHODIMP SandboxReportArray::GetBegin(uint64_t* aBegin) {
   *aBegin = mOffset;
   return NS_OK;
 }
 
 /* readonly attribute uint64_t end; */
-NS_IMETHODIMP SandboxReportArray::GetEnd(uint64_t *aEnd)
-{
+NS_IMETHODIMP SandboxReportArray::GetEnd(uint64_t* aEnd) {
   *aEnd = mOffset + mArray.Length();
   return NS_OK;
 }
 
 /* mozISandboxReport getElement (in uint64_t aIndex); */
-NS_IMETHODIMP SandboxReportArray::GetElement(uint64_t aIndex, mozISandboxReport ** aRetval)
-{
+NS_IMETHODIMP SandboxReportArray::GetElement(uint64_t aIndex,
+                                             mozISandboxReport** aRetval) {
   uint64_t relIndex = aIndex - mOffset;
   if (relIndex >= mArray.Length()) {
     return NS_ERROR_INVALID_ARG;
   }
 
   nsCOMPtr<mozISandboxReport> wrapper =
-    new SandboxReportWrapper(mArray[relIndex]);
+      new SandboxReportWrapper(mArray[relIndex]);
   wrapper.forget(aRetval);
   return NS_OK;
 }
 
-class SandboxReporterWrapper final : public mozISandboxReporter
-{
-public:
+class SandboxReporterWrapper final : public mozISandboxReporter {
+ public:
   NS_DECL_ISUPPORTS
   NS_DECL_MOZISANDBOXREPORTER
 
   SandboxReporterWrapper() = default;
 
-private:
+ private:
   ~SandboxReporterWrapper() = default;
 };
 
 NS_IMPL_ISUPPORTS(SandboxReporterWrapper, mozISandboxReporter)
 
 /* mozISandboxReportArray snapshot(); */
-NS_IMETHODIMP SandboxReporterWrapper::Snapshot(mozISandboxReportArray** aRetval)
-{
+NS_IMETHODIMP SandboxReporterWrapper::Snapshot(
+    mozISandboxReportArray** aRetval) {
   if (!XRE_IsParentProcess()) {
     return NS_ERROR_NOT_AVAILABLE;
   }
 
   nsCOMPtr<mozISandboxReportArray> wrapper =
-    new SandboxReportArray(SandboxReporter::Singleton()->GetSnapshot());
+      new SandboxReportArray(SandboxReporter::Singleton()->GetSnapshot());
   wrapper.forget(aRetval);
   return NS_OK;
 }
 
-NS_GENERIC_FACTORY_CONSTRUCTOR(SandboxReporterWrapper)
+}  // namespace mozilla
 
-NS_DEFINE_NAMED_CID(MOZ_SANDBOX_REPORTER_CID);
-
-static const mozilla::Module::CIDEntry kSandboxReporterCIDs[] = {
-  { &kMOZ_SANDBOX_REPORTER_CID, false, nullptr,
-    SandboxReporterWrapperConstructor },
-  { nullptr }
-};
-
-static const mozilla::Module::ContractIDEntry kSandboxReporterContracts[] = {
-  { MOZ_SANDBOX_REPORTER_CONTRACTID, &kMOZ_SANDBOX_REPORTER_CID },
-  { nullptr }
-};
-
-static const mozilla::Module kSandboxReporterModule = {
-  mozilla::Module::kVersion,
-  kSandboxReporterCIDs,
-  kSandboxReporterContracts
-};
-
-NSMODULE_DEFN(SandboxReporterModule) = &kSandboxReporterModule;
-
-} // namespace mozilla
+NS_IMPL_COMPONENT_FACTORY(mozISandboxReporter) {
+  return MakeAndAddRef<SandboxReporterWrapper>().downcast<nsISupports>();
+}
