@@ -1,11 +1,7 @@
 /* Any copyright is dedicated to the Public Domain.
    http://creativecommons.org/publicdomain/zero/1.0/ */
 
-ChromeUtils.import("resource://gre/modules/Log.jsm");
-ChromeUtils.import("resource://services-sync/constants.js");
-ChromeUtils.import("resource://services-sync/service.js");
-ChromeUtils.import("resource://services-sync/policies.js");
-ChromeUtils.import("resource://services-sync/util.js");
+const { Service } = ChromeUtils.import("resource://services-sync/service.js");
 
 Log.repository.rootLogger.addAppender(new Log.DumpAppender());
 
@@ -36,9 +32,9 @@ add_task(async function test_offline() {
 
 function setup() {
   let janeHelper = track_collections_helper();
-  let janeU      = janeHelper.with_updated_collection;
+  let janeU = janeHelper.with_updated_collection;
   let johnHelper = track_collections_helper();
-  let johnU      = johnHelper.with_updated_collection;
+  let johnU = johnHelper.with_updated_collection;
 
   let server = httpd_setup({
     "/1.1/johndoe/info/collections": login_handling(johnHelper.handler),
@@ -47,10 +43,22 @@ function setup() {
     // We need these handlers because we test login, and login
     // is where keys are generated or fetched.
     // TODO: have Jane fetch her keys, not generate them...
-    "/1.1/johndoe/storage/crypto/keys": johnU("crypto", new ServerWBO("keys").handler()),
-    "/1.1/johndoe/storage/meta/global": johnU("meta", new ServerWBO("global").handler()),
-    "/1.1/janedoe/storage/crypto/keys": janeU("crypto", new ServerWBO("keys").handler()),
-    "/1.1/janedoe/storage/meta/global": janeU("meta", new ServerWBO("global").handler())
+    "/1.1/johndoe/storage/crypto/keys": johnU(
+      "crypto",
+      new ServerWBO("keys").handler()
+    ),
+    "/1.1/johndoe/storage/meta/global": johnU(
+      "meta",
+      new ServerWBO("global").handler()
+    ),
+    "/1.1/janedoe/storage/crypto/keys": janeU(
+      "crypto",
+      new ServerWBO("keys").handler()
+    ),
+    "/1.1/janedoe/storage/meta/global": janeU(
+      "meta",
+      new ServerWBO("global").handler()
+    ),
   });
 
   return server;
@@ -91,17 +99,6 @@ add_task(async function test_login_logout() {
     Assert.equal(Service.status.login, LOGIN_SUCCEEDED);
     Assert.ok(Service.isLoggedIn);
 
-    _("Profile refresh edge case: FxA configured but prefs reset");
-    await Service.startOver();
-    let config = makeIdentityConfig({ username: "johndoe" }, server);
-    config.fxaccount.token.endpoint = server.baseURI + "/1.1/" + config.username + "/";
-    configureFxAccountIdentity(Service.identity, config);
-
-    await Service.login();
-    Assert.equal(Service.status.service, STATUS_OK);
-    Assert.equal(Service.status.login, LOGIN_SUCCEEDED);
-    Assert.ok(Service.isLoggedIn);
-
     _("Logout.");
     Service.logout();
     Assert.ok(!Service.isLoggedIn);
@@ -109,7 +106,6 @@ add_task(async function test_login_logout() {
     _("Logging out again won't do any harm.");
     Service.logout();
     Assert.ok(!Service.isLoggedIn);
-
   } finally {
     Svc.Prefs.resetBranch("");
     await promiseStopServer(server);
@@ -182,13 +178,18 @@ add_task(async function test_login_on_sync() {
 
     // Testing exception handling if master password dialog is canceled.
     // Do this by monkeypatching.
-    Service.identity.unlockAndVerifyAuthState = () => Promise.resolve(MASTER_PASSWORD_LOCKED);
+    Service.identity.unlockAndVerifyAuthState = () =>
+      Promise.resolve(MASTER_PASSWORD_LOCKED);
 
     let cSTCalled = false;
     let lockedSyncCalled = false;
 
-    Service.scheduler.clearSyncTriggers = function() { cSTCalled = true; };
-    Service._lockedSync = async function() { lockedSyncCalled = true; };
+    Service.scheduler.clearSyncTriggers = function() {
+      cSTCalled = true;
+    };
+    Service._lockedSync = async function() {
+      lockedSyncCalled = true;
+    };
 
     _("If master password is canceled, login fails and we report lockage.");
     Assert.ok(!(await Service.login()));
@@ -206,7 +207,6 @@ add_task(async function test_login_on_sync() {
 
     // N.B., a bunch of methods are stubbed at this point. Be careful putting
     // new tests after this point!
-
   } finally {
     Svc.Prefs.resetBranch("");
     await promiseStopServer(server);
