@@ -22,7 +22,6 @@ module.exports = {
     SpecialPowers: false,
     TestUtils: false,
     XPCNativeWrapper: false,
-    XULDocument: false,
     addLoadEvent: false,
     add_task: false,
     content: false,
@@ -42,7 +41,6 @@ module.exports = {
     is: false,
     isnot: false,
     ok: false,
-    privateNoteIntentionalCrash: false,
     record: false,
     registerCleanupFunction: false,
     requestLongerTimeout: false,
