@@ -13,19 +13,20 @@
 // -----------------------------------------------------------------------------
 
 module.exports = function(context) {
-
   // ---------------------------------------------------------------------------
   // Public
   //  --------------------------------------------------------------------------
 
   return {
     "Program > FunctionDeclaration": function(node) {
-      if (node.id.name === "run_test" &&
-          node.body.type === "BlockStatement" &&
-          node.body.body.length === 1 &&
-          node.body.body[0].type === "ExpressionStatement" &&
-          node.body.body[0].expression.type === "CallExpression" &&
-          node.body.body[0].expression.callee.name === "run_next_test") {
+      if (
+        node.id.name === "run_test" &&
+        node.body.type === "BlockStatement" &&
+        node.body.body.length === 1 &&
+        node.body.body[0].type === "ExpressionStatement" &&
+        node.body.body[0].expression.type === "CallExpression" &&
+        node.body.body[0].expression.callee.name === "run_next_test"
+      ) {
         context.report({
           node,
           fix: fixer => {
@@ -52,17 +53,18 @@ module.exports = function(context) {
             return fixer.removeRange([
               // If there's no startNode, we fall back to zero, i.e. start of
               // file.
-              startNode ? (startNode.end + 1) : 0,
+              startNode ? startNode.end + 1 : 0,
               // We know the function is a block and it'll end with }. Normally
               // there's a new line after that, so just advance past it. This
               // may be slightly not dodgy in some cases, but covers the existing
               // cases.
-              node.end + 1
+              node.end + 1,
             ]);
           },
-          message: "Useless run_test function - only contains run_next_test; whole function can be removed"
+          message:
+            "Useless run_test function - only contains run_next_test; whole function can be removed",
         });
       }
-    }
+    },
   };
 };
