@@ -22,6 +22,7 @@
 #include <mach/vm_statistics.h>
 #include <sys/time.h>
 #include <sys/resource.h>
+#include <sys/syscall.h>
 #include <sys/types.h>
 #include <sys/sysctl.h>
 #include <stdarg.h>
@@ -32,24 +33,20 @@
 
 // this port is based off of v8 svn revision 9837
 
-/* static */ int
-Thread::GetCurrentId()
-{
-  return gettid();
+int profiler_current_process_id() { return getpid(); }
+
+int profiler_current_thread_id() {
+  return static_cast<int>(static_cast<pid_t>(syscall(SYS_thread_selfid)));
 }
 
-void*
-GetStackTop(void* aGuess)
-{
+void* GetStackTop(void* aGuess) {
   pthread_t thread = pthread_self();
   return pthread_get_stackaddr_np(thread);
 }
 
-class PlatformData
-{
-public:
-  explicit PlatformData(int aThreadId) : mProfiledThread(mach_thread_self())
-  {
+class PlatformData {
+ public:
+  explicit PlatformData(int aThreadId) : mProfiledThread(mach_thread_self()) {
     MOZ_COUNT_CTOR(PlatformData);
   }
 
@@ -62,7 +59,7 @@ public:
 
   thread_act_t ProfiledThread() { return mProfiledThread; }
 
-private:
+ private:
   // Note: for mProfiledThread Mach primitives are used instead of pthread's
   // because the latter doesn't provide thread manipulation primitives required.
   // For details, consult "Mac OS X Internals" book, Section 7.3.
@@ -72,23 +69,16 @@ private:
 ////////////////////////////////////////////////////////////////////////
 // BEGIN Sampler target specifics
 
-Sampler::Sampler(PSLockRef aLock)
-{
-}
+Sampler::Sampler(PSLockRef aLock) {}
 
-void
-Sampler::Disable(PSLockRef aLock)
-{
-}
+void Sampler::Disable(PSLockRef aLock) {}
 
-template<typename Func>
-void
-Sampler::SuspendAndSampleAndResumeThread(PSLockRef aLock,
-                                         const RegisteredThread& aRegisteredThread,
-                                         const Func& aProcessRegs)
-{
+template <typename Func>
+void Sampler::SuspendAndSampleAndResumeThread(
+    PSLockRef aLock, const RegisteredThread& aRegisteredThread,
+    const TimeStamp& aNow, const Func& aProcessRegs) {
   thread_act_t samplee_thread =
-    aRegisteredThread.GetPlatformData()->ProfiledThread();
+      aRegisteredThread.GetPlatformData()->ProfiledThread();
 
   //----------------------------------------------------------------//
   // Suspend the samplee thread and get its context.
@@ -114,14 +104,13 @@ Sampler::SuspendAndSampleAndResumeThread(PSLockRef aLock,
   thread_state_flavor_t flavor = x86_THREAD_STATE64;
   x86_thread_state64_t state;
   mach_msg_type_number_t count = x86_THREAD_STATE64_COUNT;
-# if __DARWIN_UNIX03
-#  define REGISTER_FIELD(name) __r ## name
-# else
-#  define REGISTER_FIELD(name) r ## name
-# endif  // __DARWIN_UNIX03
+#if __DARWIN_UNIX03
+#  define REGISTER_FIELD(name) __r##name
+#else
+#  define REGISTER_FIELD(name) r##name
+#endif  // __DARWIN_UNIX03
 
-  if (thread_get_state(samplee_thread,
-                       flavor,
+  if (thread_get_state(samplee_thread, flavor,
                        reinterpret_cast<natural_t*>(&state),
                        &count) == KERN_SUCCESS) {
     Registers regs;
@@ -130,7 +119,7 @@ Sampler::SuspendAndSampleAndResumeThread(PSLockRef aLock,
     regs.mFP = reinterpret_cast<Address>(state.REGISTER_FIELD(bp));
     regs.mLR = 0;
 
-    aProcessRegs(regs);
+    aProcessRegs(regs, aNow);
   }
 
 #undef REGISTER_FIELD
@@ -151,9 +140,7 @@ Sampler::SuspendAndSampleAndResumeThread(PSLockRef aLock,
 ////////////////////////////////////////////////////////////////////////
 // BEGIN SamplerThread target specifics
 
-static void*
-ThreadEntry(void* aArg)
-{
+static void* ThreadEntry(void* aArg) {
   auto thread = static_cast<SamplerThread*>(aArg);
   thread->Run();
   return nullptr;
@@ -161,25 +148,26 @@ ThreadEntry(void* aArg)
 
 SamplerThread::SamplerThread(PSLockRef aLock, uint32_t aActivityGeneration,
                              double aIntervalMilliseconds)
-  : Sampler(aLock)
-  , mActivityGeneration(aActivityGeneration)
-  , mIntervalMicroseconds(
-      std::max(1, int(floor(aIntervalMilliseconds * 1000 + 0.5))))
-{
+    : mSampler(aLock),
+      mActivityGeneration(aActivityGeneration),
+      mIntervalMicroseconds(
+          std::max(1, int(floor(aIntervalMilliseconds * 1000 + 0.5)))),
+      mThread{nullptr} {
   pthread_attr_t* attr_ptr = nullptr;
   if (pthread_create(&mThread, attr_ptr, ThreadEntry, this) != 0) {
     MOZ_CRASH("pthread_create failed");
   }
 }
 
-SamplerThread::~SamplerThread()
-{
+SamplerThread::~SamplerThread() {
   pthread_join(mThread, nullptr);
+  // Just in the unlikely case some callbacks were added between the end of the
+  // thread and now.
+  InvokePostSamplingCallbacks(std::move(mPostSamplingCallbackList),
+                              SamplingState::JustStopped);
 }
 
-void
-SamplerThread::SleepMicro(uint32_t aMicroseconds)
-{
+void SamplerThread::SleepMicro(uint32_t aMicroseconds) {
   usleep(aMicroseconds);
   // FIXME: the OSX 10.12 page for usleep says "The usleep() function is
   // obsolescent.  Use nanosleep(2) instead."  This implementation could be
@@ -187,37 +175,24 @@ SamplerThread::SleepMicro(uint32_t aMicroseconds)
   // case where the usleep call is interrupted by a signal.
 }
 
-void
-SamplerThread::Stop(PSLockRef aLock)
-{
-  Sampler::Disable(aLock);
-}
+void SamplerThread::Stop(PSLockRef aLock) { mSampler.Disable(aLock); }
 
 // END SamplerThread target specifics
 ////////////////////////////////////////////////////////////////////////
 
-static void
-PlatformInit(PSLockRef aLock)
-{
-}
+static void PlatformInit(PSLockRef aLock) {}
 
 #if defined(HAVE_NATIVE_UNWIND)
-void
-Registers::SyncPopulate()
-{
-  asm (
+void Registers::SyncPopulate() {
+  asm(
       // Compute caller's %rsp by adding to %rbp:
       // 8 bytes for previous %rbp, 8 bytes for return address
       "leaq 0x10(%%rbp), %0\n\t"
       // Dereference %rbp to get previous %rbp
       "movq (%%rbp), %1\n\t"
-      :
-      "=r"(mSP),
-      "=r"(mFP)
-  );
-  mPC = reinterpret_cast<Address>(__builtin_extract_return_addr(
-                                    __builtin_return_address(0)));
+      : "=r"(mSP), "=r"(mFP));
+  mPC = reinterpret_cast<Address>(
+      __builtin_extract_return_addr(__builtin_return_address(0)));
   mLR = 0;
 }
 #endif
-
