@@ -10,11 +10,11 @@ import sys
 from collections import defaultdict
 
 import mozpack.path as mozpath
+import six
 from moztest.resolve import TestResolver
 
-from .. import preset
 from ..cli import BaseTryParser
-from ..vcs import VCSHelper
+from ..push import build, push_to_try
 
 here = os.path.abspath(os.path.dirname(__file__))
 
@@ -24,6 +24,7 @@ class SyntaxParser(BaseTryParser):
     arguments = [
         [['paths'],
          {'nargs': '*',
+          'default': [],
           'help': 'Paths to search for tests to run on try.',
           }],
         [['-b', '--build'],
@@ -113,7 +114,7 @@ class SyntaxParser(BaseTryParser):
         '--setenv': {
             'action': 'append',
             'dest': 'setenv',
-            'help': 'Set the corresponding variable in the test environment for'
+            'help': 'Set the corresponding variable in the test environment for '
                     'applicable harnesses.',
         },
         '-f': {
@@ -147,7 +148,7 @@ class SyntaxParser(BaseTryParser):
             'help': 'Upload XDB compilation db files generated by hazard build',
         },
     }
-    templates = ['chemspill-prio']
+    task_configs = []
 
     def __init__(self, *args, **kwargs):
         BaseTryParser.__init__(self, *args, **kwargs)
@@ -158,7 +159,7 @@ class SyntaxParser(BaseTryParser):
 
 
 class TryArgumentTokenizer(object):
-    symbols = [("seperator", ","),
+    symbols = [("separator", ","),
                ("list_start", "\["),
                ("list_end", "\]"),
                ("item", "([^,\[\]\s][^,\[\]]+)"),
@@ -207,7 +208,7 @@ class TryArgumentParser(object):
 
     def consume(self):
         try:
-            self.token = self.tokens.next()
+            self.token = next(self.tokens)
         except StopIteration:
             self.token = (self.EOF, None)
 
@@ -222,7 +223,7 @@ class TryArgumentParser(object):
             self.data[value] = []
         self.current_item = value
         self.consume()
-        if self.token[0] == "seperator":
+        if self.token[0] == "separator":
             self.consume()
         elif self.token[0] == "list_start":
             self.consume()
@@ -237,7 +238,7 @@ class TryArgumentParser(object):
         value = self.token[1].strip()
         self.data[self.current_item].append(value)
         self.consume()
-        if self.token[0] == "seperator":
+        if self.token[0] == "separator":
             self.consume()
         elif self.token[0] == "list_end":
             self.consume()
@@ -246,7 +247,7 @@ class TryArgumentParser(object):
             raise ValueError
 
     def after_list_end_state(self):
-        self.expect("seperator")
+        self.expect("separator")
         self.consume()
         self.state = self.item_state
 
@@ -272,6 +273,7 @@ class AutoTry(object):
                             'mochitest-devtools-chrome-e10s-1'],
         'crashtest': ['crashtest', 'crashtest-e10s'],
         'reftest': ['reftest', 'reftest-e10s'],
+        'remote': ['mochitest-remote'],
         'web-platform-tests': ['web-platform-tests-1'],
     }
 
@@ -296,6 +298,8 @@ class AutoTry(object):
         "cppunit",
         "crashtest",
         "firefox-ui-functional",
+        "geckoview",
+        "geckoview-junit",
         "gtest",
         "jittest",
         "jsreftest",
@@ -303,15 +307,14 @@ class AutoTry(object):
         "marionette-e10s",
         "mochitests",
         "reftest",
+        "robocop",
         "web-platform-tests",
         "xpcshell",
     ]
 
-    def __init__(self, topsrcdir, mach_context):
-        self.topsrcdir = topsrcdir
+    def __init__(self):
+        self.topsrcdir = build.topsrcdir
         self._resolver = None
-        self.mach_context = mach_context
-        self.vcs = VCSHelper.create()
 
     @property
     def resolver(self):
@@ -319,7 +322,8 @@ class AutoTry(object):
             self._resolver = TestResolver.from_environment(cwd=here)
         return self._resolver
 
-    def split_try_string(self, data):
+    @classmethod
+    def split_try_string(cls, data):
         return re.findall(r'(?:\[.*?\]|\S)+', data)
 
     def paths_by_flavor(self, paths=None, tags=None):
@@ -387,7 +391,7 @@ class AutoTry(object):
 
         suites = tests if not intersection else {}
         paths = set()
-        for flavor, flavor_tests in paths_by_flavor.iteritems():
+        for flavor, flavor_tests in six.iteritems(paths_by_flavor):
             suite = self.flavor_suites[flavor]
             if suite not in suites and (not intersection or suite in tests):
                 for job_name in self.flavor_jobs[flavor]:
@@ -447,7 +451,7 @@ class AutoTry(object):
             parts.append("--try-test-paths %s" % " ".join(sorted(paths)))
 
         args_by_dest = {v['dest']: k for k, v in SyntaxParser.pass_through_arguments.items()}
-        for dest, value in extras.iteritems():
+        for dest, value in six.iteritems(extras):
             assert dest in args_by_dest
             arg = args_by_dest[dest]
             action = SyntaxParser.pass_through_arguments[arg]['action']
@@ -467,11 +471,11 @@ class AutoTry(object):
         rv = defaultdict(list)
         for item in items:
             parsed = parse_arg(item)
-            for key, values in parsed.iteritems():
+            for key, values in six.iteritems(parsed):
                 rv[key].extend(values)
 
         if not allow_subitems:
-            if not all(item == [] for item in rv.itervalues()):
+            if not all(item == [] for item in six.itervalues(rv)):
                 raise ValueError("Unexpected subitems in argument")
             return rv.keys()
         else:
@@ -542,22 +546,6 @@ class AutoTry(object):
         return kwargs["builds"], platforms, tests, talos, jobs, paths, tags, extra_args
 
     def run(self, **kwargs):
-        if kwargs["mod_presets"]:
-            getattr(preset, kwargs["mod_presets"])(section='try')
-            sys.exit()
-
-        if kwargs["preset"]:
-            value = preset.load(kwargs["preset"], section='try')[0]
-            defaults = vars(SyntaxParser().parse_args(self.split_try_string(value)))
-
-            if defaults is None:
-                print("No saved configuration called %s found in autotry.ini" % kwargs["preset"],
-                      file=sys.stderr)
-
-            for key, value in kwargs.iteritems():
-                if value in (None, []) and key in defaults:
-                    kwargs[key] = defaults[key]
-
         if not any(kwargs[item] for item in ("paths", "tests", "tags")):
             if kwargs['detect_paths']:
                 res = self.resolver.get_outgoing_metadata()
@@ -611,16 +599,16 @@ class AutoTry(object):
 
         if kwargs["verbose"] and paths_by_flavor:
             print('The following tests will be selected: ')
-            for flavor, paths in paths_by_flavor.iteritems():
+            for flavor, paths in six.iteritems(paths_by_flavor):
                 print("%s: %s" % (flavor, ",".join(paths)))
 
         if kwargs["verbose"]:
             print('The following try syntax was calculated:\n%s' % msg)
 
-        self.vcs.push_to_try('syntax', kwargs["message"].format(msg=msg), push=kwargs['push'],
-                             closed_tree=kwargs["closed_tree"])
+        push_to_try('syntax', kwargs["message"].format(msg=msg), push=kwargs['push'],
+                    closed_tree=kwargs["closed_tree"])
 
-        if kwargs["save"]:
-            assert msg.startswith("try: ")
-            msg = msg[len("try: "):]
-            preset.save('try', kwargs["save"], msg)
+
+def run(**kwargs):
+    at = AutoTry()
+    return at.run(**kwargs)
