@@ -8,13 +8,12 @@
 # Author: Darin Fisher
 #
 
-# shellcheck disable=SC1090
-. "$(dirname "$0")/common.sh"
+. $(dirname "$0")/common.sh
 
 # -----------------------------------------------------------------------------
 
 print_usage() {
-  notice "Usage: $(basename "$0") [OPTIONS] ARCHIVE DIRECTORY"
+  notice "Usage: $(basename $0) [OPTIONS] ARCHIVE DIRECTORY"
 }
 
 if [ $# = 0 ]; then
@@ -22,26 +21,32 @@ if [ $# = 0 ]; then
   exit 1
 fi
 
-if [ "$1" = -h ]; then
+if [ $1 = -h ]; then
   print_usage
   notice ""
   notice "The contents of DIRECTORY will be stored in ARCHIVE."
   notice ""
   notice "Options:"
   notice "  -h  show this help text"
+  notice "  -q  be less verbose"
   notice ""
   exit 1
 fi
 
+if [ $1 = -q ]; then
+  QUIET=1
+  shift
+fi
+
 # -----------------------------------------------------------------------------
 
 archive="$1"
 targetdir="$2"
 # Prevent the workdir from being inside the targetdir so it isn't included in
 # the update mar.
-if [ "$(echo "$targetdir" | grep -c '\/$')" = 1 ]; then
+if [ $(echo "$targetdir" | grep -c '\/$') = 1 ]; then
   # Remove the /
-  targetdir=${targetdir%/}
+  targetdir=$(echo "$targetdir" | sed -e 's:\/$::')
 fi
 workdir="$targetdir.work"
 updatemanifestv2="$workdir/updatev2.manifest"
@@ -51,7 +56,7 @@ targetfiles="updatev2.manifest updatev3.manifest"
 mkdir -p "$workdir"
 
 # Generate a list of all files in the target directory.
-pushd "$targetdir" || exit 1
+pushd "$targetdir"
 if test $? -ne 0 ; then
   exit 1
 fi
@@ -63,14 +68,13 @@ if [ ! -f "precomplete" ]; then
   fi
 fi
 
-declare -a files
 list_files files
 
-popd || exit 1
+popd
 
 # Add the type of update to the beginning of the update manifests.
-: > "$updatemanifestv2"
-: > "$updatemanifestv3"
+> "$updatemanifestv2"
+> "$updatemanifestv3"
 notice ""
 notice "Adding type instruction to update manifests"
 notice "       type complete"
@@ -79,9 +83,11 @@ echo "type \"complete\"" >> "$updatemanifestv3"
 
 notice ""
 notice "Adding file add instructions to update manifests"
+num_files=${#files[*]}
+
+for ((i=0; $i<$num_files; i=$i+1)); do
+  f="${files[$i]}"
 
-for f in "${files[@]}"
-do
   if check_for_add_if_not_update "$f"; then
     make_add_if_not_instruction "$f" "$updatemanifestv3"
     if check_for_add_to_manifestv2 "$f"; then
@@ -96,7 +102,7 @@ do
   if [[ -n $MAR_OLD_FORMAT ]]; then
     $BZIP2 -cz9 "$targetdir/$f" > "$workdir/$f"
   else
-    $XZ --compress --x86 --lzma2 --format=xz --check=crc64 --force --stdout "$targetdir/$f" > "$workdir/$f"
+    $XZ --compress $BCJ_OPTIONS --lzma2 --format=xz --check=crc64 --force --stdout "$targetdir/$f" > "$workdir/$f"
   fi
   copy_perm "$targetdir/$f" "$workdir/$f"
 
@@ -112,8 +118,8 @@ if [[ -n $MAR_OLD_FORMAT ]]; then
   $BZIP2 -z9 "$updatemanifestv2" && mv -f "$updatemanifestv2.bz2" "$updatemanifestv2"
   $BZIP2 -z9 "$updatemanifestv3" && mv -f "$updatemanifestv3.bz2" "$updatemanifestv3"
 else
-  $XZ --compress --x86 --lzma2 --format=xz --check=crc64 --force "$updatemanifestv2" && mv -f "$updatemanifestv2.xz" "$updatemanifestv2"
-  $XZ --compress --x86 --lzma2 --format=xz --check=crc64 --force "$updatemanifestv3" && mv -f "$updatemanifestv3.xz" "$updatemanifestv3"
+  $XZ --compress $BCJ_OPTIONS --lzma2 --format=xz --check=crc64 --force "$updatemanifestv2" && mv -f "$updatemanifestv2.xz" "$updatemanifestv2"
+  $XZ --compress $BCJ_OPTIONS --lzma2 --format=xz --check=crc64 --force "$updatemanifestv3" && mv -f "$updatemanifestv3.xz" "$updatemanifestv3"
 fi
 
 mar_command="$MAR"
@@ -121,6 +127,10 @@ if [[ -n $MOZ_PRODUCT_VERSION ]]
 then
   mar_command="$mar_command -V $MOZ_PRODUCT_VERSION"
 fi
+if [[ -n $MAR_CHANNEL_ID ]]
+then
+  mar_command="$mar_command -H $MAR_CHANNEL_ID"
+fi
 mar_command="$mar_command -C \"$workdir\" -c output.mar"
 eval "$mar_command $targetfiles"
 mv -f "$workdir/output.mar" "$archive"
